Author:      Patrick Trentin <patrick.trentin@unitn.it>
Institution: University of Trento

IMPORTANT: the commands listed hereafter should be executed
from within the `floating_point` directory.

# 1. Experiment Setup:

Modify the value of variable BASE_DIR, contained in the .bashrc file, to point
at the FS location in which this file is contained. E.g.,

    export BASE_DIR="/home/account_name/floatingpoint_test/"

# 2. Launching the Experiment:

- Load the environment variables and scripts with

    ~$ source .bashrc

- Protect your computer from trashing, and run the script

    ~$ nohup ./monitor_ram.sh > ram.log 2>&1 &

  in a separate terminal

- Protect your experiment from the zombie invasion, and run the script

    ~$ nohup ./pkiller.sh optimathsat > nohup.log 2>&1 &

  in a separate terminal

- Launch the experiment

    ~$ nohup run_experiment.sh o300 to600 1 600 &

# 3. Collect the Data:

    ~$ python results/uniques_stats_plot.py *.out

# 4. Generate a different benchmark-set

    ~$ source .bashrc
    ~$ run_translation.sh bench obench
    # optional: randomly select at-most 300
    #           formulas from each group
    ~$ run_filter.sh obench o300 1 300


###############################################################################
###############################################################################
###############################################################################

run_experiment.sh <src_dir> <dst_dir> <overwrite> <timeout>
    <src_dir>:   benchmarks directory; just the name, no path/slashes
                 IMPORTANT: must be in the same directory from which
                            the command is being executed.
    <dst_dir>:   output directory; just the name, no path/slashes
                 IMPORTANT: must be in the same directory from which
                            the command is being executed.
    <overwrite>: if equal 1, overwrite any result file if already
                 existing by executing the solver again;
                 if equal 0, do not overwrite result files if
                 already present and skip executing the solver;
    <timeout>:   Number of seconds each solver is allowed to run
                 to solve a given problem before being killed;
                 This number should be the smaller than the timeout
                 value set in the `pkiller.sh` script (if used). 

    This command executes the experiment on a fixed list of configurations.

###############################################################################

run_filter.sh <src_dir> <dst_dir> <overwrite> <threshold>
    <src_dir>:   directory of all benchmarks; just the name, no path/slashes
                 IMPORTANT: must be in the same directory from which
                            the command is being executed.
    <dst_dir>:   directory of benchmarks to be used in the experiment;
                 just the name, no path/slashes
                 IMPORTANT: must be in the same directory from which
                            the command is being executed.
    <overwrite>: delete <dst_dir> if it already exists before executing
                 this command
    <threshold>: upper-bound on the number of benchmarks randomly selected
                 from each of the five categories.

    This command randomly selects the benchmarks to be used in the experiment.

###############################################################################

pkiller.sh

    This command kills any instance of optimathsat executing for over 10 minutes.
    This is only meant to be an extra safe guard against rogues executions that
    may affect the time needed to complete the experiment. Such rogue executions
    should not exist in the first place, because run_experiment.sh uses the
    timeout command to kill optimathsat when it reaches its deadline.

###############################################################################

monitor_ram.sh

    Checks that tue RAM usage does not exceed 80% of the installed memory. If
    that is the case, kills the process using the largest amount of memory
    (possibly even optimathsat). Linux instances with a swap partition experience
    a heavy amount of thrashing and become unusable when the RAM is full.

###############################################################################

python results/uniques_stats_plot.py <result_01.out> ... <result_NN.out>

    Collects the runtime statistics contained in the output files generated
    along the experiment (in the same location from which `run_experiment.sh`
    is being executed), generating a summary table and the corresponding
    plots (if any).

###############################################################################
###############################################################################
###############################################################################
