#!/bin/bash

#
## FUNCTIONS
#
function apply_file () {	# applied over each .mzn file detected
	# src_file=${1} ;

	src_file=${1};

	if [[ ${src_file} =~ \.sat\.smt2 ]]; then
		return
	fi

	if [[ ${src_file} =~ \.unsat\.smt2 ]]; then
		return
	fi

    if [[ ${src_file} =~ \.fp_to_bv\.smt2 ]]; then
        return
    fi


	dst_file=$(echo ${src_file} \
			| sed 's#'${SRC_DIR}'#'${DST_DIR}'#' \
			| sed 's#\.smt2$#\.out#')

	test_formula "${src_file}" "${dst_file}"
	return
}
function apply_dir () {
    re=${1} ; func=${2} ;
    shift ; shift ;

	echo "Directory "`pwd`" ..."

	test $(( $(find . -maxdepth 1 -type d | wc -l) - 1)) -gt 0
	has_subdir=$? # 0 if has sub-dir, 1 otherwise

    test -n "$(find . -name '*.dzn')"
    mzn_only=$((1 - $?)) # 0 if there are only .mzn files, 1 otherwise

	for file in `ls -A ${re} 2>/dev/null`
	do
		eval '${func} `pwd`/${file} ${@} ${has_subdir} ${mzn_only}'
	done
}
function apply_rec () {
    pushd ${1} 1>/dev/null ;
    shift ;

    eval "${@}" ;

    for dir in `ls -d */ 2>/dev/null`
    do
        apply_rec `pwd`/${dir} ${@} ;
    done
    popd 1>/dev/null
}
function cp_dirtree () {
	dest_dir=`pwd | sed 's#'${SRC_DIR}'#'${DST_DIR}'#'`
	if [[ ! ( -d ${dest_dir} ) ]]; then
		mkdir ${dest_dir}
	fi
}

#
## PROBLEM HANDLERS
#


function test_formula()
{
	formula_count=$((formula_count + 1))

	# 0: nonnapapera
	# 1: ciccio
    # theory.fp.mode -- 0: lazy, 1: eager, 2: acdcl

    ########################################################################################
    ########################################################################################
    ########################################################################################

    # REDUCTION TO OMT(BV)

    test_optimathsat_reduction "${@}" "${formula_count}" "smt2" "omsat_e_r_obvbs_ff" \
        -opt.verbose=True \
        -theory.fp.mode=1 \
        -opt.theory.bv.engine=obvbs \
        -opt.theory.bv.polarity=False \
        -opt.theory.bv.branch_preference=False \
        -model_generation=True

    ########################################################################################
    ########################################################################################
    ########################################################################################

    ### LAZY + OMT + LIN
	test_optimathsat "${@}" "${formula_count}" "smt2" "omsat_l_omt_cf" \
		-opt.verbose=True \
		-opt.strategy=lin \
        -theory.fp.mode=0 \
        -opt.theory.fp.engine=omt \
		-opt.theory.fp.polarity=False \
		-opt.theory.fp.branch_preference=False \
		-opt.theory.fp.safe_bits_only=False \
        -theory.fp.bv_combination_enabled=False \
		-model_generation=True

    ### EAGER + OMT + LIN
	test_optimathsat "${@}" "${formula_count}" "smt2" "omsat_e_omt_fff" \
		-opt.verbose=True \
		-opt.strategy=lin \
        -theory.fp.mode=1 \
        -opt.theory.fp.engine=omt \
		-opt.theory.fp.polarity=False \
		-opt.theory.fp.branch_preference=False \
		-opt.theory.fp.safe_bits_only=False \
		-model_generation=True
	test_optimathsat "${@}" "${formula_count}" "smt2" "omsat_e_omt_ftf" \
		-opt.verbose=True \
		-opt.strategy=lin \
        -theory.fp.mode=1 \
        -opt.theory.fp.engine=omt \
		-opt.theory.fp.polarity=False \
		-opt.theory.fp.branch_preference=True \
		-opt.theory.fp.safe_bits_only=False \
		-model_generation=True
	test_optimathsat "${@}" "${formula_count}" "smt2" "omsat_e_omt_tff" \
		-opt.verbose=True \
		-opt.strategy=lin \
        -theory.fp.mode=1 \
        -opt.theory.fp.engine=omt \
		-opt.theory.fp.polarity=True \
		-opt.theory.fp.branch_preference=False \
		-opt.theory.fp.safe_bits_only=False \
		-model_generation=True
	test_optimathsat "${@}" "${formula_count}" "smt2" "omsat_e_omt_ttf" \
		-opt.verbose=True \
		-opt.strategy=lin \
        -theory.fp.mode=1 \
        -opt.theory.fp.engine=omt \
		-opt.theory.fp.polarity=True \
		-opt.theory.fp.branch_preference=True \
		-opt.theory.fp.safe_bits_only=False \
		-model_generation=True

    ########################################################################################
    ########################################################################################
    ########################################################################################

    ### LAZY + OMT + BIN
	test_optimathsat "${@}" "${formula_count}" "smt2" "omsat_l_omt_bin_cf" \
		-opt.verbose=True \
		-opt.strategy=bin \
        -theory.fp.mode=0 \
        -opt.theory.fp.engine=omt \
		-opt.theory.fp.polarity=False \
		-opt.theory.fp.branch_preference=False \
		-opt.theory.fp.safe_bits_only=False \
        -theory.fp.bv_combination_enabled=False \
		-model_generation=True

    ### EAGER + OMT + BIN
	test_optimathsat "${@}" "${formula_count}" "smt2" "omsat_e_omt_bin_fff" \
		-opt.verbose=True \
		-opt.strategy=bin \
        -theory.fp.mode=1 \
        -opt.theory.fp.engine=omt \
		-opt.theory.fp.polarity=False \
		-opt.theory.fp.branch_preference=False \
		-opt.theory.fp.safe_bits_only=False \
		-model_generation=True
	test_optimathsat "${@}" "${formula_count}" "smt2" "omsat_e_omt_bin_ftf" \
		-opt.verbose=True \
		-opt.strategy=bin \
        -theory.fp.mode=1 \
        -opt.theory.fp.engine=omt \
		-opt.theory.fp.polarity=False \
		-opt.theory.fp.branch_preference=True \
		-opt.theory.fp.safe_bits_only=False \
		-model_generation=True
	test_optimathsat "${@}" "${formula_count}" "smt2" "omsat_e_omt_bin_tff" \
		-opt.verbose=True \
		-opt.strategy=bin \
        -theory.fp.mode=1 \
        -opt.theory.fp.engine=omt \
		-opt.theory.fp.polarity=True \
		-opt.theory.fp.branch_preference=False \
		-opt.theory.fp.safe_bits_only=False \
		-model_generation=True
	test_optimathsat "${@}" "${formula_count}" "smt2" "omsat_e_omt_bin_ttf" \
		-opt.verbose=True \
		-opt.strategy=bin \
        -theory.fp.mode=1 \
        -opt.theory.fp.engine=omt \
		-opt.theory.fp.polarity=True \
		-opt.theory.fp.branch_preference=True \
		-opt.theory.fp.safe_bits_only=False \
		-model_generation=True

    ########################################################################################
    ########################################################################################
    ########################################################################################

    ### EAGER + OFPBS
	test_optimathsat "${@}" "${formula_count}" "smt2" "omsat_e_ofpbs_fff" \
		-opt.verbose=True \
		-opt.strategy=lin \
        -theory.fp.mode=1 \
        -opt.theory.fp.engine=ofpbs \
		-opt.theory.fp.polarity=False \
		-opt.theory.fp.branch_preference=False \
		-opt.theory.fp.safe_bits_only=False \
		-model_generation=True
	test_optimathsat "${@}" "${formula_count}" "smt2" "omsat_e_ofpbs_ftf" \
		-opt.verbose=True \
		-opt.strategy=lin \
        -theory.fp.mode=1 \
        -opt.theory.fp.engine=ofpbs \
		-opt.theory.fp.polarity=False \
		-opt.theory.fp.branch_preference=True \
		-opt.theory.fp.safe_bits_only=False \
		-model_generation=True
	test_optimathsat "${@}" "${formula_count}" "smt2" "omsat_e_ofpbs_tff" \
		-opt.verbose=True \
		-opt.strategy=lin \
        -theory.fp.mode=1 \
        -opt.theory.fp.engine=ofpbs \
		-opt.theory.fp.polarity=True \
		-opt.theory.fp.branch_preference=False \
		-opt.theory.fp.safe_bits_only=False \
		-model_generation=True
	test_optimathsat "${@}" "${formula_count}" "smt2" "omsat_e_ofpbs_ttf" \
		-opt.verbose=True \
		-opt.strategy=lin \
        -theory.fp.mode=1 \
        -opt.theory.fp.engine=ofpbs \
		-opt.theory.fp.polarity=True \
		-opt.theory.fp.branch_preference=True \
		-opt.theory.fp.safe_bits_only=False \
		-model_generation=True

    ### EAGER + OFPBS + SAFE ONLY
	test_optimathsat "${@}" "${formula_count}" "smt2" "omsat_e_ofpbs_fft" \
		-opt.verbose=True \
		-opt.strategy=lin \
        -theory.fp.mode=1 \
        -opt.theory.fp.engine=ofpbs \
		-opt.theory.fp.polarity=False \
		-opt.theory.fp.branch_preference=False \
		-opt.theory.fp.safe_bits_only=True \
		-model_generation=True
	test_optimathsat "${@}" "${formula_count}" "smt2" "omsat_e_ofpbs_ftt" \
		-opt.verbose=True \
		-opt.strategy=lin \
        -theory.fp.mode=1 \
        -opt.theory.fp.engine=ofpbs \
		-opt.theory.fp.polarity=False \
		-opt.theory.fp.branch_preference=True \
		-opt.theory.fp.safe_bits_only=True \
		-model_generation=True
	test_optimathsat "${@}" "${formula_count}" "smt2" "omsat_e_ofpbs_tft" \
		-opt.verbose=True \
		-opt.strategy=lin \
        -theory.fp.mode=1 \
        -opt.theory.fp.engine=ofpbs \
		-opt.theory.fp.polarity=True \
		-opt.theory.fp.branch_preference=False \
		-opt.theory.fp.safe_bits_only=True \
		-model_generation=True
	test_optimathsat "${@}" "${formula_count}" "smt2" "omsat_e_ofpbs_ttt" \
		-opt.verbose=True \
		-opt.strategy=lin \
        -theory.fp.mode=1 \
        -opt.theory.fp.engine=ofpbs \
		-opt.theory.fp.polarity=True \
		-opt.theory.fp.branch_preference=True \
		-opt.theory.fp.safe_bits_only=True \
		-model_generation=True

    ########################################################################################
    ########################################################################################
    ########################################################################################

    # ONLY SAT

    test_mathsat "${@}" "${formula_count}" "smt2" "smt_only" \
        -opt.verbose=True \
        -theory.fp.mode=1 \
        -model_generation=True

    ########################################################################################
    ########################################################################################
    ########################################################################################

    return

    # The following configurations have not been included in the official experiment.

    ########################################################################################
    ########################################################################################
    ########################################################################################

    ### LAZY + OFPBLS
	test_optimathsat "${@}" "${formula_count}" "smt2" "omsat_l_ofpbls_cf" \
		-opt.verbose=True \
		-opt.strategy=lin \
        -theory.fp.mode=0 \
        -opt.theory.fp.engine=ofpbls \
		-opt.theory.fp.polarity=False \
		-opt.theory.fp.branch_preference=False \
		-opt.theory.fp.safe_bits_only=False \
        -theory.fp.bv_combination_enabled=False \
		-model_generation=True

    ### EAGER + OFPBLS
	test_optimathsat "${@}" "${formula_count}" "smt2" "omsat_e_ofpbls_cf" \
		-opt.verbose=True \
		-opt.strategy=lin \
        -theory.fp.mode=1 \
        -opt.theory.fp.engine=ofpbls \
		-opt.theory.fp.polarity=False \
		-opt.theory.fp.branch_preference=False \
		-opt.theory.fp.safe_bits_only=False \
        -theory.fp.bv_combination_enabled=False \
		-model_generation=True

    ### ACDCL + OFPBLS
	test_optimathsat "${@}" "${formula_count}" "smt2" "omsat_a_ofpbls_cf" \
		-opt.verbose=True \
		-opt.strategy=lin \
        -theory.fp.mode=2 \
        -opt.theory.fp.engine=ofpbls \
		-opt.theory.fp.polarity=False \
		-opt.theory.fp.branch_preference=False \
		-opt.theory.fp.safe_bits_only=False \
        -theory.fp.bv_combination_enabled=False \
		-model_generation=True

    ########################################################################################
    ########################################################################################
    ########################################################################################
}

function test_mathsat()
{
	local tool_ret=1
	local src_file="${1}"
	local dst_file=$(echo "${2}" | sed 's#\.out#.'${5}'.out#')
	local uid="${3}"
	local label="${5}"
    local mst_file=$(echo "${1}" | sed 's#\.smt2#.mst.out#')

    cp "${src_file}" "${mst_file}"
    sed -i 's@(maximize \(.*\))@;(maximize \1)@;s@(minimize \(.*\))@;(minimize \1)@' "${mst_file}"

	run_optimathsat "${mst_file}" "${dst_file}" "${OVERWRITE}" "${TIMEOUT}" ${@:6}
	tool_ret=$?

    rm "${mst_file}"

    print_statistics "${src_file}" "${dst_file}" "${uid}" "${label}" "mathsat" \
                     "${tool_ret}" "0" "0" \
                     "${TIMEOUT}" >> "${INIT_DIR}/mathsat_${label}.out"
}

function test_optimathsat()
{
	local tool_ret=1
	local sat_ret=1
	local unsat_ret=1
	local src_file="${1}"
	local dst_file=$(echo "${2}" | sed 's#\.out#.'${5}'.out#')
	local uid="${3}"
	local label="${5}"

	run_optimathsat "${src_file}" "${dst_file}" "${OVERWRITE}" "${TIMEOUT}" ${@:6}
	tool_ret=$?

	if (( ! ${tool_ret} )); then
		check_sat "${src_file}" "${dst_file}"
		sat_ret=$?

		check_unsat "${src_file}" "${dst_file}"
		unsat_ret=$?
	fi

    print_statistics "${1}" "${dst_file}" "${3}" "${5}" "optimathsat" \
                     "${tool_ret}" "${sat_ret}" "${unsat_ret}" \
                     "${TIMEOUT}" >> "${INIT_DIR}/optimathsat_${label}.out"
}

function test_z3()
{
    local tool_ret=1
    local sat_ret=1
    local unsat_ret=1
    local src_file="${1}"
    local dst_file=$(echo "${2}" | sed 's#\.out#.'${5}'.out#')
    local uid="${3}"
    local label="${5}"

    run_z3 "${src_file}" "${dst_file}" "${OVERWRITE}" "${TIMEOUT}" ${@:6}
    tool_ret=$?

    if (( ! ${tool_ret} )); then
        check_sat "${src_file}" "${dst_file}"
		sat_ret=$?

		check_unsat "${src_file}" "${dst_file}"
		unsat_ret=$?
	fi

    print_statistics "${1}" "${dst_file}" "${3}" "${5}" "z3" \
                     "${tool_ret}" "${sat_ret}" "${unsat_ret}" \
                     "${TIMEOUT}" >> "${INIT_DIR}/z3_${label}.out"

}


# ${1} : smt2 file
# ${2} : out file
# ${3} : formula number
# ${4} : configuration group
# ${5} : search tool
# ${6} : search tool exit status
# ${7} : SAT test exit status
# ${8} : UNSAT test exit status
# ${9} : timeout
function print_statistics()
{
    echo -en "${3} : "    # formula number
    echo -en "${4} : "    # configuration group
    echo -en "${5} : "    # search tool
    echo -en "${6} : "    # search tool exit status (0: good)
    echo -en "${7} : "    # SAT test exit status    (0: good)
    echo -en "${8} : "    # UNSAT test exit status  (0: good)

    result "${2}"
    echo -en "${result} : "  # sat/unsat/unknown

    tot_time "${2}"
    echo -en "${tot_time} : " # search tool time
    echo -en "${9} : "        # timeout

    echo -en "${1} : "
    echo -en "${2} : "

    if [[ "${result}" =~ ^SAT$ ]]; then
        opt_value "${2}"
        echo -en "${opt_value} : " # optimum value, if any
    else
        echo -en "na : "
    fi

    echo -en "\n"
}

# ${1} : smt2 file
# ${2} : out file
# ${3} : formula number
# ${4} : configuration group
# ${5} : search tool
# ${6} : search tool exit status
# ${9} : timeout
function print_status()
{
    echo -en "${3} : "    # formula number
    echo -en "${4} : "    # configuration group
    echo -en "${5} : "    # search tool
    echo -en "${6} : "    # search tool exit status (0: good)
    echo -en "na : "
    echo -en "na : "

    result "${2}"
    echo -en "${result} : "  # sat/unsat/unknown

    tot_time "${2}"
    echo -en "${tot_time} : " # search tool time
    echo -en "${9} : "        # timeout

    echo -en "${1} : "
    echo -en "${2} : "

    echo -en "na : "

    echo -en "\n"
}


function check_sat()
{
	local tool_ret=1
	local smt2_file="${1}"
	local smt2_out_file="${2}"
	local objective=`grep -E "minimize|maximize" "${smt2_file}" | sed 's/(minimize //;s/(maximize //;s/)//'`

    opt_value "${smt2_out_file}"
    local value="${opt_value}"

    if [[ "${value}" = "unsat" ]]; then
        return 0;
    fi

	local src_file=$(echo "${smt2_file}" | sed 's#\.smt2#\.sat\.smt2#')
	local dst_file=$(echo "${src_file}" \
				| sed 's#'${SRC_DIR}'#'${DST_DIR}'#' \
				| sed 's#\.smt2$#\.out#')

	if [[ ${OVERWRITE} -eq 1 || ! -f ${dst_file} ]]; then
        if grep -q "maximize" "${smt2_file}"; then
    		cat "${smt2_file}" | sed "s@(maximize\(.*\)@(assert (= ${value}\1)@" > "${src_file}"
        else
    		cat "${smt2_file}" | sed "s@(minimize\(.*\)@(assert (= ${value}\1)@" > "${src_file}"
        fi
        cat "${smt2_out_file}" | grep  -E "  \(|\( \(" | sed 's/^( /  /;s/ )$//;s/^  (/(assert (= /;s/$/)/' > to_add.txt
        sed -n -i -e '/(check-sat)/r to_add.txt' -e 1x -e '2,${x;p}' -e '${x;p}' "${src_file}"
        rm to_add.txt
        sed -i "s/(set-info :status sat)/;(set-info :status sat)/" "${src_file}"
        sed -i "s/(get-objectives)/;(get-objectives)/" "${src_file}"
	fi

	run_mathsat "${src_file}" "${dst_file}" "${OVERWRITE}" "${TIMEOUT}"
	tool_ret=$?

	if (( ! ${tool_ret} )); then
		grep -q "^sat" "${dst_file}"
		return $?
	else
		return 1;
	fi
}

function check_unsat()
{
	local tool_ret=1
	local smt2_file="${1}"
	local smt2_out_file="${2}"
	local objective=`grep -E "minimize|maximize" "${smt2_file}" | sed 's/(minimize //;s/(maximize //;s/)//'`

    opt_value "${smt2_out_file}"
    local value="${opt_value}"

	local src_file=$(echo "${smt2_file}" | sed 's#\.smt2#\.unsat\.smt2#')
	local dst_file=$(echo "${src_file}" \
				| sed 's#'${SRC_DIR}'#'${DST_DIR}'#' \
				| sed 's#\.smt2$#\.out#')

	if [[ ${OVERWRITE} -eq 1 || ! -f ${dst_file} ]]; then

        if [[ "${value}" = "unsat" ]]; then
            if grep -q "maximize" "${smt2_file}"; then
        		cat "${smt2_file}" | sed "s@.*maximize.*@@" > "${src_file}"
            else
    		    cat "${smt2_file}" | sed "s@.*minimize.*@@" > "${src_file}"
            fi
        else
            if grep -q "maximize" "${smt2_file}"; then
        		cat "${smt2_file}" | sed "s@(maximize\(.*\)@(assert (fp.lt ${value}\1)@" > "${src_file}"
            else
    	    	cat "${smt2_file}" | sed "s@(minimize\(.*\)@(assert (fp.gt ${value}\1)@" > "${src_file}"
            fi
        fi
        sed -i "s/(set-info :status sat)/(set-info :status unsat)/" "${src_file}"
        sed -i "s/(get-model)/;(get-model)/" "${src_file}"
        sed -i "s/(get-objectives)/;(get-objectives)/" "${src_file}"
	fi

	run_mathsat "${src_file}" "${dst_file}" "${OVERWRITE}" "${TIMEOUT}"
	tool_ret=$?

	if (( ! ${tool_ret} )); then
		grep -q "^unsat" "${dst_file}"
		return $?
	else
		return 1;
	fi
}



function reset_counters()
{
	formula_count=$((0))

	OPTIMATHSAT_ERRORS=$((0))
	OPTIMATHSAT_TIMEOUTS=$((0))
	OPTIMATHSAT_KILLED=$((0))
	OPTIMATHSAT_OKS=$((0))
	OPTIMATHSAT_UNKNOWNS=$((0))
	OPTIMATHSAT_SKIPPED=$((0))
	OPTIMATHSAT_SIGSEVS=$((0))

	MATHSAT_ERRORS=$((0))
	MATHSAT_TIMEOUTS=$((0))
	MATHSAT_KILLED=$((0))
	MATHSAT_OKS=$((0))
	MATHSAT_UNKNOWNS=$((0))
	MATHSAT_SKIPPED=$((0))
	MATHSAT_SIGSEVS=$((0))

	Z3_ERRORS=$((0))
	Z3_TIMEOUTS=$((0))
	Z3_KILLED=$((0))
	Z3_OKS=$((0))
	Z3_UNKNOWNS=$((0))
	Z3_SKIPPED=$((0))
	Z3_SIGSEVS=$((0))
}

function load_resources()
{
	source run_optimathsat.sh
	source run_mathsat.sh
}

## file containing result from time
#
function usr_time()
{
	usr_time="ERROR"

	grep -q "User time (seconds):" ${1} || return 1;

	usr_time=$(grep "User time (seconds):" ${1} | grep -oE "[[:digit:]]+\.[[:digit:]]+")
	return 0;
}

function sys_time()
{
	sys_time="ERROR"

	grep -q "System time (seconds):" ${1} || return 1;

	sys_time=$(grep "System time (seconds):" ${1} | grep -oE "[[:digit:]]+\.[[:digit:]]+")
	return 0;
}

function tot_time()
{
	tot_time="ERROR"

	usr_time ${1} || return 1;
	sys_time ${1} || return 1;

	tot_time=$(python -c "print ${usr_time} + ${sys_time}")
	return 0;
}

function result()
{
	result="ERROR"

	grep -q "^sat" ${1}     && result="SAT"
	grep -q "^unsat" ${1}   && result="UNSAT"
	grep -q "^unknown" ${1} && result="UNKNOWN"

	[[ "${result}" =~ ERROR ]] && return 1

	return 0;
}

function opt_value()
{
	opt_value="ERROR"

    opt_value=$(awk 'f&&NR==f+1; /objectives/ {f=NR}' "${1}" | cut -d\  -f3-6 | sed 's/)//')

    if [[ -z "${opt_value}" ]]; then
        opt_value="---"
    fi

	return 0;
}

#
## REDUCTION
#

function test_optimathsat_reduction()
{
    local tool_ret=1
    local sat_ret=1
    local unsat_ret=1
    local src_file="${1}"
    local dst_file=$(echo "${2}" | sed 's#\.out#.'${5}'.out#')
    local uid="${3}"
    local label="${5}"

    local tmp_file=$(echo "${src_file}" | sed 's#\.smt2#.fp_to_bv.smt2#')

    fp_to_bv.py "${src_file}" > "${tmp_file}"

    run_optimathsat "${tmp_file}" "${dst_file}" "${OVERWRITE}" "${TIMEOUT}" ${@:6}
    tool_ret=$?

    if (( ! ${tool_ret} )); then
        src_file="${1}"
        check_sat_reduction "${src_file}" "${dst_file}" "${tmp_file}"
        sat_ret=$?

        src_file="${1}"
        check_unsat_reduction "${src_file}" "${dst_file}" "${tmp_file}"
        unsat_ret=$?
    fi

    print_statistics_reduction "${1}" "${dst_file}" "${3}" "${5}" "optimathsat" \
                               "${tool_ret}" "${sat_ret}" "${unsat_ret}" \
                               "${TIMEOUT}" "${tmp_file}" >> "${INIT_DIR}/optimathsat_${label}.out"
}

function opt_value_reduction()
{
    opt_value="ERROR"

    opt_value=$(grep "^(objectives" -B 2 "${1}" | head -n 1 | cut -d\  -f3-6 | sed 's/)//')

    if [[ -z "${opt_value}" ]]; then
        opt_value="---"
    fi

    return 0;
}

function check_sat_reduction()
{
    local tool_ret=1
    local smt2_file="${1}"
    local smt2_out_file="${2}"
    local reduction_file="${3}"
    local objective=`grep -E "minimize|maximize" "${smt2_file}" | sed 's/(minimize //;s/(maximize //;s/)//'`

    opt_value "${smt2_out_file}"
    local value="${opt_value}"

    if [[ "${value}" = "unsat" ]]; then
        return 0;
    fi

    opt_value_reduction "${smt2_out_file}"
    value="${opt_value}"

    local src_file=$(echo "${smt2_file}" | sed 's#\.smt2#\.sat\.smt2#')
    local dst_file=$(echo "${src_file}" \
                | sed 's#'${SRC_DIR}'#'${DST_DIR}'#' \
                | sed 's#\.smt2$#\.out#')

    if [[ ${OVERWRITE} -eq 1 || ! -f ${dst_file} ]]; then
        if grep -q "maximize" "${smt2_file}"; then
            cat "${smt2_file}" | sed "s@(maximize\(.*\)@(assert (= ${value}\1)@" > "${src_file}"
        else
            cat "${smt2_file}" | sed "s@(minimize\(.*\)@(assert (= ${value}\1)@" > "${src_file}"
        fi
        cat "${smt2_out_file}" | grep  -E "  \(|\( \(" | sed 's/^( /  /;s/ )$//;s/^  (/(assert (= /;s/$/)/' | grep "my_new_bv_" --invert-match > to_add.txt
        sed -n -i -e '/(check-sat)/r to_add.txt' -e 1x -e '2,${x;p}' -e '${x;p}' "${src_file}"
        rm to_add.txt
        sed -i "s/(set-info :status sat)/;(set-info :status sat)/" "${src_file}"
        sed -i "s/(get-objectives)/;(get-objectives)/" "${src_file}"
    fi

    run_mathsat "${src_file}" "${dst_file}" "${OVERWRITE}" "${TIMEOUT}"
    tool_ret=$?

    if (( ! ${tool_ret} )); then
        grep -q "^sat" "${dst_file}"
        return $?
    else
        return 1;
    fi
}

function check_unsat_reduction()
{
    local tool_ret=1
    local smt2_file="${1}"
    local smt2_out_file="${2}"
    local reduction_file="${3}"
    local objective=`grep -E "minimize|maximize" "${smt2_file}" | sed 's/(minimize //;s/(maximize //;s/)//'`

    opt_value "${smt2_out_file}"
    local value="${opt_value}"

    if [[ "${value}" = "unsat" ]]; then
        return 0;
    else
        opt_value_reduction "${smt2_out_file}"
        value="${opt_value}"
    fi

    local src_file=$(echo "${smt2_file}" | sed 's#\.smt2#\.unsat\.smt2#')
    local dst_file=$(echo "${src_file}" \
                | sed 's#'${SRC_DIR}'#'${DST_DIR}'#' \
                | sed 's#\.smt2$#\.out#')

    if [[ ${OVERWRITE} -eq 1 || ! -f ${dst_file} ]]; then

        if [[ "${value}" = "unsat" ]]; then
            if grep -q "maximize" "${smt2_file}"; then
                cat "${smt2_file}" | sed "s@.*maximize.*@@" > "${src_file}"
            else
                cat "${smt2_file}" | sed "s@.*minimize.*@@" > "${src_file}"
            fi
        else
            if grep -q "maximize" "${smt2_file}"; then
                cat "${smt2_file}" | sed "s@(maximize\(.*\)@(assert (fp.lt ${value}\1)@" > "${src_file}"
            else
                cat "${smt2_file}" | sed "s@(minimize\(.*\)@(assert (fp.gt ${value}\1)@" > "${src_file}"
            fi
        fi
        sed -i "s/(set-info :status sat)/(set-info :status unsat)/" "${src_file}"
        sed -i "s/(get-model)/;(get-model)/" "${src_file}"
        sed -i "s/(get-objectives)/;(get-objectives)/" "${src_file}"
    fi

    run_mathsat "${src_file}" "${dst_file}" "${OVERWRITE}" "${TIMEOUT}"
    tool_ret=$?

    if (( ! ${tool_ret} )); then
        grep -q "^unsat" "${dst_file}"
        return $?
    else
        return 1;
    fi
}


# ${1} : smt2 file
# ${2} : out file
# ${3} : formula number
# ${4} : configuration group
# ${5} : search tool
# ${6} : search tool exit status
# ${7} : SAT test exit status
# ${8} : UNSAT test exit status
# ${9} : timeout
# ${10} : reduction smt2 file
function print_statistics_reduction()
{
    echo -en "${3} : "    # formula number
    echo -en "${4} : "    # configuration group
    echo -en "${5} : "    # search tool
    echo -en "${6} : "    # search tool exit status (0: good)
    echo -en "${7} : "    # SAT test exit status    (0: good)
    echo -en "${8} : "    # UNSAT test exit status  (0: good)

    result "${2}"
    echo -en "${result} : "  # sat/unsat/unknown

    tot_time "${2}"
    echo -en "${tot_time} : " # search tool time
    echo -en "${9} : "        # timeout

    echo -en "${1} : "
    echo -en "${2} : "

    if [[ "${result}" =~ ^SAT$ ]]; then
        opt_value_reduction "${2}"
        echo -en "${opt_value} : " # optimum value, if any
    else
        echo -en "na : "
    fi

    echo -en "\n"
}

#
## MAIN
#

## run_main
#	- ${1}: src_dir
#	- ${2}: dst_dir
#	- ${3}: overwrite
#	- ${4}: timeout
#
function run_main ()
{
	reset_counters

	export INIT_DIR=$(pwd)
	export SRC_REG='\*.smt2'
	export SRC_DIR="${1}"
	export DST_DIR="${2}"
	export OVERWRITE="${3}"
	export TIMEOUT="${4}"

	[ "${OVERWRITE}" ] || OVERWRITE="1" ;
	[ "${TIMEOUT}" ]   || TIMEOUT="0" ;

	[[ ! ( -d "${DST_DIR}" ) ]] && mkdir "${DST_DIR}"

	load_resources

	dirs -c
	apply_rec ${SRC_DIR} cp_dirtree
	apply_rec ${SRC_DIR} apply_dir ${SRC_REG} apply_file
}

if [[ "${BASH_SOURCE[0]}" == "${0}" ]]; then
	run_main "${@}"
else
	:
fi
