#!/bin/bash

#
## FUNCTIONS
#
function apply_file () {	# applied over each .mzn file detected
	local src_file="${1}";

	if [[ ${src_file} =~ \.sat\.smt2 ]]; then
		return
	fi

	if [[ ${src_file} =~ \.unsat\.smt2 ]]; then
		return
	fi

	local dst_file=$(echo ${src_file} \
			        | sed 's#'${SRC_DIR}'#'${DST_DIR}'#')

	cp "${src_file}" "${dst_file}"
	return
}
function apply_dir () {
    re=${1} ; func=${2} ;
    shift ; shift ;

	echo "Directory "`pwd`" ..."

	test $(( $(find . -maxdepth 1 -type d | wc -l) - 1)) -gt 0
	has_subdir=$? # 0 if has sub-dir, 1 otherwise

    test -n "$(find . -name '*.dzn')"
    mzn_only=$((1 - $?)) # 0 if there are only .mzn files, 1 otherwise

	for file in `ls -A ${re} 2>/dev/null`
	do
		eval '${func} `pwd`/${file} ${@} ${has_subdir} ${mzn_only}'
	done
}
function apply_rec () {
    pushd ${1} 1>/dev/null ;
    shift ;

    eval "${@}" ;

    for dir in `ls -d */ 2>/dev/null`
    do
        apply_rec `pwd`/${dir} ${@} ;
    done
    popd 1>/dev/null
}
function cp_dirtree () {
	dest_dir=`pwd | sed 's#'${SRC_DIR}'#'${DST_DIR}'#'`
	if [[ ! ( -d ${dest_dir} ) ]]; then
		mkdir ${dest_dir}
	fi
}

#
## FUNCTIONALITY
#

function filter_copy()
{
    local src_dir="${1}"
    local dst_dir="${1}"

    local num_files=$(find "${src_dir}" -name "*.smt2" -printf '%i\n' | wc -l)

    if (( num_files <= THRESHOLD )); then
    	echo "Directory "$(pwd)"/${src_dir} ... (copy all)"
    	apply_rec "${src_dir}" apply_dir ${SRC_REG} apply_file 1>/dev/null
    else
    	echo "Directory "$(pwd)"/${src_dir} ... (copy some)"

        export -f apply_file

        find "${src_dir}" -type f -name "*.smt2" |\
            shuf -n $(shuf -i"${THRESHOLD}"-"${THRESHOLD}" -n1) |\
            tr '\n' '\0' | \
             xargs -I % -0 -n1 bash -c 'apply_file %'
    fi
}

#
## MAIN
#

## run_main
#	- ${1}: src_dir
#	- ${2}: dst_dir
#	- ${3}: overwrite
#	- ${4}: timeout
#
function run_main ()
{
	export INIT_DIR=$(pwd)
	export SRC_REG='\*.smt2'
	export SRC_DIR="${1}"
	export DST_DIR="${2}"
	export OVERWRITE="${3}"
	export THRESHOLD="${4}"

	[ "${OVERWRITE}" ] || OVERWRITE="1" ;
	[ "${THRESHOLD}" ] || THRESHOLD="0" ;

    if (( 0 < OVERWRITE )); then
        if [[ ( -d "${DST_DIR}" ) ]]; then
            echo "deletion of \"${DST_DIR}\", confirm [enter]"; read
            rm -r "${DST_DIR}"
        fi
    fi

	[[ ! ( -d "${DST_DIR}" ) ]] && mkdir "${DST_DIR}"

	dirs -c
	apply_rec ${SRC_DIR} cp_dirtree
    filter_copy "${SRC_DIR}/griggio" "${DST_DIR}/griggio"
    filter_copy "${SRC_DIR}/heizmann" "${DST_DIR}/heizmann"
    filter_copy "${SRC_DIR}/ramalho" "${DST_DIR}/ramalho"
    filter_copy "${SRC_DIR}/schanda" "${DST_DIR}/schanda"
    filter_copy "${SRC_DIR}/wintersteiger" "${DST_DIR}/wintersteiger"
}

if [[ "${BASH_SOURCE[0]}" == "${0}" ]]; then
	run_main "${@}"
else
	:
fi
