; -*- SMT2 -*-
;
; Author: Patrick Trentin <patrick.trentin@unitn.it>
;
; This file is part of OptiMathSAT.
;
; ADAPTIVE SEARCH:
;     In OptiMathSAT, each objective is associated with a search
;     interval in the form [lower, upper], which can be specified
;     using ':lower' and ':upper' directives when it is defined.
;     In order to use adaptive search, the user needs to provide
;     at least one over-approximation of a bound for the objective
;     function which would be more-than-optimal wrt. the optimization
;     direction.
;     (for more details, see: smtlib2_bounds_semantics.smt2)
;
;     In order to use adaptive search, the following option must be set:
;
;         (set-option :config opt.strategy=ada)
;
;     Similarly to binary search, adaptive search takes fewer
;     search-steps to converge towards the optimal solution than linear
;     search. However, adaptive search can be cheaper than binary search
;     as it uses an internal heuristic to decide when to perform a linear
;     step rather than a binary step.
;
;     When using adaptive-search, it might be convenient to enable the
;     verbose flag on the optimization search as follows:
;
;         (set-option :config opt.verbose=true)
;
;     The following options also affect the adaptive search algorithm
;     when it performs binary steps:
;
; -opt.bin.first_step_linear=BOOL
;          If true, the first search step is forced to be linear. (default: true)
; -opt.bin.max_consecutive=INT
;          Sets the maximum number (> 0) of consecutive binary search steps 
;          before a linear search step is performed for LAR/LIA objectives. 
;          (default: 1) 
; -opt.bin.pivot_position=FLOAT
;          Sets the position of the pivoting cut in binary search. Requires a 
;          value in the interval ]0, 1] interval. (default: 0.5) 
;

; ; ; ; ; ; ; ; ; ; ; ; ; ; ; ; ; ; ; ; ; ; ;
; ; ; ; ; ; ; ; ; ; ; ; ; ; ; ; ; ; ; ; ; ; ;
; ; ; ; ; ; ; ; ; ; ; ; ; ; ; ; ; ; ; ; ; ; ;

(set-option :config opt.strategy=ada)
(set-option :config opt.bin.first_step_linear=false)
(set-option :config opt.bin.max_consecutive=5)
(set-option :config opt.bin.pivot_position=0.5)
(set-option :config opt.verbose=true)

(declare-fun course_load__ARRAY__1 () Real)
(declare-fun course_load__ARRAY__2 () Real)
(declare-fun course_load__ARRAY__3 () Real)
(declare-fun course_load__ARRAY__4 () Real)
(declare-fun course_load__ARRAY__5 () Real)
(declare-fun course_load__ARRAY__6 () Real)
(declare-fun course_load__ARRAY__7 () Real)
(declare-fun course_load__ARRAY__8 () Real)
(declare-fun course_load__ARRAY__9 () Real)
(declare-fun course_load__ARRAY__10 () Real)
(declare-fun course_load__ARRAY__11 () Real)
(declare-fun course_load__ARRAY__12 () Real)
(declare-fun course_load__ARRAY__13 () Real)
(declare-fun course_load__ARRAY__14 () Real)
(declare-fun course_load__ARRAY__15 () Real)
(declare-fun course_load__ARRAY__16 () Real)
(declare-fun course_load__ARRAY__17 () Real)
(declare-fun course_load__ARRAY__18 () Real)
(declare-fun course_load__ARRAY__19 () Real)
(declare-fun course_load__ARRAY__20 () Real)
(declare-fun course_load__ARRAY__21 () Real)
(declare-fun course_load__ARRAY__22 () Real)
(declare-fun course_load__ARRAY__23 () Real)
(declare-fun course_load__ARRAY__24 () Real)
(declare-fun course_load__ARRAY__25 () Real)
(declare-fun course_load__ARRAY__26 () Real)
(declare-fun course_load__ARRAY__27 () Real)
(declare-fun course_load__ARRAY__28 () Real)
(declare-fun course_load__ARRAY__29 () Real)
(declare-fun course_load__ARRAY__30 () Real)
(declare-fun course_load__ARRAY__31 () Real)
(declare-fun course_load__ARRAY__32 () Real)
(declare-fun course_load__ARRAY__33 () Real)
(declare-fun course_load__ARRAY__34 () Real)
(declare-fun course_load__ARRAY__35 () Real)
(declare-fun course_load__ARRAY__36 () Real)
(declare-fun course_load__ARRAY__37 () Real)
(declare-fun course_load__ARRAY__38 () Real)
(declare-fun course_load__ARRAY__39 () Real)
(declare-fun course_load__ARRAY__40 () Real)
(declare-fun course_load__ARRAY__41 () Real)
(declare-fun course_load__ARRAY__42 () Real)
(declare-fun course_load__ARRAY__43 () Real)
(declare-fun course_load__ARRAY__44 () Real)
(declare-fun course_load__ARRAY__45 () Real)
(declare-fun course_load__ARRAY__46 () Real)
(declare-fun course_load__ARRAY__47 () Real)
(declare-fun course_load__ARRAY__48 () Real)
(declare-fun course_load__ARRAY__49 () Real)
(declare-fun course_load__ARRAY__50 () Real)
(assert (= course_load__ARRAY__1 9))
(assert (= course_load__ARRAY__2 1))
(assert (= course_load__ARRAY__3 2))
(assert (= course_load__ARRAY__4 10))
(assert (= course_load__ARRAY__5 3))
(assert (= course_load__ARRAY__6 4))
(assert (= course_load__ARRAY__7 1))
(assert (= course_load__ARRAY__8 10))
(assert (= course_load__ARRAY__9 4))
(assert (= course_load__ARRAY__10 1))
(assert (= course_load__ARRAY__11 1))
(assert (= course_load__ARRAY__12 5))
(assert (= course_load__ARRAY__13 10))
(assert (= course_load__ARRAY__14 9))
(assert (= course_load__ARRAY__15 1))
(assert (= course_load__ARRAY__16 8))
(assert (= course_load__ARRAY__17 5))
(assert (= course_load__ARRAY__18 2))
(assert (= course_load__ARRAY__19 8))
(assert (= course_load__ARRAY__20 6))
(assert (= course_load__ARRAY__21 9))
(assert (= course_load__ARRAY__22 5))
(assert (= course_load__ARRAY__23 10))
(assert (= course_load__ARRAY__24 7))
(assert (= course_load__ARRAY__25 8))
(assert (= course_load__ARRAY__26 3))
(assert (= course_load__ARRAY__27 7))
(assert (= course_load__ARRAY__28 4))
(assert (= course_load__ARRAY__29 10))
(assert (= course_load__ARRAY__30 3))
(assert (= course_load__ARRAY__31 1))
(assert (= course_load__ARRAY__32 10))
(assert (= course_load__ARRAY__33 6))
(assert (= course_load__ARRAY__34 2))
(assert (= course_load__ARRAY__35 9))
(assert (= course_load__ARRAY__36 8))
(assert (= course_load__ARRAY__37 7))
(assert (= course_load__ARRAY__38 2))
(assert (= course_load__ARRAY__39 7))
(assert (= course_load__ARRAY__40 3))
(assert (= course_load__ARRAY__41 4))
(assert (= course_load__ARRAY__42 8))
(assert (= course_load__ARRAY__43 9))
(assert (= course_load__ARRAY__44 3))
(assert (= course_load__ARRAY__45 8))
(assert (= course_load__ARRAY__46 1))
(assert (= course_load__ARRAY__47 2))
(assert (= course_load__ARRAY__48 3))
(assert (= course_load__ARRAY__49 2))
(assert (= course_load__ARRAY__50 9))
(declare-fun BOOL____00001 () Bool)
(declare-fun BOOL____00003 () Bool)
(declare-fun BOOL____00005 () Bool)
(declare-fun BOOL____00007 () Bool)
(declare-fun BOOL____00009 () Bool)
(declare-fun BOOL____00011 () Bool)
(declare-fun BOOL____00013 () Bool)
(declare-fun BOOL____00015 () Bool)
(declare-fun BOOL____00017 () Bool)
(declare-fun BOOL____00019 () Bool)
(declare-fun BOOL____00021 () Bool)
(declare-fun BOOL____00023 () Bool)
(declare-fun BOOL____00025 () Bool)
(declare-fun BOOL____00027 () Bool)
(declare-fun BOOL____00029 () Bool)
(declare-fun BOOL____00031 () Bool)
(declare-fun BOOL____00033 () Bool)
(declare-fun BOOL____00035 () Bool)
(declare-fun BOOL____00037 () Bool)
(declare-fun BOOL____00039 () Bool)
(declare-fun BOOL____00041 () Bool)
(declare-fun BOOL____00043 () Bool)
(declare-fun BOOL____00045 () Bool)
(declare-fun BOOL____00047 () Bool)
(declare-fun BOOL____00049 () Bool)
(declare-fun BOOL____00051 () Bool)
(declare-fun BOOL____00053 () Bool)
(declare-fun BOOL____00055 () Bool)
(declare-fun BOOL____00057 () Bool)
(declare-fun BOOL____00059 () Bool)
(declare-fun BOOL____00061 () Bool)
(declare-fun BOOL____00063 () Bool)
(declare-fun BOOL____00065 () Bool)
(declare-fun BOOL____00067 () Bool)
(declare-fun BOOL____00069 () Bool)
(declare-fun BOOL____00071 () Bool)
(declare-fun BOOL____00073 () Bool)
(declare-fun BOOL____00075 () Bool)
(declare-fun BOOL____00077 () Bool)
(declare-fun BOOL____00079 () Bool)
(declare-fun BOOL____00081 () Bool)
(declare-fun BOOL____00083 () Bool)
(declare-fun BOOL____00085 () Bool)
(declare-fun BOOL____00087 () Bool)
(declare-fun BOOL____00089 () Bool)
(declare-fun BOOL____00091 () Bool)
(declare-fun BOOL____00093 () Bool)
(declare-fun BOOL____00095 () Bool)
(declare-fun BOOL____00097 () Bool)
(declare-fun BOOL____00099 () Bool)
(declare-fun BOOL____00101 () Bool)
(declare-fun BOOL____00103 () Bool)
(declare-fun BOOL____00105 () Bool)
(declare-fun BOOL____00107 () Bool)
(declare-fun BOOL____00109 () Bool)
(declare-fun BOOL____00111 () Bool)
(declare-fun BOOL____00113 () Bool)
(declare-fun BOOL____00115 () Bool)
(declare-fun BOOL____00117 () Bool)
(declare-fun BOOL____00119 () Bool)
(declare-fun BOOL____00121 () Bool)
(declare-fun BOOL____00123 () Bool)
(declare-fun BOOL____00125 () Bool)
(declare-fun BOOL____00127 () Bool)
(declare-fun BOOL____00129 () Bool)
(declare-fun BOOL____00131 () Bool)
(declare-fun BOOL____00133 () Bool)
(declare-fun BOOL____00135 () Bool)
(declare-fun BOOL____00137 () Bool)
(declare-fun BOOL____00139 () Bool)
(declare-fun BOOL____00141 () Bool)
(declare-fun BOOL____00143 () Bool)
(declare-fun BOOL____00145 () Bool)
(declare-fun BOOL____00147 () Bool)
(declare-fun BOOL____00149 () Bool)
(declare-fun BOOL____00151 () Bool)
(declare-fun BOOL____00153 () Bool)
(declare-fun BOOL____00155 () Bool)
(declare-fun BOOL____00157 () Bool)
(declare-fun BOOL____00159 () Bool)
(declare-fun BOOL____00161 () Bool)
(declare-fun BOOL____00163 () Bool)
(declare-fun BOOL____00165 () Bool)
(declare-fun BOOL____00167 () Bool)
(declare-fun BOOL____00169 () Bool)
(declare-fun BOOL____00171 () Bool)
(declare-fun BOOL____00173 () Bool)
(declare-fun BOOL____00175 () Bool)
(declare-fun BOOL____00177 () Bool)
(declare-fun BOOL____00179 () Bool)
(declare-fun BOOL____00181 () Bool)
(declare-fun BOOL____00183 () Bool)
(declare-fun BOOL____00185 () Bool)
(declare-fun BOOL____00187 () Bool)
(declare-fun BOOL____00189 () Bool)
(declare-fun BOOL____00191 () Bool)
(declare-fun BOOL____00193 () Bool)
(declare-fun BOOL____00195 () Bool)
(declare-fun BOOL____00197 () Bool)
(declare-fun BOOL____00199 () Bool)
(declare-fun BOOL____00201 () Bool)
(declare-fun BOOL____00203 () Bool)
(declare-fun BOOL____00205 () Bool)
(declare-fun BOOL____00207 () Bool)
(declare-fun BOOL____00209 () Bool)
(declare-fun BOOL____00211 () Bool)
(declare-fun BOOL____00213 () Bool)
(declare-fun BOOL____00215 () Bool)
(declare-fun BOOL____00217 () Bool)
(declare-fun BOOL____00219 () Bool)
(declare-fun BOOL____00221 () Bool)
(declare-fun BOOL____00223 () Bool)
(declare-fun BOOL____00225 () Bool)
(declare-fun BOOL____00227 () Bool)
(declare-fun BOOL____00229 () Bool)
(declare-fun BOOL____00231 () Bool)
(declare-fun BOOL____00233 () Bool)
(declare-fun BOOL____00235 () Bool)
(declare-fun BOOL____00237 () Bool)
(declare-fun BOOL____00239 () Bool)
(declare-fun BOOL____00241 () Bool)
(declare-fun BOOL____00243 () Bool)
(declare-fun BOOL____00245 () Bool)
(declare-fun BOOL____00247 () Bool)
(declare-fun BOOL____00249 () Bool)
(declare-fun BOOL____00251 () Bool)
(declare-fun BOOL____00253 () Bool)
(declare-fun BOOL____00255 () Bool)
(declare-fun BOOL____00257 () Bool)
(declare-fun BOOL____00259 () Bool)
(declare-fun BOOL____00261 () Bool)
(declare-fun BOOL____00263 () Bool)
(declare-fun BOOL____00265 () Bool)
(declare-fun BOOL____00267 () Bool)
(declare-fun BOOL____00269 () Bool)
(declare-fun BOOL____00271 () Bool)
(declare-fun BOOL____00273 () Bool)
(declare-fun BOOL____00275 () Bool)
(declare-fun BOOL____00277 () Bool)
(declare-fun BOOL____00279 () Bool)
(declare-fun BOOL____00281 () Bool)
(declare-fun BOOL____00283 () Bool)
(declare-fun BOOL____00285 () Bool)
(declare-fun BOOL____00287 () Bool)
(declare-fun BOOL____00289 () Bool)
(declare-fun BOOL____00291 () Bool)
(declare-fun BOOL____00293 () Bool)
(declare-fun BOOL____00295 () Bool)
(declare-fun BOOL____00297 () Bool)
(declare-fun BOOL____00299 () Bool)
(declare-fun BOOL____00301 () Bool)
(declare-fun BOOL____00303 () Bool)
(declare-fun BOOL____00305 () Bool)
(declare-fun BOOL____00307 () Bool)
(declare-fun BOOL____00309 () Bool)
(declare-fun BOOL____00311 () Bool)
(declare-fun BOOL____00313 () Bool)
(declare-fun BOOL____00315 () Bool)
(declare-fun BOOL____00317 () Bool)
(declare-fun BOOL____00319 () Bool)
(declare-fun BOOL____00321 () Bool)
(declare-fun BOOL____00323 () Bool)
(declare-fun BOOL____00325 () Bool)
(declare-fun BOOL____00327 () Bool)
(declare-fun BOOL____00329 () Bool)
(declare-fun BOOL____00331 () Bool)
(declare-fun BOOL____00333 () Bool)
(declare-fun BOOL____00335 () Bool)
(declare-fun BOOL____00337 () Bool)
(declare-fun BOOL____00339 () Bool)
(declare-fun BOOL____00341 () Bool)
(declare-fun BOOL____00343 () Bool)
(declare-fun BOOL____00345 () Bool)
(declare-fun BOOL____00347 () Bool)
(declare-fun BOOL____00349 () Bool)
(declare-fun BOOL____00351 () Bool)
(declare-fun BOOL____00353 () Bool)
(declare-fun BOOL____00355 () Bool)
(declare-fun BOOL____00357 () Bool)
(declare-fun BOOL____00359 () Bool)
(declare-fun BOOL____00361 () Bool)
(declare-fun BOOL____00363 () Bool)
(declare-fun BOOL____00365 () Bool)
(declare-fun BOOL____00367 () Bool)
(declare-fun BOOL____00369 () Bool)
(declare-fun BOOL____00371 () Bool)
(declare-fun BOOL____00373 () Bool)
(declare-fun BOOL____00375 () Bool)
(declare-fun BOOL____00377 () Bool)
(declare-fun BOOL____00379 () Bool)
(declare-fun BOOL____00381 () Bool)
(declare-fun BOOL____00383 () Bool)
(declare-fun BOOL____00385 () Bool)
(declare-fun BOOL____00387 () Bool)
(declare-fun BOOL____00389 () Bool)
(declare-fun BOOL____00391 () Bool)
(declare-fun BOOL____00393 () Bool)
(declare-fun BOOL____00395 () Bool)
(declare-fun BOOL____00397 () Bool)
(declare-fun BOOL____00399 () Bool)
(declare-fun BOOL____00401 () Bool)
(declare-fun BOOL____00403 () Bool)
(declare-fun BOOL____00405 () Bool)
(declare-fun BOOL____00407 () Bool)
(declare-fun BOOL____00409 () Bool)
(declare-fun BOOL____00411 () Bool)
(declare-fun BOOL____00413 () Bool)
(declare-fun BOOL____00415 () Bool)
(declare-fun BOOL____00417 () Bool)
(declare-fun BOOL____00419 () Bool)
(declare-fun BOOL____00421 () Bool)
(declare-fun BOOL____00423 () Bool)
(declare-fun BOOL____00425 () Bool)
(declare-fun BOOL____00427 () Bool)
(declare-fun BOOL____00429 () Bool)
(declare-fun BOOL____00431 () Bool)
(declare-fun BOOL____00433 () Bool)
(declare-fun BOOL____00435 () Bool)
(declare-fun BOOL____00437 () Bool)
(declare-fun BOOL____00439 () Bool)
(declare-fun BOOL____00441 () Bool)
(declare-fun BOOL____00443 () Bool)
(declare-fun BOOL____00445 () Bool)
(declare-fun BOOL____00447 () Bool)
(declare-fun BOOL____00449 () Bool)
(declare-fun BOOL____00451 () Bool)
(declare-fun BOOL____00453 () Bool)
(declare-fun BOOL____00455 () Bool)
(declare-fun BOOL____00457 () Bool)
(declare-fun BOOL____00459 () Bool)
(declare-fun BOOL____00461 () Bool)
(declare-fun BOOL____00463 () Bool)
(declare-fun BOOL____00465 () Bool)
(declare-fun BOOL____00467 () Bool)
(declare-fun BOOL____00469 () Bool)
(declare-fun BOOL____00471 () Bool)
(declare-fun BOOL____00473 () Bool)
(declare-fun BOOL____00475 () Bool)
(declare-fun BOOL____00477 () Bool)
(declare-fun BOOL____00479 () Bool)
(declare-fun BOOL____00481 () Bool)
(declare-fun BOOL____00483 () Bool)
(declare-fun BOOL____00485 () Bool)
(declare-fun BOOL____00487 () Bool)
(declare-fun BOOL____00489 () Bool)
(declare-fun BOOL____00491 () Bool)
(declare-fun BOOL____00493 () Bool)
(declare-fun BOOL____00495 () Bool)
(declare-fun BOOL____00497 () Bool)
(declare-fun BOOL____00499 () Bool)
(declare-fun BOOL____00501 () Bool)
(declare-fun BOOL____00503 () Bool)
(declare-fun BOOL____00505 () Bool)
(declare-fun BOOL____00507 () Bool)
(declare-fun BOOL____00509 () Bool)
(declare-fun BOOL____00511 () Bool)
(declare-fun BOOL____00513 () Bool)
(declare-fun BOOL____00515 () Bool)
(declare-fun BOOL____00517 () Bool)
(declare-fun BOOL____00519 () Bool)
(declare-fun BOOL____00521 () Bool)
(declare-fun BOOL____00523 () Bool)
(declare-fun BOOL____00525 () Bool)
(declare-fun BOOL____00527 () Bool)
(declare-fun BOOL____00529 () Bool)
(declare-fun BOOL____00531 () Bool)
(declare-fun BOOL____00533 () Bool)
(declare-fun BOOL____00535 () Bool)
(declare-fun BOOL____00537 () Bool)
(declare-fun BOOL____00539 () Bool)
(declare-fun BOOL____00541 () Bool)
(declare-fun BOOL____00543 () Bool)
(declare-fun BOOL____00545 () Bool)
(declare-fun BOOL____00547 () Bool)
(declare-fun BOOL____00549 () Bool)
(declare-fun BOOL____00551 () Bool)
(declare-fun BOOL____00553 () Bool)
(declare-fun BOOL____00555 () Bool)
(declare-fun BOOL____00557 () Bool)
(declare-fun BOOL____00559 () Bool)
(declare-fun BOOL____00561 () Bool)
(declare-fun BOOL____00563 () Bool)
(declare-fun BOOL____00565 () Bool)
(declare-fun BOOL____00567 () Bool)
(declare-fun BOOL____00569 () Bool)
(declare-fun BOOL____00571 () Bool)
(declare-fun BOOL____00573 () Bool)
(declare-fun BOOL____00575 () Bool)
(declare-fun BOOL____00577 () Bool)
(declare-fun BOOL____00579 () Bool)
(declare-fun BOOL____00581 () Bool)
(declare-fun BOOL____00583 () Bool)
(declare-fun BOOL____00585 () Bool)
(declare-fun BOOL____00587 () Bool)
(declare-fun BOOL____00589 () Bool)
(declare-fun BOOL____00591 () Bool)
(declare-fun BOOL____00593 () Bool)
(declare-fun BOOL____00595 () Bool)
(declare-fun BOOL____00597 () Bool)
(declare-fun BOOL____00599 () Bool)
(declare-fun BOOL____00601 () Bool)
(declare-fun BOOL____00603 () Bool)
(declare-fun BOOL____00605 () Bool)
(declare-fun BOOL____00607 () Bool)
(declare-fun BOOL____00609 () Bool)
(declare-fun BOOL____00611 () Bool)
(declare-fun BOOL____00613 () Bool)
(declare-fun BOOL____00615 () Bool)
(declare-fun BOOL____00617 () Bool)
(declare-fun BOOL____00619 () Bool)
(declare-fun BOOL____00621 () Bool)
(declare-fun BOOL____00623 () Bool)
(declare-fun BOOL____00625 () Bool)
(declare-fun BOOL____00627 () Bool)
(declare-fun BOOL____00629 () Bool)
(declare-fun BOOL____00631 () Bool)
(declare-fun BOOL____00633 () Bool)
(declare-fun BOOL____00635 () Bool)
(declare-fun BOOL____00637 () Bool)
(declare-fun BOOL____00639 () Bool)
(declare-fun BOOL____00641 () Bool)
(declare-fun BOOL____00643 () Bool)
(declare-fun BOOL____00645 () Bool)
(declare-fun BOOL____00647 () Bool)
(declare-fun BOOL____00649 () Bool)
(declare-fun BOOL____00651 () Bool)
(declare-fun BOOL____00653 () Bool)
(declare-fun BOOL____00655 () Bool)
(declare-fun BOOL____00657 () Bool)
(declare-fun BOOL____00659 () Bool)
(declare-fun BOOL____00661 () Bool)
(declare-fun BOOL____00663 () Bool)
(declare-fun BOOL____00665 () Bool)
(declare-fun BOOL____00667 () Bool)
(declare-fun BOOL____00669 () Bool)
(declare-fun BOOL____00671 () Bool)
(declare-fun BOOL____00673 () Bool)
(declare-fun BOOL____00675 () Bool)
(declare-fun BOOL____00677 () Bool)
(declare-fun BOOL____00679 () Bool)
(declare-fun BOOL____00681 () Bool)
(declare-fun BOOL____00683 () Bool)
(declare-fun BOOL____00685 () Bool)
(declare-fun BOOL____00687 () Bool)
(declare-fun BOOL____00689 () Bool)
(declare-fun BOOL____00691 () Bool)
(declare-fun BOOL____00693 () Bool)
(declare-fun BOOL____00695 () Bool)
(declare-fun BOOL____00697 () Bool)
(declare-fun BOOL____00699 () Bool)
(declare-fun BOOL____00701 () Bool)
(declare-fun BOOL____00703 () Bool)
(declare-fun BOOL____00705 () Bool)
(declare-fun BOOL____00707 () Bool)
(declare-fun BOOL____00709 () Bool)
(declare-fun BOOL____00711 () Bool)
(declare-fun BOOL____00713 () Bool)
(declare-fun BOOL____00715 () Bool)
(declare-fun BOOL____00717 () Bool)
(declare-fun BOOL____00719 () Bool)
(declare-fun BOOL____00721 () Bool)
(declare-fun BOOL____00723 () Bool)
(declare-fun BOOL____00725 () Bool)
(declare-fun BOOL____00727 () Bool)
(declare-fun BOOL____00729 () Bool)
(declare-fun BOOL____00731 () Bool)
(declare-fun BOOL____00733 () Bool)
(declare-fun BOOL____00735 () Bool)
(declare-fun BOOL____00737 () Bool)
(declare-fun BOOL____00739 () Bool)
(declare-fun BOOL____00741 () Bool)
(declare-fun BOOL____00743 () Bool)
(declare-fun BOOL____00745 () Bool)
(declare-fun BOOL____00747 () Bool)
(declare-fun BOOL____00749 () Bool)
(declare-fun BOOL____00751 () Bool)
(declare-fun BOOL____00753 () Bool)
(declare-fun BOOL____00755 () Bool)
(declare-fun BOOL____00757 () Bool)
(declare-fun BOOL____00759 () Bool)
(declare-fun BOOL____00761 () Bool)
(declare-fun BOOL____00763 () Bool)
(declare-fun BOOL____00765 () Bool)
(declare-fun BOOL____00767 () Bool)
(declare-fun BOOL____00769 () Bool)
(declare-fun BOOL____00771 () Bool)
(declare-fun BOOL____00773 () Bool)
(declare-fun BOOL____00775 () Bool)
(declare-fun BOOL____00777 () Bool)
(declare-fun BOOL____00779 () Bool)
(declare-fun BOOL____00781 () Bool)
(declare-fun BOOL____00783 () Bool)
(declare-fun BOOL____00785 () Bool)
(declare-fun BOOL____00787 () Bool)
(declare-fun BOOL____00789 () Bool)
(declare-fun BOOL____00791 () Bool)
(declare-fun BOOL____00793 () Bool)
(declare-fun BOOL____00795 () Bool)
(declare-fun BOOL____00797 () Bool)
(declare-fun BOOL____00799 () Bool)
(declare-fun BOOL____00801 () Bool)
(declare-fun BOOL____00803 () Bool)
(declare-fun BOOL____00805 () Bool)
(declare-fun BOOL____00807 () Bool)
(declare-fun BOOL____00809 () Bool)
(declare-fun BOOL____00811 () Bool)
(declare-fun BOOL____00813 () Bool)
(declare-fun BOOL____00815 () Bool)
(declare-fun BOOL____00817 () Bool)
(declare-fun BOOL____00819 () Bool)
(declare-fun BOOL____00821 () Bool)
(declare-fun BOOL____00823 () Bool)
(declare-fun BOOL____00825 () Bool)
(declare-fun BOOL____00827 () Bool)
(declare-fun BOOL____00829 () Bool)
(declare-fun BOOL____00831 () Bool)
(declare-fun BOOL____00833 () Bool)
(declare-fun BOOL____00835 () Bool)
(declare-fun BOOL____00837 () Bool)
(declare-fun BOOL____00839 () Bool)
(declare-fun BOOL____00841 () Bool)
(declare-fun BOOL____00843 () Bool)
(declare-fun BOOL____00845 () Bool)
(declare-fun BOOL____00847 () Bool)
(declare-fun BOOL____00849 () Bool)
(declare-fun BOOL____00851 () Bool)
(declare-fun BOOL____00853 () Bool)
(declare-fun BOOL____00855 () Bool)
(declare-fun BOOL____00857 () Bool)
(declare-fun BOOL____00859 () Bool)
(declare-fun BOOL____00861 () Bool)
(declare-fun BOOL____00863 () Bool)
(declare-fun BOOL____00865 () Bool)
(declare-fun BOOL____00867 () Bool)
(declare-fun BOOL____00869 () Bool)
(declare-fun BOOL____00871 () Bool)
(declare-fun BOOL____00873 () Bool)
(declare-fun BOOL____00875 () Bool)
(declare-fun BOOL____00877 () Bool)
(declare-fun BOOL____00879 () Bool)
(declare-fun BOOL____00881 () Bool)
(declare-fun BOOL____00883 () Bool)
(declare-fun BOOL____00885 () Bool)
(declare-fun BOOL____00887 () Bool)
(declare-fun BOOL____00889 () Bool)
(declare-fun BOOL____00891 () Bool)
(declare-fun BOOL____00893 () Bool)
(declare-fun BOOL____00895 () Bool)
(declare-fun BOOL____00897 () Bool)
(declare-fun BOOL____00899 () Bool)
(declare-fun BOOL____00901 () Bool)
(declare-fun BOOL____00903 () Bool)
(declare-fun BOOL____00905 () Bool)
(declare-fun BOOL____00907 () Bool)
(declare-fun BOOL____00909 () Bool)
(declare-fun BOOL____00911 () Bool)
(declare-fun BOOL____00913 () Bool)
(declare-fun BOOL____00915 () Bool)
(declare-fun BOOL____00917 () Bool)
(declare-fun BOOL____00919 () Bool)
(declare-fun BOOL____00921 () Bool)
(declare-fun BOOL____00923 () Bool)
(declare-fun BOOL____00925 () Bool)
(declare-fun BOOL____00927 () Bool)
(declare-fun BOOL____00929 () Bool)
(declare-fun BOOL____00931 () Bool)
(declare-fun BOOL____00933 () Bool)
(declare-fun BOOL____00935 () Bool)
(declare-fun BOOL____00937 () Bool)
(declare-fun BOOL____00939 () Bool)
(declare-fun BOOL____00941 () Bool)
(declare-fun BOOL____00943 () Bool)
(declare-fun BOOL____00945 () Bool)
(declare-fun BOOL____00947 () Bool)
(declare-fun BOOL____00949 () Bool)
(declare-fun BOOL____00951 () Bool)
(declare-fun BOOL____00953 () Bool)
(declare-fun BOOL____00955 () Bool)
(declare-fun BOOL____00957 () Bool)
(declare-fun BOOL____00959 () Bool)
(declare-fun BOOL____00961 () Bool)
(declare-fun BOOL____00963 () Bool)
(declare-fun BOOL____00965 () Bool)
(declare-fun BOOL____00967 () Bool)
(declare-fun BOOL____00969 () Bool)
(declare-fun BOOL____00971 () Bool)
(declare-fun BOOL____00973 () Bool)
(declare-fun BOOL____00975 () Bool)
(declare-fun BOOL____00977 () Bool)
(declare-fun BOOL____00979 () Bool)
(declare-fun BOOL____00981 () Bool)
(declare-fun BOOL____00983 () Bool)
(declare-fun BOOL____00985 () Bool)
(declare-fun BOOL____00987 () Bool)
(declare-fun BOOL____00989 () Bool)
(declare-fun BOOL____00991 () Bool)
(declare-fun BOOL____00993 () Bool)
(declare-fun BOOL____00995 () Bool)
(declare-fun BOOL____00997 () Bool)
(declare-fun BOOL____00999 () Bool)
(declare-fun BOOL____01001 () Bool)
(declare-fun BOOL____01003 () Bool)
(declare-fun BOOL____01005 () Bool)
(declare-fun BOOL____01007 () Bool)
(declare-fun BOOL____01009 () Bool)
(declare-fun BOOL____01011 () Bool)
(declare-fun BOOL____01013 () Bool)
(declare-fun BOOL____01015 () Bool)
(declare-fun BOOL____01017 () Bool)
(declare-fun BOOL____01019 () Bool)
(declare-fun BOOL____01021 () Bool)
(declare-fun BOOL____01023 () Bool)
(declare-fun BOOL____01025 () Bool)
(declare-fun BOOL____01027 () Bool)
(declare-fun BOOL____01029 () Bool)
(declare-fun BOOL____01031 () Bool)
(declare-fun BOOL____01033 () Bool)
(declare-fun BOOL____01035 () Bool)
(declare-fun BOOL____01037 () Bool)
(declare-fun BOOL____01039 () Bool)
(declare-fun BOOL____01041 () Bool)
(declare-fun BOOL____01043 () Bool)
(declare-fun BOOL____01045 () Bool)
(declare-fun BOOL____01047 () Bool)
(declare-fun BOOL____01049 () Bool)
(declare-fun BOOL____01051 () Bool)
(declare-fun BOOL____01053 () Bool)
(declare-fun BOOL____01055 () Bool)
(declare-fun BOOL____01057 () Bool)
(declare-fun BOOL____01059 () Bool)
(declare-fun BOOL____01061 () Bool)
(declare-fun BOOL____01063 () Bool)
(declare-fun BOOL____01065 () Bool)
(declare-fun BOOL____01067 () Bool)
(declare-fun BOOL____01069 () Bool)
(declare-fun BOOL____01071 () Bool)
(declare-fun BOOL____01073 () Bool)
(declare-fun BOOL____01075 () Bool)
(declare-fun BOOL____01077 () Bool)
(declare-fun BOOL____01079 () Bool)
(declare-fun BOOL____01081 () Bool)
(declare-fun BOOL____01083 () Bool)
(declare-fun BOOL____01085 () Bool)
(declare-fun BOOL____01087 () Bool)
(declare-fun BOOL____01089 () Bool)
(declare-fun BOOL____01091 () Bool)
(declare-fun BOOL____01093 () Bool)
(declare-fun BOOL____01095 () Bool)
(declare-fun BOOL____01097 () Bool)
(declare-fun BOOL____01099 () Bool)
(declare-fun BOOL____01101 () Bool)
(declare-fun BOOL____01103 () Bool)
(declare-fun BOOL____01105 () Bool)
(declare-fun BOOL____01107 () Bool)
(declare-fun BOOL____01109 () Bool)
(declare-fun BOOL____01111 () Bool)
(declare-fun BOOL____01113 () Bool)
(declare-fun BOOL____01115 () Bool)
(declare-fun BOOL____01117 () Bool)
(declare-fun BOOL____01119 () Bool)
(declare-fun BOOL____01121 () Bool)
(declare-fun BOOL____01123 () Bool)
(declare-fun BOOL____01125 () Bool)
(declare-fun BOOL____01127 () Bool)
(declare-fun BOOL____01129 () Bool)
(declare-fun BOOL____01131 () Bool)
(declare-fun BOOL____01133 () Bool)
(declare-fun BOOL____01135 () Bool)
(declare-fun BOOL____01137 () Bool)
(declare-fun BOOL____01139 () Bool)
(declare-fun BOOL____01141 () Bool)
(declare-fun BOOL____01143 () Bool)
(declare-fun BOOL____01145 () Bool)
(declare-fun BOOL____01147 () Bool)
(declare-fun BOOL____01149 () Bool)
(declare-fun BOOL____01151 () Bool)
(declare-fun BOOL____01153 () Bool)
(declare-fun BOOL____01155 () Bool)
(declare-fun BOOL____01157 () Bool)
(declare-fun BOOL____01159 () Bool)
(declare-fun BOOL____01161 () Bool)
(declare-fun BOOL____01163 () Bool)
(declare-fun BOOL____01165 () Bool)
(declare-fun BOOL____01167 () Bool)
(declare-fun BOOL____01169 () Bool)
(declare-fun BOOL____01171 () Bool)
(declare-fun BOOL____01173 () Bool)
(declare-fun BOOL____01175 () Bool)
(declare-fun BOOL____01177 () Bool)
(declare-fun BOOL____01179 () Bool)
(declare-fun BOOL____01181 () Bool)
(declare-fun BOOL____01183 () Bool)
(declare-fun BOOL____01185 () Bool)
(declare-fun BOOL____01187 () Bool)
(declare-fun BOOL____01189 () Bool)
(declare-fun BOOL____01191 () Bool)
(declare-fun BOOL____01193 () Bool)
(declare-fun BOOL____01195 () Bool)
(declare-fun BOOL____01197 () Bool)
(declare-fun BOOL____01199 () Bool)
(declare-fun BOOL____01201 () Bool)
(declare-fun BOOL____01203 () Bool)
(declare-fun BOOL____01205 () Bool)
(declare-fun BOOL____01207 () Bool)
(declare-fun BOOL____01209 () Bool)
(declare-fun BOOL____01211 () Bool)
(declare-fun BOOL____01213 () Bool)
(declare-fun BOOL____01215 () Bool)
(declare-fun BOOL____01217 () Bool)
(declare-fun BOOL____01219 () Bool)
(declare-fun BOOL____01221 () Bool)
(declare-fun BOOL____01223 () Bool)
(declare-fun BOOL____01225 () Bool)
(declare-fun BOOL____01227 () Bool)
(declare-fun BOOL____01229 () Bool)
(declare-fun BOOL____01231 () Bool)
(declare-fun BOOL____01233 () Bool)
(declare-fun BOOL____01235 () Bool)
(declare-fun BOOL____01237 () Bool)
(declare-fun BOOL____01239 () Bool)
(declare-fun BOOL____01241 () Bool)
(declare-fun BOOL____01243 () Bool)
(declare-fun BOOL____01245 () Bool)
(declare-fun BOOL____01247 () Bool)
(declare-fun BOOL____01249 () Bool)
(declare-fun BOOL____01251 () Bool)
(declare-fun BOOL____01253 () Bool)
(declare-fun BOOL____01255 () Bool)
(declare-fun BOOL____01257 () Bool)
(declare-fun BOOL____01259 () Bool)
(declare-fun BOOL____01261 () Bool)
(declare-fun BOOL____01263 () Bool)
(declare-fun BOOL____01265 () Bool)
(declare-fun BOOL____01267 () Bool)
(declare-fun BOOL____01269 () Bool)
(declare-fun BOOL____01271 () Bool)
(declare-fun BOOL____01273 () Bool)
(declare-fun BOOL____01275 () Bool)
(declare-fun BOOL____01277 () Bool)
(declare-fun BOOL____01279 () Bool)
(declare-fun BOOL____01281 () Bool)
(declare-fun BOOL____01283 () Bool)
(declare-fun BOOL____01285 () Bool)
(declare-fun BOOL____01287 () Bool)
(declare-fun BOOL____01289 () Bool)
(declare-fun BOOL____01291 () Bool)
(declare-fun BOOL____01293 () Bool)
(declare-fun BOOL____01295 () Bool)
(declare-fun BOOL____01297 () Bool)
(declare-fun BOOL____01299 () Bool)
(declare-fun BOOL____01301 () Bool)
(declare-fun BOOL____01303 () Bool)
(declare-fun BOOL____01305 () Bool)
(declare-fun BOOL____01307 () Bool)
(declare-fun BOOL____01309 () Bool)
(declare-fun BOOL____01311 () Bool)
(declare-fun BOOL____01313 () Bool)
(declare-fun BOOL____01315 () Bool)
(declare-fun BOOL____01317 () Bool)
(declare-fun BOOL____01319 () Bool)
(declare-fun BOOL____01321 () Bool)
(declare-fun BOOL____01323 () Bool)
(declare-fun BOOL____01325 () Bool)
(declare-fun BOOL____01327 () Bool)
(declare-fun BOOL____01329 () Bool)
(declare-fun BOOL____01331 () Bool)
(declare-fun BOOL____01333 () Bool)
(declare-fun BOOL____01335 () Bool)
(declare-fun BOOL____01337 () Bool)
(declare-fun BOOL____01339 () Bool)
(declare-fun BOOL____01341 () Bool)
(declare-fun BOOL____01343 () Bool)
(declare-fun BOOL____01345 () Bool)
(declare-fun BOOL____01347 () Bool)
(declare-fun BOOL____01349 () Bool)
(declare-fun BOOL____01351 () Bool)
(declare-fun BOOL____01353 () Bool)
(declare-fun BOOL____01355 () Bool)
(declare-fun BOOL____01357 () Bool)
(declare-fun BOOL____01359 () Bool)
(declare-fun BOOL____01361 () Bool)
(declare-fun BOOL____01363 () Bool)
(declare-fun BOOL____01365 () Bool)
(declare-fun BOOL____01367 () Bool)
(declare-fun BOOL____01369 () Bool)
(declare-fun BOOL____01371 () Bool)
(declare-fun BOOL____01373 () Bool)
(declare-fun BOOL____01375 () Bool)
(declare-fun BOOL____01377 () Bool)
(declare-fun BOOL____01379 () Bool)
(declare-fun BOOL____01381 () Bool)
(declare-fun BOOL____01383 () Bool)
(declare-fun BOOL____01385 () Bool)
(declare-fun BOOL____01387 () Bool)
(declare-fun BOOL____01389 () Bool)
(declare-fun BOOL____01391 () Bool)
(declare-fun BOOL____01393 () Bool)
(declare-fun BOOL____01395 () Bool)
(declare-fun BOOL____01397 () Bool)
(declare-fun BOOL____01399 () Bool)
(declare-fun BOOL____01401 () Bool)
(declare-fun BOOL____01403 () Bool)
(declare-fun BOOL____01405 () Bool)
(declare-fun BOOL____01407 () Bool)
(declare-fun BOOL____01409 () Bool)
(declare-fun BOOL____01411 () Bool)
(declare-fun BOOL____01413 () Bool)
(declare-fun BOOL____01415 () Bool)
(declare-fun BOOL____01417 () Bool)
(declare-fun BOOL____01419 () Bool)
(declare-fun BOOL____01421 () Bool)
(declare-fun BOOL____01423 () Bool)
(declare-fun BOOL____01425 () Bool)
(declare-fun BOOL____01427 () Bool)
(declare-fun BOOL____01429 () Bool)
(declare-fun BOOL____01431 () Bool)
(declare-fun BOOL____01433 () Bool)
(declare-fun BOOL____01435 () Bool)
(declare-fun BOOL____01437 () Bool)
(declare-fun BOOL____01439 () Bool)
(declare-fun BOOL____01441 () Bool)
(declare-fun BOOL____01443 () Bool)
(declare-fun BOOL____01445 () Bool)
(declare-fun BOOL____01447 () Bool)
(declare-fun BOOL____01449 () Bool)
(declare-fun BOOL____01451 () Bool)
(declare-fun BOOL____01453 () Bool)
(declare-fun BOOL____01455 () Bool)
(declare-fun BOOL____01457 () Bool)
(declare-fun BOOL____01459 () Bool)
(declare-fun BOOL____01461 () Bool)
(declare-fun BOOL____01463 () Bool)
(declare-fun BOOL____01465 () Bool)
(declare-fun BOOL____01467 () Bool)
(declare-fun BOOL____01469 () Bool)
(declare-fun BOOL____01471 () Bool)
(declare-fun BOOL____01473 () Bool)
(declare-fun BOOL____01475 () Bool)
(declare-fun BOOL____01477 () Bool)
(declare-fun BOOL____01479 () Bool)
(declare-fun BOOL____01481 () Bool)
(declare-fun BOOL____01483 () Bool)
(declare-fun BOOL____01485 () Bool)
(declare-fun BOOL____01487 () Bool)
(declare-fun BOOL____01489 () Bool)
(declare-fun BOOL____01491 () Bool)
(declare-fun BOOL____01493 () Bool)
(declare-fun BOOL____01495 () Bool)
(declare-fun BOOL____01497 () Bool)
(declare-fun BOOL____01499 () Bool)
(declare-fun BOOL____01501 () Bool)
(declare-fun BOOL____01503 () Bool)
(declare-fun BOOL____01505 () Bool)
(declare-fun BOOL____01507 () Bool)
(declare-fun BOOL____01509 () Bool)
(declare-fun BOOL____01511 () Bool)
(declare-fun BOOL____01513 () Bool)
(declare-fun BOOL____01515 () Bool)
(declare-fun BOOL____01517 () Bool)
(declare-fun BOOL____01519 () Bool)
(declare-fun BOOL____01521 () Bool)
(declare-fun BOOL____01523 () Bool)
(declare-fun BOOL____01525 () Bool)
(declare-fun BOOL____01527 () Bool)
(declare-fun BOOL____01529 () Bool)
(declare-fun BOOL____01531 () Bool)
(declare-fun BOOL____01533 () Bool)
(declare-fun BOOL____01535 () Bool)
(declare-fun BOOL____01537 () Bool)
(declare-fun BOOL____01539 () Bool)
(declare-fun BOOL____01541 () Bool)
(declare-fun BOOL____01543 () Bool)
(declare-fun BOOL____01545 () Bool)
(declare-fun BOOL____01547 () Bool)
(declare-fun BOOL____01549 () Bool)
(declare-fun BOOL____01551 () Bool)
(declare-fun BOOL____01553 () Bool)
(declare-fun BOOL____01555 () Bool)
(declare-fun BOOL____01557 () Bool)
(declare-fun BOOL____01559 () Bool)
(declare-fun BOOL____01561 () Bool)
(declare-fun BOOL____01563 () Bool)
(declare-fun BOOL____01565 () Bool)
(declare-fun BOOL____01567 () Bool)
(declare-fun BOOL____01569 () Bool)
(declare-fun BOOL____01571 () Bool)
(declare-fun BOOL____01573 () Bool)
(declare-fun BOOL____01575 () Bool)
(declare-fun BOOL____01577 () Bool)
(declare-fun BOOL____01579 () Bool)
(declare-fun BOOL____01581 () Bool)
(declare-fun BOOL____01583 () Bool)
(declare-fun BOOL____01585 () Bool)
(declare-fun BOOL____01587 () Bool)
(declare-fun BOOL____01589 () Bool)
(declare-fun BOOL____01591 () Bool)
(declare-fun BOOL____01593 () Bool)
(declare-fun BOOL____01595 () Bool)
(declare-fun BOOL____01597 () Bool)
(declare-fun BOOL____01599 () Bool)
(declare-fun BOOL____01601 () Bool)
(declare-fun BOOL____01603 () Bool)
(declare-fun BOOL____01605 () Bool)
(declare-fun BOOL____01607 () Bool)
(declare-fun BOOL____01609 () Bool)
(declare-fun BOOL____01611 () Bool)
(declare-fun BOOL____01613 () Bool)
(declare-fun BOOL____01615 () Bool)
(declare-fun BOOL____01617 () Bool)
(declare-fun BOOL____01619 () Bool)
(declare-fun BOOL____01621 () Bool)
(declare-fun BOOL____01623 () Bool)
(declare-fun BOOL____01625 () Bool)
(declare-fun BOOL____01627 () Bool)
(declare-fun BOOL____01629 () Bool)
(declare-fun BOOL____01631 () Bool)
(declare-fun BOOL____01633 () Bool)
(declare-fun BOOL____01635 () Bool)
(declare-fun BOOL____01637 () Bool)
(declare-fun BOOL____01639 () Bool)
(declare-fun BOOL____01641 () Bool)
(declare-fun BOOL____01643 () Bool)
(declare-fun BOOL____01645 () Bool)
(declare-fun BOOL____01647 () Bool)
(declare-fun BOOL____01649 () Bool)
(declare-fun BOOL____01651 () Bool)
(declare-fun BOOL____01653 () Bool)
(declare-fun BOOL____01655 () Bool)
(declare-fun BOOL____01657 () Bool)
(declare-fun BOOL____01659 () Bool)
(declare-fun BOOL____01661 () Bool)
(declare-fun BOOL____01663 () Bool)
(declare-fun BOOL____01665 () Bool)
(declare-fun BOOL____01667 () Bool)
(declare-fun BOOL____01669 () Bool)
(declare-fun BOOL____01671 () Bool)
(declare-fun BOOL____01673 () Bool)
(declare-fun BOOL____01675 () Bool)
(declare-fun BOOL____01677 () Bool)
(declare-fun BOOL____01679 () Bool)
(declare-fun BOOL____01681 () Bool)
(declare-fun BOOL____01683 () Bool)
(declare-fun BOOL____01685 () Bool)
(declare-fun BOOL____01687 () Bool)
(declare-fun BOOL____01689 () Bool)
(declare-fun BOOL____01691 () Bool)
(declare-fun BOOL____01693 () Bool)
(declare-fun BOOL____01695 () Bool)
(declare-fun BOOL____01697 () Bool)
(declare-fun BOOL____01699 () Bool)
(declare-fun BOOL____01701 () Bool)
(declare-fun BOOL____01703 () Bool)
(declare-fun BOOL____01705 () Bool)
(declare-fun BOOL____01707 () Bool)
(declare-fun BOOL____01709 () Bool)
(declare-fun BOOL____01711 () Bool)
(declare-fun BOOL____01713 () Bool)
(declare-fun BOOL____01715 () Bool)
(declare-fun BOOL____01717 () Bool)
(declare-fun BOOL____01719 () Bool)
(declare-fun BOOL____01721 () Bool)
(declare-fun BOOL____01723 () Bool)
(declare-fun BOOL____01725 () Bool)
(declare-fun BOOL____01727 () Bool)
(declare-fun BOOL____01729 () Bool)
(declare-fun BOOL____01731 () Bool)
(declare-fun BOOL____01733 () Bool)
(declare-fun BOOL____01735 () Bool)
(declare-fun BOOL____01737 () Bool)
(declare-fun BOOL____01739 () Bool)
(declare-fun BOOL____01741 () Bool)
(declare-fun BOOL____01743 () Bool)
(declare-fun BOOL____01745 () Bool)
(declare-fun BOOL____01747 () Bool)
(declare-fun BOOL____01749 () Bool)
(declare-fun BOOL____01751 () Bool)
(declare-fun BOOL____01753 () Bool)
(declare-fun BOOL____01755 () Bool)
(declare-fun BOOL____01757 () Bool)
(declare-fun BOOL____01759 () Bool)
(declare-fun BOOL____01761 () Bool)
(declare-fun BOOL____01763 () Bool)
(declare-fun BOOL____01765 () Bool)
(declare-fun BOOL____01767 () Bool)
(declare-fun BOOL____01769 () Bool)
(declare-fun BOOL____01771 () Bool)
(declare-fun BOOL____01773 () Bool)
(declare-fun BOOL____01775 () Bool)
(declare-fun BOOL____01777 () Bool)
(declare-fun BOOL____01779 () Bool)
(declare-fun BOOL____01781 () Bool)
(declare-fun BOOL____01783 () Bool)
(declare-fun BOOL____01785 () Bool)
(declare-fun BOOL____01787 () Bool)
(declare-fun BOOL____01789 () Bool)
(declare-fun BOOL____01791 () Bool)
(declare-fun BOOL____01793 () Bool)
(declare-fun BOOL____01795 () Bool)
(declare-fun BOOL____01797 () Bool)
(declare-fun BOOL____01799 () Bool)
(declare-fun BOOL____01801 () Bool)
(declare-fun BOOL____01803 () Bool)
(declare-fun BOOL____01805 () Bool)
(declare-fun BOOL____01807 () Bool)
(declare-fun BOOL____01809 () Bool)
(declare-fun BOOL____01811 () Bool)
(declare-fun BOOL____01813 () Bool)
(declare-fun BOOL____01815 () Bool)
(declare-fun BOOL____01817 () Bool)
(declare-fun BOOL____01819 () Bool)
(declare-fun BOOL____01821 () Bool)
(declare-fun BOOL____01823 () Bool)
(declare-fun BOOL____01825 () Bool)
(declare-fun BOOL____01827 () Bool)
(declare-fun BOOL____01829 () Bool)
(declare-fun BOOL____01831 () Bool)
(declare-fun BOOL____01833 () Bool)
(declare-fun BOOL____01835 () Bool)
(declare-fun BOOL____01837 () Bool)
(declare-fun BOOL____01839 () Bool)
(declare-fun BOOL____01841 () Bool)
(declare-fun BOOL____01843 () Bool)
(declare-fun BOOL____01845 () Bool)
(declare-fun BOOL____01847 () Bool)
(declare-fun BOOL____01849 () Bool)
(declare-fun BOOL____01851 () Bool)
(declare-fun BOOL____01853 () Bool)
(declare-fun BOOL____01855 () Bool)
(declare-fun BOOL____01857 () Bool)
(declare-fun BOOL____01859 () Bool)
(declare-fun BOOL____01861 () Bool)
(declare-fun BOOL____01863 () Bool)
(declare-fun BOOL____01865 () Bool)
(declare-fun BOOL____01867 () Bool)
(declare-fun BOOL____01869 () Bool)
(declare-fun BOOL____01871 () Bool)
(declare-fun BOOL____01873 () Bool)
(declare-fun BOOL____01875 () Bool)
(declare-fun BOOL____01877 () Bool)
(declare-fun BOOL____01879 () Bool)
(declare-fun BOOL____01881 () Bool)
(declare-fun BOOL____01883 () Bool)
(declare-fun BOOL____01885 () Bool)
(declare-fun BOOL____01887 () Bool)
(declare-fun BOOL____01889 () Bool)
(declare-fun BOOL____01891 () Bool)
(declare-fun BOOL____01893 () Bool)
(declare-fun BOOL____01895 () Bool)
(declare-fun BOOL____01897 () Bool)
(declare-fun BOOL____01899 () Bool)
(declare-fun Real____00002 () Real)
(assert (and (<= 0 Real____00002) (<= Real____00002 1)))
(declare-fun Real____00004 () Real)
(assert (and (<= 0 Real____00004) (<= Real____00004 1)))
(declare-fun Real____00006 () Real)
(assert (and (<= 0 Real____00006) (<= Real____00006 1)))
(declare-fun Real____00008 () Real)
(assert (and (<= 0 Real____00008) (<= Real____00008 1)))
(declare-fun Real____00010 () Real)
(assert (and (<= 0 Real____00010) (<= Real____00010 1)))
(declare-fun Real____00012 () Real)
(assert (and (<= 0 Real____00012) (<= Real____00012 1)))
(declare-fun Real____00014 () Real)
(assert (and (<= 0 Real____00014) (<= Real____00014 1)))
(declare-fun Real____00016 () Real)
(assert (and (<= 0 Real____00016) (<= Real____00016 1)))
(declare-fun Real____00018 () Real)
(assert (and (<= 0 Real____00018) (<= Real____00018 1)))
(declare-fun Real____00020 () Real)
(assert (and (<= 0 Real____00020) (<= Real____00020 1)))
(declare-fun Real____00022 () Real)
(assert (and (<= 0 Real____00022) (<= Real____00022 1)))
(declare-fun Real____00024 () Real)
(assert (and (<= 0 Real____00024) (<= Real____00024 1)))
(declare-fun Real____00026 () Real)
(assert (and (<= 0 Real____00026) (<= Real____00026 1)))
(declare-fun Real____00028 () Real)
(assert (and (<= 0 Real____00028) (<= Real____00028 1)))
(declare-fun Real____00030 () Real)
(assert (and (<= 0 Real____00030) (<= Real____00030 1)))
(declare-fun Real____00032 () Real)
(assert (and (<= 0 Real____00032) (<= Real____00032 1)))
(declare-fun Real____00034 () Real)
(assert (and (<= 0 Real____00034) (<= Real____00034 1)))
(declare-fun Real____00036 () Real)
(assert (and (<= 0 Real____00036) (<= Real____00036 1)))
(declare-fun Real____00038 () Real)
(assert (and (<= 0 Real____00038) (<= Real____00038 1)))
(declare-fun Real____00040 () Real)
(assert (and (<= 0 Real____00040) (<= Real____00040 1)))
(declare-fun Real____00042 () Real)
(assert (and (<= 0 Real____00042) (<= Real____00042 1)))
(declare-fun Real____00044 () Real)
(assert (and (<= 0 Real____00044) (<= Real____00044 1)))
(declare-fun Real____00046 () Real)
(assert (and (<= 0 Real____00046) (<= Real____00046 1)))
(declare-fun Real____00048 () Real)
(assert (and (<= 0 Real____00048) (<= Real____00048 1)))
(declare-fun Real____00050 () Real)
(assert (and (<= 0 Real____00050) (<= Real____00050 1)))
(declare-fun Real____00052 () Real)
(assert (and (<= 0 Real____00052) (<= Real____00052 1)))
(declare-fun Real____00054 () Real)
(assert (and (<= 0 Real____00054) (<= Real____00054 1)))
(declare-fun Real____00056 () Real)
(assert (and (<= 0 Real____00056) (<= Real____00056 1)))
(declare-fun Real____00058 () Real)
(assert (and (<= 0 Real____00058) (<= Real____00058 1)))
(declare-fun Real____00060 () Real)
(assert (and (<= 0 Real____00060) (<= Real____00060 1)))
(declare-fun Real____00062 () Real)
(assert (and (<= 0 Real____00062) (<= Real____00062 1)))
(declare-fun Real____00064 () Real)
(assert (and (<= 0 Real____00064) (<= Real____00064 1)))
(declare-fun Real____00066 () Real)
(assert (and (<= 0 Real____00066) (<= Real____00066 1)))
(declare-fun Real____00068 () Real)
(assert (and (<= 0 Real____00068) (<= Real____00068 1)))
(declare-fun Real____00070 () Real)
(assert (and (<= 0 Real____00070) (<= Real____00070 1)))
(declare-fun Real____00072 () Real)
(assert (and (<= 0 Real____00072) (<= Real____00072 1)))
(declare-fun Real____00074 () Real)
(assert (and (<= 0 Real____00074) (<= Real____00074 1)))
(declare-fun Real____00076 () Real)
(assert (and (<= 0 Real____00076) (<= Real____00076 1)))
(declare-fun Real____00078 () Real)
(assert (and (<= 0 Real____00078) (<= Real____00078 1)))
(declare-fun Real____00080 () Real)
(assert (and (<= 0 Real____00080) (<= Real____00080 1)))
(declare-fun Real____00082 () Real)
(assert (and (<= 0 Real____00082) (<= Real____00082 1)))
(declare-fun Real____00084 () Real)
(assert (and (<= 0 Real____00084) (<= Real____00084 1)))
(declare-fun Real____00086 () Real)
(assert (and (<= 0 Real____00086) (<= Real____00086 1)))
(declare-fun Real____00088 () Real)
(assert (and (<= 0 Real____00088) (<= Real____00088 1)))
(declare-fun Real____00090 () Real)
(assert (and (<= 0 Real____00090) (<= Real____00090 1)))
(declare-fun Real____00092 () Real)
(assert (and (<= 0 Real____00092) (<= Real____00092 1)))
(declare-fun Real____00094 () Real)
(assert (and (<= 0 Real____00094) (<= Real____00094 1)))
(declare-fun Real____00096 () Real)
(assert (and (<= 0 Real____00096) (<= Real____00096 1)))
(declare-fun Real____00098 () Real)
(assert (and (<= 0 Real____00098) (<= Real____00098 1)))
(declare-fun Real____00100 () Real)
(assert (and (<= 0 Real____00100) (<= Real____00100 1)))
(declare-fun Real____00102 () Real)
(assert (and (<= 0 Real____00102) (<= Real____00102 1)))
(declare-fun Real____00104 () Real)
(assert (and (<= 0 Real____00104) (<= Real____00104 1)))
(declare-fun Real____00106 () Real)
(assert (and (<= 0 Real____00106) (<= Real____00106 1)))
(declare-fun Real____00108 () Real)
(assert (and (<= 0 Real____00108) (<= Real____00108 1)))
(declare-fun Real____00110 () Real)
(assert (and (<= 0 Real____00110) (<= Real____00110 1)))
(declare-fun Real____00112 () Real)
(assert (and (<= 0 Real____00112) (<= Real____00112 1)))
(declare-fun Real____00114 () Real)
(assert (and (<= 0 Real____00114) (<= Real____00114 1)))
(declare-fun Real____00116 () Real)
(assert (and (<= 0 Real____00116) (<= Real____00116 1)))
(declare-fun Real____00118 () Real)
(assert (and (<= 0 Real____00118) (<= Real____00118 1)))
(declare-fun Real____00120 () Real)
(assert (and (<= 0 Real____00120) (<= Real____00120 1)))
(declare-fun Real____00122 () Real)
(assert (and (<= 0 Real____00122) (<= Real____00122 1)))
(declare-fun Real____00124 () Real)
(assert (and (<= 0 Real____00124) (<= Real____00124 1)))
(declare-fun Real____00126 () Real)
(assert (and (<= 0 Real____00126) (<= Real____00126 1)))
(declare-fun Real____00128 () Real)
(assert (and (<= 0 Real____00128) (<= Real____00128 1)))
(declare-fun Real____00130 () Real)
(assert (and (<= 0 Real____00130) (<= Real____00130 1)))
(declare-fun Real____00132 () Real)
(assert (and (<= 0 Real____00132) (<= Real____00132 1)))
(declare-fun Real____00134 () Real)
(assert (and (<= 0 Real____00134) (<= Real____00134 1)))
(declare-fun Real____00136 () Real)
(assert (and (<= 0 Real____00136) (<= Real____00136 1)))
(declare-fun Real____00138 () Real)
(assert (and (<= 0 Real____00138) (<= Real____00138 1)))
(declare-fun Real____00140 () Real)
(assert (and (<= 0 Real____00140) (<= Real____00140 1)))
(declare-fun Real____00142 () Real)
(assert (and (<= 0 Real____00142) (<= Real____00142 1)))
(declare-fun Real____00144 () Real)
(assert (and (<= 0 Real____00144) (<= Real____00144 1)))
(declare-fun Real____00146 () Real)
(assert (and (<= 0 Real____00146) (<= Real____00146 1)))
(declare-fun Real____00148 () Real)
(assert (and (<= 0 Real____00148) (<= Real____00148 1)))
(declare-fun Real____00150 () Real)
(assert (and (<= 0 Real____00150) (<= Real____00150 1)))
(declare-fun Real____00152 () Real)
(assert (and (<= 0 Real____00152) (<= Real____00152 1)))
(declare-fun Real____00154 () Real)
(assert (and (<= 0 Real____00154) (<= Real____00154 1)))
(declare-fun Real____00156 () Real)
(assert (and (<= 0 Real____00156) (<= Real____00156 1)))
(declare-fun Real____00158 () Real)
(assert (and (<= 0 Real____00158) (<= Real____00158 1)))
(declare-fun Real____00160 () Real)
(assert (and (<= 0 Real____00160) (<= Real____00160 1)))
(declare-fun Real____00162 () Real)
(assert (and (<= 0 Real____00162) (<= Real____00162 1)))
(declare-fun Real____00164 () Real)
(assert (and (<= 0 Real____00164) (<= Real____00164 1)))
(declare-fun Real____00166 () Real)
(assert (and (<= 0 Real____00166) (<= Real____00166 1)))
(declare-fun Real____00168 () Real)
(assert (and (<= 0 Real____00168) (<= Real____00168 1)))
(declare-fun Real____00170 () Real)
(assert (and (<= 0 Real____00170) (<= Real____00170 1)))
(declare-fun Real____00172 () Real)
(assert (and (<= 0 Real____00172) (<= Real____00172 1)))
(declare-fun Real____00174 () Real)
(assert (and (<= 0 Real____00174) (<= Real____00174 1)))
(declare-fun Real____00176 () Real)
(assert (and (<= 0 Real____00176) (<= Real____00176 1)))
(declare-fun Real____00178 () Real)
(assert (and (<= 0 Real____00178) (<= Real____00178 1)))
(declare-fun Real____00180 () Real)
(assert (and (<= 0 Real____00180) (<= Real____00180 1)))
(declare-fun Real____00182 () Real)
(assert (and (<= 0 Real____00182) (<= Real____00182 1)))
(declare-fun Real____00184 () Real)
(assert (and (<= 0 Real____00184) (<= Real____00184 1)))
(declare-fun Real____00186 () Real)
(assert (and (<= 0 Real____00186) (<= Real____00186 1)))
(declare-fun Real____00188 () Real)
(assert (and (<= 0 Real____00188) (<= Real____00188 1)))
(declare-fun Real____00190 () Real)
(assert (and (<= 0 Real____00190) (<= Real____00190 1)))
(declare-fun Real____00192 () Real)
(assert (and (<= 0 Real____00192) (<= Real____00192 1)))
(declare-fun Real____00194 () Real)
(assert (and (<= 0 Real____00194) (<= Real____00194 1)))
(declare-fun Real____00196 () Real)
(assert (and (<= 0 Real____00196) (<= Real____00196 1)))
(declare-fun Real____00198 () Real)
(assert (and (<= 0 Real____00198) (<= Real____00198 1)))
(declare-fun Real____00200 () Real)
(assert (and (<= 0 Real____00200) (<= Real____00200 1)))
(declare-fun Real____00202 () Real)
(assert (and (<= 0 Real____00202) (<= Real____00202 1)))
(declare-fun Real____00204 () Real)
(assert (and (<= 0 Real____00204) (<= Real____00204 1)))
(declare-fun Real____00206 () Real)
(assert (and (<= 0 Real____00206) (<= Real____00206 1)))
(declare-fun Real____00208 () Real)
(assert (and (<= 0 Real____00208) (<= Real____00208 1)))
(declare-fun Real____00210 () Real)
(assert (and (<= 0 Real____00210) (<= Real____00210 1)))
(declare-fun Real____00212 () Real)
(assert (and (<= 0 Real____00212) (<= Real____00212 1)))
(declare-fun Real____00214 () Real)
(assert (and (<= 0 Real____00214) (<= Real____00214 1)))
(declare-fun Real____00216 () Real)
(assert (and (<= 0 Real____00216) (<= Real____00216 1)))
(declare-fun Real____00218 () Real)
(assert (and (<= 0 Real____00218) (<= Real____00218 1)))
(declare-fun Real____00220 () Real)
(assert (and (<= 0 Real____00220) (<= Real____00220 1)))
(declare-fun Real____00222 () Real)
(assert (and (<= 0 Real____00222) (<= Real____00222 1)))
(declare-fun Real____00224 () Real)
(assert (and (<= 0 Real____00224) (<= Real____00224 1)))
(declare-fun Real____00226 () Real)
(assert (and (<= 0 Real____00226) (<= Real____00226 1)))
(declare-fun Real____00228 () Real)
(assert (and (<= 0 Real____00228) (<= Real____00228 1)))
(declare-fun Real____00230 () Real)
(assert (and (<= 0 Real____00230) (<= Real____00230 1)))
(declare-fun Real____00232 () Real)
(assert (and (<= 0 Real____00232) (<= Real____00232 1)))
(declare-fun Real____00234 () Real)
(assert (and (<= 0 Real____00234) (<= Real____00234 1)))
(declare-fun Real____00236 () Real)
(assert (and (<= 0 Real____00236) (<= Real____00236 1)))
(declare-fun Real____00238 () Real)
(assert (and (<= 0 Real____00238) (<= Real____00238 1)))
(declare-fun Real____00240 () Real)
(assert (and (<= 0 Real____00240) (<= Real____00240 1)))
(declare-fun Real____00242 () Real)
(assert (and (<= 0 Real____00242) (<= Real____00242 1)))
(declare-fun Real____00244 () Real)
(assert (and (<= 0 Real____00244) (<= Real____00244 1)))
(declare-fun Real____00246 () Real)
(assert (and (<= 0 Real____00246) (<= Real____00246 1)))
(declare-fun Real____00248 () Real)
(assert (and (<= 0 Real____00248) (<= Real____00248 1)))
(declare-fun Real____00250 () Real)
(assert (and (<= 0 Real____00250) (<= Real____00250 1)))
(declare-fun Real____00252 () Real)
(assert (and (<= 0 Real____00252) (<= Real____00252 1)))
(declare-fun Real____00254 () Real)
(assert (and (<= 0 Real____00254) (<= Real____00254 1)))
(declare-fun Real____00256 () Real)
(assert (and (<= 0 Real____00256) (<= Real____00256 1)))
(declare-fun Real____00258 () Real)
(assert (and (<= 0 Real____00258) (<= Real____00258 1)))
(declare-fun Real____00260 () Real)
(assert (and (<= 0 Real____00260) (<= Real____00260 1)))
(declare-fun Real____00262 () Real)
(assert (and (<= 0 Real____00262) (<= Real____00262 1)))
(declare-fun Real____00264 () Real)
(assert (and (<= 0 Real____00264) (<= Real____00264 1)))
(declare-fun Real____00266 () Real)
(assert (and (<= 0 Real____00266) (<= Real____00266 1)))
(declare-fun Real____00268 () Real)
(assert (and (<= 0 Real____00268) (<= Real____00268 1)))
(declare-fun Real____00270 () Real)
(assert (and (<= 0 Real____00270) (<= Real____00270 1)))
(declare-fun Real____00272 () Real)
(assert (and (<= 0 Real____00272) (<= Real____00272 1)))
(declare-fun Real____00274 () Real)
(assert (and (<= 0 Real____00274) (<= Real____00274 1)))
(declare-fun Real____00276 () Real)
(assert (and (<= 0 Real____00276) (<= Real____00276 1)))
(declare-fun Real____00278 () Real)
(assert (and (<= 0 Real____00278) (<= Real____00278 1)))
(declare-fun Real____00280 () Real)
(assert (and (<= 0 Real____00280) (<= Real____00280 1)))
(declare-fun Real____00282 () Real)
(assert (and (<= 0 Real____00282) (<= Real____00282 1)))
(declare-fun Real____00284 () Real)
(assert (and (<= 0 Real____00284) (<= Real____00284 1)))
(declare-fun Real____00286 () Real)
(assert (and (<= 0 Real____00286) (<= Real____00286 1)))
(declare-fun Real____00288 () Real)
(assert (and (<= 0 Real____00288) (<= Real____00288 1)))
(declare-fun Real____00290 () Real)
(assert (and (<= 0 Real____00290) (<= Real____00290 1)))
(declare-fun Real____00292 () Real)
(assert (and (<= 0 Real____00292) (<= Real____00292 1)))
(declare-fun Real____00294 () Real)
(assert (and (<= 0 Real____00294) (<= Real____00294 1)))
(declare-fun Real____00296 () Real)
(assert (and (<= 0 Real____00296) (<= Real____00296 1)))
(declare-fun Real____00298 () Real)
(assert (and (<= 0 Real____00298) (<= Real____00298 1)))
(declare-fun Real____00300 () Real)
(assert (and (<= 0 Real____00300) (<= Real____00300 1)))
(declare-fun Real____00302 () Real)
(assert (and (<= 0 Real____00302) (<= Real____00302 1)))
(declare-fun Real____00304 () Real)
(assert (and (<= 0 Real____00304) (<= Real____00304 1)))
(declare-fun Real____00306 () Real)
(assert (and (<= 0 Real____00306) (<= Real____00306 1)))
(declare-fun Real____00308 () Real)
(assert (and (<= 0 Real____00308) (<= Real____00308 1)))
(declare-fun Real____00310 () Real)
(assert (and (<= 0 Real____00310) (<= Real____00310 1)))
(declare-fun Real____00312 () Real)
(assert (and (<= 0 Real____00312) (<= Real____00312 1)))
(declare-fun Real____00314 () Real)
(assert (and (<= 0 Real____00314) (<= Real____00314 1)))
(declare-fun Real____00316 () Real)
(assert (and (<= 0 Real____00316) (<= Real____00316 1)))
(declare-fun Real____00318 () Real)
(assert (and (<= 0 Real____00318) (<= Real____00318 1)))
(declare-fun Real____00320 () Real)
(assert (and (<= 0 Real____00320) (<= Real____00320 1)))
(declare-fun Real____00322 () Real)
(assert (and (<= 0 Real____00322) (<= Real____00322 1)))
(declare-fun Real____00324 () Real)
(assert (and (<= 0 Real____00324) (<= Real____00324 1)))
(declare-fun Real____00326 () Real)
(assert (and (<= 0 Real____00326) (<= Real____00326 1)))
(declare-fun Real____00328 () Real)
(assert (and (<= 0 Real____00328) (<= Real____00328 1)))
(declare-fun Real____00330 () Real)
(assert (and (<= 0 Real____00330) (<= Real____00330 1)))
(declare-fun Real____00332 () Real)
(assert (and (<= 0 Real____00332) (<= Real____00332 1)))
(declare-fun Real____00334 () Real)
(assert (and (<= 0 Real____00334) (<= Real____00334 1)))
(declare-fun Real____00336 () Real)
(assert (and (<= 0 Real____00336) (<= Real____00336 1)))
(declare-fun Real____00338 () Real)
(assert (and (<= 0 Real____00338) (<= Real____00338 1)))
(declare-fun Real____00340 () Real)
(assert (and (<= 0 Real____00340) (<= Real____00340 1)))
(declare-fun Real____00342 () Real)
(assert (and (<= 0 Real____00342) (<= Real____00342 1)))
(declare-fun Real____00344 () Real)
(assert (and (<= 0 Real____00344) (<= Real____00344 1)))
(declare-fun Real____00346 () Real)
(assert (and (<= 0 Real____00346) (<= Real____00346 1)))
(declare-fun Real____00348 () Real)
(assert (and (<= 0 Real____00348) (<= Real____00348 1)))
(declare-fun Real____00350 () Real)
(assert (and (<= 0 Real____00350) (<= Real____00350 1)))
(declare-fun Real____00352 () Real)
(assert (and (<= 0 Real____00352) (<= Real____00352 1)))
(declare-fun Real____00354 () Real)
(assert (and (<= 0 Real____00354) (<= Real____00354 1)))
(declare-fun Real____00356 () Real)
(assert (and (<= 0 Real____00356) (<= Real____00356 1)))
(declare-fun Real____00358 () Real)
(assert (and (<= 0 Real____00358) (<= Real____00358 1)))
(declare-fun Real____00360 () Real)
(assert (and (<= 0 Real____00360) (<= Real____00360 1)))
(declare-fun Real____00362 () Real)
(assert (and (<= 0 Real____00362) (<= Real____00362 1)))
(declare-fun Real____00364 () Real)
(assert (and (<= 0 Real____00364) (<= Real____00364 1)))
(declare-fun Real____00366 () Real)
(assert (and (<= 0 Real____00366) (<= Real____00366 1)))
(declare-fun Real____00368 () Real)
(assert (and (<= 0 Real____00368) (<= Real____00368 1)))
(declare-fun Real____00370 () Real)
(assert (and (<= 0 Real____00370) (<= Real____00370 1)))
(declare-fun Real____00372 () Real)
(assert (and (<= 0 Real____00372) (<= Real____00372 1)))
(declare-fun Real____00374 () Real)
(assert (and (<= 0 Real____00374) (<= Real____00374 1)))
(declare-fun Real____00376 () Real)
(assert (and (<= 0 Real____00376) (<= Real____00376 1)))
(declare-fun Real____00378 () Real)
(assert (and (<= 0 Real____00378) (<= Real____00378 1)))
(declare-fun Real____00380 () Real)
(assert (and (<= 0 Real____00380) (<= Real____00380 1)))
(declare-fun Real____00382 () Real)
(assert (and (<= 0 Real____00382) (<= Real____00382 1)))
(declare-fun Real____00384 () Real)
(assert (and (<= 0 Real____00384) (<= Real____00384 1)))
(declare-fun Real____00386 () Real)
(assert (and (<= 0 Real____00386) (<= Real____00386 1)))
(declare-fun Real____00388 () Real)
(assert (and (<= 0 Real____00388) (<= Real____00388 1)))
(declare-fun Real____00390 () Real)
(assert (and (<= 0 Real____00390) (<= Real____00390 1)))
(declare-fun Real____00392 () Real)
(assert (and (<= 0 Real____00392) (<= Real____00392 1)))
(declare-fun Real____00394 () Real)
(assert (and (<= 0 Real____00394) (<= Real____00394 1)))
(declare-fun Real____00396 () Real)
(assert (and (<= 0 Real____00396) (<= Real____00396 1)))
(declare-fun Real____00398 () Real)
(assert (and (<= 0 Real____00398) (<= Real____00398 1)))
(declare-fun Real____00400 () Real)
(assert (and (<= 0 Real____00400) (<= Real____00400 1)))
(declare-fun Real____00402 () Real)
(assert (and (<= 0 Real____00402) (<= Real____00402 1)))
(declare-fun Real____00404 () Real)
(assert (and (<= 0 Real____00404) (<= Real____00404 1)))
(declare-fun Real____00406 () Real)
(assert (and (<= 0 Real____00406) (<= Real____00406 1)))
(declare-fun Real____00408 () Real)
(assert (and (<= 0 Real____00408) (<= Real____00408 1)))
(declare-fun Real____00410 () Real)
(assert (and (<= 0 Real____00410) (<= Real____00410 1)))
(declare-fun Real____00412 () Real)
(assert (and (<= 0 Real____00412) (<= Real____00412 1)))
(declare-fun Real____00414 () Real)
(assert (and (<= 0 Real____00414) (<= Real____00414 1)))
(declare-fun Real____00416 () Real)
(assert (and (<= 0 Real____00416) (<= Real____00416 1)))
(declare-fun Real____00418 () Real)
(assert (and (<= 0 Real____00418) (<= Real____00418 1)))
(declare-fun Real____00420 () Real)
(assert (and (<= 0 Real____00420) (<= Real____00420 1)))
(declare-fun Real____00422 () Real)
(assert (and (<= 0 Real____00422) (<= Real____00422 1)))
(declare-fun Real____00424 () Real)
(assert (and (<= 0 Real____00424) (<= Real____00424 1)))
(declare-fun Real____00426 () Real)
(assert (and (<= 0 Real____00426) (<= Real____00426 1)))
(declare-fun Real____00428 () Real)
(assert (and (<= 0 Real____00428) (<= Real____00428 1)))
(declare-fun Real____00430 () Real)
(assert (and (<= 0 Real____00430) (<= Real____00430 1)))
(declare-fun Real____00432 () Real)
(assert (and (<= 0 Real____00432) (<= Real____00432 1)))
(declare-fun Real____00434 () Real)
(assert (and (<= 0 Real____00434) (<= Real____00434 1)))
(declare-fun Real____00436 () Real)
(assert (and (<= 0 Real____00436) (<= Real____00436 1)))
(declare-fun Real____00438 () Real)
(assert (and (<= 0 Real____00438) (<= Real____00438 1)))
(declare-fun Real____00440 () Real)
(assert (and (<= 0 Real____00440) (<= Real____00440 1)))
(declare-fun Real____00442 () Real)
(assert (and (<= 0 Real____00442) (<= Real____00442 1)))
(declare-fun Real____00444 () Real)
(assert (and (<= 0 Real____00444) (<= Real____00444 1)))
(declare-fun Real____00446 () Real)
(assert (and (<= 0 Real____00446) (<= Real____00446 1)))
(declare-fun Real____00448 () Real)
(assert (and (<= 0 Real____00448) (<= Real____00448 1)))
(declare-fun Real____00450 () Real)
(assert (and (<= 0 Real____00450) (<= Real____00450 1)))
(declare-fun Real____00452 () Real)
(assert (and (<= 0 Real____00452) (<= Real____00452 1)))
(declare-fun Real____00454 () Real)
(assert (and (<= 0 Real____00454) (<= Real____00454 1)))
(declare-fun Real____00456 () Real)
(assert (and (<= 0 Real____00456) (<= Real____00456 1)))
(declare-fun Real____00458 () Real)
(assert (and (<= 0 Real____00458) (<= Real____00458 1)))
(declare-fun Real____00460 () Real)
(assert (and (<= 0 Real____00460) (<= Real____00460 1)))
(declare-fun Real____00462 () Real)
(assert (and (<= 0 Real____00462) (<= Real____00462 1)))
(declare-fun Real____00464 () Real)
(assert (and (<= 0 Real____00464) (<= Real____00464 1)))
(declare-fun Real____00466 () Real)
(assert (and (<= 0 Real____00466) (<= Real____00466 1)))
(declare-fun Real____00468 () Real)
(assert (and (<= 0 Real____00468) (<= Real____00468 1)))
(declare-fun Real____00470 () Real)
(assert (and (<= 0 Real____00470) (<= Real____00470 1)))
(declare-fun Real____00472 () Real)
(assert (and (<= 0 Real____00472) (<= Real____00472 1)))
(declare-fun Real____00474 () Real)
(assert (and (<= 0 Real____00474) (<= Real____00474 1)))
(declare-fun Real____00476 () Real)
(assert (and (<= 0 Real____00476) (<= Real____00476 1)))
(declare-fun Real____00478 () Real)
(assert (and (<= 0 Real____00478) (<= Real____00478 1)))
(declare-fun Real____00480 () Real)
(assert (and (<= 0 Real____00480) (<= Real____00480 1)))
(declare-fun Real____00482 () Real)
(assert (and (<= 0 Real____00482) (<= Real____00482 1)))
(declare-fun Real____00484 () Real)
(assert (and (<= 0 Real____00484) (<= Real____00484 1)))
(declare-fun Real____00486 () Real)
(assert (and (<= 0 Real____00486) (<= Real____00486 1)))
(declare-fun Real____00488 () Real)
(assert (and (<= 0 Real____00488) (<= Real____00488 1)))
(declare-fun Real____00490 () Real)
(assert (and (<= 0 Real____00490) (<= Real____00490 1)))
(declare-fun Real____00492 () Real)
(assert (and (<= 0 Real____00492) (<= Real____00492 1)))
(declare-fun Real____00494 () Real)
(assert (and (<= 0 Real____00494) (<= Real____00494 1)))
(declare-fun Real____00496 () Real)
(assert (and (<= 0 Real____00496) (<= Real____00496 1)))
(declare-fun Real____00498 () Real)
(assert (and (<= 0 Real____00498) (<= Real____00498 1)))
(declare-fun Real____00500 () Real)
(assert (and (<= 0 Real____00500) (<= Real____00500 1)))
(declare-fun Real____00502 () Real)
(assert (and (<= 0 Real____00502) (<= Real____00502 1)))
(declare-fun Real____00504 () Real)
(assert (and (<= 0 Real____00504) (<= Real____00504 1)))
(declare-fun Real____00506 () Real)
(assert (and (<= 0 Real____00506) (<= Real____00506 1)))
(declare-fun Real____00508 () Real)
(assert (and (<= 0 Real____00508) (<= Real____00508 1)))
(declare-fun Real____00510 () Real)
(assert (and (<= 0 Real____00510) (<= Real____00510 1)))
(declare-fun Real____00512 () Real)
(assert (and (<= 0 Real____00512) (<= Real____00512 1)))
(declare-fun Real____00514 () Real)
(assert (and (<= 0 Real____00514) (<= Real____00514 1)))
(declare-fun Real____00516 () Real)
(assert (and (<= 0 Real____00516) (<= Real____00516 1)))
(declare-fun Real____00518 () Real)
(assert (and (<= 0 Real____00518) (<= Real____00518 1)))
(declare-fun Real____00520 () Real)
(assert (and (<= 0 Real____00520) (<= Real____00520 1)))
(declare-fun Real____00522 () Real)
(assert (and (<= 0 Real____00522) (<= Real____00522 1)))
(declare-fun Real____00524 () Real)
(assert (and (<= 0 Real____00524) (<= Real____00524 1)))
(declare-fun Real____00526 () Real)
(assert (and (<= 0 Real____00526) (<= Real____00526 1)))
(declare-fun Real____00528 () Real)
(assert (and (<= 0 Real____00528) (<= Real____00528 1)))
(declare-fun Real____00530 () Real)
(assert (and (<= 0 Real____00530) (<= Real____00530 1)))
(declare-fun Real____00532 () Real)
(assert (and (<= 0 Real____00532) (<= Real____00532 1)))
(declare-fun Real____00534 () Real)
(assert (and (<= 0 Real____00534) (<= Real____00534 1)))
(declare-fun Real____00536 () Real)
(assert (and (<= 0 Real____00536) (<= Real____00536 1)))
(declare-fun Real____00538 () Real)
(assert (and (<= 0 Real____00538) (<= Real____00538 1)))
(declare-fun Real____00540 () Real)
(assert (and (<= 0 Real____00540) (<= Real____00540 1)))
(declare-fun Real____00542 () Real)
(assert (and (<= 0 Real____00542) (<= Real____00542 1)))
(declare-fun Real____00544 () Real)
(assert (and (<= 0 Real____00544) (<= Real____00544 1)))
(declare-fun Real____00546 () Real)
(assert (and (<= 0 Real____00546) (<= Real____00546 1)))
(declare-fun Real____00548 () Real)
(assert (and (<= 0 Real____00548) (<= Real____00548 1)))
(declare-fun Real____00550 () Real)
(assert (and (<= 0 Real____00550) (<= Real____00550 1)))
(declare-fun Real____00552 () Real)
(assert (and (<= 0 Real____00552) (<= Real____00552 1)))
(declare-fun Real____00554 () Real)
(assert (and (<= 0 Real____00554) (<= Real____00554 1)))
(declare-fun Real____00556 () Real)
(assert (and (<= 0 Real____00556) (<= Real____00556 1)))
(declare-fun Real____00558 () Real)
(assert (and (<= 0 Real____00558) (<= Real____00558 1)))
(declare-fun Real____00560 () Real)
(assert (and (<= 0 Real____00560) (<= Real____00560 1)))
(declare-fun Real____00562 () Real)
(assert (and (<= 0 Real____00562) (<= Real____00562 1)))
(declare-fun Real____00564 () Real)
(assert (and (<= 0 Real____00564) (<= Real____00564 1)))
(declare-fun Real____00566 () Real)
(assert (and (<= 0 Real____00566) (<= Real____00566 1)))
(declare-fun Real____00568 () Real)
(assert (and (<= 0 Real____00568) (<= Real____00568 1)))
(declare-fun Real____00570 () Real)
(assert (and (<= 0 Real____00570) (<= Real____00570 1)))
(declare-fun Real____00572 () Real)
(assert (and (<= 0 Real____00572) (<= Real____00572 1)))
(declare-fun Real____00574 () Real)
(assert (and (<= 0 Real____00574) (<= Real____00574 1)))
(declare-fun Real____00576 () Real)
(assert (and (<= 0 Real____00576) (<= Real____00576 1)))
(declare-fun Real____00578 () Real)
(assert (and (<= 0 Real____00578) (<= Real____00578 1)))
(declare-fun Real____00580 () Real)
(assert (and (<= 0 Real____00580) (<= Real____00580 1)))
(declare-fun Real____00582 () Real)
(assert (and (<= 0 Real____00582) (<= Real____00582 1)))
(declare-fun Real____00584 () Real)
(assert (and (<= 0 Real____00584) (<= Real____00584 1)))
(declare-fun Real____00586 () Real)
(assert (and (<= 0 Real____00586) (<= Real____00586 1)))
(declare-fun Real____00588 () Real)
(assert (and (<= 0 Real____00588) (<= Real____00588 1)))
(declare-fun Real____00590 () Real)
(assert (and (<= 0 Real____00590) (<= Real____00590 1)))
(declare-fun Real____00592 () Real)
(assert (and (<= 0 Real____00592) (<= Real____00592 1)))
(declare-fun Real____00594 () Real)
(assert (and (<= 0 Real____00594) (<= Real____00594 1)))
(declare-fun Real____00596 () Real)
(assert (and (<= 0 Real____00596) (<= Real____00596 1)))
(declare-fun Real____00598 () Real)
(assert (and (<= 0 Real____00598) (<= Real____00598 1)))
(declare-fun Real____00600 () Real)
(assert (and (<= 0 Real____00600) (<= Real____00600 1)))
(declare-fun Real____00602 () Real)
(assert (and (<= 0 Real____00602) (<= Real____00602 1)))
(declare-fun Real____00604 () Real)
(assert (and (<= 0 Real____00604) (<= Real____00604 1)))
(declare-fun Real____00606 () Real)
(assert (and (<= 0 Real____00606) (<= Real____00606 1)))
(declare-fun Real____00608 () Real)
(assert (and (<= 0 Real____00608) (<= Real____00608 1)))
(declare-fun Real____00610 () Real)
(assert (and (<= 0 Real____00610) (<= Real____00610 1)))
(declare-fun Real____00612 () Real)
(assert (and (<= 0 Real____00612) (<= Real____00612 1)))
(declare-fun Real____00614 () Real)
(assert (and (<= 0 Real____00614) (<= Real____00614 1)))
(declare-fun Real____00616 () Real)
(assert (and (<= 0 Real____00616) (<= Real____00616 1)))
(declare-fun Real____00618 () Real)
(assert (and (<= 0 Real____00618) (<= Real____00618 1)))
(declare-fun Real____00620 () Real)
(assert (and (<= 0 Real____00620) (<= Real____00620 1)))
(declare-fun Real____00622 () Real)
(assert (and (<= 0 Real____00622) (<= Real____00622 1)))
(declare-fun Real____00624 () Real)
(assert (and (<= 0 Real____00624) (<= Real____00624 1)))
(declare-fun Real____00626 () Real)
(assert (and (<= 0 Real____00626) (<= Real____00626 1)))
(declare-fun Real____00628 () Real)
(assert (and (<= 0 Real____00628) (<= Real____00628 1)))
(declare-fun Real____00630 () Real)
(assert (and (<= 0 Real____00630) (<= Real____00630 1)))
(declare-fun Real____00632 () Real)
(assert (and (<= 0 Real____00632) (<= Real____00632 1)))
(declare-fun Real____00634 () Real)
(assert (and (<= 0 Real____00634) (<= Real____00634 1)))
(declare-fun Real____00636 () Real)
(assert (and (<= 0 Real____00636) (<= Real____00636 1)))
(declare-fun Real____00638 () Real)
(assert (and (<= 0 Real____00638) (<= Real____00638 1)))
(declare-fun Real____00640 () Real)
(assert (and (<= 0 Real____00640) (<= Real____00640 1)))
(declare-fun Real____00642 () Real)
(assert (and (<= 0 Real____00642) (<= Real____00642 1)))
(declare-fun Real____00644 () Real)
(assert (and (<= 0 Real____00644) (<= Real____00644 1)))
(declare-fun Real____00646 () Real)
(assert (and (<= 0 Real____00646) (<= Real____00646 1)))
(declare-fun Real____00648 () Real)
(assert (and (<= 0 Real____00648) (<= Real____00648 1)))
(declare-fun Real____00650 () Real)
(assert (and (<= 0 Real____00650) (<= Real____00650 1)))
(declare-fun Real____00652 () Real)
(assert (and (<= 0 Real____00652) (<= Real____00652 1)))
(declare-fun Real____00654 () Real)
(assert (and (<= 0 Real____00654) (<= Real____00654 1)))
(declare-fun Real____00656 () Real)
(assert (and (<= 0 Real____00656) (<= Real____00656 1)))
(declare-fun Real____00658 () Real)
(assert (and (<= 0 Real____00658) (<= Real____00658 1)))
(declare-fun Real____00660 () Real)
(assert (and (<= 0 Real____00660) (<= Real____00660 1)))
(declare-fun Real____00662 () Real)
(assert (and (<= 0 Real____00662) (<= Real____00662 1)))
(declare-fun Real____00664 () Real)
(assert (and (<= 0 Real____00664) (<= Real____00664 1)))
(declare-fun Real____00666 () Real)
(assert (and (<= 0 Real____00666) (<= Real____00666 1)))
(declare-fun Real____00668 () Real)
(assert (and (<= 0 Real____00668) (<= Real____00668 1)))
(declare-fun Real____00670 () Real)
(assert (and (<= 0 Real____00670) (<= Real____00670 1)))
(declare-fun Real____00672 () Real)
(assert (and (<= 0 Real____00672) (<= Real____00672 1)))
(declare-fun Real____00674 () Real)
(assert (and (<= 0 Real____00674) (<= Real____00674 1)))
(declare-fun Real____00676 () Real)
(assert (and (<= 0 Real____00676) (<= Real____00676 1)))
(declare-fun Real____00678 () Real)
(assert (and (<= 0 Real____00678) (<= Real____00678 1)))
(declare-fun Real____00680 () Real)
(assert (and (<= 0 Real____00680) (<= Real____00680 1)))
(declare-fun Real____00682 () Real)
(assert (and (<= 0 Real____00682) (<= Real____00682 1)))
(declare-fun Real____00684 () Real)
(assert (and (<= 0 Real____00684) (<= Real____00684 1)))
(declare-fun Real____00686 () Real)
(assert (and (<= 0 Real____00686) (<= Real____00686 1)))
(declare-fun Real____00688 () Real)
(assert (and (<= 0 Real____00688) (<= Real____00688 1)))
(declare-fun Real____00690 () Real)
(assert (and (<= 0 Real____00690) (<= Real____00690 1)))
(declare-fun Real____00692 () Real)
(assert (and (<= 0 Real____00692) (<= Real____00692 1)))
(declare-fun Real____00694 () Real)
(assert (and (<= 0 Real____00694) (<= Real____00694 1)))
(declare-fun Real____00696 () Real)
(assert (and (<= 0 Real____00696) (<= Real____00696 1)))
(declare-fun Real____00698 () Real)
(assert (and (<= 0 Real____00698) (<= Real____00698 1)))
(declare-fun Real____00700 () Real)
(assert (and (<= 0 Real____00700) (<= Real____00700 1)))
(declare-fun Real____00702 () Real)
(assert (and (<= 0 Real____00702) (<= Real____00702 1)))
(declare-fun Real____00704 () Real)
(assert (and (<= 0 Real____00704) (<= Real____00704 1)))
(declare-fun Real____00706 () Real)
(assert (and (<= 0 Real____00706) (<= Real____00706 1)))
(declare-fun Real____00708 () Real)
(assert (and (<= 0 Real____00708) (<= Real____00708 1)))
(declare-fun Real____00710 () Real)
(assert (and (<= 0 Real____00710) (<= Real____00710 1)))
(declare-fun Real____00712 () Real)
(assert (and (<= 0 Real____00712) (<= Real____00712 1)))
(declare-fun Real____00714 () Real)
(assert (and (<= 0 Real____00714) (<= Real____00714 1)))
(declare-fun Real____00716 () Real)
(assert (and (<= 0 Real____00716) (<= Real____00716 1)))
(declare-fun Real____00718 () Real)
(assert (and (<= 0 Real____00718) (<= Real____00718 1)))
(declare-fun Real____00720 () Real)
(assert (and (<= 0 Real____00720) (<= Real____00720 1)))
(declare-fun Real____00722 () Real)
(assert (and (<= 0 Real____00722) (<= Real____00722 1)))
(declare-fun Real____00724 () Real)
(assert (and (<= 0 Real____00724) (<= Real____00724 1)))
(declare-fun Real____00726 () Real)
(assert (and (<= 0 Real____00726) (<= Real____00726 1)))
(declare-fun Real____00728 () Real)
(assert (and (<= 0 Real____00728) (<= Real____00728 1)))
(declare-fun Real____00730 () Real)
(assert (and (<= 0 Real____00730) (<= Real____00730 1)))
(declare-fun Real____00732 () Real)
(assert (and (<= 0 Real____00732) (<= Real____00732 1)))
(declare-fun Real____00734 () Real)
(assert (and (<= 0 Real____00734) (<= Real____00734 1)))
(declare-fun Real____00736 () Real)
(assert (and (<= 0 Real____00736) (<= Real____00736 1)))
(declare-fun Real____00738 () Real)
(assert (and (<= 0 Real____00738) (<= Real____00738 1)))
(declare-fun Real____00740 () Real)
(assert (and (<= 0 Real____00740) (<= Real____00740 1)))
(declare-fun Real____00742 () Real)
(assert (and (<= 0 Real____00742) (<= Real____00742 1)))
(declare-fun Real____00744 () Real)
(assert (and (<= 0 Real____00744) (<= Real____00744 1)))
(declare-fun Real____00746 () Real)
(assert (and (<= 0 Real____00746) (<= Real____00746 1)))
(declare-fun Real____00748 () Real)
(assert (and (<= 0 Real____00748) (<= Real____00748 1)))
(declare-fun Real____00750 () Real)
(assert (and (<= 0 Real____00750) (<= Real____00750 1)))
(declare-fun Real____00752 () Real)
(assert (and (<= 0 Real____00752) (<= Real____00752 1)))
(declare-fun Real____00754 () Real)
(assert (and (<= 0 Real____00754) (<= Real____00754 1)))
(declare-fun Real____00756 () Real)
(assert (and (<= 0 Real____00756) (<= Real____00756 1)))
(declare-fun Real____00758 () Real)
(assert (and (<= 0 Real____00758) (<= Real____00758 1)))
(declare-fun Real____00760 () Real)
(assert (and (<= 0 Real____00760) (<= Real____00760 1)))
(declare-fun Real____00762 () Real)
(assert (and (<= 0 Real____00762) (<= Real____00762 1)))
(declare-fun Real____00764 () Real)
(assert (and (<= 0 Real____00764) (<= Real____00764 1)))
(declare-fun Real____00766 () Real)
(assert (and (<= 0 Real____00766) (<= Real____00766 1)))
(declare-fun Real____00768 () Real)
(assert (and (<= 0 Real____00768) (<= Real____00768 1)))
(declare-fun Real____00770 () Real)
(assert (and (<= 0 Real____00770) (<= Real____00770 1)))
(declare-fun Real____00772 () Real)
(assert (and (<= 0 Real____00772) (<= Real____00772 1)))
(declare-fun Real____00774 () Real)
(assert (and (<= 0 Real____00774) (<= Real____00774 1)))
(declare-fun Real____00776 () Real)
(assert (and (<= 0 Real____00776) (<= Real____00776 1)))
(declare-fun Real____00778 () Real)
(assert (and (<= 0 Real____00778) (<= Real____00778 1)))
(declare-fun Real____00780 () Real)
(assert (and (<= 0 Real____00780) (<= Real____00780 1)))
(declare-fun Real____00782 () Real)
(assert (and (<= 0 Real____00782) (<= Real____00782 1)))
(declare-fun Real____00784 () Real)
(assert (and (<= 0 Real____00784) (<= Real____00784 1)))
(declare-fun Real____00786 () Real)
(assert (and (<= 0 Real____00786) (<= Real____00786 1)))
(declare-fun Real____00788 () Real)
(assert (and (<= 0 Real____00788) (<= Real____00788 1)))
(declare-fun Real____00790 () Real)
(assert (and (<= 0 Real____00790) (<= Real____00790 1)))
(declare-fun Real____00792 () Real)
(assert (and (<= 0 Real____00792) (<= Real____00792 1)))
(declare-fun Real____00794 () Real)
(assert (and (<= 0 Real____00794) (<= Real____00794 1)))
(declare-fun Real____00796 () Real)
(assert (and (<= 0 Real____00796) (<= Real____00796 1)))
(declare-fun Real____00798 () Real)
(assert (and (<= 0 Real____00798) (<= Real____00798 1)))
(declare-fun Real____00800 () Real)
(assert (and (<= 0 Real____00800) (<= Real____00800 1)))
(declare-fun Real____00802 () Real)
(assert (and (<= 0 Real____00802) (<= Real____00802 1)))
(declare-fun Real____00804 () Real)
(assert (and (<= 0 Real____00804) (<= Real____00804 1)))
(declare-fun Real____00806 () Real)
(assert (and (<= 0 Real____00806) (<= Real____00806 1)))
(declare-fun Real____00808 () Real)
(assert (and (<= 0 Real____00808) (<= Real____00808 1)))
(declare-fun Real____00810 () Real)
(assert (and (<= 0 Real____00810) (<= Real____00810 1)))
(declare-fun Real____00812 () Real)
(assert (and (<= 0 Real____00812) (<= Real____00812 1)))
(declare-fun Real____00814 () Real)
(assert (and (<= 0 Real____00814) (<= Real____00814 1)))
(declare-fun Real____00816 () Real)
(assert (and (<= 0 Real____00816) (<= Real____00816 1)))
(declare-fun Real____00818 () Real)
(assert (and (<= 0 Real____00818) (<= Real____00818 1)))
(declare-fun Real____00820 () Real)
(assert (and (<= 0 Real____00820) (<= Real____00820 1)))
(declare-fun Real____00822 () Real)
(assert (and (<= 0 Real____00822) (<= Real____00822 1)))
(declare-fun Real____00824 () Real)
(assert (and (<= 0 Real____00824) (<= Real____00824 1)))
(declare-fun Real____00826 () Real)
(assert (and (<= 0 Real____00826) (<= Real____00826 1)))
(declare-fun Real____00828 () Real)
(assert (and (<= 0 Real____00828) (<= Real____00828 1)))
(declare-fun Real____00830 () Real)
(assert (and (<= 0 Real____00830) (<= Real____00830 1)))
(declare-fun Real____00832 () Real)
(assert (and (<= 0 Real____00832) (<= Real____00832 1)))
(declare-fun Real____00834 () Real)
(assert (and (<= 0 Real____00834) (<= Real____00834 1)))
(declare-fun Real____00836 () Real)
(assert (and (<= 0 Real____00836) (<= Real____00836 1)))
(declare-fun Real____00838 () Real)
(assert (and (<= 0 Real____00838) (<= Real____00838 1)))
(declare-fun Real____00840 () Real)
(assert (and (<= 0 Real____00840) (<= Real____00840 1)))
(declare-fun Real____00842 () Real)
(assert (and (<= 0 Real____00842) (<= Real____00842 1)))
(declare-fun Real____00844 () Real)
(assert (and (<= 0 Real____00844) (<= Real____00844 1)))
(declare-fun Real____00846 () Real)
(assert (and (<= 0 Real____00846) (<= Real____00846 1)))
(declare-fun Real____00848 () Real)
(assert (and (<= 0 Real____00848) (<= Real____00848 1)))
(declare-fun Real____00850 () Real)
(assert (and (<= 0 Real____00850) (<= Real____00850 1)))
(declare-fun Real____00852 () Real)
(assert (and (<= 0 Real____00852) (<= Real____00852 1)))
(declare-fun Real____00854 () Real)
(assert (and (<= 0 Real____00854) (<= Real____00854 1)))
(declare-fun Real____00856 () Real)
(assert (and (<= 0 Real____00856) (<= Real____00856 1)))
(declare-fun Real____00858 () Real)
(assert (and (<= 0 Real____00858) (<= Real____00858 1)))
(declare-fun Real____00860 () Real)
(assert (and (<= 0 Real____00860) (<= Real____00860 1)))
(declare-fun Real____00862 () Real)
(assert (and (<= 0 Real____00862) (<= Real____00862 1)))
(declare-fun Real____00864 () Real)
(assert (and (<= 0 Real____00864) (<= Real____00864 1)))
(declare-fun Real____00866 () Real)
(assert (and (<= 0 Real____00866) (<= Real____00866 1)))
(declare-fun Real____00868 () Real)
(assert (and (<= 0 Real____00868) (<= Real____00868 1)))
(declare-fun Real____00870 () Real)
(assert (and (<= 0 Real____00870) (<= Real____00870 1)))
(declare-fun Real____00872 () Real)
(assert (and (<= 0 Real____00872) (<= Real____00872 1)))
(declare-fun Real____00874 () Real)
(assert (and (<= 0 Real____00874) (<= Real____00874 1)))
(declare-fun Real____00876 () Real)
(assert (and (<= 0 Real____00876) (<= Real____00876 1)))
(declare-fun Real____00878 () Real)
(assert (and (<= 0 Real____00878) (<= Real____00878 1)))
(declare-fun Real____00880 () Real)
(assert (and (<= 0 Real____00880) (<= Real____00880 1)))
(declare-fun Real____00882 () Real)
(assert (and (<= 0 Real____00882) (<= Real____00882 1)))
(declare-fun Real____00884 () Real)
(assert (and (<= 0 Real____00884) (<= Real____00884 1)))
(declare-fun Real____00886 () Real)
(assert (and (<= 0 Real____00886) (<= Real____00886 1)))
(declare-fun Real____00888 () Real)
(assert (and (<= 0 Real____00888) (<= Real____00888 1)))
(declare-fun Real____00890 () Real)
(assert (and (<= 0 Real____00890) (<= Real____00890 1)))
(declare-fun Real____00892 () Real)
(assert (and (<= 0 Real____00892) (<= Real____00892 1)))
(declare-fun Real____00894 () Real)
(assert (and (<= 0 Real____00894) (<= Real____00894 1)))
(declare-fun Real____00896 () Real)
(assert (and (<= 0 Real____00896) (<= Real____00896 1)))
(declare-fun Real____00898 () Real)
(assert (and (<= 0 Real____00898) (<= Real____00898 1)))
(declare-fun Real____00900 () Real)
(assert (and (<= 0 Real____00900) (<= Real____00900 1)))
;; Declaring array [1..50] of type Real, named course_period.
(declare-fun course_period__ARRAY__1 () Real)
(declare-fun course_period__ARRAY__2 () Real)
(declare-fun course_period__ARRAY__3 () Real)
(declare-fun course_period__ARRAY__4 () Real)
(declare-fun course_period__ARRAY__5 () Real)
(declare-fun course_period__ARRAY__6 () Real)
(declare-fun course_period__ARRAY__7 () Real)
(declare-fun course_period__ARRAY__8 () Real)
(declare-fun course_period__ARRAY__9 () Real)
(declare-fun course_period__ARRAY__10 () Real)
(declare-fun course_period__ARRAY__11 () Real)
(declare-fun course_period__ARRAY__12 () Real)
(declare-fun course_period__ARRAY__13 () Real)
(declare-fun course_period__ARRAY__14 () Real)
(declare-fun course_period__ARRAY__15 () Real)
(declare-fun course_period__ARRAY__16 () Real)
(declare-fun course_period__ARRAY__17 () Real)
(declare-fun course_period__ARRAY__18 () Real)
(declare-fun course_period__ARRAY__19 () Real)
(declare-fun course_period__ARRAY__20 () Real)
(declare-fun course_period__ARRAY__21 () Real)
(declare-fun course_period__ARRAY__22 () Real)
(declare-fun course_period__ARRAY__23 () Real)
(declare-fun course_period__ARRAY__24 () Real)
(declare-fun course_period__ARRAY__25 () Real)
(declare-fun course_period__ARRAY__26 () Real)
(declare-fun course_period__ARRAY__27 () Real)
(declare-fun course_period__ARRAY__28 () Real)
(declare-fun course_period__ARRAY__29 () Real)
(declare-fun course_period__ARRAY__30 () Real)
(declare-fun course_period__ARRAY__31 () Real)
(declare-fun course_period__ARRAY__32 () Real)
(declare-fun course_period__ARRAY__33 () Real)
(declare-fun course_period__ARRAY__34 () Real)
(declare-fun course_period__ARRAY__35 () Real)
(declare-fun course_period__ARRAY__36 () Real)
(declare-fun course_period__ARRAY__37 () Real)
(declare-fun course_period__ARRAY__38 () Real)
(declare-fun course_period__ARRAY__39 () Real)
(declare-fun course_period__ARRAY__40 () Real)
(declare-fun course_period__ARRAY__41 () Real)
(declare-fun course_period__ARRAY__42 () Real)
(declare-fun course_period__ARRAY__43 () Real)
(declare-fun course_period__ARRAY__44 () Real)
(declare-fun course_period__ARRAY__45 () Real)
(declare-fun course_period__ARRAY__46 () Real)
(declare-fun course_period__ARRAY__47 () Real)
(declare-fun course_period__ARRAY__48 () Real)
(declare-fun course_period__ARRAY__49 () Real)
(declare-fun course_period__ARRAY__50 () Real)
(assert (and (<= 1 course_period__ARRAY__1) (<= course_period__ARRAY__1 10)))
(assert (and (<= 1 course_period__ARRAY__2) (<= course_period__ARRAY__2 10)))
(assert (and (<= 1 course_period__ARRAY__3) (<= course_period__ARRAY__3 10)))
(assert (and (<= 1 course_period__ARRAY__4) (<= course_period__ARRAY__4 10)))
(assert (and (<= 1 course_period__ARRAY__5) (<= course_period__ARRAY__5 10)))
(assert (and (<= 1 course_period__ARRAY__6) (<= course_period__ARRAY__6 10)))
(assert (and (<= 1 course_period__ARRAY__7) (<= course_period__ARRAY__7 10)))
(assert (and (<= 1 course_period__ARRAY__8) (<= course_period__ARRAY__8 10)))
(assert (and (<= 1 course_period__ARRAY__9) (<= course_period__ARRAY__9 10)))
(assert (and (<= 1 course_period__ARRAY__10) (<= course_period__ARRAY__10 10)))
(assert (and (<= 1 course_period__ARRAY__11) (<= course_period__ARRAY__11 10)))
(assert (and (<= 1 course_period__ARRAY__12) (<= course_period__ARRAY__12 10)))
(assert (and (<= 1 course_period__ARRAY__13) (<= course_period__ARRAY__13 10)))
(assert (and (<= 1 course_period__ARRAY__14) (<= course_period__ARRAY__14 10)))
(assert (and (<= 1 course_period__ARRAY__15) (<= course_period__ARRAY__15 10)))
(assert (and (<= 1 course_period__ARRAY__16) (<= course_period__ARRAY__16 10)))
(assert (and (<= 1 course_period__ARRAY__17) (<= course_period__ARRAY__17 10)))
(assert (and (<= 1 course_period__ARRAY__18) (<= course_period__ARRAY__18 10)))
(assert (and (<= 1 course_period__ARRAY__19) (<= course_period__ARRAY__19 10)))
(assert (and (<= 1 course_period__ARRAY__20) (<= course_period__ARRAY__20 10)))
(assert (and (<= 1 course_period__ARRAY__21) (<= course_period__ARRAY__21 10)))
(assert (and (<= 1 course_period__ARRAY__22) (<= course_period__ARRAY__22 10)))
(assert (and (<= 1 course_period__ARRAY__23) (<= course_period__ARRAY__23 10)))
(assert (and (<= 1 course_period__ARRAY__24) (<= course_period__ARRAY__24 10)))
(assert (and (<= 1 course_period__ARRAY__25) (<= course_period__ARRAY__25 10)))
(assert (and (<= 1 course_period__ARRAY__26) (<= course_period__ARRAY__26 10)))
(assert (and (<= 1 course_period__ARRAY__27) (<= course_period__ARRAY__27 10)))
(assert (and (<= 1 course_period__ARRAY__28) (<= course_period__ARRAY__28 10)))
(assert (and (<= 1 course_period__ARRAY__29) (<= course_period__ARRAY__29 10)))
(assert (and (<= 1 course_period__ARRAY__30) (<= course_period__ARRAY__30 10)))
(assert (and (<= 1 course_period__ARRAY__31) (<= course_period__ARRAY__31 10)))
(assert (and (<= 1 course_period__ARRAY__32) (<= course_period__ARRAY__32 10)))
(assert (and (<= 1 course_period__ARRAY__33) (<= course_period__ARRAY__33 10)))
(assert (and (<= 1 course_period__ARRAY__34) (<= course_period__ARRAY__34 10)))
(assert (and (<= 1 course_period__ARRAY__35) (<= course_period__ARRAY__35 10)))
(assert (and (<= 1 course_period__ARRAY__36) (<= course_period__ARRAY__36 10)))
(assert (and (<= 1 course_period__ARRAY__37) (<= course_period__ARRAY__37 10)))
(assert (and (<= 1 course_period__ARRAY__38) (<= course_period__ARRAY__38 10)))
(assert (and (<= 1 course_period__ARRAY__39) (<= course_period__ARRAY__39 10)))
(assert (and (<= 1 course_period__ARRAY__40) (<= course_period__ARRAY__40 10)))
(assert (and (<= 1 course_period__ARRAY__41) (<= course_period__ARRAY__41 10)))
(assert (and (<= 1 course_period__ARRAY__42) (<= course_period__ARRAY__42 10)))
(assert (and (<= 1 course_period__ARRAY__43) (<= course_period__ARRAY__43 10)))
(assert (and (<= 1 course_period__ARRAY__44) (<= course_period__ARRAY__44 10)))
(assert (and (<= 1 course_period__ARRAY__45) (<= course_period__ARRAY__45 10)))
(assert (and (<= 1 course_period__ARRAY__46) (<= course_period__ARRAY__46 10)))
(assert (and (<= 1 course_period__ARRAY__47) (<= course_period__ARRAY__47 10)))
(assert (and (<= 1 course_period__ARRAY__48) (<= course_period__ARRAY__48 10)))
(assert (and (<= 1 course_period__ARRAY__49) (<= course_period__ARRAY__49 10)))
(assert (and (<= 1 course_period__ARRAY__50) (<= course_period__ARRAY__50 10)))
;; Declaring array [1..10] of type Real, named load.
(declare-fun load__ARRAY__1 () Real)
(declare-fun load__ARRAY__2 () Real)
(declare-fun load__ARRAY__3 () Real)
(declare-fun load__ARRAY__4 () Real)
(declare-fun load__ARRAY__5 () Real)
(declare-fun load__ARRAY__6 () Real)
(declare-fun load__ARRAY__7 () Real)
(declare-fun load__ARRAY__8 () Real)
(declare-fun load__ARRAY__9 () Real)
(declare-fun load__ARRAY__10 () Real)
(assert (and (<= 2 load__ARRAY__1) (<= load__ARRAY__1 100)))
(assert (and (<= 2 load__ARRAY__2) (<= load__ARRAY__2 100)))
(assert (and (<= 2 load__ARRAY__3) (<= load__ARRAY__3 100)))
(assert (and (<= 2 load__ARRAY__4) (<= load__ARRAY__4 100)))
(assert (and (<= 2 load__ARRAY__5) (<= load__ARRAY__5 100)))
(assert (and (<= 2 load__ARRAY__6) (<= load__ARRAY__6 100)))
(assert (and (<= 2 load__ARRAY__7) (<= load__ARRAY__7 100)))
(assert (and (<= 2 load__ARRAY__8) (<= load__ARRAY__8 100)))
(assert (and (<= 2 load__ARRAY__9) (<= load__ARRAY__9 100)))
(assert (and (<= 2 load__ARRAY__10) (<= load__ARRAY__10 100)))
(declare-fun objective () Real)
(assert (and (<= 2 objective) (<= objective 100)))
;; Declaring array [1..500] of type Real, named x.
(declare-fun x__ARRAY__1 () Real)
(declare-fun x__ARRAY__2 () Real)
(declare-fun x__ARRAY__3 () Real)
(declare-fun x__ARRAY__4 () Real)
(declare-fun x__ARRAY__5 () Real)
(declare-fun x__ARRAY__6 () Real)
(declare-fun x__ARRAY__7 () Real)
(declare-fun x__ARRAY__8 () Real)
(declare-fun x__ARRAY__9 () Real)
(declare-fun x__ARRAY__10 () Real)
(declare-fun x__ARRAY__11 () Real)
(declare-fun x__ARRAY__12 () Real)
(declare-fun x__ARRAY__13 () Real)
(declare-fun x__ARRAY__14 () Real)
(declare-fun x__ARRAY__15 () Real)
(declare-fun x__ARRAY__16 () Real)
(declare-fun x__ARRAY__17 () Real)
(declare-fun x__ARRAY__18 () Real)
(declare-fun x__ARRAY__19 () Real)
(declare-fun x__ARRAY__20 () Real)
(declare-fun x__ARRAY__21 () Real)
(declare-fun x__ARRAY__22 () Real)
(declare-fun x__ARRAY__23 () Real)
(declare-fun x__ARRAY__24 () Real)
(declare-fun x__ARRAY__25 () Real)
(declare-fun x__ARRAY__26 () Real)
(declare-fun x__ARRAY__27 () Real)
(declare-fun x__ARRAY__28 () Real)
(declare-fun x__ARRAY__29 () Real)
(declare-fun x__ARRAY__30 () Real)
(declare-fun x__ARRAY__31 () Real)
(declare-fun x__ARRAY__32 () Real)
(declare-fun x__ARRAY__33 () Real)
(declare-fun x__ARRAY__34 () Real)
(declare-fun x__ARRAY__35 () Real)
(declare-fun x__ARRAY__36 () Real)
(declare-fun x__ARRAY__37 () Real)
(declare-fun x__ARRAY__38 () Real)
(declare-fun x__ARRAY__39 () Real)
(declare-fun x__ARRAY__40 () Real)
(declare-fun x__ARRAY__41 () Real)
(declare-fun x__ARRAY__42 () Real)
(declare-fun x__ARRAY__43 () Real)
(declare-fun x__ARRAY__44 () Real)
(declare-fun x__ARRAY__45 () Real)
(declare-fun x__ARRAY__46 () Real)
(declare-fun x__ARRAY__47 () Real)
(declare-fun x__ARRAY__48 () Real)
(declare-fun x__ARRAY__49 () Real)
(declare-fun x__ARRAY__50 () Real)
(declare-fun x__ARRAY__51 () Real)
(declare-fun x__ARRAY__52 () Real)
(declare-fun x__ARRAY__53 () Real)
(declare-fun x__ARRAY__54 () Real)
(declare-fun x__ARRAY__55 () Real)
(declare-fun x__ARRAY__56 () Real)
(declare-fun x__ARRAY__57 () Real)
(declare-fun x__ARRAY__58 () Real)
(declare-fun x__ARRAY__59 () Real)
(declare-fun x__ARRAY__60 () Real)
(declare-fun x__ARRAY__61 () Real)
(declare-fun x__ARRAY__62 () Real)
(declare-fun x__ARRAY__63 () Real)
(declare-fun x__ARRAY__64 () Real)
(declare-fun x__ARRAY__65 () Real)
(declare-fun x__ARRAY__66 () Real)
(declare-fun x__ARRAY__67 () Real)
(declare-fun x__ARRAY__68 () Real)
(declare-fun x__ARRAY__69 () Real)
(declare-fun x__ARRAY__70 () Real)
(declare-fun x__ARRAY__71 () Real)
(declare-fun x__ARRAY__72 () Real)
(declare-fun x__ARRAY__73 () Real)
(declare-fun x__ARRAY__74 () Real)
(declare-fun x__ARRAY__75 () Real)
(declare-fun x__ARRAY__76 () Real)
(declare-fun x__ARRAY__77 () Real)
(declare-fun x__ARRAY__78 () Real)
(declare-fun x__ARRAY__79 () Real)
(declare-fun x__ARRAY__80 () Real)
(declare-fun x__ARRAY__81 () Real)
(declare-fun x__ARRAY__82 () Real)
(declare-fun x__ARRAY__83 () Real)
(declare-fun x__ARRAY__84 () Real)
(declare-fun x__ARRAY__85 () Real)
(declare-fun x__ARRAY__86 () Real)
(declare-fun x__ARRAY__87 () Real)
(declare-fun x__ARRAY__88 () Real)
(declare-fun x__ARRAY__89 () Real)
(declare-fun x__ARRAY__90 () Real)
(declare-fun x__ARRAY__91 () Real)
(declare-fun x__ARRAY__92 () Real)
(declare-fun x__ARRAY__93 () Real)
(declare-fun x__ARRAY__94 () Real)
(declare-fun x__ARRAY__95 () Real)
(declare-fun x__ARRAY__96 () Real)
(declare-fun x__ARRAY__97 () Real)
(declare-fun x__ARRAY__98 () Real)
(declare-fun x__ARRAY__99 () Real)
(declare-fun x__ARRAY__100 () Real)
(declare-fun x__ARRAY__101 () Real)
(declare-fun x__ARRAY__102 () Real)
(declare-fun x__ARRAY__103 () Real)
(declare-fun x__ARRAY__104 () Real)
(declare-fun x__ARRAY__105 () Real)
(declare-fun x__ARRAY__106 () Real)
(declare-fun x__ARRAY__107 () Real)
(declare-fun x__ARRAY__108 () Real)
(declare-fun x__ARRAY__109 () Real)
(declare-fun x__ARRAY__110 () Real)
(declare-fun x__ARRAY__111 () Real)
(declare-fun x__ARRAY__112 () Real)
(declare-fun x__ARRAY__113 () Real)
(declare-fun x__ARRAY__114 () Real)
(declare-fun x__ARRAY__115 () Real)
(declare-fun x__ARRAY__116 () Real)
(declare-fun x__ARRAY__117 () Real)
(declare-fun x__ARRAY__118 () Real)
(declare-fun x__ARRAY__119 () Real)
(declare-fun x__ARRAY__120 () Real)
(declare-fun x__ARRAY__121 () Real)
(declare-fun x__ARRAY__122 () Real)
(declare-fun x__ARRAY__123 () Real)
(declare-fun x__ARRAY__124 () Real)
(declare-fun x__ARRAY__125 () Real)
(declare-fun x__ARRAY__126 () Real)
(declare-fun x__ARRAY__127 () Real)
(declare-fun x__ARRAY__128 () Real)
(declare-fun x__ARRAY__129 () Real)
(declare-fun x__ARRAY__130 () Real)
(declare-fun x__ARRAY__131 () Real)
(declare-fun x__ARRAY__132 () Real)
(declare-fun x__ARRAY__133 () Real)
(declare-fun x__ARRAY__134 () Real)
(declare-fun x__ARRAY__135 () Real)
(declare-fun x__ARRAY__136 () Real)
(declare-fun x__ARRAY__137 () Real)
(declare-fun x__ARRAY__138 () Real)
(declare-fun x__ARRAY__139 () Real)
(declare-fun x__ARRAY__140 () Real)
(declare-fun x__ARRAY__141 () Real)
(declare-fun x__ARRAY__142 () Real)
(declare-fun x__ARRAY__143 () Real)
(declare-fun x__ARRAY__144 () Real)
(declare-fun x__ARRAY__145 () Real)
(declare-fun x__ARRAY__146 () Real)
(declare-fun x__ARRAY__147 () Real)
(declare-fun x__ARRAY__148 () Real)
(declare-fun x__ARRAY__149 () Real)
(declare-fun x__ARRAY__150 () Real)
(declare-fun x__ARRAY__151 () Real)
(declare-fun x__ARRAY__152 () Real)
(declare-fun x__ARRAY__153 () Real)
(declare-fun x__ARRAY__154 () Real)
(declare-fun x__ARRAY__155 () Real)
(declare-fun x__ARRAY__156 () Real)
(declare-fun x__ARRAY__157 () Real)
(declare-fun x__ARRAY__158 () Real)
(declare-fun x__ARRAY__159 () Real)
(declare-fun x__ARRAY__160 () Real)
(declare-fun x__ARRAY__161 () Real)
(declare-fun x__ARRAY__162 () Real)
(declare-fun x__ARRAY__163 () Real)
(declare-fun x__ARRAY__164 () Real)
(declare-fun x__ARRAY__165 () Real)
(declare-fun x__ARRAY__166 () Real)
(declare-fun x__ARRAY__167 () Real)
(declare-fun x__ARRAY__168 () Real)
(declare-fun x__ARRAY__169 () Real)
(declare-fun x__ARRAY__170 () Real)
(declare-fun x__ARRAY__171 () Real)
(declare-fun x__ARRAY__172 () Real)
(declare-fun x__ARRAY__173 () Real)
(declare-fun x__ARRAY__174 () Real)
(declare-fun x__ARRAY__175 () Real)
(declare-fun x__ARRAY__176 () Real)
(declare-fun x__ARRAY__177 () Real)
(declare-fun x__ARRAY__178 () Real)
(declare-fun x__ARRAY__179 () Real)
(declare-fun x__ARRAY__180 () Real)
(declare-fun x__ARRAY__181 () Real)
(declare-fun x__ARRAY__182 () Real)
(declare-fun x__ARRAY__183 () Real)
(declare-fun x__ARRAY__184 () Real)
(declare-fun x__ARRAY__185 () Real)
(declare-fun x__ARRAY__186 () Real)
(declare-fun x__ARRAY__187 () Real)
(declare-fun x__ARRAY__188 () Real)
(declare-fun x__ARRAY__189 () Real)
(declare-fun x__ARRAY__190 () Real)
(declare-fun x__ARRAY__191 () Real)
(declare-fun x__ARRAY__192 () Real)
(declare-fun x__ARRAY__193 () Real)
(declare-fun x__ARRAY__194 () Real)
(declare-fun x__ARRAY__195 () Real)
(declare-fun x__ARRAY__196 () Real)
(declare-fun x__ARRAY__197 () Real)
(declare-fun x__ARRAY__198 () Real)
(declare-fun x__ARRAY__199 () Real)
(declare-fun x__ARRAY__200 () Real)
(declare-fun x__ARRAY__201 () Real)
(declare-fun x__ARRAY__202 () Real)
(declare-fun x__ARRAY__203 () Real)
(declare-fun x__ARRAY__204 () Real)
(declare-fun x__ARRAY__205 () Real)
(declare-fun x__ARRAY__206 () Real)
(declare-fun x__ARRAY__207 () Real)
(declare-fun x__ARRAY__208 () Real)
(declare-fun x__ARRAY__209 () Real)
(declare-fun x__ARRAY__210 () Real)
(declare-fun x__ARRAY__211 () Real)
(declare-fun x__ARRAY__212 () Real)
(declare-fun x__ARRAY__213 () Real)
(declare-fun x__ARRAY__214 () Real)
(declare-fun x__ARRAY__215 () Real)
(declare-fun x__ARRAY__216 () Real)
(declare-fun x__ARRAY__217 () Real)
(declare-fun x__ARRAY__218 () Real)
(declare-fun x__ARRAY__219 () Real)
(declare-fun x__ARRAY__220 () Real)
(declare-fun x__ARRAY__221 () Real)
(declare-fun x__ARRAY__222 () Real)
(declare-fun x__ARRAY__223 () Real)
(declare-fun x__ARRAY__224 () Real)
(declare-fun x__ARRAY__225 () Real)
(declare-fun x__ARRAY__226 () Real)
(declare-fun x__ARRAY__227 () Real)
(declare-fun x__ARRAY__228 () Real)
(declare-fun x__ARRAY__229 () Real)
(declare-fun x__ARRAY__230 () Real)
(declare-fun x__ARRAY__231 () Real)
(declare-fun x__ARRAY__232 () Real)
(declare-fun x__ARRAY__233 () Real)
(declare-fun x__ARRAY__234 () Real)
(declare-fun x__ARRAY__235 () Real)
(declare-fun x__ARRAY__236 () Real)
(declare-fun x__ARRAY__237 () Real)
(declare-fun x__ARRAY__238 () Real)
(declare-fun x__ARRAY__239 () Real)
(declare-fun x__ARRAY__240 () Real)
(declare-fun x__ARRAY__241 () Real)
(declare-fun x__ARRAY__242 () Real)
(declare-fun x__ARRAY__243 () Real)
(declare-fun x__ARRAY__244 () Real)
(declare-fun x__ARRAY__245 () Real)
(declare-fun x__ARRAY__246 () Real)
(declare-fun x__ARRAY__247 () Real)
(declare-fun x__ARRAY__248 () Real)
(declare-fun x__ARRAY__249 () Real)
(declare-fun x__ARRAY__250 () Real)
(declare-fun x__ARRAY__251 () Real)
(declare-fun x__ARRAY__252 () Real)
(declare-fun x__ARRAY__253 () Real)
(declare-fun x__ARRAY__254 () Real)
(declare-fun x__ARRAY__255 () Real)
(declare-fun x__ARRAY__256 () Real)
(declare-fun x__ARRAY__257 () Real)
(declare-fun x__ARRAY__258 () Real)
(declare-fun x__ARRAY__259 () Real)
(declare-fun x__ARRAY__260 () Real)
(declare-fun x__ARRAY__261 () Real)
(declare-fun x__ARRAY__262 () Real)
(declare-fun x__ARRAY__263 () Real)
(declare-fun x__ARRAY__264 () Real)
(declare-fun x__ARRAY__265 () Real)
(declare-fun x__ARRAY__266 () Real)
(declare-fun x__ARRAY__267 () Real)
(declare-fun x__ARRAY__268 () Real)
(declare-fun x__ARRAY__269 () Real)
(declare-fun x__ARRAY__270 () Real)
(declare-fun x__ARRAY__271 () Real)
(declare-fun x__ARRAY__272 () Real)
(declare-fun x__ARRAY__273 () Real)
(declare-fun x__ARRAY__274 () Real)
(declare-fun x__ARRAY__275 () Real)
(declare-fun x__ARRAY__276 () Real)
(declare-fun x__ARRAY__277 () Real)
(declare-fun x__ARRAY__278 () Real)
(declare-fun x__ARRAY__279 () Real)
(declare-fun x__ARRAY__280 () Real)
(declare-fun x__ARRAY__281 () Real)
(declare-fun x__ARRAY__282 () Real)
(declare-fun x__ARRAY__283 () Real)
(declare-fun x__ARRAY__284 () Real)
(declare-fun x__ARRAY__285 () Real)
(declare-fun x__ARRAY__286 () Real)
(declare-fun x__ARRAY__287 () Real)
(declare-fun x__ARRAY__288 () Real)
(declare-fun x__ARRAY__289 () Real)
(declare-fun x__ARRAY__290 () Real)
(declare-fun x__ARRAY__291 () Real)
(declare-fun x__ARRAY__292 () Real)
(declare-fun x__ARRAY__293 () Real)
(declare-fun x__ARRAY__294 () Real)
(declare-fun x__ARRAY__295 () Real)
(declare-fun x__ARRAY__296 () Real)
(declare-fun x__ARRAY__297 () Real)
(declare-fun x__ARRAY__298 () Real)
(declare-fun x__ARRAY__299 () Real)
(declare-fun x__ARRAY__300 () Real)
(declare-fun x__ARRAY__301 () Real)
(declare-fun x__ARRAY__302 () Real)
(declare-fun x__ARRAY__303 () Real)
(declare-fun x__ARRAY__304 () Real)
(declare-fun x__ARRAY__305 () Real)
(declare-fun x__ARRAY__306 () Real)
(declare-fun x__ARRAY__307 () Real)
(declare-fun x__ARRAY__308 () Real)
(declare-fun x__ARRAY__309 () Real)
(declare-fun x__ARRAY__310 () Real)
(declare-fun x__ARRAY__311 () Real)
(declare-fun x__ARRAY__312 () Real)
(declare-fun x__ARRAY__313 () Real)
(declare-fun x__ARRAY__314 () Real)
(declare-fun x__ARRAY__315 () Real)
(declare-fun x__ARRAY__316 () Real)
(declare-fun x__ARRAY__317 () Real)
(declare-fun x__ARRAY__318 () Real)
(declare-fun x__ARRAY__319 () Real)
(declare-fun x__ARRAY__320 () Real)
(declare-fun x__ARRAY__321 () Real)
(declare-fun x__ARRAY__322 () Real)
(declare-fun x__ARRAY__323 () Real)
(declare-fun x__ARRAY__324 () Real)
(declare-fun x__ARRAY__325 () Real)
(declare-fun x__ARRAY__326 () Real)
(declare-fun x__ARRAY__327 () Real)
(declare-fun x__ARRAY__328 () Real)
(declare-fun x__ARRAY__329 () Real)
(declare-fun x__ARRAY__330 () Real)
(declare-fun x__ARRAY__331 () Real)
(declare-fun x__ARRAY__332 () Real)
(declare-fun x__ARRAY__333 () Real)
(declare-fun x__ARRAY__334 () Real)
(declare-fun x__ARRAY__335 () Real)
(declare-fun x__ARRAY__336 () Real)
(declare-fun x__ARRAY__337 () Real)
(declare-fun x__ARRAY__338 () Real)
(declare-fun x__ARRAY__339 () Real)
(declare-fun x__ARRAY__340 () Real)
(declare-fun x__ARRAY__341 () Real)
(declare-fun x__ARRAY__342 () Real)
(declare-fun x__ARRAY__343 () Real)
(declare-fun x__ARRAY__344 () Real)
(declare-fun x__ARRAY__345 () Real)
(declare-fun x__ARRAY__346 () Real)
(declare-fun x__ARRAY__347 () Real)
(declare-fun x__ARRAY__348 () Real)
(declare-fun x__ARRAY__349 () Real)
(declare-fun x__ARRAY__350 () Real)
(declare-fun x__ARRAY__351 () Real)
(declare-fun x__ARRAY__352 () Real)
(declare-fun x__ARRAY__353 () Real)
(declare-fun x__ARRAY__354 () Real)
(declare-fun x__ARRAY__355 () Real)
(declare-fun x__ARRAY__356 () Real)
(declare-fun x__ARRAY__357 () Real)
(declare-fun x__ARRAY__358 () Real)
(declare-fun x__ARRAY__359 () Real)
(declare-fun x__ARRAY__360 () Real)
(declare-fun x__ARRAY__361 () Real)
(declare-fun x__ARRAY__362 () Real)
(declare-fun x__ARRAY__363 () Real)
(declare-fun x__ARRAY__364 () Real)
(declare-fun x__ARRAY__365 () Real)
(declare-fun x__ARRAY__366 () Real)
(declare-fun x__ARRAY__367 () Real)
(declare-fun x__ARRAY__368 () Real)
(declare-fun x__ARRAY__369 () Real)
(declare-fun x__ARRAY__370 () Real)
(declare-fun x__ARRAY__371 () Real)
(declare-fun x__ARRAY__372 () Real)
(declare-fun x__ARRAY__373 () Real)
(declare-fun x__ARRAY__374 () Real)
(declare-fun x__ARRAY__375 () Real)
(declare-fun x__ARRAY__376 () Real)
(declare-fun x__ARRAY__377 () Real)
(declare-fun x__ARRAY__378 () Real)
(declare-fun x__ARRAY__379 () Real)
(declare-fun x__ARRAY__380 () Real)
(declare-fun x__ARRAY__381 () Real)
(declare-fun x__ARRAY__382 () Real)
(declare-fun x__ARRAY__383 () Real)
(declare-fun x__ARRAY__384 () Real)
(declare-fun x__ARRAY__385 () Real)
(declare-fun x__ARRAY__386 () Real)
(declare-fun x__ARRAY__387 () Real)
(declare-fun x__ARRAY__388 () Real)
(declare-fun x__ARRAY__389 () Real)
(declare-fun x__ARRAY__390 () Real)
(declare-fun x__ARRAY__391 () Real)
(declare-fun x__ARRAY__392 () Real)
(declare-fun x__ARRAY__393 () Real)
(declare-fun x__ARRAY__394 () Real)
(declare-fun x__ARRAY__395 () Real)
(declare-fun x__ARRAY__396 () Real)
(declare-fun x__ARRAY__397 () Real)
(declare-fun x__ARRAY__398 () Real)
(declare-fun x__ARRAY__399 () Real)
(declare-fun x__ARRAY__400 () Real)
(declare-fun x__ARRAY__401 () Real)
(declare-fun x__ARRAY__402 () Real)
(declare-fun x__ARRAY__403 () Real)
(declare-fun x__ARRAY__404 () Real)
(declare-fun x__ARRAY__405 () Real)
(declare-fun x__ARRAY__406 () Real)
(declare-fun x__ARRAY__407 () Real)
(declare-fun x__ARRAY__408 () Real)
(declare-fun x__ARRAY__409 () Real)
(declare-fun x__ARRAY__410 () Real)
(declare-fun x__ARRAY__411 () Real)
(declare-fun x__ARRAY__412 () Real)
(declare-fun x__ARRAY__413 () Real)
(declare-fun x__ARRAY__414 () Real)
(declare-fun x__ARRAY__415 () Real)
(declare-fun x__ARRAY__416 () Real)
(declare-fun x__ARRAY__417 () Real)
(declare-fun x__ARRAY__418 () Real)
(declare-fun x__ARRAY__419 () Real)
(declare-fun x__ARRAY__420 () Real)
(declare-fun x__ARRAY__421 () Real)
(declare-fun x__ARRAY__422 () Real)
(declare-fun x__ARRAY__423 () Real)
(declare-fun x__ARRAY__424 () Real)
(declare-fun x__ARRAY__425 () Real)
(declare-fun x__ARRAY__426 () Real)
(declare-fun x__ARRAY__427 () Real)
(declare-fun x__ARRAY__428 () Real)
(declare-fun x__ARRAY__429 () Real)
(declare-fun x__ARRAY__430 () Real)
(declare-fun x__ARRAY__431 () Real)
(declare-fun x__ARRAY__432 () Real)
(declare-fun x__ARRAY__433 () Real)
(declare-fun x__ARRAY__434 () Real)
(declare-fun x__ARRAY__435 () Real)
(declare-fun x__ARRAY__436 () Real)
(declare-fun x__ARRAY__437 () Real)
(declare-fun x__ARRAY__438 () Real)
(declare-fun x__ARRAY__439 () Real)
(declare-fun x__ARRAY__440 () Real)
(declare-fun x__ARRAY__441 () Real)
(declare-fun x__ARRAY__442 () Real)
(declare-fun x__ARRAY__443 () Real)
(declare-fun x__ARRAY__444 () Real)
(declare-fun x__ARRAY__445 () Real)
(declare-fun x__ARRAY__446 () Real)
(declare-fun x__ARRAY__447 () Real)
(declare-fun x__ARRAY__448 () Real)
(declare-fun x__ARRAY__449 () Real)
(declare-fun x__ARRAY__450 () Real)
(declare-fun x__ARRAY__451 () Real)
(declare-fun x__ARRAY__452 () Real)
(declare-fun x__ARRAY__453 () Real)
(declare-fun x__ARRAY__454 () Real)
(declare-fun x__ARRAY__455 () Real)
(declare-fun x__ARRAY__456 () Real)
(declare-fun x__ARRAY__457 () Real)
(declare-fun x__ARRAY__458 () Real)
(declare-fun x__ARRAY__459 () Real)
(declare-fun x__ARRAY__460 () Real)
(declare-fun x__ARRAY__461 () Real)
(declare-fun x__ARRAY__462 () Real)
(declare-fun x__ARRAY__463 () Real)
(declare-fun x__ARRAY__464 () Real)
(declare-fun x__ARRAY__465 () Real)
(declare-fun x__ARRAY__466 () Real)
(declare-fun x__ARRAY__467 () Real)
(declare-fun x__ARRAY__468 () Real)
(declare-fun x__ARRAY__469 () Real)
(declare-fun x__ARRAY__470 () Real)
(declare-fun x__ARRAY__471 () Real)
(declare-fun x__ARRAY__472 () Real)
(declare-fun x__ARRAY__473 () Real)
(declare-fun x__ARRAY__474 () Real)
(declare-fun x__ARRAY__475 () Real)
(declare-fun x__ARRAY__476 () Real)
(declare-fun x__ARRAY__477 () Real)
(declare-fun x__ARRAY__478 () Real)
(declare-fun x__ARRAY__479 () Real)
(declare-fun x__ARRAY__480 () Real)
(declare-fun x__ARRAY__481 () Real)
(declare-fun x__ARRAY__482 () Real)
(declare-fun x__ARRAY__483 () Real)
(declare-fun x__ARRAY__484 () Real)
(declare-fun x__ARRAY__485 () Real)
(declare-fun x__ARRAY__486 () Real)
(declare-fun x__ARRAY__487 () Real)
(declare-fun x__ARRAY__488 () Real)
(declare-fun x__ARRAY__489 () Real)
(declare-fun x__ARRAY__490 () Real)
(declare-fun x__ARRAY__491 () Real)
(declare-fun x__ARRAY__492 () Real)
(declare-fun x__ARRAY__493 () Real)
(declare-fun x__ARRAY__494 () Real)
(declare-fun x__ARRAY__495 () Real)
(declare-fun x__ARRAY__496 () Real)
(declare-fun x__ARRAY__497 () Real)
(declare-fun x__ARRAY__498 () Real)
(declare-fun x__ARRAY__499 () Real)
(declare-fun x__ARRAY__500 () Real)
(assert (and (<= 0 x__ARRAY__1) (<= x__ARRAY__1 1)))
(assert (and (<= 0 x__ARRAY__2) (<= x__ARRAY__2 1)))
(assert (and (<= 0 x__ARRAY__3) (<= x__ARRAY__3 1)))
(assert (and (<= 0 x__ARRAY__4) (<= x__ARRAY__4 1)))
(assert (and (<= 0 x__ARRAY__5) (<= x__ARRAY__5 1)))
(assert (and (<= 0 x__ARRAY__6) (<= x__ARRAY__6 1)))
(assert (and (<= 0 x__ARRAY__7) (<= x__ARRAY__7 1)))
(assert (and (<= 0 x__ARRAY__8) (<= x__ARRAY__8 1)))
(assert (and (<= 0 x__ARRAY__9) (<= x__ARRAY__9 1)))
(assert (and (<= 0 x__ARRAY__10) (<= x__ARRAY__10 1)))
(assert (and (<= 0 x__ARRAY__11) (<= x__ARRAY__11 1)))
(assert (and (<= 0 x__ARRAY__12) (<= x__ARRAY__12 1)))
(assert (and (<= 0 x__ARRAY__13) (<= x__ARRAY__13 1)))
(assert (and (<= 0 x__ARRAY__14) (<= x__ARRAY__14 1)))
(assert (and (<= 0 x__ARRAY__15) (<= x__ARRAY__15 1)))
(assert (and (<= 0 x__ARRAY__16) (<= x__ARRAY__16 1)))
(assert (and (<= 0 x__ARRAY__17) (<= x__ARRAY__17 1)))
(assert (and (<= 0 x__ARRAY__18) (<= x__ARRAY__18 1)))
(assert (and (<= 0 x__ARRAY__19) (<= x__ARRAY__19 1)))
(assert (and (<= 0 x__ARRAY__20) (<= x__ARRAY__20 1)))
(assert (and (<= 0 x__ARRAY__21) (<= x__ARRAY__21 1)))
(assert (and (<= 0 x__ARRAY__22) (<= x__ARRAY__22 1)))
(assert (and (<= 0 x__ARRAY__23) (<= x__ARRAY__23 1)))
(assert (and (<= 0 x__ARRAY__24) (<= x__ARRAY__24 1)))
(assert (and (<= 0 x__ARRAY__25) (<= x__ARRAY__25 1)))
(assert (and (<= 0 x__ARRAY__26) (<= x__ARRAY__26 1)))
(assert (and (<= 0 x__ARRAY__27) (<= x__ARRAY__27 1)))
(assert (and (<= 0 x__ARRAY__28) (<= x__ARRAY__28 1)))
(assert (and (<= 0 x__ARRAY__29) (<= x__ARRAY__29 1)))
(assert (and (<= 0 x__ARRAY__30) (<= x__ARRAY__30 1)))
(assert (and (<= 0 x__ARRAY__31) (<= x__ARRAY__31 1)))
(assert (and (<= 0 x__ARRAY__32) (<= x__ARRAY__32 1)))
(assert (and (<= 0 x__ARRAY__33) (<= x__ARRAY__33 1)))
(assert (and (<= 0 x__ARRAY__34) (<= x__ARRAY__34 1)))
(assert (and (<= 0 x__ARRAY__35) (<= x__ARRAY__35 1)))
(assert (and (<= 0 x__ARRAY__36) (<= x__ARRAY__36 1)))
(assert (and (<= 0 x__ARRAY__37) (<= x__ARRAY__37 1)))
(assert (and (<= 0 x__ARRAY__38) (<= x__ARRAY__38 1)))
(assert (and (<= 0 x__ARRAY__39) (<= x__ARRAY__39 1)))
(assert (and (<= 0 x__ARRAY__40) (<= x__ARRAY__40 1)))
(assert (and (<= 0 x__ARRAY__41) (<= x__ARRAY__41 1)))
(assert (and (<= 0 x__ARRAY__42) (<= x__ARRAY__42 1)))
(assert (and (<= 0 x__ARRAY__43) (<= x__ARRAY__43 1)))
(assert (and (<= 0 x__ARRAY__44) (<= x__ARRAY__44 1)))
(assert (and (<= 0 x__ARRAY__45) (<= x__ARRAY__45 1)))
(assert (and (<= 0 x__ARRAY__46) (<= x__ARRAY__46 1)))
(assert (and (<= 0 x__ARRAY__47) (<= x__ARRAY__47 1)))
(assert (and (<= 0 x__ARRAY__48) (<= x__ARRAY__48 1)))
(assert (and (<= 0 x__ARRAY__49) (<= x__ARRAY__49 1)))
(assert (and (<= 0 x__ARRAY__50) (<= x__ARRAY__50 1)))
(assert (and (<= 0 x__ARRAY__51) (<= x__ARRAY__51 1)))
(assert (and (<= 0 x__ARRAY__52) (<= x__ARRAY__52 1)))
(assert (and (<= 0 x__ARRAY__53) (<= x__ARRAY__53 1)))
(assert (and (<= 0 x__ARRAY__54) (<= x__ARRAY__54 1)))
(assert (and (<= 0 x__ARRAY__55) (<= x__ARRAY__55 1)))
(assert (and (<= 0 x__ARRAY__56) (<= x__ARRAY__56 1)))
(assert (and (<= 0 x__ARRAY__57) (<= x__ARRAY__57 1)))
(assert (and (<= 0 x__ARRAY__58) (<= x__ARRAY__58 1)))
(assert (and (<= 0 x__ARRAY__59) (<= x__ARRAY__59 1)))
(assert (and (<= 0 x__ARRAY__60) (<= x__ARRAY__60 1)))
(assert (and (<= 0 x__ARRAY__61) (<= x__ARRAY__61 1)))
(assert (and (<= 0 x__ARRAY__62) (<= x__ARRAY__62 1)))
(assert (and (<= 0 x__ARRAY__63) (<= x__ARRAY__63 1)))
(assert (and (<= 0 x__ARRAY__64) (<= x__ARRAY__64 1)))
(assert (and (<= 0 x__ARRAY__65) (<= x__ARRAY__65 1)))
(assert (and (<= 0 x__ARRAY__66) (<= x__ARRAY__66 1)))
(assert (and (<= 0 x__ARRAY__67) (<= x__ARRAY__67 1)))
(assert (and (<= 0 x__ARRAY__68) (<= x__ARRAY__68 1)))
(assert (and (<= 0 x__ARRAY__69) (<= x__ARRAY__69 1)))
(assert (and (<= 0 x__ARRAY__70) (<= x__ARRAY__70 1)))
(assert (and (<= 0 x__ARRAY__71) (<= x__ARRAY__71 1)))
(assert (and (<= 0 x__ARRAY__72) (<= x__ARRAY__72 1)))
(assert (and (<= 0 x__ARRAY__73) (<= x__ARRAY__73 1)))
(assert (and (<= 0 x__ARRAY__74) (<= x__ARRAY__74 1)))
(assert (and (<= 0 x__ARRAY__75) (<= x__ARRAY__75 1)))
(assert (and (<= 0 x__ARRAY__76) (<= x__ARRAY__76 1)))
(assert (and (<= 0 x__ARRAY__77) (<= x__ARRAY__77 1)))
(assert (and (<= 0 x__ARRAY__78) (<= x__ARRAY__78 1)))
(assert (and (<= 0 x__ARRAY__79) (<= x__ARRAY__79 1)))
(assert (and (<= 0 x__ARRAY__80) (<= x__ARRAY__80 1)))
(assert (and (<= 0 x__ARRAY__81) (<= x__ARRAY__81 1)))
(assert (and (<= 0 x__ARRAY__82) (<= x__ARRAY__82 1)))
(assert (and (<= 0 x__ARRAY__83) (<= x__ARRAY__83 1)))
(assert (and (<= 0 x__ARRAY__84) (<= x__ARRAY__84 1)))
(assert (and (<= 0 x__ARRAY__85) (<= x__ARRAY__85 1)))
(assert (and (<= 0 x__ARRAY__86) (<= x__ARRAY__86 1)))
(assert (and (<= 0 x__ARRAY__87) (<= x__ARRAY__87 1)))
(assert (and (<= 0 x__ARRAY__88) (<= x__ARRAY__88 1)))
(assert (and (<= 0 x__ARRAY__89) (<= x__ARRAY__89 1)))
(assert (and (<= 0 x__ARRAY__90) (<= x__ARRAY__90 1)))
(assert (and (<= 0 x__ARRAY__91) (<= x__ARRAY__91 1)))
(assert (and (<= 0 x__ARRAY__92) (<= x__ARRAY__92 1)))
(assert (and (<= 0 x__ARRAY__93) (<= x__ARRAY__93 1)))
(assert (and (<= 0 x__ARRAY__94) (<= x__ARRAY__94 1)))
(assert (and (<= 0 x__ARRAY__95) (<= x__ARRAY__95 1)))
(assert (and (<= 0 x__ARRAY__96) (<= x__ARRAY__96 1)))
(assert (and (<= 0 x__ARRAY__97) (<= x__ARRAY__97 1)))
(assert (and (<= 0 x__ARRAY__98) (<= x__ARRAY__98 1)))
(assert (and (<= 0 x__ARRAY__99) (<= x__ARRAY__99 1)))
(assert (and (<= 0 x__ARRAY__100) (<= x__ARRAY__100 1)))
(assert (and (<= 0 x__ARRAY__101) (<= x__ARRAY__101 1)))
(assert (and (<= 0 x__ARRAY__102) (<= x__ARRAY__102 1)))
(assert (and (<= 0 x__ARRAY__103) (<= x__ARRAY__103 1)))
(assert (and (<= 0 x__ARRAY__104) (<= x__ARRAY__104 1)))
(assert (and (<= 0 x__ARRAY__105) (<= x__ARRAY__105 1)))
(assert (and (<= 0 x__ARRAY__106) (<= x__ARRAY__106 1)))
(assert (and (<= 0 x__ARRAY__107) (<= x__ARRAY__107 1)))
(assert (and (<= 0 x__ARRAY__108) (<= x__ARRAY__108 1)))
(assert (and (<= 0 x__ARRAY__109) (<= x__ARRAY__109 1)))
(assert (and (<= 0 x__ARRAY__110) (<= x__ARRAY__110 1)))
(assert (and (<= 0 x__ARRAY__111) (<= x__ARRAY__111 1)))
(assert (and (<= 0 x__ARRAY__112) (<= x__ARRAY__112 1)))
(assert (and (<= 0 x__ARRAY__113) (<= x__ARRAY__113 1)))
(assert (and (<= 0 x__ARRAY__114) (<= x__ARRAY__114 1)))
(assert (and (<= 0 x__ARRAY__115) (<= x__ARRAY__115 1)))
(assert (and (<= 0 x__ARRAY__116) (<= x__ARRAY__116 1)))
(assert (and (<= 0 x__ARRAY__117) (<= x__ARRAY__117 1)))
(assert (and (<= 0 x__ARRAY__118) (<= x__ARRAY__118 1)))
(assert (and (<= 0 x__ARRAY__119) (<= x__ARRAY__119 1)))
(assert (and (<= 0 x__ARRAY__120) (<= x__ARRAY__120 1)))
(assert (and (<= 0 x__ARRAY__121) (<= x__ARRAY__121 1)))
(assert (and (<= 0 x__ARRAY__122) (<= x__ARRAY__122 1)))
(assert (and (<= 0 x__ARRAY__123) (<= x__ARRAY__123 1)))
(assert (and (<= 0 x__ARRAY__124) (<= x__ARRAY__124 1)))
(assert (and (<= 0 x__ARRAY__125) (<= x__ARRAY__125 1)))
(assert (and (<= 0 x__ARRAY__126) (<= x__ARRAY__126 1)))
(assert (and (<= 0 x__ARRAY__127) (<= x__ARRAY__127 1)))
(assert (and (<= 0 x__ARRAY__128) (<= x__ARRAY__128 1)))
(assert (and (<= 0 x__ARRAY__129) (<= x__ARRAY__129 1)))
(assert (and (<= 0 x__ARRAY__130) (<= x__ARRAY__130 1)))
(assert (and (<= 0 x__ARRAY__131) (<= x__ARRAY__131 1)))
(assert (and (<= 0 x__ARRAY__132) (<= x__ARRAY__132 1)))
(assert (and (<= 0 x__ARRAY__133) (<= x__ARRAY__133 1)))
(assert (and (<= 0 x__ARRAY__134) (<= x__ARRAY__134 1)))
(assert (and (<= 0 x__ARRAY__135) (<= x__ARRAY__135 1)))
(assert (and (<= 0 x__ARRAY__136) (<= x__ARRAY__136 1)))
(assert (and (<= 0 x__ARRAY__137) (<= x__ARRAY__137 1)))
(assert (and (<= 0 x__ARRAY__138) (<= x__ARRAY__138 1)))
(assert (and (<= 0 x__ARRAY__139) (<= x__ARRAY__139 1)))
(assert (and (<= 0 x__ARRAY__140) (<= x__ARRAY__140 1)))
(assert (and (<= 0 x__ARRAY__141) (<= x__ARRAY__141 1)))
(assert (and (<= 0 x__ARRAY__142) (<= x__ARRAY__142 1)))
(assert (and (<= 0 x__ARRAY__143) (<= x__ARRAY__143 1)))
(assert (and (<= 0 x__ARRAY__144) (<= x__ARRAY__144 1)))
(assert (and (<= 0 x__ARRAY__145) (<= x__ARRAY__145 1)))
(assert (and (<= 0 x__ARRAY__146) (<= x__ARRAY__146 1)))
(assert (and (<= 0 x__ARRAY__147) (<= x__ARRAY__147 1)))
(assert (and (<= 0 x__ARRAY__148) (<= x__ARRAY__148 1)))
(assert (and (<= 0 x__ARRAY__149) (<= x__ARRAY__149 1)))
(assert (and (<= 0 x__ARRAY__150) (<= x__ARRAY__150 1)))
(assert (and (<= 0 x__ARRAY__151) (<= x__ARRAY__151 1)))
(assert (and (<= 0 x__ARRAY__152) (<= x__ARRAY__152 1)))
(assert (and (<= 0 x__ARRAY__153) (<= x__ARRAY__153 1)))
(assert (and (<= 0 x__ARRAY__154) (<= x__ARRAY__154 1)))
(assert (and (<= 0 x__ARRAY__155) (<= x__ARRAY__155 1)))
(assert (and (<= 0 x__ARRAY__156) (<= x__ARRAY__156 1)))
(assert (and (<= 0 x__ARRAY__157) (<= x__ARRAY__157 1)))
(assert (and (<= 0 x__ARRAY__158) (<= x__ARRAY__158 1)))
(assert (and (<= 0 x__ARRAY__159) (<= x__ARRAY__159 1)))
(assert (and (<= 0 x__ARRAY__160) (<= x__ARRAY__160 1)))
(assert (and (<= 0 x__ARRAY__161) (<= x__ARRAY__161 1)))
(assert (and (<= 0 x__ARRAY__162) (<= x__ARRAY__162 1)))
(assert (and (<= 0 x__ARRAY__163) (<= x__ARRAY__163 1)))
(assert (and (<= 0 x__ARRAY__164) (<= x__ARRAY__164 1)))
(assert (and (<= 0 x__ARRAY__165) (<= x__ARRAY__165 1)))
(assert (and (<= 0 x__ARRAY__166) (<= x__ARRAY__166 1)))
(assert (and (<= 0 x__ARRAY__167) (<= x__ARRAY__167 1)))
(assert (and (<= 0 x__ARRAY__168) (<= x__ARRAY__168 1)))
(assert (and (<= 0 x__ARRAY__169) (<= x__ARRAY__169 1)))
(assert (and (<= 0 x__ARRAY__170) (<= x__ARRAY__170 1)))
(assert (and (<= 0 x__ARRAY__171) (<= x__ARRAY__171 1)))
(assert (and (<= 0 x__ARRAY__172) (<= x__ARRAY__172 1)))
(assert (and (<= 0 x__ARRAY__173) (<= x__ARRAY__173 1)))
(assert (and (<= 0 x__ARRAY__174) (<= x__ARRAY__174 1)))
(assert (and (<= 0 x__ARRAY__175) (<= x__ARRAY__175 1)))
(assert (and (<= 0 x__ARRAY__176) (<= x__ARRAY__176 1)))
(assert (and (<= 0 x__ARRAY__177) (<= x__ARRAY__177 1)))
(assert (and (<= 0 x__ARRAY__178) (<= x__ARRAY__178 1)))
(assert (and (<= 0 x__ARRAY__179) (<= x__ARRAY__179 1)))
(assert (and (<= 0 x__ARRAY__180) (<= x__ARRAY__180 1)))
(assert (and (<= 0 x__ARRAY__181) (<= x__ARRAY__181 1)))
(assert (and (<= 0 x__ARRAY__182) (<= x__ARRAY__182 1)))
(assert (and (<= 0 x__ARRAY__183) (<= x__ARRAY__183 1)))
(assert (and (<= 0 x__ARRAY__184) (<= x__ARRAY__184 1)))
(assert (and (<= 0 x__ARRAY__185) (<= x__ARRAY__185 1)))
(assert (and (<= 0 x__ARRAY__186) (<= x__ARRAY__186 1)))
(assert (and (<= 0 x__ARRAY__187) (<= x__ARRAY__187 1)))
(assert (and (<= 0 x__ARRAY__188) (<= x__ARRAY__188 1)))
(assert (and (<= 0 x__ARRAY__189) (<= x__ARRAY__189 1)))
(assert (and (<= 0 x__ARRAY__190) (<= x__ARRAY__190 1)))
(assert (and (<= 0 x__ARRAY__191) (<= x__ARRAY__191 1)))
(assert (and (<= 0 x__ARRAY__192) (<= x__ARRAY__192 1)))
(assert (and (<= 0 x__ARRAY__193) (<= x__ARRAY__193 1)))
(assert (and (<= 0 x__ARRAY__194) (<= x__ARRAY__194 1)))
(assert (and (<= 0 x__ARRAY__195) (<= x__ARRAY__195 1)))
(assert (and (<= 0 x__ARRAY__196) (<= x__ARRAY__196 1)))
(assert (and (<= 0 x__ARRAY__197) (<= x__ARRAY__197 1)))
(assert (and (<= 0 x__ARRAY__198) (<= x__ARRAY__198 1)))
(assert (and (<= 0 x__ARRAY__199) (<= x__ARRAY__199 1)))
(assert (and (<= 0 x__ARRAY__200) (<= x__ARRAY__200 1)))
(assert (and (<= 0 x__ARRAY__201) (<= x__ARRAY__201 1)))
(assert (and (<= 0 x__ARRAY__202) (<= x__ARRAY__202 1)))
(assert (and (<= 0 x__ARRAY__203) (<= x__ARRAY__203 1)))
(assert (and (<= 0 x__ARRAY__204) (<= x__ARRAY__204 1)))
(assert (and (<= 0 x__ARRAY__205) (<= x__ARRAY__205 1)))
(assert (and (<= 0 x__ARRAY__206) (<= x__ARRAY__206 1)))
(assert (and (<= 0 x__ARRAY__207) (<= x__ARRAY__207 1)))
(assert (and (<= 0 x__ARRAY__208) (<= x__ARRAY__208 1)))
(assert (and (<= 0 x__ARRAY__209) (<= x__ARRAY__209 1)))
(assert (and (<= 0 x__ARRAY__210) (<= x__ARRAY__210 1)))
(assert (and (<= 0 x__ARRAY__211) (<= x__ARRAY__211 1)))
(assert (and (<= 0 x__ARRAY__212) (<= x__ARRAY__212 1)))
(assert (and (<= 0 x__ARRAY__213) (<= x__ARRAY__213 1)))
(assert (and (<= 0 x__ARRAY__214) (<= x__ARRAY__214 1)))
(assert (and (<= 0 x__ARRAY__215) (<= x__ARRAY__215 1)))
(assert (and (<= 0 x__ARRAY__216) (<= x__ARRAY__216 1)))
(assert (and (<= 0 x__ARRAY__217) (<= x__ARRAY__217 1)))
(assert (and (<= 0 x__ARRAY__218) (<= x__ARRAY__218 1)))
(assert (and (<= 0 x__ARRAY__219) (<= x__ARRAY__219 1)))
(assert (and (<= 0 x__ARRAY__220) (<= x__ARRAY__220 1)))
(assert (and (<= 0 x__ARRAY__221) (<= x__ARRAY__221 1)))
(assert (and (<= 0 x__ARRAY__222) (<= x__ARRAY__222 1)))
(assert (and (<= 0 x__ARRAY__223) (<= x__ARRAY__223 1)))
(assert (and (<= 0 x__ARRAY__224) (<= x__ARRAY__224 1)))
(assert (and (<= 0 x__ARRAY__225) (<= x__ARRAY__225 1)))
(assert (and (<= 0 x__ARRAY__226) (<= x__ARRAY__226 1)))
(assert (and (<= 0 x__ARRAY__227) (<= x__ARRAY__227 1)))
(assert (and (<= 0 x__ARRAY__228) (<= x__ARRAY__228 1)))
(assert (and (<= 0 x__ARRAY__229) (<= x__ARRAY__229 1)))
(assert (and (<= 0 x__ARRAY__230) (<= x__ARRAY__230 1)))
(assert (and (<= 0 x__ARRAY__231) (<= x__ARRAY__231 1)))
(assert (and (<= 0 x__ARRAY__232) (<= x__ARRAY__232 1)))
(assert (and (<= 0 x__ARRAY__233) (<= x__ARRAY__233 1)))
(assert (and (<= 0 x__ARRAY__234) (<= x__ARRAY__234 1)))
(assert (and (<= 0 x__ARRAY__235) (<= x__ARRAY__235 1)))
(assert (and (<= 0 x__ARRAY__236) (<= x__ARRAY__236 1)))
(assert (and (<= 0 x__ARRAY__237) (<= x__ARRAY__237 1)))
(assert (and (<= 0 x__ARRAY__238) (<= x__ARRAY__238 1)))
(assert (and (<= 0 x__ARRAY__239) (<= x__ARRAY__239 1)))
(assert (and (<= 0 x__ARRAY__240) (<= x__ARRAY__240 1)))
(assert (and (<= 0 x__ARRAY__241) (<= x__ARRAY__241 1)))
(assert (and (<= 0 x__ARRAY__242) (<= x__ARRAY__242 1)))
(assert (and (<= 0 x__ARRAY__243) (<= x__ARRAY__243 1)))
(assert (and (<= 0 x__ARRAY__244) (<= x__ARRAY__244 1)))
(assert (and (<= 0 x__ARRAY__245) (<= x__ARRAY__245 1)))
(assert (and (<= 0 x__ARRAY__246) (<= x__ARRAY__246 1)))
(assert (and (<= 0 x__ARRAY__247) (<= x__ARRAY__247 1)))
(assert (and (<= 0 x__ARRAY__248) (<= x__ARRAY__248 1)))
(assert (and (<= 0 x__ARRAY__249) (<= x__ARRAY__249 1)))
(assert (and (<= 0 x__ARRAY__250) (<= x__ARRAY__250 1)))
(assert (and (<= 0 x__ARRAY__251) (<= x__ARRAY__251 1)))
(assert (and (<= 0 x__ARRAY__252) (<= x__ARRAY__252 1)))
(assert (and (<= 0 x__ARRAY__253) (<= x__ARRAY__253 1)))
(assert (and (<= 0 x__ARRAY__254) (<= x__ARRAY__254 1)))
(assert (and (<= 0 x__ARRAY__255) (<= x__ARRAY__255 1)))
(assert (and (<= 0 x__ARRAY__256) (<= x__ARRAY__256 1)))
(assert (and (<= 0 x__ARRAY__257) (<= x__ARRAY__257 1)))
(assert (and (<= 0 x__ARRAY__258) (<= x__ARRAY__258 1)))
(assert (and (<= 0 x__ARRAY__259) (<= x__ARRAY__259 1)))
(assert (and (<= 0 x__ARRAY__260) (<= x__ARRAY__260 1)))
(assert (and (<= 0 x__ARRAY__261) (<= x__ARRAY__261 1)))
(assert (and (<= 0 x__ARRAY__262) (<= x__ARRAY__262 1)))
(assert (and (<= 0 x__ARRAY__263) (<= x__ARRAY__263 1)))
(assert (and (<= 0 x__ARRAY__264) (<= x__ARRAY__264 1)))
(assert (and (<= 0 x__ARRAY__265) (<= x__ARRAY__265 1)))
(assert (and (<= 0 x__ARRAY__266) (<= x__ARRAY__266 1)))
(assert (and (<= 0 x__ARRAY__267) (<= x__ARRAY__267 1)))
(assert (and (<= 0 x__ARRAY__268) (<= x__ARRAY__268 1)))
(assert (and (<= 0 x__ARRAY__269) (<= x__ARRAY__269 1)))
(assert (and (<= 0 x__ARRAY__270) (<= x__ARRAY__270 1)))
(assert (and (<= 0 x__ARRAY__271) (<= x__ARRAY__271 1)))
(assert (and (<= 0 x__ARRAY__272) (<= x__ARRAY__272 1)))
(assert (and (<= 0 x__ARRAY__273) (<= x__ARRAY__273 1)))
(assert (and (<= 0 x__ARRAY__274) (<= x__ARRAY__274 1)))
(assert (and (<= 0 x__ARRAY__275) (<= x__ARRAY__275 1)))
(assert (and (<= 0 x__ARRAY__276) (<= x__ARRAY__276 1)))
(assert (and (<= 0 x__ARRAY__277) (<= x__ARRAY__277 1)))
(assert (and (<= 0 x__ARRAY__278) (<= x__ARRAY__278 1)))
(assert (and (<= 0 x__ARRAY__279) (<= x__ARRAY__279 1)))
(assert (and (<= 0 x__ARRAY__280) (<= x__ARRAY__280 1)))
(assert (and (<= 0 x__ARRAY__281) (<= x__ARRAY__281 1)))
(assert (and (<= 0 x__ARRAY__282) (<= x__ARRAY__282 1)))
(assert (and (<= 0 x__ARRAY__283) (<= x__ARRAY__283 1)))
(assert (and (<= 0 x__ARRAY__284) (<= x__ARRAY__284 1)))
(assert (and (<= 0 x__ARRAY__285) (<= x__ARRAY__285 1)))
(assert (and (<= 0 x__ARRAY__286) (<= x__ARRAY__286 1)))
(assert (and (<= 0 x__ARRAY__287) (<= x__ARRAY__287 1)))
(assert (and (<= 0 x__ARRAY__288) (<= x__ARRAY__288 1)))
(assert (and (<= 0 x__ARRAY__289) (<= x__ARRAY__289 1)))
(assert (and (<= 0 x__ARRAY__290) (<= x__ARRAY__290 1)))
(assert (and (<= 0 x__ARRAY__291) (<= x__ARRAY__291 1)))
(assert (and (<= 0 x__ARRAY__292) (<= x__ARRAY__292 1)))
(assert (and (<= 0 x__ARRAY__293) (<= x__ARRAY__293 1)))
(assert (and (<= 0 x__ARRAY__294) (<= x__ARRAY__294 1)))
(assert (and (<= 0 x__ARRAY__295) (<= x__ARRAY__295 1)))
(assert (and (<= 0 x__ARRAY__296) (<= x__ARRAY__296 1)))
(assert (and (<= 0 x__ARRAY__297) (<= x__ARRAY__297 1)))
(assert (and (<= 0 x__ARRAY__298) (<= x__ARRAY__298 1)))
(assert (and (<= 0 x__ARRAY__299) (<= x__ARRAY__299 1)))
(assert (and (<= 0 x__ARRAY__300) (<= x__ARRAY__300 1)))
(assert (and (<= 0 x__ARRAY__301) (<= x__ARRAY__301 1)))
(assert (and (<= 0 x__ARRAY__302) (<= x__ARRAY__302 1)))
(assert (and (<= 0 x__ARRAY__303) (<= x__ARRAY__303 1)))
(assert (and (<= 0 x__ARRAY__304) (<= x__ARRAY__304 1)))
(assert (and (<= 0 x__ARRAY__305) (<= x__ARRAY__305 1)))
(assert (and (<= 0 x__ARRAY__306) (<= x__ARRAY__306 1)))
(assert (and (<= 0 x__ARRAY__307) (<= x__ARRAY__307 1)))
(assert (and (<= 0 x__ARRAY__308) (<= x__ARRAY__308 1)))
(assert (and (<= 0 x__ARRAY__309) (<= x__ARRAY__309 1)))
(assert (and (<= 0 x__ARRAY__310) (<= x__ARRAY__310 1)))
(assert (and (<= 0 x__ARRAY__311) (<= x__ARRAY__311 1)))
(assert (and (<= 0 x__ARRAY__312) (<= x__ARRAY__312 1)))
(assert (and (<= 0 x__ARRAY__313) (<= x__ARRAY__313 1)))
(assert (and (<= 0 x__ARRAY__314) (<= x__ARRAY__314 1)))
(assert (and (<= 0 x__ARRAY__315) (<= x__ARRAY__315 1)))
(assert (and (<= 0 x__ARRAY__316) (<= x__ARRAY__316 1)))
(assert (and (<= 0 x__ARRAY__317) (<= x__ARRAY__317 1)))
(assert (and (<= 0 x__ARRAY__318) (<= x__ARRAY__318 1)))
(assert (and (<= 0 x__ARRAY__319) (<= x__ARRAY__319 1)))
(assert (and (<= 0 x__ARRAY__320) (<= x__ARRAY__320 1)))
(assert (and (<= 0 x__ARRAY__321) (<= x__ARRAY__321 1)))
(assert (and (<= 0 x__ARRAY__322) (<= x__ARRAY__322 1)))
(assert (and (<= 0 x__ARRAY__323) (<= x__ARRAY__323 1)))
(assert (and (<= 0 x__ARRAY__324) (<= x__ARRAY__324 1)))
(assert (and (<= 0 x__ARRAY__325) (<= x__ARRAY__325 1)))
(assert (and (<= 0 x__ARRAY__326) (<= x__ARRAY__326 1)))
(assert (and (<= 0 x__ARRAY__327) (<= x__ARRAY__327 1)))
(assert (and (<= 0 x__ARRAY__328) (<= x__ARRAY__328 1)))
(assert (and (<= 0 x__ARRAY__329) (<= x__ARRAY__329 1)))
(assert (and (<= 0 x__ARRAY__330) (<= x__ARRAY__330 1)))
(assert (and (<= 0 x__ARRAY__331) (<= x__ARRAY__331 1)))
(assert (and (<= 0 x__ARRAY__332) (<= x__ARRAY__332 1)))
(assert (and (<= 0 x__ARRAY__333) (<= x__ARRAY__333 1)))
(assert (and (<= 0 x__ARRAY__334) (<= x__ARRAY__334 1)))
(assert (and (<= 0 x__ARRAY__335) (<= x__ARRAY__335 1)))
(assert (and (<= 0 x__ARRAY__336) (<= x__ARRAY__336 1)))
(assert (and (<= 0 x__ARRAY__337) (<= x__ARRAY__337 1)))
(assert (and (<= 0 x__ARRAY__338) (<= x__ARRAY__338 1)))
(assert (and (<= 0 x__ARRAY__339) (<= x__ARRAY__339 1)))
(assert (and (<= 0 x__ARRAY__340) (<= x__ARRAY__340 1)))
(assert (and (<= 0 x__ARRAY__341) (<= x__ARRAY__341 1)))
(assert (and (<= 0 x__ARRAY__342) (<= x__ARRAY__342 1)))
(assert (and (<= 0 x__ARRAY__343) (<= x__ARRAY__343 1)))
(assert (and (<= 0 x__ARRAY__344) (<= x__ARRAY__344 1)))
(assert (and (<= 0 x__ARRAY__345) (<= x__ARRAY__345 1)))
(assert (and (<= 0 x__ARRAY__346) (<= x__ARRAY__346 1)))
(assert (and (<= 0 x__ARRAY__347) (<= x__ARRAY__347 1)))
(assert (and (<= 0 x__ARRAY__348) (<= x__ARRAY__348 1)))
(assert (and (<= 0 x__ARRAY__349) (<= x__ARRAY__349 1)))
(assert (and (<= 0 x__ARRAY__350) (<= x__ARRAY__350 1)))
(assert (and (<= 0 x__ARRAY__351) (<= x__ARRAY__351 1)))
(assert (and (<= 0 x__ARRAY__352) (<= x__ARRAY__352 1)))
(assert (and (<= 0 x__ARRAY__353) (<= x__ARRAY__353 1)))
(assert (and (<= 0 x__ARRAY__354) (<= x__ARRAY__354 1)))
(assert (and (<= 0 x__ARRAY__355) (<= x__ARRAY__355 1)))
(assert (and (<= 0 x__ARRAY__356) (<= x__ARRAY__356 1)))
(assert (and (<= 0 x__ARRAY__357) (<= x__ARRAY__357 1)))
(assert (and (<= 0 x__ARRAY__358) (<= x__ARRAY__358 1)))
(assert (and (<= 0 x__ARRAY__359) (<= x__ARRAY__359 1)))
(assert (and (<= 0 x__ARRAY__360) (<= x__ARRAY__360 1)))
(assert (and (<= 0 x__ARRAY__361) (<= x__ARRAY__361 1)))
(assert (and (<= 0 x__ARRAY__362) (<= x__ARRAY__362 1)))
(assert (and (<= 0 x__ARRAY__363) (<= x__ARRAY__363 1)))
(assert (and (<= 0 x__ARRAY__364) (<= x__ARRAY__364 1)))
(assert (and (<= 0 x__ARRAY__365) (<= x__ARRAY__365 1)))
(assert (and (<= 0 x__ARRAY__366) (<= x__ARRAY__366 1)))
(assert (and (<= 0 x__ARRAY__367) (<= x__ARRAY__367 1)))
(assert (and (<= 0 x__ARRAY__368) (<= x__ARRAY__368 1)))
(assert (and (<= 0 x__ARRAY__369) (<= x__ARRAY__369 1)))
(assert (and (<= 0 x__ARRAY__370) (<= x__ARRAY__370 1)))
(assert (and (<= 0 x__ARRAY__371) (<= x__ARRAY__371 1)))
(assert (and (<= 0 x__ARRAY__372) (<= x__ARRAY__372 1)))
(assert (and (<= 0 x__ARRAY__373) (<= x__ARRAY__373 1)))
(assert (and (<= 0 x__ARRAY__374) (<= x__ARRAY__374 1)))
(assert (and (<= 0 x__ARRAY__375) (<= x__ARRAY__375 1)))
(assert (and (<= 0 x__ARRAY__376) (<= x__ARRAY__376 1)))
(assert (and (<= 0 x__ARRAY__377) (<= x__ARRAY__377 1)))
(assert (and (<= 0 x__ARRAY__378) (<= x__ARRAY__378 1)))
(assert (and (<= 0 x__ARRAY__379) (<= x__ARRAY__379 1)))
(assert (and (<= 0 x__ARRAY__380) (<= x__ARRAY__380 1)))
(assert (and (<= 0 x__ARRAY__381) (<= x__ARRAY__381 1)))
(assert (and (<= 0 x__ARRAY__382) (<= x__ARRAY__382 1)))
(assert (and (<= 0 x__ARRAY__383) (<= x__ARRAY__383 1)))
(assert (and (<= 0 x__ARRAY__384) (<= x__ARRAY__384 1)))
(assert (and (<= 0 x__ARRAY__385) (<= x__ARRAY__385 1)))
(assert (and (<= 0 x__ARRAY__386) (<= x__ARRAY__386 1)))
(assert (and (<= 0 x__ARRAY__387) (<= x__ARRAY__387 1)))
(assert (and (<= 0 x__ARRAY__388) (<= x__ARRAY__388 1)))
(assert (and (<= 0 x__ARRAY__389) (<= x__ARRAY__389 1)))
(assert (and (<= 0 x__ARRAY__390) (<= x__ARRAY__390 1)))
(assert (and (<= 0 x__ARRAY__391) (<= x__ARRAY__391 1)))
(assert (and (<= 0 x__ARRAY__392) (<= x__ARRAY__392 1)))
(assert (and (<= 0 x__ARRAY__393) (<= x__ARRAY__393 1)))
(assert (and (<= 0 x__ARRAY__394) (<= x__ARRAY__394 1)))
(assert (and (<= 0 x__ARRAY__395) (<= x__ARRAY__395 1)))
(assert (and (<= 0 x__ARRAY__396) (<= x__ARRAY__396 1)))
(assert (and (<= 0 x__ARRAY__397) (<= x__ARRAY__397 1)))
(assert (and (<= 0 x__ARRAY__398) (<= x__ARRAY__398 1)))
(assert (and (<= 0 x__ARRAY__399) (<= x__ARRAY__399 1)))
(assert (and (<= 0 x__ARRAY__400) (<= x__ARRAY__400 1)))
(assert (and (<= 0 x__ARRAY__401) (<= x__ARRAY__401 1)))
(assert (and (<= 0 x__ARRAY__402) (<= x__ARRAY__402 1)))
(assert (and (<= 0 x__ARRAY__403) (<= x__ARRAY__403 1)))
(assert (and (<= 0 x__ARRAY__404) (<= x__ARRAY__404 1)))
(assert (and (<= 0 x__ARRAY__405) (<= x__ARRAY__405 1)))
(assert (and (<= 0 x__ARRAY__406) (<= x__ARRAY__406 1)))
(assert (and (<= 0 x__ARRAY__407) (<= x__ARRAY__407 1)))
(assert (and (<= 0 x__ARRAY__408) (<= x__ARRAY__408 1)))
(assert (and (<= 0 x__ARRAY__409) (<= x__ARRAY__409 1)))
(assert (and (<= 0 x__ARRAY__410) (<= x__ARRAY__410 1)))
(assert (and (<= 0 x__ARRAY__411) (<= x__ARRAY__411 1)))
(assert (and (<= 0 x__ARRAY__412) (<= x__ARRAY__412 1)))
(assert (and (<= 0 x__ARRAY__413) (<= x__ARRAY__413 1)))
(assert (and (<= 0 x__ARRAY__414) (<= x__ARRAY__414 1)))
(assert (and (<= 0 x__ARRAY__415) (<= x__ARRAY__415 1)))
(assert (and (<= 0 x__ARRAY__416) (<= x__ARRAY__416 1)))
(assert (and (<= 0 x__ARRAY__417) (<= x__ARRAY__417 1)))
(assert (and (<= 0 x__ARRAY__418) (<= x__ARRAY__418 1)))
(assert (and (<= 0 x__ARRAY__419) (<= x__ARRAY__419 1)))
(assert (and (<= 0 x__ARRAY__420) (<= x__ARRAY__420 1)))
(assert (and (<= 0 x__ARRAY__421) (<= x__ARRAY__421 1)))
(assert (and (<= 0 x__ARRAY__422) (<= x__ARRAY__422 1)))
(assert (and (<= 0 x__ARRAY__423) (<= x__ARRAY__423 1)))
(assert (and (<= 0 x__ARRAY__424) (<= x__ARRAY__424 1)))
(assert (and (<= 0 x__ARRAY__425) (<= x__ARRAY__425 1)))
(assert (and (<= 0 x__ARRAY__426) (<= x__ARRAY__426 1)))
(assert (and (<= 0 x__ARRAY__427) (<= x__ARRAY__427 1)))
(assert (and (<= 0 x__ARRAY__428) (<= x__ARRAY__428 1)))
(assert (and (<= 0 x__ARRAY__429) (<= x__ARRAY__429 1)))
(assert (and (<= 0 x__ARRAY__430) (<= x__ARRAY__430 1)))
(assert (and (<= 0 x__ARRAY__431) (<= x__ARRAY__431 1)))
(assert (and (<= 0 x__ARRAY__432) (<= x__ARRAY__432 1)))
(assert (and (<= 0 x__ARRAY__433) (<= x__ARRAY__433 1)))
(assert (and (<= 0 x__ARRAY__434) (<= x__ARRAY__434 1)))
(assert (and (<= 0 x__ARRAY__435) (<= x__ARRAY__435 1)))
(assert (and (<= 0 x__ARRAY__436) (<= x__ARRAY__436 1)))
(assert (and (<= 0 x__ARRAY__437) (<= x__ARRAY__437 1)))
(assert (and (<= 0 x__ARRAY__438) (<= x__ARRAY__438 1)))
(assert (and (<= 0 x__ARRAY__439) (<= x__ARRAY__439 1)))
(assert (and (<= 0 x__ARRAY__440) (<= x__ARRAY__440 1)))
(assert (and (<= 0 x__ARRAY__441) (<= x__ARRAY__441 1)))
(assert (and (<= 0 x__ARRAY__442) (<= x__ARRAY__442 1)))
(assert (and (<= 0 x__ARRAY__443) (<= x__ARRAY__443 1)))
(assert (and (<= 0 x__ARRAY__444) (<= x__ARRAY__444 1)))
(assert (and (<= 0 x__ARRAY__445) (<= x__ARRAY__445 1)))
(assert (and (<= 0 x__ARRAY__446) (<= x__ARRAY__446 1)))
(assert (and (<= 0 x__ARRAY__447) (<= x__ARRAY__447 1)))
(assert (and (<= 0 x__ARRAY__448) (<= x__ARRAY__448 1)))
(assert (and (<= 0 x__ARRAY__449) (<= x__ARRAY__449 1)))
(assert (and (<= 0 x__ARRAY__450) (<= x__ARRAY__450 1)))
(assert (and (<= 0 x__ARRAY__451) (<= x__ARRAY__451 1)))
(assert (and (<= 0 x__ARRAY__452) (<= x__ARRAY__452 1)))
(assert (and (<= 0 x__ARRAY__453) (<= x__ARRAY__453 1)))
(assert (and (<= 0 x__ARRAY__454) (<= x__ARRAY__454 1)))
(assert (and (<= 0 x__ARRAY__455) (<= x__ARRAY__455 1)))
(assert (and (<= 0 x__ARRAY__456) (<= x__ARRAY__456 1)))
(assert (and (<= 0 x__ARRAY__457) (<= x__ARRAY__457 1)))
(assert (and (<= 0 x__ARRAY__458) (<= x__ARRAY__458 1)))
(assert (and (<= 0 x__ARRAY__459) (<= x__ARRAY__459 1)))
(assert (and (<= 0 x__ARRAY__460) (<= x__ARRAY__460 1)))
(assert (and (<= 0 x__ARRAY__461) (<= x__ARRAY__461 1)))
(assert (and (<= 0 x__ARRAY__462) (<= x__ARRAY__462 1)))
(assert (and (<= 0 x__ARRAY__463) (<= x__ARRAY__463 1)))
(assert (and (<= 0 x__ARRAY__464) (<= x__ARRAY__464 1)))
(assert (and (<= 0 x__ARRAY__465) (<= x__ARRAY__465 1)))
(assert (and (<= 0 x__ARRAY__466) (<= x__ARRAY__466 1)))
(assert (and (<= 0 x__ARRAY__467) (<= x__ARRAY__467 1)))
(assert (and (<= 0 x__ARRAY__468) (<= x__ARRAY__468 1)))
(assert (and (<= 0 x__ARRAY__469) (<= x__ARRAY__469 1)))
(assert (and (<= 0 x__ARRAY__470) (<= x__ARRAY__470 1)))
(assert (and (<= 0 x__ARRAY__471) (<= x__ARRAY__471 1)))
(assert (and (<= 0 x__ARRAY__472) (<= x__ARRAY__472 1)))
(assert (and (<= 0 x__ARRAY__473) (<= x__ARRAY__473 1)))
(assert (and (<= 0 x__ARRAY__474) (<= x__ARRAY__474 1)))
(assert (and (<= 0 x__ARRAY__475) (<= x__ARRAY__475 1)))
(assert (and (<= 0 x__ARRAY__476) (<= x__ARRAY__476 1)))
(assert (and (<= 0 x__ARRAY__477) (<= x__ARRAY__477 1)))
(assert (and (<= 0 x__ARRAY__478) (<= x__ARRAY__478 1)))
(assert (and (<= 0 x__ARRAY__479) (<= x__ARRAY__479 1)))
(assert (and (<= 0 x__ARRAY__480) (<= x__ARRAY__480 1)))
(assert (and (<= 0 x__ARRAY__481) (<= x__ARRAY__481 1)))
(assert (and (<= 0 x__ARRAY__482) (<= x__ARRAY__482 1)))
(assert (and (<= 0 x__ARRAY__483) (<= x__ARRAY__483 1)))
(assert (and (<= 0 x__ARRAY__484) (<= x__ARRAY__484 1)))
(assert (and (<= 0 x__ARRAY__485) (<= x__ARRAY__485 1)))
(assert (and (<= 0 x__ARRAY__486) (<= x__ARRAY__486 1)))
(assert (and (<= 0 x__ARRAY__487) (<= x__ARRAY__487 1)))
(assert (and (<= 0 x__ARRAY__488) (<= x__ARRAY__488 1)))
(assert (and (<= 0 x__ARRAY__489) (<= x__ARRAY__489 1)))
(assert (and (<= 0 x__ARRAY__490) (<= x__ARRAY__490 1)))
(assert (and (<= 0 x__ARRAY__491) (<= x__ARRAY__491 1)))
(assert (and (<= 0 x__ARRAY__492) (<= x__ARRAY__492 1)))
(assert (and (<= 0 x__ARRAY__493) (<= x__ARRAY__493 1)))
(assert (and (<= 0 x__ARRAY__494) (<= x__ARRAY__494 1)))
(assert (and (<= 0 x__ARRAY__495) (<= x__ARRAY__495 1)))
(assert (and (<= 0 x__ARRAY__496) (<= x__ARRAY__496 1)))
(assert (and (<= 0 x__ARRAY__497) (<= x__ARRAY__497 1)))
(assert (and (<= 0 x__ARRAY__498) (<= x__ARRAY__498 1)))
(assert (and (<= 0 x__ARRAY__499) (<= x__ARRAY__499 1)))
(assert (and (<= 0 x__ARRAY__500) (<= x__ARRAY__500 1)))
;; >> ConstraReal_Items
(assert (and (and (=> BOOL____00001 (= Real____00002 1)) (=> (= Real____00002 1) BOOL____00001)) (and (=> (not BOOL____00001) (= Real____00002 0)) (=> (= Real____00002 0) (not BOOL____00001)))))
(assert (and (and (=> BOOL____00003 (= Real____00004 1)) (=> (= Real____00004 1) BOOL____00003)) (and (=> (not BOOL____00003) (= Real____00004 0)) (=> (= Real____00004 0) (not BOOL____00003)))))
(assert (and (and (=> BOOL____00005 (= Real____00006 1)) (=> (= Real____00006 1) BOOL____00005)) (and (=> (not BOOL____00005) (= Real____00006 0)) (=> (= Real____00006 0) (not BOOL____00005)))))
(assert (and (and (=> BOOL____00007 (= Real____00008 1)) (=> (= Real____00008 1) BOOL____00007)) (and (=> (not BOOL____00007) (= Real____00008 0)) (=> (= Real____00008 0) (not BOOL____00007)))))
(assert (and (and (=> BOOL____00009 (= Real____00010 1)) (=> (= Real____00010 1) BOOL____00009)) (and (=> (not BOOL____00009) (= Real____00010 0)) (=> (= Real____00010 0) (not BOOL____00009)))))
(assert (and (and (=> BOOL____00011 (= Real____00012 1)) (=> (= Real____00012 1) BOOL____00011)) (and (=> (not BOOL____00011) (= Real____00012 0)) (=> (= Real____00012 0) (not BOOL____00011)))))
(assert (and (and (=> BOOL____00013 (= Real____00014 1)) (=> (= Real____00014 1) BOOL____00013)) (and (=> (not BOOL____00013) (= Real____00014 0)) (=> (= Real____00014 0) (not BOOL____00013)))))
(assert (and (and (=> BOOL____00015 (= Real____00016 1)) (=> (= Real____00016 1) BOOL____00015)) (and (=> (not BOOL____00015) (= Real____00016 0)) (=> (= Real____00016 0) (not BOOL____00015)))))
(assert (and (and (=> BOOL____00017 (= Real____00018 1)) (=> (= Real____00018 1) BOOL____00017)) (and (=> (not BOOL____00017) (= Real____00018 0)) (=> (= Real____00018 0) (not BOOL____00017)))))
(assert (and (and (=> BOOL____00019 (= Real____00020 1)) (=> (= Real____00020 1) BOOL____00019)) (and (=> (not BOOL____00019) (= Real____00020 0)) (=> (= Real____00020 0) (not BOOL____00019)))))
(assert (and (and (=> BOOL____00021 (= Real____00022 1)) (=> (= Real____00022 1) BOOL____00021)) (and (=> (not BOOL____00021) (= Real____00022 0)) (=> (= Real____00022 0) (not BOOL____00021)))))
(assert (and (and (=> BOOL____00023 (= Real____00024 1)) (=> (= Real____00024 1) BOOL____00023)) (and (=> (not BOOL____00023) (= Real____00024 0)) (=> (= Real____00024 0) (not BOOL____00023)))))
(assert (and (and (=> BOOL____00025 (= Real____00026 1)) (=> (= Real____00026 1) BOOL____00025)) (and (=> (not BOOL____00025) (= Real____00026 0)) (=> (= Real____00026 0) (not BOOL____00025)))))
(assert (and (and (=> BOOL____00027 (= Real____00028 1)) (=> (= Real____00028 1) BOOL____00027)) (and (=> (not BOOL____00027) (= Real____00028 0)) (=> (= Real____00028 0) (not BOOL____00027)))))
(assert (and (and (=> BOOL____00029 (= Real____00030 1)) (=> (= Real____00030 1) BOOL____00029)) (and (=> (not BOOL____00029) (= Real____00030 0)) (=> (= Real____00030 0) (not BOOL____00029)))))
(assert (and (and (=> BOOL____00031 (= Real____00032 1)) (=> (= Real____00032 1) BOOL____00031)) (and (=> (not BOOL____00031) (= Real____00032 0)) (=> (= Real____00032 0) (not BOOL____00031)))))
(assert (and (and (=> BOOL____00033 (= Real____00034 1)) (=> (= Real____00034 1) BOOL____00033)) (and (=> (not BOOL____00033) (= Real____00034 0)) (=> (= Real____00034 0) (not BOOL____00033)))))
(assert (and (and (=> BOOL____00035 (= Real____00036 1)) (=> (= Real____00036 1) BOOL____00035)) (and (=> (not BOOL____00035) (= Real____00036 0)) (=> (= Real____00036 0) (not BOOL____00035)))))
(assert (and (and (=> BOOL____00037 (= Real____00038 1)) (=> (= Real____00038 1) BOOL____00037)) (and (=> (not BOOL____00037) (= Real____00038 0)) (=> (= Real____00038 0) (not BOOL____00037)))))
(assert (and (and (=> BOOL____00039 (= Real____00040 1)) (=> (= Real____00040 1) BOOL____00039)) (and (=> (not BOOL____00039) (= Real____00040 0)) (=> (= Real____00040 0) (not BOOL____00039)))))
(assert (and (and (=> BOOL____00041 (= Real____00042 1)) (=> (= Real____00042 1) BOOL____00041)) (and (=> (not BOOL____00041) (= Real____00042 0)) (=> (= Real____00042 0) (not BOOL____00041)))))
(assert (and (and (=> BOOL____00043 (= Real____00044 1)) (=> (= Real____00044 1) BOOL____00043)) (and (=> (not BOOL____00043) (= Real____00044 0)) (=> (= Real____00044 0) (not BOOL____00043)))))
(assert (and (and (=> BOOL____00045 (= Real____00046 1)) (=> (= Real____00046 1) BOOL____00045)) (and (=> (not BOOL____00045) (= Real____00046 0)) (=> (= Real____00046 0) (not BOOL____00045)))))
(assert (and (and (=> BOOL____00047 (= Real____00048 1)) (=> (= Real____00048 1) BOOL____00047)) (and (=> (not BOOL____00047) (= Real____00048 0)) (=> (= Real____00048 0) (not BOOL____00047)))))
(assert (and (and (=> BOOL____00049 (= Real____00050 1)) (=> (= Real____00050 1) BOOL____00049)) (and (=> (not BOOL____00049) (= Real____00050 0)) (=> (= Real____00050 0) (not BOOL____00049)))))
(assert (and (and (=> BOOL____00051 (= Real____00052 1)) (=> (= Real____00052 1) BOOL____00051)) (and (=> (not BOOL____00051) (= Real____00052 0)) (=> (= Real____00052 0) (not BOOL____00051)))))
(assert (and (and (=> BOOL____00053 (= Real____00054 1)) (=> (= Real____00054 1) BOOL____00053)) (and (=> (not BOOL____00053) (= Real____00054 0)) (=> (= Real____00054 0) (not BOOL____00053)))))
(assert (and (and (=> BOOL____00055 (= Real____00056 1)) (=> (= Real____00056 1) BOOL____00055)) (and (=> (not BOOL____00055) (= Real____00056 0)) (=> (= Real____00056 0) (not BOOL____00055)))))
(assert (and (and (=> BOOL____00057 (= Real____00058 1)) (=> (= Real____00058 1) BOOL____00057)) (and (=> (not BOOL____00057) (= Real____00058 0)) (=> (= Real____00058 0) (not BOOL____00057)))))
(assert (and (and (=> BOOL____00059 (= Real____00060 1)) (=> (= Real____00060 1) BOOL____00059)) (and (=> (not BOOL____00059) (= Real____00060 0)) (=> (= Real____00060 0) (not BOOL____00059)))))
(assert (and (and (=> BOOL____00061 (= Real____00062 1)) (=> (= Real____00062 1) BOOL____00061)) (and (=> (not BOOL____00061) (= Real____00062 0)) (=> (= Real____00062 0) (not BOOL____00061)))))
(assert (and (and (=> BOOL____00063 (= Real____00064 1)) (=> (= Real____00064 1) BOOL____00063)) (and (=> (not BOOL____00063) (= Real____00064 0)) (=> (= Real____00064 0) (not BOOL____00063)))))
(assert (and (and (=> BOOL____00065 (= Real____00066 1)) (=> (= Real____00066 1) BOOL____00065)) (and (=> (not BOOL____00065) (= Real____00066 0)) (=> (= Real____00066 0) (not BOOL____00065)))))
(assert (and (and (=> BOOL____00067 (= Real____00068 1)) (=> (= Real____00068 1) BOOL____00067)) (and (=> (not BOOL____00067) (= Real____00068 0)) (=> (= Real____00068 0) (not BOOL____00067)))))
(assert (and (and (=> BOOL____00069 (= Real____00070 1)) (=> (= Real____00070 1) BOOL____00069)) (and (=> (not BOOL____00069) (= Real____00070 0)) (=> (= Real____00070 0) (not BOOL____00069)))))
(assert (and (and (=> BOOL____00071 (= Real____00072 1)) (=> (= Real____00072 1) BOOL____00071)) (and (=> (not BOOL____00071) (= Real____00072 0)) (=> (= Real____00072 0) (not BOOL____00071)))))
(assert (and (and (=> BOOL____00073 (= Real____00074 1)) (=> (= Real____00074 1) BOOL____00073)) (and (=> (not BOOL____00073) (= Real____00074 0)) (=> (= Real____00074 0) (not BOOL____00073)))))
(assert (and (and (=> BOOL____00075 (= Real____00076 1)) (=> (= Real____00076 1) BOOL____00075)) (and (=> (not BOOL____00075) (= Real____00076 0)) (=> (= Real____00076 0) (not BOOL____00075)))))
(assert (and (and (=> BOOL____00077 (= Real____00078 1)) (=> (= Real____00078 1) BOOL____00077)) (and (=> (not BOOL____00077) (= Real____00078 0)) (=> (= Real____00078 0) (not BOOL____00077)))))
(assert (and (and (=> BOOL____00079 (= Real____00080 1)) (=> (= Real____00080 1) BOOL____00079)) (and (=> (not BOOL____00079) (= Real____00080 0)) (=> (= Real____00080 0) (not BOOL____00079)))))
(assert (and (and (=> BOOL____00081 (= Real____00082 1)) (=> (= Real____00082 1) BOOL____00081)) (and (=> (not BOOL____00081) (= Real____00082 0)) (=> (= Real____00082 0) (not BOOL____00081)))))
(assert (and (and (=> BOOL____00083 (= Real____00084 1)) (=> (= Real____00084 1) BOOL____00083)) (and (=> (not BOOL____00083) (= Real____00084 0)) (=> (= Real____00084 0) (not BOOL____00083)))))
(assert (and (and (=> BOOL____00085 (= Real____00086 1)) (=> (= Real____00086 1) BOOL____00085)) (and (=> (not BOOL____00085) (= Real____00086 0)) (=> (= Real____00086 0) (not BOOL____00085)))))
(assert (and (and (=> BOOL____00087 (= Real____00088 1)) (=> (= Real____00088 1) BOOL____00087)) (and (=> (not BOOL____00087) (= Real____00088 0)) (=> (= Real____00088 0) (not BOOL____00087)))))
(assert (and (and (=> BOOL____00089 (= Real____00090 1)) (=> (= Real____00090 1) BOOL____00089)) (and (=> (not BOOL____00089) (= Real____00090 0)) (=> (= Real____00090 0) (not BOOL____00089)))))
(assert (and (and (=> BOOL____00091 (= Real____00092 1)) (=> (= Real____00092 1) BOOL____00091)) (and (=> (not BOOL____00091) (= Real____00092 0)) (=> (= Real____00092 0) (not BOOL____00091)))))
(assert (and (and (=> BOOL____00093 (= Real____00094 1)) (=> (= Real____00094 1) BOOL____00093)) (and (=> (not BOOL____00093) (= Real____00094 0)) (=> (= Real____00094 0) (not BOOL____00093)))))
(assert (and (and (=> BOOL____00095 (= Real____00096 1)) (=> (= Real____00096 1) BOOL____00095)) (and (=> (not BOOL____00095) (= Real____00096 0)) (=> (= Real____00096 0) (not BOOL____00095)))))
(assert (and (and (=> BOOL____00097 (= Real____00098 1)) (=> (= Real____00098 1) BOOL____00097)) (and (=> (not BOOL____00097) (= Real____00098 0)) (=> (= Real____00098 0) (not BOOL____00097)))))
(assert (and (and (=> BOOL____00099 (= Real____00100 1)) (=> (= Real____00100 1) BOOL____00099)) (and (=> (not BOOL____00099) (= Real____00100 0)) (=> (= Real____00100 0) (not BOOL____00099)))))
(assert (and (and (=> BOOL____00101 (= Real____00102 1)) (=> (= Real____00102 1) BOOL____00101)) (and (=> (not BOOL____00101) (= Real____00102 0)) (=> (= Real____00102 0) (not BOOL____00101)))))
(assert (and (and (=> BOOL____00103 (= Real____00104 1)) (=> (= Real____00104 1) BOOL____00103)) (and (=> (not BOOL____00103) (= Real____00104 0)) (=> (= Real____00104 0) (not BOOL____00103)))))
(assert (and (and (=> BOOL____00105 (= Real____00106 1)) (=> (= Real____00106 1) BOOL____00105)) (and (=> (not BOOL____00105) (= Real____00106 0)) (=> (= Real____00106 0) (not BOOL____00105)))))
(assert (and (and (=> BOOL____00107 (= Real____00108 1)) (=> (= Real____00108 1) BOOL____00107)) (and (=> (not BOOL____00107) (= Real____00108 0)) (=> (= Real____00108 0) (not BOOL____00107)))))
(assert (and (and (=> BOOL____00109 (= Real____00110 1)) (=> (= Real____00110 1) BOOL____00109)) (and (=> (not BOOL____00109) (= Real____00110 0)) (=> (= Real____00110 0) (not BOOL____00109)))))
(assert (and (and (=> BOOL____00111 (= Real____00112 1)) (=> (= Real____00112 1) BOOL____00111)) (and (=> (not BOOL____00111) (= Real____00112 0)) (=> (= Real____00112 0) (not BOOL____00111)))))
(assert (and (and (=> BOOL____00113 (= Real____00114 1)) (=> (= Real____00114 1) BOOL____00113)) (and (=> (not BOOL____00113) (= Real____00114 0)) (=> (= Real____00114 0) (not BOOL____00113)))))
(assert (and (and (=> BOOL____00115 (= Real____00116 1)) (=> (= Real____00116 1) BOOL____00115)) (and (=> (not BOOL____00115) (= Real____00116 0)) (=> (= Real____00116 0) (not BOOL____00115)))))
(assert (and (and (=> BOOL____00117 (= Real____00118 1)) (=> (= Real____00118 1) BOOL____00117)) (and (=> (not BOOL____00117) (= Real____00118 0)) (=> (= Real____00118 0) (not BOOL____00117)))))
(assert (and (and (=> BOOL____00119 (= Real____00120 1)) (=> (= Real____00120 1) BOOL____00119)) (and (=> (not BOOL____00119) (= Real____00120 0)) (=> (= Real____00120 0) (not BOOL____00119)))))
(assert (and (and (=> BOOL____00121 (= Real____00122 1)) (=> (= Real____00122 1) BOOL____00121)) (and (=> (not BOOL____00121) (= Real____00122 0)) (=> (= Real____00122 0) (not BOOL____00121)))))
(assert (and (and (=> BOOL____00123 (= Real____00124 1)) (=> (= Real____00124 1) BOOL____00123)) (and (=> (not BOOL____00123) (= Real____00124 0)) (=> (= Real____00124 0) (not BOOL____00123)))))
(assert (and (and (=> BOOL____00125 (= Real____00126 1)) (=> (= Real____00126 1) BOOL____00125)) (and (=> (not BOOL____00125) (= Real____00126 0)) (=> (= Real____00126 0) (not BOOL____00125)))))
(assert (and (and (=> BOOL____00127 (= Real____00128 1)) (=> (= Real____00128 1) BOOL____00127)) (and (=> (not BOOL____00127) (= Real____00128 0)) (=> (= Real____00128 0) (not BOOL____00127)))))
(assert (and (and (=> BOOL____00129 (= Real____00130 1)) (=> (= Real____00130 1) BOOL____00129)) (and (=> (not BOOL____00129) (= Real____00130 0)) (=> (= Real____00130 0) (not BOOL____00129)))))
(assert (and (and (=> BOOL____00131 (= Real____00132 1)) (=> (= Real____00132 1) BOOL____00131)) (and (=> (not BOOL____00131) (= Real____00132 0)) (=> (= Real____00132 0) (not BOOL____00131)))))
(assert (and (and (=> BOOL____00133 (= Real____00134 1)) (=> (= Real____00134 1) BOOL____00133)) (and (=> (not BOOL____00133) (= Real____00134 0)) (=> (= Real____00134 0) (not BOOL____00133)))))
(assert (and (and (=> BOOL____00135 (= Real____00136 1)) (=> (= Real____00136 1) BOOL____00135)) (and (=> (not BOOL____00135) (= Real____00136 0)) (=> (= Real____00136 0) (not BOOL____00135)))))
(assert (and (and (=> BOOL____00137 (= Real____00138 1)) (=> (= Real____00138 1) BOOL____00137)) (and (=> (not BOOL____00137) (= Real____00138 0)) (=> (= Real____00138 0) (not BOOL____00137)))))
(assert (and (and (=> BOOL____00139 (= Real____00140 1)) (=> (= Real____00140 1) BOOL____00139)) (and (=> (not BOOL____00139) (= Real____00140 0)) (=> (= Real____00140 0) (not BOOL____00139)))))
(assert (and (and (=> BOOL____00141 (= Real____00142 1)) (=> (= Real____00142 1) BOOL____00141)) (and (=> (not BOOL____00141) (= Real____00142 0)) (=> (= Real____00142 0) (not BOOL____00141)))))
(assert (and (and (=> BOOL____00143 (= Real____00144 1)) (=> (= Real____00144 1) BOOL____00143)) (and (=> (not BOOL____00143) (= Real____00144 0)) (=> (= Real____00144 0) (not BOOL____00143)))))
(assert (and (and (=> BOOL____00145 (= Real____00146 1)) (=> (= Real____00146 1) BOOL____00145)) (and (=> (not BOOL____00145) (= Real____00146 0)) (=> (= Real____00146 0) (not BOOL____00145)))))
(assert (and (and (=> BOOL____00147 (= Real____00148 1)) (=> (= Real____00148 1) BOOL____00147)) (and (=> (not BOOL____00147) (= Real____00148 0)) (=> (= Real____00148 0) (not BOOL____00147)))))
(assert (and (and (=> BOOL____00149 (= Real____00150 1)) (=> (= Real____00150 1) BOOL____00149)) (and (=> (not BOOL____00149) (= Real____00150 0)) (=> (= Real____00150 0) (not BOOL____00149)))))
(assert (and (and (=> BOOL____00151 (= Real____00152 1)) (=> (= Real____00152 1) BOOL____00151)) (and (=> (not BOOL____00151) (= Real____00152 0)) (=> (= Real____00152 0) (not BOOL____00151)))))
(assert (and (and (=> BOOL____00153 (= Real____00154 1)) (=> (= Real____00154 1) BOOL____00153)) (and (=> (not BOOL____00153) (= Real____00154 0)) (=> (= Real____00154 0) (not BOOL____00153)))))
(assert (and (and (=> BOOL____00155 (= Real____00156 1)) (=> (= Real____00156 1) BOOL____00155)) (and (=> (not BOOL____00155) (= Real____00156 0)) (=> (= Real____00156 0) (not BOOL____00155)))))
(assert (and (and (=> BOOL____00157 (= Real____00158 1)) (=> (= Real____00158 1) BOOL____00157)) (and (=> (not BOOL____00157) (= Real____00158 0)) (=> (= Real____00158 0) (not BOOL____00157)))))
(assert (and (and (=> BOOL____00159 (= Real____00160 1)) (=> (= Real____00160 1) BOOL____00159)) (and (=> (not BOOL____00159) (= Real____00160 0)) (=> (= Real____00160 0) (not BOOL____00159)))))
(assert (and (and (=> BOOL____00161 (= Real____00162 1)) (=> (= Real____00162 1) BOOL____00161)) (and (=> (not BOOL____00161) (= Real____00162 0)) (=> (= Real____00162 0) (not BOOL____00161)))))
(assert (and (and (=> BOOL____00163 (= Real____00164 1)) (=> (= Real____00164 1) BOOL____00163)) (and (=> (not BOOL____00163) (= Real____00164 0)) (=> (= Real____00164 0) (not BOOL____00163)))))
(assert (and (and (=> BOOL____00165 (= Real____00166 1)) (=> (= Real____00166 1) BOOL____00165)) (and (=> (not BOOL____00165) (= Real____00166 0)) (=> (= Real____00166 0) (not BOOL____00165)))))
(assert (and (and (=> BOOL____00167 (= Real____00168 1)) (=> (= Real____00168 1) BOOL____00167)) (and (=> (not BOOL____00167) (= Real____00168 0)) (=> (= Real____00168 0) (not BOOL____00167)))))
(assert (and (and (=> BOOL____00169 (= Real____00170 1)) (=> (= Real____00170 1) BOOL____00169)) (and (=> (not BOOL____00169) (= Real____00170 0)) (=> (= Real____00170 0) (not BOOL____00169)))))
(assert (and (and (=> BOOL____00171 (= Real____00172 1)) (=> (= Real____00172 1) BOOL____00171)) (and (=> (not BOOL____00171) (= Real____00172 0)) (=> (= Real____00172 0) (not BOOL____00171)))))
(assert (and (and (=> BOOL____00173 (= Real____00174 1)) (=> (= Real____00174 1) BOOL____00173)) (and (=> (not BOOL____00173) (= Real____00174 0)) (=> (= Real____00174 0) (not BOOL____00173)))))
(assert (and (and (=> BOOL____00175 (= Real____00176 1)) (=> (= Real____00176 1) BOOL____00175)) (and (=> (not BOOL____00175) (= Real____00176 0)) (=> (= Real____00176 0) (not BOOL____00175)))))
(assert (and (and (=> BOOL____00177 (= Real____00178 1)) (=> (= Real____00178 1) BOOL____00177)) (and (=> (not BOOL____00177) (= Real____00178 0)) (=> (= Real____00178 0) (not BOOL____00177)))))
(assert (and (and (=> BOOL____00179 (= Real____00180 1)) (=> (= Real____00180 1) BOOL____00179)) (and (=> (not BOOL____00179) (= Real____00180 0)) (=> (= Real____00180 0) (not BOOL____00179)))))
(assert (and (and (=> BOOL____00181 (= Real____00182 1)) (=> (= Real____00182 1) BOOL____00181)) (and (=> (not BOOL____00181) (= Real____00182 0)) (=> (= Real____00182 0) (not BOOL____00181)))))
(assert (and (and (=> BOOL____00183 (= Real____00184 1)) (=> (= Real____00184 1) BOOL____00183)) (and (=> (not BOOL____00183) (= Real____00184 0)) (=> (= Real____00184 0) (not BOOL____00183)))))
(assert (and (and (=> BOOL____00185 (= Real____00186 1)) (=> (= Real____00186 1) BOOL____00185)) (and (=> (not BOOL____00185) (= Real____00186 0)) (=> (= Real____00186 0) (not BOOL____00185)))))
(assert (and (and (=> BOOL____00187 (= Real____00188 1)) (=> (= Real____00188 1) BOOL____00187)) (and (=> (not BOOL____00187) (= Real____00188 0)) (=> (= Real____00188 0) (not BOOL____00187)))))
(assert (and (and (=> BOOL____00189 (= Real____00190 1)) (=> (= Real____00190 1) BOOL____00189)) (and (=> (not BOOL____00189) (= Real____00190 0)) (=> (= Real____00190 0) (not BOOL____00189)))))
(assert (and (and (=> BOOL____00191 (= Real____00192 1)) (=> (= Real____00192 1) BOOL____00191)) (and (=> (not BOOL____00191) (= Real____00192 0)) (=> (= Real____00192 0) (not BOOL____00191)))))
(assert (and (and (=> BOOL____00193 (= Real____00194 1)) (=> (= Real____00194 1) BOOL____00193)) (and (=> (not BOOL____00193) (= Real____00194 0)) (=> (= Real____00194 0) (not BOOL____00193)))))
(assert (and (and (=> BOOL____00195 (= Real____00196 1)) (=> (= Real____00196 1) BOOL____00195)) (and (=> (not BOOL____00195) (= Real____00196 0)) (=> (= Real____00196 0) (not BOOL____00195)))))
(assert (and (and (=> BOOL____00197 (= Real____00198 1)) (=> (= Real____00198 1) BOOL____00197)) (and (=> (not BOOL____00197) (= Real____00198 0)) (=> (= Real____00198 0) (not BOOL____00197)))))
(assert (and (and (=> BOOL____00199 (= Real____00200 1)) (=> (= Real____00200 1) BOOL____00199)) (and (=> (not BOOL____00199) (= Real____00200 0)) (=> (= Real____00200 0) (not BOOL____00199)))))
(assert (and (and (=> BOOL____00201 (= Real____00202 1)) (=> (= Real____00202 1) BOOL____00201)) (and (=> (not BOOL____00201) (= Real____00202 0)) (=> (= Real____00202 0) (not BOOL____00201)))))
(assert (and (and (=> BOOL____00203 (= Real____00204 1)) (=> (= Real____00204 1) BOOL____00203)) (and (=> (not BOOL____00203) (= Real____00204 0)) (=> (= Real____00204 0) (not BOOL____00203)))))
(assert (and (and (=> BOOL____00205 (= Real____00206 1)) (=> (= Real____00206 1) BOOL____00205)) (and (=> (not BOOL____00205) (= Real____00206 0)) (=> (= Real____00206 0) (not BOOL____00205)))))
(assert (and (and (=> BOOL____00207 (= Real____00208 1)) (=> (= Real____00208 1) BOOL____00207)) (and (=> (not BOOL____00207) (= Real____00208 0)) (=> (= Real____00208 0) (not BOOL____00207)))))
(assert (and (and (=> BOOL____00209 (= Real____00210 1)) (=> (= Real____00210 1) BOOL____00209)) (and (=> (not BOOL____00209) (= Real____00210 0)) (=> (= Real____00210 0) (not BOOL____00209)))))
(assert (and (and (=> BOOL____00211 (= Real____00212 1)) (=> (= Real____00212 1) BOOL____00211)) (and (=> (not BOOL____00211) (= Real____00212 0)) (=> (= Real____00212 0) (not BOOL____00211)))))
(assert (and (and (=> BOOL____00213 (= Real____00214 1)) (=> (= Real____00214 1) BOOL____00213)) (and (=> (not BOOL____00213) (= Real____00214 0)) (=> (= Real____00214 0) (not BOOL____00213)))))
(assert (and (and (=> BOOL____00215 (= Real____00216 1)) (=> (= Real____00216 1) BOOL____00215)) (and (=> (not BOOL____00215) (= Real____00216 0)) (=> (= Real____00216 0) (not BOOL____00215)))))
(assert (and (and (=> BOOL____00217 (= Real____00218 1)) (=> (= Real____00218 1) BOOL____00217)) (and (=> (not BOOL____00217) (= Real____00218 0)) (=> (= Real____00218 0) (not BOOL____00217)))))
(assert (and (and (=> BOOL____00219 (= Real____00220 1)) (=> (= Real____00220 1) BOOL____00219)) (and (=> (not BOOL____00219) (= Real____00220 0)) (=> (= Real____00220 0) (not BOOL____00219)))))
(assert (and (and (=> BOOL____00221 (= Real____00222 1)) (=> (= Real____00222 1) BOOL____00221)) (and (=> (not BOOL____00221) (= Real____00222 0)) (=> (= Real____00222 0) (not BOOL____00221)))))
(assert (and (and (=> BOOL____00223 (= Real____00224 1)) (=> (= Real____00224 1) BOOL____00223)) (and (=> (not BOOL____00223) (= Real____00224 0)) (=> (= Real____00224 0) (not BOOL____00223)))))
(assert (and (and (=> BOOL____00225 (= Real____00226 1)) (=> (= Real____00226 1) BOOL____00225)) (and (=> (not BOOL____00225) (= Real____00226 0)) (=> (= Real____00226 0) (not BOOL____00225)))))
(assert (and (and (=> BOOL____00227 (= Real____00228 1)) (=> (= Real____00228 1) BOOL____00227)) (and (=> (not BOOL____00227) (= Real____00228 0)) (=> (= Real____00228 0) (not BOOL____00227)))))
(assert (and (and (=> BOOL____00229 (= Real____00230 1)) (=> (= Real____00230 1) BOOL____00229)) (and (=> (not BOOL____00229) (= Real____00230 0)) (=> (= Real____00230 0) (not BOOL____00229)))))
(assert (and (and (=> BOOL____00231 (= Real____00232 1)) (=> (= Real____00232 1) BOOL____00231)) (and (=> (not BOOL____00231) (= Real____00232 0)) (=> (= Real____00232 0) (not BOOL____00231)))))
(assert (and (and (=> BOOL____00233 (= Real____00234 1)) (=> (= Real____00234 1) BOOL____00233)) (and (=> (not BOOL____00233) (= Real____00234 0)) (=> (= Real____00234 0) (not BOOL____00233)))))
(assert (and (and (=> BOOL____00235 (= Real____00236 1)) (=> (= Real____00236 1) BOOL____00235)) (and (=> (not BOOL____00235) (= Real____00236 0)) (=> (= Real____00236 0) (not BOOL____00235)))))
(assert (and (and (=> BOOL____00237 (= Real____00238 1)) (=> (= Real____00238 1) BOOL____00237)) (and (=> (not BOOL____00237) (= Real____00238 0)) (=> (= Real____00238 0) (not BOOL____00237)))))
(assert (and (and (=> BOOL____00239 (= Real____00240 1)) (=> (= Real____00240 1) BOOL____00239)) (and (=> (not BOOL____00239) (= Real____00240 0)) (=> (= Real____00240 0) (not BOOL____00239)))))
(assert (and (and (=> BOOL____00241 (= Real____00242 1)) (=> (= Real____00242 1) BOOL____00241)) (and (=> (not BOOL____00241) (= Real____00242 0)) (=> (= Real____00242 0) (not BOOL____00241)))))
(assert (and (and (=> BOOL____00243 (= Real____00244 1)) (=> (= Real____00244 1) BOOL____00243)) (and (=> (not BOOL____00243) (= Real____00244 0)) (=> (= Real____00244 0) (not BOOL____00243)))))
(assert (and (and (=> BOOL____00245 (= Real____00246 1)) (=> (= Real____00246 1) BOOL____00245)) (and (=> (not BOOL____00245) (= Real____00246 0)) (=> (= Real____00246 0) (not BOOL____00245)))))
(assert (and (and (=> BOOL____00247 (= Real____00248 1)) (=> (= Real____00248 1) BOOL____00247)) (and (=> (not BOOL____00247) (= Real____00248 0)) (=> (= Real____00248 0) (not BOOL____00247)))))
(assert (and (and (=> BOOL____00249 (= Real____00250 1)) (=> (= Real____00250 1) BOOL____00249)) (and (=> (not BOOL____00249) (= Real____00250 0)) (=> (= Real____00250 0) (not BOOL____00249)))))
(assert (and (and (=> BOOL____00251 (= Real____00252 1)) (=> (= Real____00252 1) BOOL____00251)) (and (=> (not BOOL____00251) (= Real____00252 0)) (=> (= Real____00252 0) (not BOOL____00251)))))
(assert (and (and (=> BOOL____00253 (= Real____00254 1)) (=> (= Real____00254 1) BOOL____00253)) (and (=> (not BOOL____00253) (= Real____00254 0)) (=> (= Real____00254 0) (not BOOL____00253)))))
(assert (and (and (=> BOOL____00255 (= Real____00256 1)) (=> (= Real____00256 1) BOOL____00255)) (and (=> (not BOOL____00255) (= Real____00256 0)) (=> (= Real____00256 0) (not BOOL____00255)))))
(assert (and (and (=> BOOL____00257 (= Real____00258 1)) (=> (= Real____00258 1) BOOL____00257)) (and (=> (not BOOL____00257) (= Real____00258 0)) (=> (= Real____00258 0) (not BOOL____00257)))))
(assert (and (and (=> BOOL____00259 (= Real____00260 1)) (=> (= Real____00260 1) BOOL____00259)) (and (=> (not BOOL____00259) (= Real____00260 0)) (=> (= Real____00260 0) (not BOOL____00259)))))
(assert (and (and (=> BOOL____00261 (= Real____00262 1)) (=> (= Real____00262 1) BOOL____00261)) (and (=> (not BOOL____00261) (= Real____00262 0)) (=> (= Real____00262 0) (not BOOL____00261)))))
(assert (and (and (=> BOOL____00263 (= Real____00264 1)) (=> (= Real____00264 1) BOOL____00263)) (and (=> (not BOOL____00263) (= Real____00264 0)) (=> (= Real____00264 0) (not BOOL____00263)))))
(assert (and (and (=> BOOL____00265 (= Real____00266 1)) (=> (= Real____00266 1) BOOL____00265)) (and (=> (not BOOL____00265) (= Real____00266 0)) (=> (= Real____00266 0) (not BOOL____00265)))))
(assert (and (and (=> BOOL____00267 (= Real____00268 1)) (=> (= Real____00268 1) BOOL____00267)) (and (=> (not BOOL____00267) (= Real____00268 0)) (=> (= Real____00268 0) (not BOOL____00267)))))
(assert (and (and (=> BOOL____00269 (= Real____00270 1)) (=> (= Real____00270 1) BOOL____00269)) (and (=> (not BOOL____00269) (= Real____00270 0)) (=> (= Real____00270 0) (not BOOL____00269)))))
(assert (and (and (=> BOOL____00271 (= Real____00272 1)) (=> (= Real____00272 1) BOOL____00271)) (and (=> (not BOOL____00271) (= Real____00272 0)) (=> (= Real____00272 0) (not BOOL____00271)))))
(assert (and (and (=> BOOL____00273 (= Real____00274 1)) (=> (= Real____00274 1) BOOL____00273)) (and (=> (not BOOL____00273) (= Real____00274 0)) (=> (= Real____00274 0) (not BOOL____00273)))))
(assert (and (and (=> BOOL____00275 (= Real____00276 1)) (=> (= Real____00276 1) BOOL____00275)) (and (=> (not BOOL____00275) (= Real____00276 0)) (=> (= Real____00276 0) (not BOOL____00275)))))
(assert (and (and (=> BOOL____00277 (= Real____00278 1)) (=> (= Real____00278 1) BOOL____00277)) (and (=> (not BOOL____00277) (= Real____00278 0)) (=> (= Real____00278 0) (not BOOL____00277)))))
(assert (and (and (=> BOOL____00279 (= Real____00280 1)) (=> (= Real____00280 1) BOOL____00279)) (and (=> (not BOOL____00279) (= Real____00280 0)) (=> (= Real____00280 0) (not BOOL____00279)))))
(assert (and (and (=> BOOL____00281 (= Real____00282 1)) (=> (= Real____00282 1) BOOL____00281)) (and (=> (not BOOL____00281) (= Real____00282 0)) (=> (= Real____00282 0) (not BOOL____00281)))))
(assert (and (and (=> BOOL____00283 (= Real____00284 1)) (=> (= Real____00284 1) BOOL____00283)) (and (=> (not BOOL____00283) (= Real____00284 0)) (=> (= Real____00284 0) (not BOOL____00283)))))
(assert (and (and (=> BOOL____00285 (= Real____00286 1)) (=> (= Real____00286 1) BOOL____00285)) (and (=> (not BOOL____00285) (= Real____00286 0)) (=> (= Real____00286 0) (not BOOL____00285)))))
(assert (and (and (=> BOOL____00287 (= Real____00288 1)) (=> (= Real____00288 1) BOOL____00287)) (and (=> (not BOOL____00287) (= Real____00288 0)) (=> (= Real____00288 0) (not BOOL____00287)))))
(assert (and (and (=> BOOL____00289 (= Real____00290 1)) (=> (= Real____00290 1) BOOL____00289)) (and (=> (not BOOL____00289) (= Real____00290 0)) (=> (= Real____00290 0) (not BOOL____00289)))))
(assert (and (and (=> BOOL____00291 (= Real____00292 1)) (=> (= Real____00292 1) BOOL____00291)) (and (=> (not BOOL____00291) (= Real____00292 0)) (=> (= Real____00292 0) (not BOOL____00291)))))
(assert (and (and (=> BOOL____00293 (= Real____00294 1)) (=> (= Real____00294 1) BOOL____00293)) (and (=> (not BOOL____00293) (= Real____00294 0)) (=> (= Real____00294 0) (not BOOL____00293)))))
(assert (and (and (=> BOOL____00295 (= Real____00296 1)) (=> (= Real____00296 1) BOOL____00295)) (and (=> (not BOOL____00295) (= Real____00296 0)) (=> (= Real____00296 0) (not BOOL____00295)))))
(assert (and (and (=> BOOL____00297 (= Real____00298 1)) (=> (= Real____00298 1) BOOL____00297)) (and (=> (not BOOL____00297) (= Real____00298 0)) (=> (= Real____00298 0) (not BOOL____00297)))))
(assert (and (and (=> BOOL____00299 (= Real____00300 1)) (=> (= Real____00300 1) BOOL____00299)) (and (=> (not BOOL____00299) (= Real____00300 0)) (=> (= Real____00300 0) (not BOOL____00299)))))
(assert (and (and (=> BOOL____00301 (= Real____00302 1)) (=> (= Real____00302 1) BOOL____00301)) (and (=> (not BOOL____00301) (= Real____00302 0)) (=> (= Real____00302 0) (not BOOL____00301)))))
(assert (and (and (=> BOOL____00303 (= Real____00304 1)) (=> (= Real____00304 1) BOOL____00303)) (and (=> (not BOOL____00303) (= Real____00304 0)) (=> (= Real____00304 0) (not BOOL____00303)))))
(assert (and (and (=> BOOL____00305 (= Real____00306 1)) (=> (= Real____00306 1) BOOL____00305)) (and (=> (not BOOL____00305) (= Real____00306 0)) (=> (= Real____00306 0) (not BOOL____00305)))))
(assert (and (and (=> BOOL____00307 (= Real____00308 1)) (=> (= Real____00308 1) BOOL____00307)) (and (=> (not BOOL____00307) (= Real____00308 0)) (=> (= Real____00308 0) (not BOOL____00307)))))
(assert (and (and (=> BOOL____00309 (= Real____00310 1)) (=> (= Real____00310 1) BOOL____00309)) (and (=> (not BOOL____00309) (= Real____00310 0)) (=> (= Real____00310 0) (not BOOL____00309)))))
(assert (and (and (=> BOOL____00311 (= Real____00312 1)) (=> (= Real____00312 1) BOOL____00311)) (and (=> (not BOOL____00311) (= Real____00312 0)) (=> (= Real____00312 0) (not BOOL____00311)))))
(assert (and (and (=> BOOL____00313 (= Real____00314 1)) (=> (= Real____00314 1) BOOL____00313)) (and (=> (not BOOL____00313) (= Real____00314 0)) (=> (= Real____00314 0) (not BOOL____00313)))))
(assert (and (and (=> BOOL____00315 (= Real____00316 1)) (=> (= Real____00316 1) BOOL____00315)) (and (=> (not BOOL____00315) (= Real____00316 0)) (=> (= Real____00316 0) (not BOOL____00315)))))
(assert (and (and (=> BOOL____00317 (= Real____00318 1)) (=> (= Real____00318 1) BOOL____00317)) (and (=> (not BOOL____00317) (= Real____00318 0)) (=> (= Real____00318 0) (not BOOL____00317)))))
(assert (and (and (=> BOOL____00319 (= Real____00320 1)) (=> (= Real____00320 1) BOOL____00319)) (and (=> (not BOOL____00319) (= Real____00320 0)) (=> (= Real____00320 0) (not BOOL____00319)))))
(assert (and (and (=> BOOL____00321 (= Real____00322 1)) (=> (= Real____00322 1) BOOL____00321)) (and (=> (not BOOL____00321) (= Real____00322 0)) (=> (= Real____00322 0) (not BOOL____00321)))))
(assert (and (and (=> BOOL____00323 (= Real____00324 1)) (=> (= Real____00324 1) BOOL____00323)) (and (=> (not BOOL____00323) (= Real____00324 0)) (=> (= Real____00324 0) (not BOOL____00323)))))
(assert (and (and (=> BOOL____00325 (= Real____00326 1)) (=> (= Real____00326 1) BOOL____00325)) (and (=> (not BOOL____00325) (= Real____00326 0)) (=> (= Real____00326 0) (not BOOL____00325)))))
(assert (and (and (=> BOOL____00327 (= Real____00328 1)) (=> (= Real____00328 1) BOOL____00327)) (and (=> (not BOOL____00327) (= Real____00328 0)) (=> (= Real____00328 0) (not BOOL____00327)))))
(assert (and (and (=> BOOL____00329 (= Real____00330 1)) (=> (= Real____00330 1) BOOL____00329)) (and (=> (not BOOL____00329) (= Real____00330 0)) (=> (= Real____00330 0) (not BOOL____00329)))))
(assert (and (and (=> BOOL____00331 (= Real____00332 1)) (=> (= Real____00332 1) BOOL____00331)) (and (=> (not BOOL____00331) (= Real____00332 0)) (=> (= Real____00332 0) (not BOOL____00331)))))
(assert (and (and (=> BOOL____00333 (= Real____00334 1)) (=> (= Real____00334 1) BOOL____00333)) (and (=> (not BOOL____00333) (= Real____00334 0)) (=> (= Real____00334 0) (not BOOL____00333)))))
(assert (and (and (=> BOOL____00335 (= Real____00336 1)) (=> (= Real____00336 1) BOOL____00335)) (and (=> (not BOOL____00335) (= Real____00336 0)) (=> (= Real____00336 0) (not BOOL____00335)))))
(assert (and (and (=> BOOL____00337 (= Real____00338 1)) (=> (= Real____00338 1) BOOL____00337)) (and (=> (not BOOL____00337) (= Real____00338 0)) (=> (= Real____00338 0) (not BOOL____00337)))))
(assert (and (and (=> BOOL____00339 (= Real____00340 1)) (=> (= Real____00340 1) BOOL____00339)) (and (=> (not BOOL____00339) (= Real____00340 0)) (=> (= Real____00340 0) (not BOOL____00339)))))
(assert (and (and (=> BOOL____00341 (= Real____00342 1)) (=> (= Real____00342 1) BOOL____00341)) (and (=> (not BOOL____00341) (= Real____00342 0)) (=> (= Real____00342 0) (not BOOL____00341)))))
(assert (and (and (=> BOOL____00343 (= Real____00344 1)) (=> (= Real____00344 1) BOOL____00343)) (and (=> (not BOOL____00343) (= Real____00344 0)) (=> (= Real____00344 0) (not BOOL____00343)))))
(assert (and (and (=> BOOL____00345 (= Real____00346 1)) (=> (= Real____00346 1) BOOL____00345)) (and (=> (not BOOL____00345) (= Real____00346 0)) (=> (= Real____00346 0) (not BOOL____00345)))))
(assert (and (and (=> BOOL____00347 (= Real____00348 1)) (=> (= Real____00348 1) BOOL____00347)) (and (=> (not BOOL____00347) (= Real____00348 0)) (=> (= Real____00348 0) (not BOOL____00347)))))
(assert (and (and (=> BOOL____00349 (= Real____00350 1)) (=> (= Real____00350 1) BOOL____00349)) (and (=> (not BOOL____00349) (= Real____00350 0)) (=> (= Real____00350 0) (not BOOL____00349)))))
(assert (and (and (=> BOOL____00351 (= Real____00352 1)) (=> (= Real____00352 1) BOOL____00351)) (and (=> (not BOOL____00351) (= Real____00352 0)) (=> (= Real____00352 0) (not BOOL____00351)))))
(assert (and (and (=> BOOL____00353 (= Real____00354 1)) (=> (= Real____00354 1) BOOL____00353)) (and (=> (not BOOL____00353) (= Real____00354 0)) (=> (= Real____00354 0) (not BOOL____00353)))))
(assert (and (and (=> BOOL____00355 (= Real____00356 1)) (=> (= Real____00356 1) BOOL____00355)) (and (=> (not BOOL____00355) (= Real____00356 0)) (=> (= Real____00356 0) (not BOOL____00355)))))
(assert (and (and (=> BOOL____00357 (= Real____00358 1)) (=> (= Real____00358 1) BOOL____00357)) (and (=> (not BOOL____00357) (= Real____00358 0)) (=> (= Real____00358 0) (not BOOL____00357)))))
(assert (and (and (=> BOOL____00359 (= Real____00360 1)) (=> (= Real____00360 1) BOOL____00359)) (and (=> (not BOOL____00359) (= Real____00360 0)) (=> (= Real____00360 0) (not BOOL____00359)))))
(assert (and (and (=> BOOL____00361 (= Real____00362 1)) (=> (= Real____00362 1) BOOL____00361)) (and (=> (not BOOL____00361) (= Real____00362 0)) (=> (= Real____00362 0) (not BOOL____00361)))))
(assert (and (and (=> BOOL____00363 (= Real____00364 1)) (=> (= Real____00364 1) BOOL____00363)) (and (=> (not BOOL____00363) (= Real____00364 0)) (=> (= Real____00364 0) (not BOOL____00363)))))
(assert (and (and (=> BOOL____00365 (= Real____00366 1)) (=> (= Real____00366 1) BOOL____00365)) (and (=> (not BOOL____00365) (= Real____00366 0)) (=> (= Real____00366 0) (not BOOL____00365)))))
(assert (and (and (=> BOOL____00367 (= Real____00368 1)) (=> (= Real____00368 1) BOOL____00367)) (and (=> (not BOOL____00367) (= Real____00368 0)) (=> (= Real____00368 0) (not BOOL____00367)))))
(assert (and (and (=> BOOL____00369 (= Real____00370 1)) (=> (= Real____00370 1) BOOL____00369)) (and (=> (not BOOL____00369) (= Real____00370 0)) (=> (= Real____00370 0) (not BOOL____00369)))))
(assert (and (and (=> BOOL____00371 (= Real____00372 1)) (=> (= Real____00372 1) BOOL____00371)) (and (=> (not BOOL____00371) (= Real____00372 0)) (=> (= Real____00372 0) (not BOOL____00371)))))
(assert (and (and (=> BOOL____00373 (= Real____00374 1)) (=> (= Real____00374 1) BOOL____00373)) (and (=> (not BOOL____00373) (= Real____00374 0)) (=> (= Real____00374 0) (not BOOL____00373)))))
(assert (and (and (=> BOOL____00375 (= Real____00376 1)) (=> (= Real____00376 1) BOOL____00375)) (and (=> (not BOOL____00375) (= Real____00376 0)) (=> (= Real____00376 0) (not BOOL____00375)))))
(assert (and (and (=> BOOL____00377 (= Real____00378 1)) (=> (= Real____00378 1) BOOL____00377)) (and (=> (not BOOL____00377) (= Real____00378 0)) (=> (= Real____00378 0) (not BOOL____00377)))))
(assert (and (and (=> BOOL____00379 (= Real____00380 1)) (=> (= Real____00380 1) BOOL____00379)) (and (=> (not BOOL____00379) (= Real____00380 0)) (=> (= Real____00380 0) (not BOOL____00379)))))
(assert (and (and (=> BOOL____00381 (= Real____00382 1)) (=> (= Real____00382 1) BOOL____00381)) (and (=> (not BOOL____00381) (= Real____00382 0)) (=> (= Real____00382 0) (not BOOL____00381)))))
(assert (and (and (=> BOOL____00383 (= Real____00384 1)) (=> (= Real____00384 1) BOOL____00383)) (and (=> (not BOOL____00383) (= Real____00384 0)) (=> (= Real____00384 0) (not BOOL____00383)))))
(assert (and (and (=> BOOL____00385 (= Real____00386 1)) (=> (= Real____00386 1) BOOL____00385)) (and (=> (not BOOL____00385) (= Real____00386 0)) (=> (= Real____00386 0) (not BOOL____00385)))))
(assert (and (and (=> BOOL____00387 (= Real____00388 1)) (=> (= Real____00388 1) BOOL____00387)) (and (=> (not BOOL____00387) (= Real____00388 0)) (=> (= Real____00388 0) (not BOOL____00387)))))
(assert (and (and (=> BOOL____00389 (= Real____00390 1)) (=> (= Real____00390 1) BOOL____00389)) (and (=> (not BOOL____00389) (= Real____00390 0)) (=> (= Real____00390 0) (not BOOL____00389)))))
(assert (and (and (=> BOOL____00391 (= Real____00392 1)) (=> (= Real____00392 1) BOOL____00391)) (and (=> (not BOOL____00391) (= Real____00392 0)) (=> (= Real____00392 0) (not BOOL____00391)))))
(assert (and (and (=> BOOL____00393 (= Real____00394 1)) (=> (= Real____00394 1) BOOL____00393)) (and (=> (not BOOL____00393) (= Real____00394 0)) (=> (= Real____00394 0) (not BOOL____00393)))))
(assert (and (and (=> BOOL____00395 (= Real____00396 1)) (=> (= Real____00396 1) BOOL____00395)) (and (=> (not BOOL____00395) (= Real____00396 0)) (=> (= Real____00396 0) (not BOOL____00395)))))
(assert (and (and (=> BOOL____00397 (= Real____00398 1)) (=> (= Real____00398 1) BOOL____00397)) (and (=> (not BOOL____00397) (= Real____00398 0)) (=> (= Real____00398 0) (not BOOL____00397)))))
(assert (and (and (=> BOOL____00399 (= Real____00400 1)) (=> (= Real____00400 1) BOOL____00399)) (and (=> (not BOOL____00399) (= Real____00400 0)) (=> (= Real____00400 0) (not BOOL____00399)))))
(assert (and (and (=> BOOL____00401 (= Real____00402 1)) (=> (= Real____00402 1) BOOL____00401)) (and (=> (not BOOL____00401) (= Real____00402 0)) (=> (= Real____00402 0) (not BOOL____00401)))))
(assert (and (and (=> BOOL____00403 (= Real____00404 1)) (=> (= Real____00404 1) BOOL____00403)) (and (=> (not BOOL____00403) (= Real____00404 0)) (=> (= Real____00404 0) (not BOOL____00403)))))
(assert (and (and (=> BOOL____00405 (= Real____00406 1)) (=> (= Real____00406 1) BOOL____00405)) (and (=> (not BOOL____00405) (= Real____00406 0)) (=> (= Real____00406 0) (not BOOL____00405)))))
(assert (and (and (=> BOOL____00407 (= Real____00408 1)) (=> (= Real____00408 1) BOOL____00407)) (and (=> (not BOOL____00407) (= Real____00408 0)) (=> (= Real____00408 0) (not BOOL____00407)))))
(assert (and (and (=> BOOL____00409 (= Real____00410 1)) (=> (= Real____00410 1) BOOL____00409)) (and (=> (not BOOL____00409) (= Real____00410 0)) (=> (= Real____00410 0) (not BOOL____00409)))))
(assert (and (and (=> BOOL____00411 (= Real____00412 1)) (=> (= Real____00412 1) BOOL____00411)) (and (=> (not BOOL____00411) (= Real____00412 0)) (=> (= Real____00412 0) (not BOOL____00411)))))
(assert (and (and (=> BOOL____00413 (= Real____00414 1)) (=> (= Real____00414 1) BOOL____00413)) (and (=> (not BOOL____00413) (= Real____00414 0)) (=> (= Real____00414 0) (not BOOL____00413)))))
(assert (and (and (=> BOOL____00415 (= Real____00416 1)) (=> (= Real____00416 1) BOOL____00415)) (and (=> (not BOOL____00415) (= Real____00416 0)) (=> (= Real____00416 0) (not BOOL____00415)))))
(assert (and (and (=> BOOL____00417 (= Real____00418 1)) (=> (= Real____00418 1) BOOL____00417)) (and (=> (not BOOL____00417) (= Real____00418 0)) (=> (= Real____00418 0) (not BOOL____00417)))))
(assert (and (and (=> BOOL____00419 (= Real____00420 1)) (=> (= Real____00420 1) BOOL____00419)) (and (=> (not BOOL____00419) (= Real____00420 0)) (=> (= Real____00420 0) (not BOOL____00419)))))
(assert (and (and (=> BOOL____00421 (= Real____00422 1)) (=> (= Real____00422 1) BOOL____00421)) (and (=> (not BOOL____00421) (= Real____00422 0)) (=> (= Real____00422 0) (not BOOL____00421)))))
(assert (and (and (=> BOOL____00423 (= Real____00424 1)) (=> (= Real____00424 1) BOOL____00423)) (and (=> (not BOOL____00423) (= Real____00424 0)) (=> (= Real____00424 0) (not BOOL____00423)))))
(assert (and (and (=> BOOL____00425 (= Real____00426 1)) (=> (= Real____00426 1) BOOL____00425)) (and (=> (not BOOL____00425) (= Real____00426 0)) (=> (= Real____00426 0) (not BOOL____00425)))))
(assert (and (and (=> BOOL____00427 (= Real____00428 1)) (=> (= Real____00428 1) BOOL____00427)) (and (=> (not BOOL____00427) (= Real____00428 0)) (=> (= Real____00428 0) (not BOOL____00427)))))
(assert (and (and (=> BOOL____00429 (= Real____00430 1)) (=> (= Real____00430 1) BOOL____00429)) (and (=> (not BOOL____00429) (= Real____00430 0)) (=> (= Real____00430 0) (not BOOL____00429)))))
(assert (and (and (=> BOOL____00431 (= Real____00432 1)) (=> (= Real____00432 1) BOOL____00431)) (and (=> (not BOOL____00431) (= Real____00432 0)) (=> (= Real____00432 0) (not BOOL____00431)))))
(assert (and (and (=> BOOL____00433 (= Real____00434 1)) (=> (= Real____00434 1) BOOL____00433)) (and (=> (not BOOL____00433) (= Real____00434 0)) (=> (= Real____00434 0) (not BOOL____00433)))))
(assert (and (and (=> BOOL____00435 (= Real____00436 1)) (=> (= Real____00436 1) BOOL____00435)) (and (=> (not BOOL____00435) (= Real____00436 0)) (=> (= Real____00436 0) (not BOOL____00435)))))
(assert (and (and (=> BOOL____00437 (= Real____00438 1)) (=> (= Real____00438 1) BOOL____00437)) (and (=> (not BOOL____00437) (= Real____00438 0)) (=> (= Real____00438 0) (not BOOL____00437)))))
(assert (and (and (=> BOOL____00439 (= Real____00440 1)) (=> (= Real____00440 1) BOOL____00439)) (and (=> (not BOOL____00439) (= Real____00440 0)) (=> (= Real____00440 0) (not BOOL____00439)))))
(assert (and (and (=> BOOL____00441 (= Real____00442 1)) (=> (= Real____00442 1) BOOL____00441)) (and (=> (not BOOL____00441) (= Real____00442 0)) (=> (= Real____00442 0) (not BOOL____00441)))))
(assert (and (and (=> BOOL____00443 (= Real____00444 1)) (=> (= Real____00444 1) BOOL____00443)) (and (=> (not BOOL____00443) (= Real____00444 0)) (=> (= Real____00444 0) (not BOOL____00443)))))
(assert (and (and (=> BOOL____00445 (= Real____00446 1)) (=> (= Real____00446 1) BOOL____00445)) (and (=> (not BOOL____00445) (= Real____00446 0)) (=> (= Real____00446 0) (not BOOL____00445)))))
(assert (and (and (=> BOOL____00447 (= Real____00448 1)) (=> (= Real____00448 1) BOOL____00447)) (and (=> (not BOOL____00447) (= Real____00448 0)) (=> (= Real____00448 0) (not BOOL____00447)))))
(assert (and (and (=> BOOL____00449 (= Real____00450 1)) (=> (= Real____00450 1) BOOL____00449)) (and (=> (not BOOL____00449) (= Real____00450 0)) (=> (= Real____00450 0) (not BOOL____00449)))))
(assert (and (and (=> BOOL____00451 (= Real____00452 1)) (=> (= Real____00452 1) BOOL____00451)) (and (=> (not BOOL____00451) (= Real____00452 0)) (=> (= Real____00452 0) (not BOOL____00451)))))
(assert (and (and (=> BOOL____00453 (= Real____00454 1)) (=> (= Real____00454 1) BOOL____00453)) (and (=> (not BOOL____00453) (= Real____00454 0)) (=> (= Real____00454 0) (not BOOL____00453)))))
(assert (and (and (=> BOOL____00455 (= Real____00456 1)) (=> (= Real____00456 1) BOOL____00455)) (and (=> (not BOOL____00455) (= Real____00456 0)) (=> (= Real____00456 0) (not BOOL____00455)))))
(assert (and (and (=> BOOL____00457 (= Real____00458 1)) (=> (= Real____00458 1) BOOL____00457)) (and (=> (not BOOL____00457) (= Real____00458 0)) (=> (= Real____00458 0) (not BOOL____00457)))))
(assert (and (and (=> BOOL____00459 (= Real____00460 1)) (=> (= Real____00460 1) BOOL____00459)) (and (=> (not BOOL____00459) (= Real____00460 0)) (=> (= Real____00460 0) (not BOOL____00459)))))
(assert (and (and (=> BOOL____00461 (= Real____00462 1)) (=> (= Real____00462 1) BOOL____00461)) (and (=> (not BOOL____00461) (= Real____00462 0)) (=> (= Real____00462 0) (not BOOL____00461)))))
(assert (and (and (=> BOOL____00463 (= Real____00464 1)) (=> (= Real____00464 1) BOOL____00463)) (and (=> (not BOOL____00463) (= Real____00464 0)) (=> (= Real____00464 0) (not BOOL____00463)))))
(assert (and (and (=> BOOL____00465 (= Real____00466 1)) (=> (= Real____00466 1) BOOL____00465)) (and (=> (not BOOL____00465) (= Real____00466 0)) (=> (= Real____00466 0) (not BOOL____00465)))))
(assert (and (and (=> BOOL____00467 (= Real____00468 1)) (=> (= Real____00468 1) BOOL____00467)) (and (=> (not BOOL____00467) (= Real____00468 0)) (=> (= Real____00468 0) (not BOOL____00467)))))
(assert (and (and (=> BOOL____00469 (= Real____00470 1)) (=> (= Real____00470 1) BOOL____00469)) (and (=> (not BOOL____00469) (= Real____00470 0)) (=> (= Real____00470 0) (not BOOL____00469)))))
(assert (and (and (=> BOOL____00471 (= Real____00472 1)) (=> (= Real____00472 1) BOOL____00471)) (and (=> (not BOOL____00471) (= Real____00472 0)) (=> (= Real____00472 0) (not BOOL____00471)))))
(assert (and (and (=> BOOL____00473 (= Real____00474 1)) (=> (= Real____00474 1) BOOL____00473)) (and (=> (not BOOL____00473) (= Real____00474 0)) (=> (= Real____00474 0) (not BOOL____00473)))))
(assert (and (and (=> BOOL____00475 (= Real____00476 1)) (=> (= Real____00476 1) BOOL____00475)) (and (=> (not BOOL____00475) (= Real____00476 0)) (=> (= Real____00476 0) (not BOOL____00475)))))
(assert (and (and (=> BOOL____00477 (= Real____00478 1)) (=> (= Real____00478 1) BOOL____00477)) (and (=> (not BOOL____00477) (= Real____00478 0)) (=> (= Real____00478 0) (not BOOL____00477)))))
(assert (and (and (=> BOOL____00479 (= Real____00480 1)) (=> (= Real____00480 1) BOOL____00479)) (and (=> (not BOOL____00479) (= Real____00480 0)) (=> (= Real____00480 0) (not BOOL____00479)))))
(assert (and (and (=> BOOL____00481 (= Real____00482 1)) (=> (= Real____00482 1) BOOL____00481)) (and (=> (not BOOL____00481) (= Real____00482 0)) (=> (= Real____00482 0) (not BOOL____00481)))))
(assert (and (and (=> BOOL____00483 (= Real____00484 1)) (=> (= Real____00484 1) BOOL____00483)) (and (=> (not BOOL____00483) (= Real____00484 0)) (=> (= Real____00484 0) (not BOOL____00483)))))
(assert (and (and (=> BOOL____00485 (= Real____00486 1)) (=> (= Real____00486 1) BOOL____00485)) (and (=> (not BOOL____00485) (= Real____00486 0)) (=> (= Real____00486 0) (not BOOL____00485)))))
(assert (and (and (=> BOOL____00487 (= Real____00488 1)) (=> (= Real____00488 1) BOOL____00487)) (and (=> (not BOOL____00487) (= Real____00488 0)) (=> (= Real____00488 0) (not BOOL____00487)))))
(assert (and (and (=> BOOL____00489 (= Real____00490 1)) (=> (= Real____00490 1) BOOL____00489)) (and (=> (not BOOL____00489) (= Real____00490 0)) (=> (= Real____00490 0) (not BOOL____00489)))))
(assert (and (and (=> BOOL____00491 (= Real____00492 1)) (=> (= Real____00492 1) BOOL____00491)) (and (=> (not BOOL____00491) (= Real____00492 0)) (=> (= Real____00492 0) (not BOOL____00491)))))
(assert (and (and (=> BOOL____00493 (= Real____00494 1)) (=> (= Real____00494 1) BOOL____00493)) (and (=> (not BOOL____00493) (= Real____00494 0)) (=> (= Real____00494 0) (not BOOL____00493)))))
(assert (and (and (=> BOOL____00495 (= Real____00496 1)) (=> (= Real____00496 1) BOOL____00495)) (and (=> (not BOOL____00495) (= Real____00496 0)) (=> (= Real____00496 0) (not BOOL____00495)))))
(assert (and (and (=> BOOL____00497 (= Real____00498 1)) (=> (= Real____00498 1) BOOL____00497)) (and (=> (not BOOL____00497) (= Real____00498 0)) (=> (= Real____00498 0) (not BOOL____00497)))))
(assert (and (and (=> BOOL____00499 (= Real____00500 1)) (=> (= Real____00500 1) BOOL____00499)) (and (=> (not BOOL____00499) (= Real____00500 0)) (=> (= Real____00500 0) (not BOOL____00499)))))
(assert (and (and (=> BOOL____00501 (= Real____00502 1)) (=> (= Real____00502 1) BOOL____00501)) (and (=> (not BOOL____00501) (= Real____00502 0)) (=> (= Real____00502 0) (not BOOL____00501)))))
(assert (and (and (=> BOOL____00503 (= Real____00504 1)) (=> (= Real____00504 1) BOOL____00503)) (and (=> (not BOOL____00503) (= Real____00504 0)) (=> (= Real____00504 0) (not BOOL____00503)))))
(assert (and (and (=> BOOL____00505 (= Real____00506 1)) (=> (= Real____00506 1) BOOL____00505)) (and (=> (not BOOL____00505) (= Real____00506 0)) (=> (= Real____00506 0) (not BOOL____00505)))))
(assert (and (and (=> BOOL____00507 (= Real____00508 1)) (=> (= Real____00508 1) BOOL____00507)) (and (=> (not BOOL____00507) (= Real____00508 0)) (=> (= Real____00508 0) (not BOOL____00507)))))
(assert (and (and (=> BOOL____00509 (= Real____00510 1)) (=> (= Real____00510 1) BOOL____00509)) (and (=> (not BOOL____00509) (= Real____00510 0)) (=> (= Real____00510 0) (not BOOL____00509)))))
(assert (and (and (=> BOOL____00511 (= Real____00512 1)) (=> (= Real____00512 1) BOOL____00511)) (and (=> (not BOOL____00511) (= Real____00512 0)) (=> (= Real____00512 0) (not BOOL____00511)))))
(assert (and (and (=> BOOL____00513 (= Real____00514 1)) (=> (= Real____00514 1) BOOL____00513)) (and (=> (not BOOL____00513) (= Real____00514 0)) (=> (= Real____00514 0) (not BOOL____00513)))))
(assert (and (and (=> BOOL____00515 (= Real____00516 1)) (=> (= Real____00516 1) BOOL____00515)) (and (=> (not BOOL____00515) (= Real____00516 0)) (=> (= Real____00516 0) (not BOOL____00515)))))
(assert (and (and (=> BOOL____00517 (= Real____00518 1)) (=> (= Real____00518 1) BOOL____00517)) (and (=> (not BOOL____00517) (= Real____00518 0)) (=> (= Real____00518 0) (not BOOL____00517)))))
(assert (and (and (=> BOOL____00519 (= Real____00520 1)) (=> (= Real____00520 1) BOOL____00519)) (and (=> (not BOOL____00519) (= Real____00520 0)) (=> (= Real____00520 0) (not BOOL____00519)))))
(assert (and (and (=> BOOL____00521 (= Real____00522 1)) (=> (= Real____00522 1) BOOL____00521)) (and (=> (not BOOL____00521) (= Real____00522 0)) (=> (= Real____00522 0) (not BOOL____00521)))))
(assert (and (and (=> BOOL____00523 (= Real____00524 1)) (=> (= Real____00524 1) BOOL____00523)) (and (=> (not BOOL____00523) (= Real____00524 0)) (=> (= Real____00524 0) (not BOOL____00523)))))
(assert (and (and (=> BOOL____00525 (= Real____00526 1)) (=> (= Real____00526 1) BOOL____00525)) (and (=> (not BOOL____00525) (= Real____00526 0)) (=> (= Real____00526 0) (not BOOL____00525)))))
(assert (and (and (=> BOOL____00527 (= Real____00528 1)) (=> (= Real____00528 1) BOOL____00527)) (and (=> (not BOOL____00527) (= Real____00528 0)) (=> (= Real____00528 0) (not BOOL____00527)))))
(assert (and (and (=> BOOL____00529 (= Real____00530 1)) (=> (= Real____00530 1) BOOL____00529)) (and (=> (not BOOL____00529) (= Real____00530 0)) (=> (= Real____00530 0) (not BOOL____00529)))))
(assert (and (and (=> BOOL____00531 (= Real____00532 1)) (=> (= Real____00532 1) BOOL____00531)) (and (=> (not BOOL____00531) (= Real____00532 0)) (=> (= Real____00532 0) (not BOOL____00531)))))
(assert (and (and (=> BOOL____00533 (= Real____00534 1)) (=> (= Real____00534 1) BOOL____00533)) (and (=> (not BOOL____00533) (= Real____00534 0)) (=> (= Real____00534 0) (not BOOL____00533)))))
(assert (and (and (=> BOOL____00535 (= Real____00536 1)) (=> (= Real____00536 1) BOOL____00535)) (and (=> (not BOOL____00535) (= Real____00536 0)) (=> (= Real____00536 0) (not BOOL____00535)))))
(assert (and (and (=> BOOL____00537 (= Real____00538 1)) (=> (= Real____00538 1) BOOL____00537)) (and (=> (not BOOL____00537) (= Real____00538 0)) (=> (= Real____00538 0) (not BOOL____00537)))))
(assert (and (and (=> BOOL____00539 (= Real____00540 1)) (=> (= Real____00540 1) BOOL____00539)) (and (=> (not BOOL____00539) (= Real____00540 0)) (=> (= Real____00540 0) (not BOOL____00539)))))
(assert (and (and (=> BOOL____00541 (= Real____00542 1)) (=> (= Real____00542 1) BOOL____00541)) (and (=> (not BOOL____00541) (= Real____00542 0)) (=> (= Real____00542 0) (not BOOL____00541)))))
(assert (and (and (=> BOOL____00543 (= Real____00544 1)) (=> (= Real____00544 1) BOOL____00543)) (and (=> (not BOOL____00543) (= Real____00544 0)) (=> (= Real____00544 0) (not BOOL____00543)))))
(assert (and (and (=> BOOL____00545 (= Real____00546 1)) (=> (= Real____00546 1) BOOL____00545)) (and (=> (not BOOL____00545) (= Real____00546 0)) (=> (= Real____00546 0) (not BOOL____00545)))))
(assert (and (and (=> BOOL____00547 (= Real____00548 1)) (=> (= Real____00548 1) BOOL____00547)) (and (=> (not BOOL____00547) (= Real____00548 0)) (=> (= Real____00548 0) (not BOOL____00547)))))
(assert (and (and (=> BOOL____00549 (= Real____00550 1)) (=> (= Real____00550 1) BOOL____00549)) (and (=> (not BOOL____00549) (= Real____00550 0)) (=> (= Real____00550 0) (not BOOL____00549)))))
(assert (and (and (=> BOOL____00551 (= Real____00552 1)) (=> (= Real____00552 1) BOOL____00551)) (and (=> (not BOOL____00551) (= Real____00552 0)) (=> (= Real____00552 0) (not BOOL____00551)))))
(assert (and (and (=> BOOL____00553 (= Real____00554 1)) (=> (= Real____00554 1) BOOL____00553)) (and (=> (not BOOL____00553) (= Real____00554 0)) (=> (= Real____00554 0) (not BOOL____00553)))))
(assert (and (and (=> BOOL____00555 (= Real____00556 1)) (=> (= Real____00556 1) BOOL____00555)) (and (=> (not BOOL____00555) (= Real____00556 0)) (=> (= Real____00556 0) (not BOOL____00555)))))
(assert (and (and (=> BOOL____00557 (= Real____00558 1)) (=> (= Real____00558 1) BOOL____00557)) (and (=> (not BOOL____00557) (= Real____00558 0)) (=> (= Real____00558 0) (not BOOL____00557)))))
(assert (and (and (=> BOOL____00559 (= Real____00560 1)) (=> (= Real____00560 1) BOOL____00559)) (and (=> (not BOOL____00559) (= Real____00560 0)) (=> (= Real____00560 0) (not BOOL____00559)))))
(assert (and (and (=> BOOL____00561 (= Real____00562 1)) (=> (= Real____00562 1) BOOL____00561)) (and (=> (not BOOL____00561) (= Real____00562 0)) (=> (= Real____00562 0) (not BOOL____00561)))))
(assert (and (and (=> BOOL____00563 (= Real____00564 1)) (=> (= Real____00564 1) BOOL____00563)) (and (=> (not BOOL____00563) (= Real____00564 0)) (=> (= Real____00564 0) (not BOOL____00563)))))
(assert (and (and (=> BOOL____00565 (= Real____00566 1)) (=> (= Real____00566 1) BOOL____00565)) (and (=> (not BOOL____00565) (= Real____00566 0)) (=> (= Real____00566 0) (not BOOL____00565)))))
(assert (and (and (=> BOOL____00567 (= Real____00568 1)) (=> (= Real____00568 1) BOOL____00567)) (and (=> (not BOOL____00567) (= Real____00568 0)) (=> (= Real____00568 0) (not BOOL____00567)))))
(assert (and (and (=> BOOL____00569 (= Real____00570 1)) (=> (= Real____00570 1) BOOL____00569)) (and (=> (not BOOL____00569) (= Real____00570 0)) (=> (= Real____00570 0) (not BOOL____00569)))))
(assert (and (and (=> BOOL____00571 (= Real____00572 1)) (=> (= Real____00572 1) BOOL____00571)) (and (=> (not BOOL____00571) (= Real____00572 0)) (=> (= Real____00572 0) (not BOOL____00571)))))
(assert (and (and (=> BOOL____00573 (= Real____00574 1)) (=> (= Real____00574 1) BOOL____00573)) (and (=> (not BOOL____00573) (= Real____00574 0)) (=> (= Real____00574 0) (not BOOL____00573)))))
(assert (and (and (=> BOOL____00575 (= Real____00576 1)) (=> (= Real____00576 1) BOOL____00575)) (and (=> (not BOOL____00575) (= Real____00576 0)) (=> (= Real____00576 0) (not BOOL____00575)))))
(assert (and (and (=> BOOL____00577 (= Real____00578 1)) (=> (= Real____00578 1) BOOL____00577)) (and (=> (not BOOL____00577) (= Real____00578 0)) (=> (= Real____00578 0) (not BOOL____00577)))))
(assert (and (and (=> BOOL____00579 (= Real____00580 1)) (=> (= Real____00580 1) BOOL____00579)) (and (=> (not BOOL____00579) (= Real____00580 0)) (=> (= Real____00580 0) (not BOOL____00579)))))
(assert (and (and (=> BOOL____00581 (= Real____00582 1)) (=> (= Real____00582 1) BOOL____00581)) (and (=> (not BOOL____00581) (= Real____00582 0)) (=> (= Real____00582 0) (not BOOL____00581)))))
(assert (and (and (=> BOOL____00583 (= Real____00584 1)) (=> (= Real____00584 1) BOOL____00583)) (and (=> (not BOOL____00583) (= Real____00584 0)) (=> (= Real____00584 0) (not BOOL____00583)))))
(assert (and (and (=> BOOL____00585 (= Real____00586 1)) (=> (= Real____00586 1) BOOL____00585)) (and (=> (not BOOL____00585) (= Real____00586 0)) (=> (= Real____00586 0) (not BOOL____00585)))))
(assert (and (and (=> BOOL____00587 (= Real____00588 1)) (=> (= Real____00588 1) BOOL____00587)) (and (=> (not BOOL____00587) (= Real____00588 0)) (=> (= Real____00588 0) (not BOOL____00587)))))
(assert (and (and (=> BOOL____00589 (= Real____00590 1)) (=> (= Real____00590 1) BOOL____00589)) (and (=> (not BOOL____00589) (= Real____00590 0)) (=> (= Real____00590 0) (not BOOL____00589)))))
(assert (and (and (=> BOOL____00591 (= Real____00592 1)) (=> (= Real____00592 1) BOOL____00591)) (and (=> (not BOOL____00591) (= Real____00592 0)) (=> (= Real____00592 0) (not BOOL____00591)))))
(assert (and (and (=> BOOL____00593 (= Real____00594 1)) (=> (= Real____00594 1) BOOL____00593)) (and (=> (not BOOL____00593) (= Real____00594 0)) (=> (= Real____00594 0) (not BOOL____00593)))))
(assert (and (and (=> BOOL____00595 (= Real____00596 1)) (=> (= Real____00596 1) BOOL____00595)) (and (=> (not BOOL____00595) (= Real____00596 0)) (=> (= Real____00596 0) (not BOOL____00595)))))
(assert (and (and (=> BOOL____00597 (= Real____00598 1)) (=> (= Real____00598 1) BOOL____00597)) (and (=> (not BOOL____00597) (= Real____00598 0)) (=> (= Real____00598 0) (not BOOL____00597)))))
(assert (and (and (=> BOOL____00599 (= Real____00600 1)) (=> (= Real____00600 1) BOOL____00599)) (and (=> (not BOOL____00599) (= Real____00600 0)) (=> (= Real____00600 0) (not BOOL____00599)))))
(assert (and (and (=> BOOL____00601 (= Real____00602 1)) (=> (= Real____00602 1) BOOL____00601)) (and (=> (not BOOL____00601) (= Real____00602 0)) (=> (= Real____00602 0) (not BOOL____00601)))))
(assert (and (and (=> BOOL____00603 (= Real____00604 1)) (=> (= Real____00604 1) BOOL____00603)) (and (=> (not BOOL____00603) (= Real____00604 0)) (=> (= Real____00604 0) (not BOOL____00603)))))
(assert (and (and (=> BOOL____00605 (= Real____00606 1)) (=> (= Real____00606 1) BOOL____00605)) (and (=> (not BOOL____00605) (= Real____00606 0)) (=> (= Real____00606 0) (not BOOL____00605)))))
(assert (and (and (=> BOOL____00607 (= Real____00608 1)) (=> (= Real____00608 1) BOOL____00607)) (and (=> (not BOOL____00607) (= Real____00608 0)) (=> (= Real____00608 0) (not BOOL____00607)))))
(assert (and (and (=> BOOL____00609 (= Real____00610 1)) (=> (= Real____00610 1) BOOL____00609)) (and (=> (not BOOL____00609) (= Real____00610 0)) (=> (= Real____00610 0) (not BOOL____00609)))))
(assert (and (and (=> BOOL____00611 (= Real____00612 1)) (=> (= Real____00612 1) BOOL____00611)) (and (=> (not BOOL____00611) (= Real____00612 0)) (=> (= Real____00612 0) (not BOOL____00611)))))
(assert (and (and (=> BOOL____00613 (= Real____00614 1)) (=> (= Real____00614 1) BOOL____00613)) (and (=> (not BOOL____00613) (= Real____00614 0)) (=> (= Real____00614 0) (not BOOL____00613)))))
(assert (and (and (=> BOOL____00615 (= Real____00616 1)) (=> (= Real____00616 1) BOOL____00615)) (and (=> (not BOOL____00615) (= Real____00616 0)) (=> (= Real____00616 0) (not BOOL____00615)))))
(assert (and (and (=> BOOL____00617 (= Real____00618 1)) (=> (= Real____00618 1) BOOL____00617)) (and (=> (not BOOL____00617) (= Real____00618 0)) (=> (= Real____00618 0) (not BOOL____00617)))))
(assert (and (and (=> BOOL____00619 (= Real____00620 1)) (=> (= Real____00620 1) BOOL____00619)) (and (=> (not BOOL____00619) (= Real____00620 0)) (=> (= Real____00620 0) (not BOOL____00619)))))
(assert (and (and (=> BOOL____00621 (= Real____00622 1)) (=> (= Real____00622 1) BOOL____00621)) (and (=> (not BOOL____00621) (= Real____00622 0)) (=> (= Real____00622 0) (not BOOL____00621)))))
(assert (and (and (=> BOOL____00623 (= Real____00624 1)) (=> (= Real____00624 1) BOOL____00623)) (and (=> (not BOOL____00623) (= Real____00624 0)) (=> (= Real____00624 0) (not BOOL____00623)))))
(assert (and (and (=> BOOL____00625 (= Real____00626 1)) (=> (= Real____00626 1) BOOL____00625)) (and (=> (not BOOL____00625) (= Real____00626 0)) (=> (= Real____00626 0) (not BOOL____00625)))))
(assert (and (and (=> BOOL____00627 (= Real____00628 1)) (=> (= Real____00628 1) BOOL____00627)) (and (=> (not BOOL____00627) (= Real____00628 0)) (=> (= Real____00628 0) (not BOOL____00627)))))
(assert (and (and (=> BOOL____00629 (= Real____00630 1)) (=> (= Real____00630 1) BOOL____00629)) (and (=> (not BOOL____00629) (= Real____00630 0)) (=> (= Real____00630 0) (not BOOL____00629)))))
(assert (and (and (=> BOOL____00631 (= Real____00632 1)) (=> (= Real____00632 1) BOOL____00631)) (and (=> (not BOOL____00631) (= Real____00632 0)) (=> (= Real____00632 0) (not BOOL____00631)))))
(assert (and (and (=> BOOL____00633 (= Real____00634 1)) (=> (= Real____00634 1) BOOL____00633)) (and (=> (not BOOL____00633) (= Real____00634 0)) (=> (= Real____00634 0) (not BOOL____00633)))))
(assert (and (and (=> BOOL____00635 (= Real____00636 1)) (=> (= Real____00636 1) BOOL____00635)) (and (=> (not BOOL____00635) (= Real____00636 0)) (=> (= Real____00636 0) (not BOOL____00635)))))
(assert (and (and (=> BOOL____00637 (= Real____00638 1)) (=> (= Real____00638 1) BOOL____00637)) (and (=> (not BOOL____00637) (= Real____00638 0)) (=> (= Real____00638 0) (not BOOL____00637)))))
(assert (and (and (=> BOOL____00639 (= Real____00640 1)) (=> (= Real____00640 1) BOOL____00639)) (and (=> (not BOOL____00639) (= Real____00640 0)) (=> (= Real____00640 0) (not BOOL____00639)))))
(assert (and (and (=> BOOL____00641 (= Real____00642 1)) (=> (= Real____00642 1) BOOL____00641)) (and (=> (not BOOL____00641) (= Real____00642 0)) (=> (= Real____00642 0) (not BOOL____00641)))))
(assert (and (and (=> BOOL____00643 (= Real____00644 1)) (=> (= Real____00644 1) BOOL____00643)) (and (=> (not BOOL____00643) (= Real____00644 0)) (=> (= Real____00644 0) (not BOOL____00643)))))
(assert (and (and (=> BOOL____00645 (= Real____00646 1)) (=> (= Real____00646 1) BOOL____00645)) (and (=> (not BOOL____00645) (= Real____00646 0)) (=> (= Real____00646 0) (not BOOL____00645)))))
(assert (and (and (=> BOOL____00647 (= Real____00648 1)) (=> (= Real____00648 1) BOOL____00647)) (and (=> (not BOOL____00647) (= Real____00648 0)) (=> (= Real____00648 0) (not BOOL____00647)))))
(assert (and (and (=> BOOL____00649 (= Real____00650 1)) (=> (= Real____00650 1) BOOL____00649)) (and (=> (not BOOL____00649) (= Real____00650 0)) (=> (= Real____00650 0) (not BOOL____00649)))))
(assert (and (and (=> BOOL____00651 (= Real____00652 1)) (=> (= Real____00652 1) BOOL____00651)) (and (=> (not BOOL____00651) (= Real____00652 0)) (=> (= Real____00652 0) (not BOOL____00651)))))
(assert (and (and (=> BOOL____00653 (= Real____00654 1)) (=> (= Real____00654 1) BOOL____00653)) (and (=> (not BOOL____00653) (= Real____00654 0)) (=> (= Real____00654 0) (not BOOL____00653)))))
(assert (and (and (=> BOOL____00655 (= Real____00656 1)) (=> (= Real____00656 1) BOOL____00655)) (and (=> (not BOOL____00655) (= Real____00656 0)) (=> (= Real____00656 0) (not BOOL____00655)))))
(assert (and (and (=> BOOL____00657 (= Real____00658 1)) (=> (= Real____00658 1) BOOL____00657)) (and (=> (not BOOL____00657) (= Real____00658 0)) (=> (= Real____00658 0) (not BOOL____00657)))))
(assert (and (and (=> BOOL____00659 (= Real____00660 1)) (=> (= Real____00660 1) BOOL____00659)) (and (=> (not BOOL____00659) (= Real____00660 0)) (=> (= Real____00660 0) (not BOOL____00659)))))
(assert (and (and (=> BOOL____00661 (= Real____00662 1)) (=> (= Real____00662 1) BOOL____00661)) (and (=> (not BOOL____00661) (= Real____00662 0)) (=> (= Real____00662 0) (not BOOL____00661)))))
(assert (and (and (=> BOOL____00663 (= Real____00664 1)) (=> (= Real____00664 1) BOOL____00663)) (and (=> (not BOOL____00663) (= Real____00664 0)) (=> (= Real____00664 0) (not BOOL____00663)))))
(assert (and (and (=> BOOL____00665 (= Real____00666 1)) (=> (= Real____00666 1) BOOL____00665)) (and (=> (not BOOL____00665) (= Real____00666 0)) (=> (= Real____00666 0) (not BOOL____00665)))))
(assert (and (and (=> BOOL____00667 (= Real____00668 1)) (=> (= Real____00668 1) BOOL____00667)) (and (=> (not BOOL____00667) (= Real____00668 0)) (=> (= Real____00668 0) (not BOOL____00667)))))
(assert (and (and (=> BOOL____00669 (= Real____00670 1)) (=> (= Real____00670 1) BOOL____00669)) (and (=> (not BOOL____00669) (= Real____00670 0)) (=> (= Real____00670 0) (not BOOL____00669)))))
(assert (and (and (=> BOOL____00671 (= Real____00672 1)) (=> (= Real____00672 1) BOOL____00671)) (and (=> (not BOOL____00671) (= Real____00672 0)) (=> (= Real____00672 0) (not BOOL____00671)))))
(assert (and (and (=> BOOL____00673 (= Real____00674 1)) (=> (= Real____00674 1) BOOL____00673)) (and (=> (not BOOL____00673) (= Real____00674 0)) (=> (= Real____00674 0) (not BOOL____00673)))))
(assert (and (and (=> BOOL____00675 (= Real____00676 1)) (=> (= Real____00676 1) BOOL____00675)) (and (=> (not BOOL____00675) (= Real____00676 0)) (=> (= Real____00676 0) (not BOOL____00675)))))
(assert (and (and (=> BOOL____00677 (= Real____00678 1)) (=> (= Real____00678 1) BOOL____00677)) (and (=> (not BOOL____00677) (= Real____00678 0)) (=> (= Real____00678 0) (not BOOL____00677)))))
(assert (and (and (=> BOOL____00679 (= Real____00680 1)) (=> (= Real____00680 1) BOOL____00679)) (and (=> (not BOOL____00679) (= Real____00680 0)) (=> (= Real____00680 0) (not BOOL____00679)))))
(assert (and (and (=> BOOL____00681 (= Real____00682 1)) (=> (= Real____00682 1) BOOL____00681)) (and (=> (not BOOL____00681) (= Real____00682 0)) (=> (= Real____00682 0) (not BOOL____00681)))))
(assert (and (and (=> BOOL____00683 (= Real____00684 1)) (=> (= Real____00684 1) BOOL____00683)) (and (=> (not BOOL____00683) (= Real____00684 0)) (=> (= Real____00684 0) (not BOOL____00683)))))
(assert (and (and (=> BOOL____00685 (= Real____00686 1)) (=> (= Real____00686 1) BOOL____00685)) (and (=> (not BOOL____00685) (= Real____00686 0)) (=> (= Real____00686 0) (not BOOL____00685)))))
(assert (and (and (=> BOOL____00687 (= Real____00688 1)) (=> (= Real____00688 1) BOOL____00687)) (and (=> (not BOOL____00687) (= Real____00688 0)) (=> (= Real____00688 0) (not BOOL____00687)))))
(assert (and (and (=> BOOL____00689 (= Real____00690 1)) (=> (= Real____00690 1) BOOL____00689)) (and (=> (not BOOL____00689) (= Real____00690 0)) (=> (= Real____00690 0) (not BOOL____00689)))))
(assert (and (and (=> BOOL____00691 (= Real____00692 1)) (=> (= Real____00692 1) BOOL____00691)) (and (=> (not BOOL____00691) (= Real____00692 0)) (=> (= Real____00692 0) (not BOOL____00691)))))
(assert (and (and (=> BOOL____00693 (= Real____00694 1)) (=> (= Real____00694 1) BOOL____00693)) (and (=> (not BOOL____00693) (= Real____00694 0)) (=> (= Real____00694 0) (not BOOL____00693)))))
(assert (and (and (=> BOOL____00695 (= Real____00696 1)) (=> (= Real____00696 1) BOOL____00695)) (and (=> (not BOOL____00695) (= Real____00696 0)) (=> (= Real____00696 0) (not BOOL____00695)))))
(assert (and (and (=> BOOL____00697 (= Real____00698 1)) (=> (= Real____00698 1) BOOL____00697)) (and (=> (not BOOL____00697) (= Real____00698 0)) (=> (= Real____00698 0) (not BOOL____00697)))))
(assert (and (and (=> BOOL____00699 (= Real____00700 1)) (=> (= Real____00700 1) BOOL____00699)) (and (=> (not BOOL____00699) (= Real____00700 0)) (=> (= Real____00700 0) (not BOOL____00699)))))
(assert (and (and (=> BOOL____00701 (= Real____00702 1)) (=> (= Real____00702 1) BOOL____00701)) (and (=> (not BOOL____00701) (= Real____00702 0)) (=> (= Real____00702 0) (not BOOL____00701)))))
(assert (and (and (=> BOOL____00703 (= Real____00704 1)) (=> (= Real____00704 1) BOOL____00703)) (and (=> (not BOOL____00703) (= Real____00704 0)) (=> (= Real____00704 0) (not BOOL____00703)))))
(assert (and (and (=> BOOL____00705 (= Real____00706 1)) (=> (= Real____00706 1) BOOL____00705)) (and (=> (not BOOL____00705) (= Real____00706 0)) (=> (= Real____00706 0) (not BOOL____00705)))))
(assert (and (and (=> BOOL____00707 (= Real____00708 1)) (=> (= Real____00708 1) BOOL____00707)) (and (=> (not BOOL____00707) (= Real____00708 0)) (=> (= Real____00708 0) (not BOOL____00707)))))
(assert (and (and (=> BOOL____00709 (= Real____00710 1)) (=> (= Real____00710 1) BOOL____00709)) (and (=> (not BOOL____00709) (= Real____00710 0)) (=> (= Real____00710 0) (not BOOL____00709)))))
(assert (and (and (=> BOOL____00711 (= Real____00712 1)) (=> (= Real____00712 1) BOOL____00711)) (and (=> (not BOOL____00711) (= Real____00712 0)) (=> (= Real____00712 0) (not BOOL____00711)))))
(assert (and (and (=> BOOL____00713 (= Real____00714 1)) (=> (= Real____00714 1) BOOL____00713)) (and (=> (not BOOL____00713) (= Real____00714 0)) (=> (= Real____00714 0) (not BOOL____00713)))))
(assert (and (and (=> BOOL____00715 (= Real____00716 1)) (=> (= Real____00716 1) BOOL____00715)) (and (=> (not BOOL____00715) (= Real____00716 0)) (=> (= Real____00716 0) (not BOOL____00715)))))
(assert (and (and (=> BOOL____00717 (= Real____00718 1)) (=> (= Real____00718 1) BOOL____00717)) (and (=> (not BOOL____00717) (= Real____00718 0)) (=> (= Real____00718 0) (not BOOL____00717)))))
(assert (and (and (=> BOOL____00719 (= Real____00720 1)) (=> (= Real____00720 1) BOOL____00719)) (and (=> (not BOOL____00719) (= Real____00720 0)) (=> (= Real____00720 0) (not BOOL____00719)))))
(assert (and (and (=> BOOL____00721 (= Real____00722 1)) (=> (= Real____00722 1) BOOL____00721)) (and (=> (not BOOL____00721) (= Real____00722 0)) (=> (= Real____00722 0) (not BOOL____00721)))))
(assert (and (and (=> BOOL____00723 (= Real____00724 1)) (=> (= Real____00724 1) BOOL____00723)) (and (=> (not BOOL____00723) (= Real____00724 0)) (=> (= Real____00724 0) (not BOOL____00723)))))
(assert (and (and (=> BOOL____00725 (= Real____00726 1)) (=> (= Real____00726 1) BOOL____00725)) (and (=> (not BOOL____00725) (= Real____00726 0)) (=> (= Real____00726 0) (not BOOL____00725)))))
(assert (and (and (=> BOOL____00727 (= Real____00728 1)) (=> (= Real____00728 1) BOOL____00727)) (and (=> (not BOOL____00727) (= Real____00728 0)) (=> (= Real____00728 0) (not BOOL____00727)))))
(assert (and (and (=> BOOL____00729 (= Real____00730 1)) (=> (= Real____00730 1) BOOL____00729)) (and (=> (not BOOL____00729) (= Real____00730 0)) (=> (= Real____00730 0) (not BOOL____00729)))))
(assert (and (and (=> BOOL____00731 (= Real____00732 1)) (=> (= Real____00732 1) BOOL____00731)) (and (=> (not BOOL____00731) (= Real____00732 0)) (=> (= Real____00732 0) (not BOOL____00731)))))
(assert (and (and (=> BOOL____00733 (= Real____00734 1)) (=> (= Real____00734 1) BOOL____00733)) (and (=> (not BOOL____00733) (= Real____00734 0)) (=> (= Real____00734 0) (not BOOL____00733)))))
(assert (and (and (=> BOOL____00735 (= Real____00736 1)) (=> (= Real____00736 1) BOOL____00735)) (and (=> (not BOOL____00735) (= Real____00736 0)) (=> (= Real____00736 0) (not BOOL____00735)))))
(assert (and (and (=> BOOL____00737 (= Real____00738 1)) (=> (= Real____00738 1) BOOL____00737)) (and (=> (not BOOL____00737) (= Real____00738 0)) (=> (= Real____00738 0) (not BOOL____00737)))))
(assert (and (and (=> BOOL____00739 (= Real____00740 1)) (=> (= Real____00740 1) BOOL____00739)) (and (=> (not BOOL____00739) (= Real____00740 0)) (=> (= Real____00740 0) (not BOOL____00739)))))
(assert (and (and (=> BOOL____00741 (= Real____00742 1)) (=> (= Real____00742 1) BOOL____00741)) (and (=> (not BOOL____00741) (= Real____00742 0)) (=> (= Real____00742 0) (not BOOL____00741)))))
(assert (and (and (=> BOOL____00743 (= Real____00744 1)) (=> (= Real____00744 1) BOOL____00743)) (and (=> (not BOOL____00743) (= Real____00744 0)) (=> (= Real____00744 0) (not BOOL____00743)))))
(assert (and (and (=> BOOL____00745 (= Real____00746 1)) (=> (= Real____00746 1) BOOL____00745)) (and (=> (not BOOL____00745) (= Real____00746 0)) (=> (= Real____00746 0) (not BOOL____00745)))))
(assert (and (and (=> BOOL____00747 (= Real____00748 1)) (=> (= Real____00748 1) BOOL____00747)) (and (=> (not BOOL____00747) (= Real____00748 0)) (=> (= Real____00748 0) (not BOOL____00747)))))
(assert (and (and (=> BOOL____00749 (= Real____00750 1)) (=> (= Real____00750 1) BOOL____00749)) (and (=> (not BOOL____00749) (= Real____00750 0)) (=> (= Real____00750 0) (not BOOL____00749)))))
(assert (and (and (=> BOOL____00751 (= Real____00752 1)) (=> (= Real____00752 1) BOOL____00751)) (and (=> (not BOOL____00751) (= Real____00752 0)) (=> (= Real____00752 0) (not BOOL____00751)))))
(assert (and (and (=> BOOL____00753 (= Real____00754 1)) (=> (= Real____00754 1) BOOL____00753)) (and (=> (not BOOL____00753) (= Real____00754 0)) (=> (= Real____00754 0) (not BOOL____00753)))))
(assert (and (and (=> BOOL____00755 (= Real____00756 1)) (=> (= Real____00756 1) BOOL____00755)) (and (=> (not BOOL____00755) (= Real____00756 0)) (=> (= Real____00756 0) (not BOOL____00755)))))
(assert (and (and (=> BOOL____00757 (= Real____00758 1)) (=> (= Real____00758 1) BOOL____00757)) (and (=> (not BOOL____00757) (= Real____00758 0)) (=> (= Real____00758 0) (not BOOL____00757)))))
(assert (and (and (=> BOOL____00759 (= Real____00760 1)) (=> (= Real____00760 1) BOOL____00759)) (and (=> (not BOOL____00759) (= Real____00760 0)) (=> (= Real____00760 0) (not BOOL____00759)))))
(assert (and (and (=> BOOL____00761 (= Real____00762 1)) (=> (= Real____00762 1) BOOL____00761)) (and (=> (not BOOL____00761) (= Real____00762 0)) (=> (= Real____00762 0) (not BOOL____00761)))))
(assert (and (and (=> BOOL____00763 (= Real____00764 1)) (=> (= Real____00764 1) BOOL____00763)) (and (=> (not BOOL____00763) (= Real____00764 0)) (=> (= Real____00764 0) (not BOOL____00763)))))
(assert (and (and (=> BOOL____00765 (= Real____00766 1)) (=> (= Real____00766 1) BOOL____00765)) (and (=> (not BOOL____00765) (= Real____00766 0)) (=> (= Real____00766 0) (not BOOL____00765)))))
(assert (and (and (=> BOOL____00767 (= Real____00768 1)) (=> (= Real____00768 1) BOOL____00767)) (and (=> (not BOOL____00767) (= Real____00768 0)) (=> (= Real____00768 0) (not BOOL____00767)))))
(assert (and (and (=> BOOL____00769 (= Real____00770 1)) (=> (= Real____00770 1) BOOL____00769)) (and (=> (not BOOL____00769) (= Real____00770 0)) (=> (= Real____00770 0) (not BOOL____00769)))))
(assert (and (and (=> BOOL____00771 (= Real____00772 1)) (=> (= Real____00772 1) BOOL____00771)) (and (=> (not BOOL____00771) (= Real____00772 0)) (=> (= Real____00772 0) (not BOOL____00771)))))
(assert (and (and (=> BOOL____00773 (= Real____00774 1)) (=> (= Real____00774 1) BOOL____00773)) (and (=> (not BOOL____00773) (= Real____00774 0)) (=> (= Real____00774 0) (not BOOL____00773)))))
(assert (and (and (=> BOOL____00775 (= Real____00776 1)) (=> (= Real____00776 1) BOOL____00775)) (and (=> (not BOOL____00775) (= Real____00776 0)) (=> (= Real____00776 0) (not BOOL____00775)))))
(assert (and (and (=> BOOL____00777 (= Real____00778 1)) (=> (= Real____00778 1) BOOL____00777)) (and (=> (not BOOL____00777) (= Real____00778 0)) (=> (= Real____00778 0) (not BOOL____00777)))))
(assert (and (and (=> BOOL____00779 (= Real____00780 1)) (=> (= Real____00780 1) BOOL____00779)) (and (=> (not BOOL____00779) (= Real____00780 0)) (=> (= Real____00780 0) (not BOOL____00779)))))
(assert (and (and (=> BOOL____00781 (= Real____00782 1)) (=> (= Real____00782 1) BOOL____00781)) (and (=> (not BOOL____00781) (= Real____00782 0)) (=> (= Real____00782 0) (not BOOL____00781)))))
(assert (and (and (=> BOOL____00783 (= Real____00784 1)) (=> (= Real____00784 1) BOOL____00783)) (and (=> (not BOOL____00783) (= Real____00784 0)) (=> (= Real____00784 0) (not BOOL____00783)))))
(assert (and (and (=> BOOL____00785 (= Real____00786 1)) (=> (= Real____00786 1) BOOL____00785)) (and (=> (not BOOL____00785) (= Real____00786 0)) (=> (= Real____00786 0) (not BOOL____00785)))))
(assert (and (and (=> BOOL____00787 (= Real____00788 1)) (=> (= Real____00788 1) BOOL____00787)) (and (=> (not BOOL____00787) (= Real____00788 0)) (=> (= Real____00788 0) (not BOOL____00787)))))
(assert (and (and (=> BOOL____00789 (= Real____00790 1)) (=> (= Real____00790 1) BOOL____00789)) (and (=> (not BOOL____00789) (= Real____00790 0)) (=> (= Real____00790 0) (not BOOL____00789)))))
(assert (and (and (=> BOOL____00791 (= Real____00792 1)) (=> (= Real____00792 1) BOOL____00791)) (and (=> (not BOOL____00791) (= Real____00792 0)) (=> (= Real____00792 0) (not BOOL____00791)))))
(assert (and (and (=> BOOL____00793 (= Real____00794 1)) (=> (= Real____00794 1) BOOL____00793)) (and (=> (not BOOL____00793) (= Real____00794 0)) (=> (= Real____00794 0) (not BOOL____00793)))))
(assert (and (and (=> BOOL____00795 (= Real____00796 1)) (=> (= Real____00796 1) BOOL____00795)) (and (=> (not BOOL____00795) (= Real____00796 0)) (=> (= Real____00796 0) (not BOOL____00795)))))
(assert (and (and (=> BOOL____00797 (= Real____00798 1)) (=> (= Real____00798 1) BOOL____00797)) (and (=> (not BOOL____00797) (= Real____00798 0)) (=> (= Real____00798 0) (not BOOL____00797)))))
(assert (and (and (=> BOOL____00799 (= Real____00800 1)) (=> (= Real____00800 1) BOOL____00799)) (and (=> (not BOOL____00799) (= Real____00800 0)) (=> (= Real____00800 0) (not BOOL____00799)))))
(assert (and (and (=> BOOL____00801 (= Real____00802 1)) (=> (= Real____00802 1) BOOL____00801)) (and (=> (not BOOL____00801) (= Real____00802 0)) (=> (= Real____00802 0) (not BOOL____00801)))))
(assert (and (and (=> BOOL____00803 (= Real____00804 1)) (=> (= Real____00804 1) BOOL____00803)) (and (=> (not BOOL____00803) (= Real____00804 0)) (=> (= Real____00804 0) (not BOOL____00803)))))
(assert (and (and (=> BOOL____00805 (= Real____00806 1)) (=> (= Real____00806 1) BOOL____00805)) (and (=> (not BOOL____00805) (= Real____00806 0)) (=> (= Real____00806 0) (not BOOL____00805)))))
(assert (and (and (=> BOOL____00807 (= Real____00808 1)) (=> (= Real____00808 1) BOOL____00807)) (and (=> (not BOOL____00807) (= Real____00808 0)) (=> (= Real____00808 0) (not BOOL____00807)))))
(assert (and (and (=> BOOL____00809 (= Real____00810 1)) (=> (= Real____00810 1) BOOL____00809)) (and (=> (not BOOL____00809) (= Real____00810 0)) (=> (= Real____00810 0) (not BOOL____00809)))))
(assert (and (and (=> BOOL____00811 (= Real____00812 1)) (=> (= Real____00812 1) BOOL____00811)) (and (=> (not BOOL____00811) (= Real____00812 0)) (=> (= Real____00812 0) (not BOOL____00811)))))
(assert (and (and (=> BOOL____00813 (= Real____00814 1)) (=> (= Real____00814 1) BOOL____00813)) (and (=> (not BOOL____00813) (= Real____00814 0)) (=> (= Real____00814 0) (not BOOL____00813)))))
(assert (and (and (=> BOOL____00815 (= Real____00816 1)) (=> (= Real____00816 1) BOOL____00815)) (and (=> (not BOOL____00815) (= Real____00816 0)) (=> (= Real____00816 0) (not BOOL____00815)))))
(assert (and (and (=> BOOL____00817 (= Real____00818 1)) (=> (= Real____00818 1) BOOL____00817)) (and (=> (not BOOL____00817) (= Real____00818 0)) (=> (= Real____00818 0) (not BOOL____00817)))))
(assert (and (and (=> BOOL____00819 (= Real____00820 1)) (=> (= Real____00820 1) BOOL____00819)) (and (=> (not BOOL____00819) (= Real____00820 0)) (=> (= Real____00820 0) (not BOOL____00819)))))
(assert (and (and (=> BOOL____00821 (= Real____00822 1)) (=> (= Real____00822 1) BOOL____00821)) (and (=> (not BOOL____00821) (= Real____00822 0)) (=> (= Real____00822 0) (not BOOL____00821)))))
(assert (and (and (=> BOOL____00823 (= Real____00824 1)) (=> (= Real____00824 1) BOOL____00823)) (and (=> (not BOOL____00823) (= Real____00824 0)) (=> (= Real____00824 0) (not BOOL____00823)))))
(assert (and (and (=> BOOL____00825 (= Real____00826 1)) (=> (= Real____00826 1) BOOL____00825)) (and (=> (not BOOL____00825) (= Real____00826 0)) (=> (= Real____00826 0) (not BOOL____00825)))))
(assert (and (and (=> BOOL____00827 (= Real____00828 1)) (=> (= Real____00828 1) BOOL____00827)) (and (=> (not BOOL____00827) (= Real____00828 0)) (=> (= Real____00828 0) (not BOOL____00827)))))
(assert (and (and (=> BOOL____00829 (= Real____00830 1)) (=> (= Real____00830 1) BOOL____00829)) (and (=> (not BOOL____00829) (= Real____00830 0)) (=> (= Real____00830 0) (not BOOL____00829)))))
(assert (and (and (=> BOOL____00831 (= Real____00832 1)) (=> (= Real____00832 1) BOOL____00831)) (and (=> (not BOOL____00831) (= Real____00832 0)) (=> (= Real____00832 0) (not BOOL____00831)))))
(assert (and (and (=> BOOL____00833 (= Real____00834 1)) (=> (= Real____00834 1) BOOL____00833)) (and (=> (not BOOL____00833) (= Real____00834 0)) (=> (= Real____00834 0) (not BOOL____00833)))))
(assert (and (and (=> BOOL____00835 (= Real____00836 1)) (=> (= Real____00836 1) BOOL____00835)) (and (=> (not BOOL____00835) (= Real____00836 0)) (=> (= Real____00836 0) (not BOOL____00835)))))
(assert (and (and (=> BOOL____00837 (= Real____00838 1)) (=> (= Real____00838 1) BOOL____00837)) (and (=> (not BOOL____00837) (= Real____00838 0)) (=> (= Real____00838 0) (not BOOL____00837)))))
(assert (and (and (=> BOOL____00839 (= Real____00840 1)) (=> (= Real____00840 1) BOOL____00839)) (and (=> (not BOOL____00839) (= Real____00840 0)) (=> (= Real____00840 0) (not BOOL____00839)))))
(assert (and (and (=> BOOL____00841 (= Real____00842 1)) (=> (= Real____00842 1) BOOL____00841)) (and (=> (not BOOL____00841) (= Real____00842 0)) (=> (= Real____00842 0) (not BOOL____00841)))))
(assert (and (and (=> BOOL____00843 (= Real____00844 1)) (=> (= Real____00844 1) BOOL____00843)) (and (=> (not BOOL____00843) (= Real____00844 0)) (=> (= Real____00844 0) (not BOOL____00843)))))
(assert (and (and (=> BOOL____00845 (= Real____00846 1)) (=> (= Real____00846 1) BOOL____00845)) (and (=> (not BOOL____00845) (= Real____00846 0)) (=> (= Real____00846 0) (not BOOL____00845)))))
(assert (and (and (=> BOOL____00847 (= Real____00848 1)) (=> (= Real____00848 1) BOOL____00847)) (and (=> (not BOOL____00847) (= Real____00848 0)) (=> (= Real____00848 0) (not BOOL____00847)))))
(assert (and (and (=> BOOL____00849 (= Real____00850 1)) (=> (= Real____00850 1) BOOL____00849)) (and (=> (not BOOL____00849) (= Real____00850 0)) (=> (= Real____00850 0) (not BOOL____00849)))))
(assert (and (and (=> BOOL____00851 (= Real____00852 1)) (=> (= Real____00852 1) BOOL____00851)) (and (=> (not BOOL____00851) (= Real____00852 0)) (=> (= Real____00852 0) (not BOOL____00851)))))
(assert (and (and (=> BOOL____00853 (= Real____00854 1)) (=> (= Real____00854 1) BOOL____00853)) (and (=> (not BOOL____00853) (= Real____00854 0)) (=> (= Real____00854 0) (not BOOL____00853)))))
(assert (and (and (=> BOOL____00855 (= Real____00856 1)) (=> (= Real____00856 1) BOOL____00855)) (and (=> (not BOOL____00855) (= Real____00856 0)) (=> (= Real____00856 0) (not BOOL____00855)))))
(assert (and (and (=> BOOL____00857 (= Real____00858 1)) (=> (= Real____00858 1) BOOL____00857)) (and (=> (not BOOL____00857) (= Real____00858 0)) (=> (= Real____00858 0) (not BOOL____00857)))))
(assert (and (and (=> BOOL____00859 (= Real____00860 1)) (=> (= Real____00860 1) BOOL____00859)) (and (=> (not BOOL____00859) (= Real____00860 0)) (=> (= Real____00860 0) (not BOOL____00859)))))
(assert (and (and (=> BOOL____00861 (= Real____00862 1)) (=> (= Real____00862 1) BOOL____00861)) (and (=> (not BOOL____00861) (= Real____00862 0)) (=> (= Real____00862 0) (not BOOL____00861)))))
(assert (and (and (=> BOOL____00863 (= Real____00864 1)) (=> (= Real____00864 1) BOOL____00863)) (and (=> (not BOOL____00863) (= Real____00864 0)) (=> (= Real____00864 0) (not BOOL____00863)))))
(assert (and (and (=> BOOL____00865 (= Real____00866 1)) (=> (= Real____00866 1) BOOL____00865)) (and (=> (not BOOL____00865) (= Real____00866 0)) (=> (= Real____00866 0) (not BOOL____00865)))))
(assert (and (and (=> BOOL____00867 (= Real____00868 1)) (=> (= Real____00868 1) BOOL____00867)) (and (=> (not BOOL____00867) (= Real____00868 0)) (=> (= Real____00868 0) (not BOOL____00867)))))
(assert (and (and (=> BOOL____00869 (= Real____00870 1)) (=> (= Real____00870 1) BOOL____00869)) (and (=> (not BOOL____00869) (= Real____00870 0)) (=> (= Real____00870 0) (not BOOL____00869)))))
(assert (and (and (=> BOOL____00871 (= Real____00872 1)) (=> (= Real____00872 1) BOOL____00871)) (and (=> (not BOOL____00871) (= Real____00872 0)) (=> (= Real____00872 0) (not BOOL____00871)))))
(assert (and (and (=> BOOL____00873 (= Real____00874 1)) (=> (= Real____00874 1) BOOL____00873)) (and (=> (not BOOL____00873) (= Real____00874 0)) (=> (= Real____00874 0) (not BOOL____00873)))))
(assert (and (and (=> BOOL____00875 (= Real____00876 1)) (=> (= Real____00876 1) BOOL____00875)) (and (=> (not BOOL____00875) (= Real____00876 0)) (=> (= Real____00876 0) (not BOOL____00875)))))
(assert (and (and (=> BOOL____00877 (= Real____00878 1)) (=> (= Real____00878 1) BOOL____00877)) (and (=> (not BOOL____00877) (= Real____00878 0)) (=> (= Real____00878 0) (not BOOL____00877)))))
(assert (and (and (=> BOOL____00879 (= Real____00880 1)) (=> (= Real____00880 1) BOOL____00879)) (and (=> (not BOOL____00879) (= Real____00880 0)) (=> (= Real____00880 0) (not BOOL____00879)))))
(assert (and (and (=> BOOL____00881 (= Real____00882 1)) (=> (= Real____00882 1) BOOL____00881)) (and (=> (not BOOL____00881) (= Real____00882 0)) (=> (= Real____00882 0) (not BOOL____00881)))))
(assert (and (and (=> BOOL____00883 (= Real____00884 1)) (=> (= Real____00884 1) BOOL____00883)) (and (=> (not BOOL____00883) (= Real____00884 0)) (=> (= Real____00884 0) (not BOOL____00883)))))
(assert (and (and (=> BOOL____00885 (= Real____00886 1)) (=> (= Real____00886 1) BOOL____00885)) (and (=> (not BOOL____00885) (= Real____00886 0)) (=> (= Real____00886 0) (not BOOL____00885)))))
(assert (and (and (=> BOOL____00887 (= Real____00888 1)) (=> (= Real____00888 1) BOOL____00887)) (and (=> (not BOOL____00887) (= Real____00888 0)) (=> (= Real____00888 0) (not BOOL____00887)))))
(assert (and (and (=> BOOL____00889 (= Real____00890 1)) (=> (= Real____00890 1) BOOL____00889)) (and (=> (not BOOL____00889) (= Real____00890 0)) (=> (= Real____00890 0) (not BOOL____00889)))))
(assert (and (and (=> BOOL____00891 (= Real____00892 1)) (=> (= Real____00892 1) BOOL____00891)) (and (=> (not BOOL____00891) (= Real____00892 0)) (=> (= Real____00892 0) (not BOOL____00891)))))
(assert (and (and (=> BOOL____00893 (= Real____00894 1)) (=> (= Real____00894 1) BOOL____00893)) (and (=> (not BOOL____00893) (= Real____00894 0)) (=> (= Real____00894 0) (not BOOL____00893)))))
(assert (and (and (=> BOOL____00895 (= Real____00896 1)) (=> (= Real____00896 1) BOOL____00895)) (and (=> (not BOOL____00895) (= Real____00896 0)) (=> (= Real____00896 0) (not BOOL____00895)))))
(assert (and (and (=> BOOL____00897 (= Real____00898 1)) (=> (= Real____00898 1) BOOL____00897)) (and (=> (not BOOL____00897) (= Real____00898 0)) (=> (= Real____00898 0) (not BOOL____00897)))))
(assert (and (and (=> BOOL____00899 (= Real____00900 1)) (=> (= Real____00900 1) BOOL____00899)) (and (=> (not BOOL____00899) (= Real____00900 0)) (=> (= Real____00900 0) (not BOOL____00899)))))
(assert (and (and (=> BOOL____00901 (= x__ARRAY__1 1)) (=> (= x__ARRAY__1 1) BOOL____00901)) (and (=> (not BOOL____00901) (= x__ARRAY__1 0)) (=> (= x__ARRAY__1 0) (not BOOL____00901)))))
(assert (and (and (=> BOOL____00903 (= x__ARRAY__2 1)) (=> (= x__ARRAY__2 1) BOOL____00903)) (and (=> (not BOOL____00903) (= x__ARRAY__2 0)) (=> (= x__ARRAY__2 0) (not BOOL____00903)))))
(assert (and (and (=> BOOL____00905 (= x__ARRAY__3 1)) (=> (= x__ARRAY__3 1) BOOL____00905)) (and (=> (not BOOL____00905) (= x__ARRAY__3 0)) (=> (= x__ARRAY__3 0) (not BOOL____00905)))))
(assert (and (and (=> BOOL____00907 (= x__ARRAY__4 1)) (=> (= x__ARRAY__4 1) BOOL____00907)) (and (=> (not BOOL____00907) (= x__ARRAY__4 0)) (=> (= x__ARRAY__4 0) (not BOOL____00907)))))
(assert (and (and (=> BOOL____00909 (= x__ARRAY__5 1)) (=> (= x__ARRAY__5 1) BOOL____00909)) (and (=> (not BOOL____00909) (= x__ARRAY__5 0)) (=> (= x__ARRAY__5 0) (not BOOL____00909)))))
(assert (and (and (=> BOOL____00911 (= x__ARRAY__6 1)) (=> (= x__ARRAY__6 1) BOOL____00911)) (and (=> (not BOOL____00911) (= x__ARRAY__6 0)) (=> (= x__ARRAY__6 0) (not BOOL____00911)))))
(assert (and (and (=> BOOL____00913 (= x__ARRAY__7 1)) (=> (= x__ARRAY__7 1) BOOL____00913)) (and (=> (not BOOL____00913) (= x__ARRAY__7 0)) (=> (= x__ARRAY__7 0) (not BOOL____00913)))))
(assert (and (and (=> BOOL____00915 (= x__ARRAY__8 1)) (=> (= x__ARRAY__8 1) BOOL____00915)) (and (=> (not BOOL____00915) (= x__ARRAY__8 0)) (=> (= x__ARRAY__8 0) (not BOOL____00915)))))
(assert (and (and (=> BOOL____00917 (= x__ARRAY__9 1)) (=> (= x__ARRAY__9 1) BOOL____00917)) (and (=> (not BOOL____00917) (= x__ARRAY__9 0)) (=> (= x__ARRAY__9 0) (not BOOL____00917)))))
(assert (and (and (=> BOOL____00919 (= x__ARRAY__10 1)) (=> (= x__ARRAY__10 1) BOOL____00919)) (and (=> (not BOOL____00919) (= x__ARRAY__10 0)) (=> (= x__ARRAY__10 0) (not BOOL____00919)))))
(assert (and (and (=> BOOL____00921 (= x__ARRAY__11 1)) (=> (= x__ARRAY__11 1) BOOL____00921)) (and (=> (not BOOL____00921) (= x__ARRAY__11 0)) (=> (= x__ARRAY__11 0) (not BOOL____00921)))))
(assert (and (and (=> BOOL____00923 (= x__ARRAY__12 1)) (=> (= x__ARRAY__12 1) BOOL____00923)) (and (=> (not BOOL____00923) (= x__ARRAY__12 0)) (=> (= x__ARRAY__12 0) (not BOOL____00923)))))
(assert (and (and (=> BOOL____00925 (= x__ARRAY__13 1)) (=> (= x__ARRAY__13 1) BOOL____00925)) (and (=> (not BOOL____00925) (= x__ARRAY__13 0)) (=> (= x__ARRAY__13 0) (not BOOL____00925)))))
(assert (and (and (=> BOOL____00927 (= x__ARRAY__14 1)) (=> (= x__ARRAY__14 1) BOOL____00927)) (and (=> (not BOOL____00927) (= x__ARRAY__14 0)) (=> (= x__ARRAY__14 0) (not BOOL____00927)))))
(assert (and (and (=> BOOL____00929 (= x__ARRAY__15 1)) (=> (= x__ARRAY__15 1) BOOL____00929)) (and (=> (not BOOL____00929) (= x__ARRAY__15 0)) (=> (= x__ARRAY__15 0) (not BOOL____00929)))))
(assert (and (and (=> BOOL____00931 (= x__ARRAY__16 1)) (=> (= x__ARRAY__16 1) BOOL____00931)) (and (=> (not BOOL____00931) (= x__ARRAY__16 0)) (=> (= x__ARRAY__16 0) (not BOOL____00931)))))
(assert (and (and (=> BOOL____00933 (= x__ARRAY__17 1)) (=> (= x__ARRAY__17 1) BOOL____00933)) (and (=> (not BOOL____00933) (= x__ARRAY__17 0)) (=> (= x__ARRAY__17 0) (not BOOL____00933)))))
(assert (and (and (=> BOOL____00935 (= x__ARRAY__18 1)) (=> (= x__ARRAY__18 1) BOOL____00935)) (and (=> (not BOOL____00935) (= x__ARRAY__18 0)) (=> (= x__ARRAY__18 0) (not BOOL____00935)))))
(assert (and (and (=> BOOL____00937 (= x__ARRAY__19 1)) (=> (= x__ARRAY__19 1) BOOL____00937)) (and (=> (not BOOL____00937) (= x__ARRAY__19 0)) (=> (= x__ARRAY__19 0) (not BOOL____00937)))))
(assert (and (and (=> BOOL____00939 (= x__ARRAY__20 1)) (=> (= x__ARRAY__20 1) BOOL____00939)) (and (=> (not BOOL____00939) (= x__ARRAY__20 0)) (=> (= x__ARRAY__20 0) (not BOOL____00939)))))
(assert (and (and (=> BOOL____00941 (= x__ARRAY__21 1)) (=> (= x__ARRAY__21 1) BOOL____00941)) (and (=> (not BOOL____00941) (= x__ARRAY__21 0)) (=> (= x__ARRAY__21 0) (not BOOL____00941)))))
(assert (and (and (=> BOOL____00943 (= x__ARRAY__22 1)) (=> (= x__ARRAY__22 1) BOOL____00943)) (and (=> (not BOOL____00943) (= x__ARRAY__22 0)) (=> (= x__ARRAY__22 0) (not BOOL____00943)))))
(assert (and (and (=> BOOL____00945 (= x__ARRAY__23 1)) (=> (= x__ARRAY__23 1) BOOL____00945)) (and (=> (not BOOL____00945) (= x__ARRAY__23 0)) (=> (= x__ARRAY__23 0) (not BOOL____00945)))))
(assert (and (and (=> BOOL____00947 (= x__ARRAY__24 1)) (=> (= x__ARRAY__24 1) BOOL____00947)) (and (=> (not BOOL____00947) (= x__ARRAY__24 0)) (=> (= x__ARRAY__24 0) (not BOOL____00947)))))
(assert (and (and (=> BOOL____00949 (= x__ARRAY__25 1)) (=> (= x__ARRAY__25 1) BOOL____00949)) (and (=> (not BOOL____00949) (= x__ARRAY__25 0)) (=> (= x__ARRAY__25 0) (not BOOL____00949)))))
(assert (and (and (=> BOOL____00951 (= x__ARRAY__26 1)) (=> (= x__ARRAY__26 1) BOOL____00951)) (and (=> (not BOOL____00951) (= x__ARRAY__26 0)) (=> (= x__ARRAY__26 0) (not BOOL____00951)))))
(assert (and (and (=> BOOL____00953 (= x__ARRAY__27 1)) (=> (= x__ARRAY__27 1) BOOL____00953)) (and (=> (not BOOL____00953) (= x__ARRAY__27 0)) (=> (= x__ARRAY__27 0) (not BOOL____00953)))))
(assert (and (and (=> BOOL____00955 (= x__ARRAY__28 1)) (=> (= x__ARRAY__28 1) BOOL____00955)) (and (=> (not BOOL____00955) (= x__ARRAY__28 0)) (=> (= x__ARRAY__28 0) (not BOOL____00955)))))
(assert (and (and (=> BOOL____00957 (= x__ARRAY__29 1)) (=> (= x__ARRAY__29 1) BOOL____00957)) (and (=> (not BOOL____00957) (= x__ARRAY__29 0)) (=> (= x__ARRAY__29 0) (not BOOL____00957)))))
(assert (and (and (=> BOOL____00959 (= x__ARRAY__30 1)) (=> (= x__ARRAY__30 1) BOOL____00959)) (and (=> (not BOOL____00959) (= x__ARRAY__30 0)) (=> (= x__ARRAY__30 0) (not BOOL____00959)))))
(assert (and (and (=> BOOL____00961 (= x__ARRAY__31 1)) (=> (= x__ARRAY__31 1) BOOL____00961)) (and (=> (not BOOL____00961) (= x__ARRAY__31 0)) (=> (= x__ARRAY__31 0) (not BOOL____00961)))))
(assert (and (and (=> BOOL____00963 (= x__ARRAY__32 1)) (=> (= x__ARRAY__32 1) BOOL____00963)) (and (=> (not BOOL____00963) (= x__ARRAY__32 0)) (=> (= x__ARRAY__32 0) (not BOOL____00963)))))
(assert (and (and (=> BOOL____00965 (= x__ARRAY__33 1)) (=> (= x__ARRAY__33 1) BOOL____00965)) (and (=> (not BOOL____00965) (= x__ARRAY__33 0)) (=> (= x__ARRAY__33 0) (not BOOL____00965)))))
(assert (and (and (=> BOOL____00967 (= x__ARRAY__34 1)) (=> (= x__ARRAY__34 1) BOOL____00967)) (and (=> (not BOOL____00967) (= x__ARRAY__34 0)) (=> (= x__ARRAY__34 0) (not BOOL____00967)))))
(assert (and (and (=> BOOL____00969 (= x__ARRAY__35 1)) (=> (= x__ARRAY__35 1) BOOL____00969)) (and (=> (not BOOL____00969) (= x__ARRAY__35 0)) (=> (= x__ARRAY__35 0) (not BOOL____00969)))))
(assert (and (and (=> BOOL____00971 (= x__ARRAY__36 1)) (=> (= x__ARRAY__36 1) BOOL____00971)) (and (=> (not BOOL____00971) (= x__ARRAY__36 0)) (=> (= x__ARRAY__36 0) (not BOOL____00971)))))
(assert (and (and (=> BOOL____00973 (= x__ARRAY__37 1)) (=> (= x__ARRAY__37 1) BOOL____00973)) (and (=> (not BOOL____00973) (= x__ARRAY__37 0)) (=> (= x__ARRAY__37 0) (not BOOL____00973)))))
(assert (and (and (=> BOOL____00975 (= x__ARRAY__38 1)) (=> (= x__ARRAY__38 1) BOOL____00975)) (and (=> (not BOOL____00975) (= x__ARRAY__38 0)) (=> (= x__ARRAY__38 0) (not BOOL____00975)))))
(assert (and (and (=> BOOL____00977 (= x__ARRAY__39 1)) (=> (= x__ARRAY__39 1) BOOL____00977)) (and (=> (not BOOL____00977) (= x__ARRAY__39 0)) (=> (= x__ARRAY__39 0) (not BOOL____00977)))))
(assert (and (and (=> BOOL____00979 (= x__ARRAY__40 1)) (=> (= x__ARRAY__40 1) BOOL____00979)) (and (=> (not BOOL____00979) (= x__ARRAY__40 0)) (=> (= x__ARRAY__40 0) (not BOOL____00979)))))
(assert (and (and (=> BOOL____00981 (= x__ARRAY__41 1)) (=> (= x__ARRAY__41 1) BOOL____00981)) (and (=> (not BOOL____00981) (= x__ARRAY__41 0)) (=> (= x__ARRAY__41 0) (not BOOL____00981)))))
(assert (and (and (=> BOOL____00983 (= x__ARRAY__42 1)) (=> (= x__ARRAY__42 1) BOOL____00983)) (and (=> (not BOOL____00983) (= x__ARRAY__42 0)) (=> (= x__ARRAY__42 0) (not BOOL____00983)))))
(assert (and (and (=> BOOL____00985 (= x__ARRAY__43 1)) (=> (= x__ARRAY__43 1) BOOL____00985)) (and (=> (not BOOL____00985) (= x__ARRAY__43 0)) (=> (= x__ARRAY__43 0) (not BOOL____00985)))))
(assert (and (and (=> BOOL____00987 (= x__ARRAY__44 1)) (=> (= x__ARRAY__44 1) BOOL____00987)) (and (=> (not BOOL____00987) (= x__ARRAY__44 0)) (=> (= x__ARRAY__44 0) (not BOOL____00987)))))
(assert (and (and (=> BOOL____00989 (= x__ARRAY__45 1)) (=> (= x__ARRAY__45 1) BOOL____00989)) (and (=> (not BOOL____00989) (= x__ARRAY__45 0)) (=> (= x__ARRAY__45 0) (not BOOL____00989)))))
(assert (and (and (=> BOOL____00991 (= x__ARRAY__46 1)) (=> (= x__ARRAY__46 1) BOOL____00991)) (and (=> (not BOOL____00991) (= x__ARRAY__46 0)) (=> (= x__ARRAY__46 0) (not BOOL____00991)))))
(assert (and (and (=> BOOL____00993 (= x__ARRAY__47 1)) (=> (= x__ARRAY__47 1) BOOL____00993)) (and (=> (not BOOL____00993) (= x__ARRAY__47 0)) (=> (= x__ARRAY__47 0) (not BOOL____00993)))))
(assert (and (and (=> BOOL____00995 (= x__ARRAY__48 1)) (=> (= x__ARRAY__48 1) BOOL____00995)) (and (=> (not BOOL____00995) (= x__ARRAY__48 0)) (=> (= x__ARRAY__48 0) (not BOOL____00995)))))
(assert (and (and (=> BOOL____00997 (= x__ARRAY__49 1)) (=> (= x__ARRAY__49 1) BOOL____00997)) (and (=> (not BOOL____00997) (= x__ARRAY__49 0)) (=> (= x__ARRAY__49 0) (not BOOL____00997)))))
(assert (and (and (=> BOOL____00999 (= x__ARRAY__50 1)) (=> (= x__ARRAY__50 1) BOOL____00999)) (and (=> (not BOOL____00999) (= x__ARRAY__50 0)) (=> (= x__ARRAY__50 0) (not BOOL____00999)))))
(assert (and (and (=> BOOL____01001 (= x__ARRAY__51 1)) (=> (= x__ARRAY__51 1) BOOL____01001)) (and (=> (not BOOL____01001) (= x__ARRAY__51 0)) (=> (= x__ARRAY__51 0) (not BOOL____01001)))))
(assert (and (and (=> BOOL____01003 (= x__ARRAY__52 1)) (=> (= x__ARRAY__52 1) BOOL____01003)) (and (=> (not BOOL____01003) (= x__ARRAY__52 0)) (=> (= x__ARRAY__52 0) (not BOOL____01003)))))
(assert (and (and (=> BOOL____01005 (= x__ARRAY__53 1)) (=> (= x__ARRAY__53 1) BOOL____01005)) (and (=> (not BOOL____01005) (= x__ARRAY__53 0)) (=> (= x__ARRAY__53 0) (not BOOL____01005)))))
(assert (and (and (=> BOOL____01007 (= x__ARRAY__54 1)) (=> (= x__ARRAY__54 1) BOOL____01007)) (and (=> (not BOOL____01007) (= x__ARRAY__54 0)) (=> (= x__ARRAY__54 0) (not BOOL____01007)))))
(assert (and (and (=> BOOL____01009 (= x__ARRAY__55 1)) (=> (= x__ARRAY__55 1) BOOL____01009)) (and (=> (not BOOL____01009) (= x__ARRAY__55 0)) (=> (= x__ARRAY__55 0) (not BOOL____01009)))))
(assert (and (and (=> BOOL____01011 (= x__ARRAY__56 1)) (=> (= x__ARRAY__56 1) BOOL____01011)) (and (=> (not BOOL____01011) (= x__ARRAY__56 0)) (=> (= x__ARRAY__56 0) (not BOOL____01011)))))
(assert (and (and (=> BOOL____01013 (= x__ARRAY__57 1)) (=> (= x__ARRAY__57 1) BOOL____01013)) (and (=> (not BOOL____01013) (= x__ARRAY__57 0)) (=> (= x__ARRAY__57 0) (not BOOL____01013)))))
(assert (and (and (=> BOOL____01015 (= x__ARRAY__58 1)) (=> (= x__ARRAY__58 1) BOOL____01015)) (and (=> (not BOOL____01015) (= x__ARRAY__58 0)) (=> (= x__ARRAY__58 0) (not BOOL____01015)))))
(assert (and (and (=> BOOL____01017 (= x__ARRAY__59 1)) (=> (= x__ARRAY__59 1) BOOL____01017)) (and (=> (not BOOL____01017) (= x__ARRAY__59 0)) (=> (= x__ARRAY__59 0) (not BOOL____01017)))))
(assert (and (and (=> BOOL____01019 (= x__ARRAY__60 1)) (=> (= x__ARRAY__60 1) BOOL____01019)) (and (=> (not BOOL____01019) (= x__ARRAY__60 0)) (=> (= x__ARRAY__60 0) (not BOOL____01019)))))
(assert (and (and (=> BOOL____01021 (= x__ARRAY__61 1)) (=> (= x__ARRAY__61 1) BOOL____01021)) (and (=> (not BOOL____01021) (= x__ARRAY__61 0)) (=> (= x__ARRAY__61 0) (not BOOL____01021)))))
(assert (and (and (=> BOOL____01023 (= x__ARRAY__62 1)) (=> (= x__ARRAY__62 1) BOOL____01023)) (and (=> (not BOOL____01023) (= x__ARRAY__62 0)) (=> (= x__ARRAY__62 0) (not BOOL____01023)))))
(assert (and (and (=> BOOL____01025 (= x__ARRAY__63 1)) (=> (= x__ARRAY__63 1) BOOL____01025)) (and (=> (not BOOL____01025) (= x__ARRAY__63 0)) (=> (= x__ARRAY__63 0) (not BOOL____01025)))))
(assert (and (and (=> BOOL____01027 (= x__ARRAY__64 1)) (=> (= x__ARRAY__64 1) BOOL____01027)) (and (=> (not BOOL____01027) (= x__ARRAY__64 0)) (=> (= x__ARRAY__64 0) (not BOOL____01027)))))
(assert (and (and (=> BOOL____01029 (= x__ARRAY__65 1)) (=> (= x__ARRAY__65 1) BOOL____01029)) (and (=> (not BOOL____01029) (= x__ARRAY__65 0)) (=> (= x__ARRAY__65 0) (not BOOL____01029)))))
(assert (and (and (=> BOOL____01031 (= x__ARRAY__66 1)) (=> (= x__ARRAY__66 1) BOOL____01031)) (and (=> (not BOOL____01031) (= x__ARRAY__66 0)) (=> (= x__ARRAY__66 0) (not BOOL____01031)))))
(assert (and (and (=> BOOL____01033 (= x__ARRAY__67 1)) (=> (= x__ARRAY__67 1) BOOL____01033)) (and (=> (not BOOL____01033) (= x__ARRAY__67 0)) (=> (= x__ARRAY__67 0) (not BOOL____01033)))))
(assert (and (and (=> BOOL____01035 (= x__ARRAY__68 1)) (=> (= x__ARRAY__68 1) BOOL____01035)) (and (=> (not BOOL____01035) (= x__ARRAY__68 0)) (=> (= x__ARRAY__68 0) (not BOOL____01035)))))
(assert (and (and (=> BOOL____01037 (= x__ARRAY__69 1)) (=> (= x__ARRAY__69 1) BOOL____01037)) (and (=> (not BOOL____01037) (= x__ARRAY__69 0)) (=> (= x__ARRAY__69 0) (not BOOL____01037)))))
(assert (and (and (=> BOOL____01039 (= x__ARRAY__70 1)) (=> (= x__ARRAY__70 1) BOOL____01039)) (and (=> (not BOOL____01039) (= x__ARRAY__70 0)) (=> (= x__ARRAY__70 0) (not BOOL____01039)))))
(assert (and (and (=> BOOL____01041 (= x__ARRAY__71 1)) (=> (= x__ARRAY__71 1) BOOL____01041)) (and (=> (not BOOL____01041) (= x__ARRAY__71 0)) (=> (= x__ARRAY__71 0) (not BOOL____01041)))))
(assert (and (and (=> BOOL____01043 (= x__ARRAY__72 1)) (=> (= x__ARRAY__72 1) BOOL____01043)) (and (=> (not BOOL____01043) (= x__ARRAY__72 0)) (=> (= x__ARRAY__72 0) (not BOOL____01043)))))
(assert (and (and (=> BOOL____01045 (= x__ARRAY__73 1)) (=> (= x__ARRAY__73 1) BOOL____01045)) (and (=> (not BOOL____01045) (= x__ARRAY__73 0)) (=> (= x__ARRAY__73 0) (not BOOL____01045)))))
(assert (and (and (=> BOOL____01047 (= x__ARRAY__74 1)) (=> (= x__ARRAY__74 1) BOOL____01047)) (and (=> (not BOOL____01047) (= x__ARRAY__74 0)) (=> (= x__ARRAY__74 0) (not BOOL____01047)))))
(assert (and (and (=> BOOL____01049 (= x__ARRAY__75 1)) (=> (= x__ARRAY__75 1) BOOL____01049)) (and (=> (not BOOL____01049) (= x__ARRAY__75 0)) (=> (= x__ARRAY__75 0) (not BOOL____01049)))))
(assert (and (and (=> BOOL____01051 (= x__ARRAY__76 1)) (=> (= x__ARRAY__76 1) BOOL____01051)) (and (=> (not BOOL____01051) (= x__ARRAY__76 0)) (=> (= x__ARRAY__76 0) (not BOOL____01051)))))
(assert (and (and (=> BOOL____01053 (= x__ARRAY__77 1)) (=> (= x__ARRAY__77 1) BOOL____01053)) (and (=> (not BOOL____01053) (= x__ARRAY__77 0)) (=> (= x__ARRAY__77 0) (not BOOL____01053)))))
(assert (and (and (=> BOOL____01055 (= x__ARRAY__78 1)) (=> (= x__ARRAY__78 1) BOOL____01055)) (and (=> (not BOOL____01055) (= x__ARRAY__78 0)) (=> (= x__ARRAY__78 0) (not BOOL____01055)))))
(assert (and (and (=> BOOL____01057 (= x__ARRAY__79 1)) (=> (= x__ARRAY__79 1) BOOL____01057)) (and (=> (not BOOL____01057) (= x__ARRAY__79 0)) (=> (= x__ARRAY__79 0) (not BOOL____01057)))))
(assert (and (and (=> BOOL____01059 (= x__ARRAY__80 1)) (=> (= x__ARRAY__80 1) BOOL____01059)) (and (=> (not BOOL____01059) (= x__ARRAY__80 0)) (=> (= x__ARRAY__80 0) (not BOOL____01059)))))
(assert (and (and (=> BOOL____01061 (= x__ARRAY__81 1)) (=> (= x__ARRAY__81 1) BOOL____01061)) (and (=> (not BOOL____01061) (= x__ARRAY__81 0)) (=> (= x__ARRAY__81 0) (not BOOL____01061)))))
(assert (and (and (=> BOOL____01063 (= x__ARRAY__82 1)) (=> (= x__ARRAY__82 1) BOOL____01063)) (and (=> (not BOOL____01063) (= x__ARRAY__82 0)) (=> (= x__ARRAY__82 0) (not BOOL____01063)))))
(assert (and (and (=> BOOL____01065 (= x__ARRAY__83 1)) (=> (= x__ARRAY__83 1) BOOL____01065)) (and (=> (not BOOL____01065) (= x__ARRAY__83 0)) (=> (= x__ARRAY__83 0) (not BOOL____01065)))))
(assert (and (and (=> BOOL____01067 (= x__ARRAY__84 1)) (=> (= x__ARRAY__84 1) BOOL____01067)) (and (=> (not BOOL____01067) (= x__ARRAY__84 0)) (=> (= x__ARRAY__84 0) (not BOOL____01067)))))
(assert (and (and (=> BOOL____01069 (= x__ARRAY__85 1)) (=> (= x__ARRAY__85 1) BOOL____01069)) (and (=> (not BOOL____01069) (= x__ARRAY__85 0)) (=> (= x__ARRAY__85 0) (not BOOL____01069)))))
(assert (and (and (=> BOOL____01071 (= x__ARRAY__86 1)) (=> (= x__ARRAY__86 1) BOOL____01071)) (and (=> (not BOOL____01071) (= x__ARRAY__86 0)) (=> (= x__ARRAY__86 0) (not BOOL____01071)))))
(assert (and (and (=> BOOL____01073 (= x__ARRAY__87 1)) (=> (= x__ARRAY__87 1) BOOL____01073)) (and (=> (not BOOL____01073) (= x__ARRAY__87 0)) (=> (= x__ARRAY__87 0) (not BOOL____01073)))))
(assert (and (and (=> BOOL____01075 (= x__ARRAY__88 1)) (=> (= x__ARRAY__88 1) BOOL____01075)) (and (=> (not BOOL____01075) (= x__ARRAY__88 0)) (=> (= x__ARRAY__88 0) (not BOOL____01075)))))
(assert (and (and (=> BOOL____01077 (= x__ARRAY__89 1)) (=> (= x__ARRAY__89 1) BOOL____01077)) (and (=> (not BOOL____01077) (= x__ARRAY__89 0)) (=> (= x__ARRAY__89 0) (not BOOL____01077)))))
(assert (and (and (=> BOOL____01079 (= x__ARRAY__90 1)) (=> (= x__ARRAY__90 1) BOOL____01079)) (and (=> (not BOOL____01079) (= x__ARRAY__90 0)) (=> (= x__ARRAY__90 0) (not BOOL____01079)))))
(assert (and (and (=> BOOL____01081 (= x__ARRAY__91 1)) (=> (= x__ARRAY__91 1) BOOL____01081)) (and (=> (not BOOL____01081) (= x__ARRAY__91 0)) (=> (= x__ARRAY__91 0) (not BOOL____01081)))))
(assert (and (and (=> BOOL____01083 (= x__ARRAY__92 1)) (=> (= x__ARRAY__92 1) BOOL____01083)) (and (=> (not BOOL____01083) (= x__ARRAY__92 0)) (=> (= x__ARRAY__92 0) (not BOOL____01083)))))
(assert (and (and (=> BOOL____01085 (= x__ARRAY__93 1)) (=> (= x__ARRAY__93 1) BOOL____01085)) (and (=> (not BOOL____01085) (= x__ARRAY__93 0)) (=> (= x__ARRAY__93 0) (not BOOL____01085)))))
(assert (and (and (=> BOOL____01087 (= x__ARRAY__94 1)) (=> (= x__ARRAY__94 1) BOOL____01087)) (and (=> (not BOOL____01087) (= x__ARRAY__94 0)) (=> (= x__ARRAY__94 0) (not BOOL____01087)))))
(assert (and (and (=> BOOL____01089 (= x__ARRAY__95 1)) (=> (= x__ARRAY__95 1) BOOL____01089)) (and (=> (not BOOL____01089) (= x__ARRAY__95 0)) (=> (= x__ARRAY__95 0) (not BOOL____01089)))))
(assert (and (and (=> BOOL____01091 (= x__ARRAY__96 1)) (=> (= x__ARRAY__96 1) BOOL____01091)) (and (=> (not BOOL____01091) (= x__ARRAY__96 0)) (=> (= x__ARRAY__96 0) (not BOOL____01091)))))
(assert (and (and (=> BOOL____01093 (= x__ARRAY__97 1)) (=> (= x__ARRAY__97 1) BOOL____01093)) (and (=> (not BOOL____01093) (= x__ARRAY__97 0)) (=> (= x__ARRAY__97 0) (not BOOL____01093)))))
(assert (and (and (=> BOOL____01095 (= x__ARRAY__98 1)) (=> (= x__ARRAY__98 1) BOOL____01095)) (and (=> (not BOOL____01095) (= x__ARRAY__98 0)) (=> (= x__ARRAY__98 0) (not BOOL____01095)))))
(assert (and (and (=> BOOL____01097 (= x__ARRAY__99 1)) (=> (= x__ARRAY__99 1) BOOL____01097)) (and (=> (not BOOL____01097) (= x__ARRAY__99 0)) (=> (= x__ARRAY__99 0) (not BOOL____01097)))))
(assert (and (and (=> BOOL____01099 (= x__ARRAY__100 1)) (=> (= x__ARRAY__100 1) BOOL____01099)) (and (=> (not BOOL____01099) (= x__ARRAY__100 0)) (=> (= x__ARRAY__100 0) (not BOOL____01099)))))
(assert (and (and (=> BOOL____01101 (= x__ARRAY__101 1)) (=> (= x__ARRAY__101 1) BOOL____01101)) (and (=> (not BOOL____01101) (= x__ARRAY__101 0)) (=> (= x__ARRAY__101 0) (not BOOL____01101)))))
(assert (and (and (=> BOOL____01103 (= x__ARRAY__102 1)) (=> (= x__ARRAY__102 1) BOOL____01103)) (and (=> (not BOOL____01103) (= x__ARRAY__102 0)) (=> (= x__ARRAY__102 0) (not BOOL____01103)))))
(assert (and (and (=> BOOL____01105 (= x__ARRAY__103 1)) (=> (= x__ARRAY__103 1) BOOL____01105)) (and (=> (not BOOL____01105) (= x__ARRAY__103 0)) (=> (= x__ARRAY__103 0) (not BOOL____01105)))))
(assert (and (and (=> BOOL____01107 (= x__ARRAY__104 1)) (=> (= x__ARRAY__104 1) BOOL____01107)) (and (=> (not BOOL____01107) (= x__ARRAY__104 0)) (=> (= x__ARRAY__104 0) (not BOOL____01107)))))
(assert (and (and (=> BOOL____01109 (= x__ARRAY__105 1)) (=> (= x__ARRAY__105 1) BOOL____01109)) (and (=> (not BOOL____01109) (= x__ARRAY__105 0)) (=> (= x__ARRAY__105 0) (not BOOL____01109)))))
(assert (and (and (=> BOOL____01111 (= x__ARRAY__106 1)) (=> (= x__ARRAY__106 1) BOOL____01111)) (and (=> (not BOOL____01111) (= x__ARRAY__106 0)) (=> (= x__ARRAY__106 0) (not BOOL____01111)))))
(assert (and (and (=> BOOL____01113 (= x__ARRAY__107 1)) (=> (= x__ARRAY__107 1) BOOL____01113)) (and (=> (not BOOL____01113) (= x__ARRAY__107 0)) (=> (= x__ARRAY__107 0) (not BOOL____01113)))))
(assert (and (and (=> BOOL____01115 (= x__ARRAY__108 1)) (=> (= x__ARRAY__108 1) BOOL____01115)) (and (=> (not BOOL____01115) (= x__ARRAY__108 0)) (=> (= x__ARRAY__108 0) (not BOOL____01115)))))
(assert (and (and (=> BOOL____01117 (= x__ARRAY__109 1)) (=> (= x__ARRAY__109 1) BOOL____01117)) (and (=> (not BOOL____01117) (= x__ARRAY__109 0)) (=> (= x__ARRAY__109 0) (not BOOL____01117)))))
(assert (and (and (=> BOOL____01119 (= x__ARRAY__110 1)) (=> (= x__ARRAY__110 1) BOOL____01119)) (and (=> (not BOOL____01119) (= x__ARRAY__110 0)) (=> (= x__ARRAY__110 0) (not BOOL____01119)))))
(assert (and (and (=> BOOL____01121 (= x__ARRAY__111 1)) (=> (= x__ARRAY__111 1) BOOL____01121)) (and (=> (not BOOL____01121) (= x__ARRAY__111 0)) (=> (= x__ARRAY__111 0) (not BOOL____01121)))))
(assert (and (and (=> BOOL____01123 (= x__ARRAY__112 1)) (=> (= x__ARRAY__112 1) BOOL____01123)) (and (=> (not BOOL____01123) (= x__ARRAY__112 0)) (=> (= x__ARRAY__112 0) (not BOOL____01123)))))
(assert (and (and (=> BOOL____01125 (= x__ARRAY__113 1)) (=> (= x__ARRAY__113 1) BOOL____01125)) (and (=> (not BOOL____01125) (= x__ARRAY__113 0)) (=> (= x__ARRAY__113 0) (not BOOL____01125)))))
(assert (and (and (=> BOOL____01127 (= x__ARRAY__114 1)) (=> (= x__ARRAY__114 1) BOOL____01127)) (and (=> (not BOOL____01127) (= x__ARRAY__114 0)) (=> (= x__ARRAY__114 0) (not BOOL____01127)))))
(assert (and (and (=> BOOL____01129 (= x__ARRAY__115 1)) (=> (= x__ARRAY__115 1) BOOL____01129)) (and (=> (not BOOL____01129) (= x__ARRAY__115 0)) (=> (= x__ARRAY__115 0) (not BOOL____01129)))))
(assert (and (and (=> BOOL____01131 (= x__ARRAY__116 1)) (=> (= x__ARRAY__116 1) BOOL____01131)) (and (=> (not BOOL____01131) (= x__ARRAY__116 0)) (=> (= x__ARRAY__116 0) (not BOOL____01131)))))
(assert (and (and (=> BOOL____01133 (= x__ARRAY__117 1)) (=> (= x__ARRAY__117 1) BOOL____01133)) (and (=> (not BOOL____01133) (= x__ARRAY__117 0)) (=> (= x__ARRAY__117 0) (not BOOL____01133)))))
(assert (and (and (=> BOOL____01135 (= x__ARRAY__118 1)) (=> (= x__ARRAY__118 1) BOOL____01135)) (and (=> (not BOOL____01135) (= x__ARRAY__118 0)) (=> (= x__ARRAY__118 0) (not BOOL____01135)))))
(assert (and (and (=> BOOL____01137 (= x__ARRAY__119 1)) (=> (= x__ARRAY__119 1) BOOL____01137)) (and (=> (not BOOL____01137) (= x__ARRAY__119 0)) (=> (= x__ARRAY__119 0) (not BOOL____01137)))))
(assert (and (and (=> BOOL____01139 (= x__ARRAY__120 1)) (=> (= x__ARRAY__120 1) BOOL____01139)) (and (=> (not BOOL____01139) (= x__ARRAY__120 0)) (=> (= x__ARRAY__120 0) (not BOOL____01139)))))
(assert (and (and (=> BOOL____01141 (= x__ARRAY__121 1)) (=> (= x__ARRAY__121 1) BOOL____01141)) (and (=> (not BOOL____01141) (= x__ARRAY__121 0)) (=> (= x__ARRAY__121 0) (not BOOL____01141)))))
(assert (and (and (=> BOOL____01143 (= x__ARRAY__122 1)) (=> (= x__ARRAY__122 1) BOOL____01143)) (and (=> (not BOOL____01143) (= x__ARRAY__122 0)) (=> (= x__ARRAY__122 0) (not BOOL____01143)))))
(assert (and (and (=> BOOL____01145 (= x__ARRAY__123 1)) (=> (= x__ARRAY__123 1) BOOL____01145)) (and (=> (not BOOL____01145) (= x__ARRAY__123 0)) (=> (= x__ARRAY__123 0) (not BOOL____01145)))))
(assert (and (and (=> BOOL____01147 (= x__ARRAY__124 1)) (=> (= x__ARRAY__124 1) BOOL____01147)) (and (=> (not BOOL____01147) (= x__ARRAY__124 0)) (=> (= x__ARRAY__124 0) (not BOOL____01147)))))
(assert (and (and (=> BOOL____01149 (= x__ARRAY__125 1)) (=> (= x__ARRAY__125 1) BOOL____01149)) (and (=> (not BOOL____01149) (= x__ARRAY__125 0)) (=> (= x__ARRAY__125 0) (not BOOL____01149)))))
(assert (and (and (=> BOOL____01151 (= x__ARRAY__126 1)) (=> (= x__ARRAY__126 1) BOOL____01151)) (and (=> (not BOOL____01151) (= x__ARRAY__126 0)) (=> (= x__ARRAY__126 0) (not BOOL____01151)))))
(assert (and (and (=> BOOL____01153 (= x__ARRAY__127 1)) (=> (= x__ARRAY__127 1) BOOL____01153)) (and (=> (not BOOL____01153) (= x__ARRAY__127 0)) (=> (= x__ARRAY__127 0) (not BOOL____01153)))))
(assert (and (and (=> BOOL____01155 (= x__ARRAY__128 1)) (=> (= x__ARRAY__128 1) BOOL____01155)) (and (=> (not BOOL____01155) (= x__ARRAY__128 0)) (=> (= x__ARRAY__128 0) (not BOOL____01155)))))
(assert (and (and (=> BOOL____01157 (= x__ARRAY__129 1)) (=> (= x__ARRAY__129 1) BOOL____01157)) (and (=> (not BOOL____01157) (= x__ARRAY__129 0)) (=> (= x__ARRAY__129 0) (not BOOL____01157)))))
(assert (and (and (=> BOOL____01159 (= x__ARRAY__130 1)) (=> (= x__ARRAY__130 1) BOOL____01159)) (and (=> (not BOOL____01159) (= x__ARRAY__130 0)) (=> (= x__ARRAY__130 0) (not BOOL____01159)))))
(assert (and (and (=> BOOL____01161 (= x__ARRAY__131 1)) (=> (= x__ARRAY__131 1) BOOL____01161)) (and (=> (not BOOL____01161) (= x__ARRAY__131 0)) (=> (= x__ARRAY__131 0) (not BOOL____01161)))))
(assert (and (and (=> BOOL____01163 (= x__ARRAY__132 1)) (=> (= x__ARRAY__132 1) BOOL____01163)) (and (=> (not BOOL____01163) (= x__ARRAY__132 0)) (=> (= x__ARRAY__132 0) (not BOOL____01163)))))
(assert (and (and (=> BOOL____01165 (= x__ARRAY__133 1)) (=> (= x__ARRAY__133 1) BOOL____01165)) (and (=> (not BOOL____01165) (= x__ARRAY__133 0)) (=> (= x__ARRAY__133 0) (not BOOL____01165)))))
(assert (and (and (=> BOOL____01167 (= x__ARRAY__134 1)) (=> (= x__ARRAY__134 1) BOOL____01167)) (and (=> (not BOOL____01167) (= x__ARRAY__134 0)) (=> (= x__ARRAY__134 0) (not BOOL____01167)))))
(assert (and (and (=> BOOL____01169 (= x__ARRAY__135 1)) (=> (= x__ARRAY__135 1) BOOL____01169)) (and (=> (not BOOL____01169) (= x__ARRAY__135 0)) (=> (= x__ARRAY__135 0) (not BOOL____01169)))))
(assert (and (and (=> BOOL____01171 (= x__ARRAY__136 1)) (=> (= x__ARRAY__136 1) BOOL____01171)) (and (=> (not BOOL____01171) (= x__ARRAY__136 0)) (=> (= x__ARRAY__136 0) (not BOOL____01171)))))
(assert (and (and (=> BOOL____01173 (= x__ARRAY__137 1)) (=> (= x__ARRAY__137 1) BOOL____01173)) (and (=> (not BOOL____01173) (= x__ARRAY__137 0)) (=> (= x__ARRAY__137 0) (not BOOL____01173)))))
(assert (and (and (=> BOOL____01175 (= x__ARRAY__138 1)) (=> (= x__ARRAY__138 1) BOOL____01175)) (and (=> (not BOOL____01175) (= x__ARRAY__138 0)) (=> (= x__ARRAY__138 0) (not BOOL____01175)))))
(assert (and (and (=> BOOL____01177 (= x__ARRAY__139 1)) (=> (= x__ARRAY__139 1) BOOL____01177)) (and (=> (not BOOL____01177) (= x__ARRAY__139 0)) (=> (= x__ARRAY__139 0) (not BOOL____01177)))))
(assert (and (and (=> BOOL____01179 (= x__ARRAY__140 1)) (=> (= x__ARRAY__140 1) BOOL____01179)) (and (=> (not BOOL____01179) (= x__ARRAY__140 0)) (=> (= x__ARRAY__140 0) (not BOOL____01179)))))
(assert (and (and (=> BOOL____01181 (= x__ARRAY__141 1)) (=> (= x__ARRAY__141 1) BOOL____01181)) (and (=> (not BOOL____01181) (= x__ARRAY__141 0)) (=> (= x__ARRAY__141 0) (not BOOL____01181)))))
(assert (and (and (=> BOOL____01183 (= x__ARRAY__142 1)) (=> (= x__ARRAY__142 1) BOOL____01183)) (and (=> (not BOOL____01183) (= x__ARRAY__142 0)) (=> (= x__ARRAY__142 0) (not BOOL____01183)))))
(assert (and (and (=> BOOL____01185 (= x__ARRAY__143 1)) (=> (= x__ARRAY__143 1) BOOL____01185)) (and (=> (not BOOL____01185) (= x__ARRAY__143 0)) (=> (= x__ARRAY__143 0) (not BOOL____01185)))))
(assert (and (and (=> BOOL____01187 (= x__ARRAY__144 1)) (=> (= x__ARRAY__144 1) BOOL____01187)) (and (=> (not BOOL____01187) (= x__ARRAY__144 0)) (=> (= x__ARRAY__144 0) (not BOOL____01187)))))
(assert (and (and (=> BOOL____01189 (= x__ARRAY__145 1)) (=> (= x__ARRAY__145 1) BOOL____01189)) (and (=> (not BOOL____01189) (= x__ARRAY__145 0)) (=> (= x__ARRAY__145 0) (not BOOL____01189)))))
(assert (and (and (=> BOOL____01191 (= x__ARRAY__146 1)) (=> (= x__ARRAY__146 1) BOOL____01191)) (and (=> (not BOOL____01191) (= x__ARRAY__146 0)) (=> (= x__ARRAY__146 0) (not BOOL____01191)))))
(assert (and (and (=> BOOL____01193 (= x__ARRAY__147 1)) (=> (= x__ARRAY__147 1) BOOL____01193)) (and (=> (not BOOL____01193) (= x__ARRAY__147 0)) (=> (= x__ARRAY__147 0) (not BOOL____01193)))))
(assert (and (and (=> BOOL____01195 (= x__ARRAY__148 1)) (=> (= x__ARRAY__148 1) BOOL____01195)) (and (=> (not BOOL____01195) (= x__ARRAY__148 0)) (=> (= x__ARRAY__148 0) (not BOOL____01195)))))
(assert (and (and (=> BOOL____01197 (= x__ARRAY__149 1)) (=> (= x__ARRAY__149 1) BOOL____01197)) (and (=> (not BOOL____01197) (= x__ARRAY__149 0)) (=> (= x__ARRAY__149 0) (not BOOL____01197)))))
(assert (and (and (=> BOOL____01199 (= x__ARRAY__150 1)) (=> (= x__ARRAY__150 1) BOOL____01199)) (and (=> (not BOOL____01199) (= x__ARRAY__150 0)) (=> (= x__ARRAY__150 0) (not BOOL____01199)))))
(assert (and (and (=> BOOL____01201 (= x__ARRAY__151 1)) (=> (= x__ARRAY__151 1) BOOL____01201)) (and (=> (not BOOL____01201) (= x__ARRAY__151 0)) (=> (= x__ARRAY__151 0) (not BOOL____01201)))))
(assert (and (and (=> BOOL____01203 (= x__ARRAY__152 1)) (=> (= x__ARRAY__152 1) BOOL____01203)) (and (=> (not BOOL____01203) (= x__ARRAY__152 0)) (=> (= x__ARRAY__152 0) (not BOOL____01203)))))
(assert (and (and (=> BOOL____01205 (= x__ARRAY__153 1)) (=> (= x__ARRAY__153 1) BOOL____01205)) (and (=> (not BOOL____01205) (= x__ARRAY__153 0)) (=> (= x__ARRAY__153 0) (not BOOL____01205)))))
(assert (and (and (=> BOOL____01207 (= x__ARRAY__154 1)) (=> (= x__ARRAY__154 1) BOOL____01207)) (and (=> (not BOOL____01207) (= x__ARRAY__154 0)) (=> (= x__ARRAY__154 0) (not BOOL____01207)))))
(assert (and (and (=> BOOL____01209 (= x__ARRAY__155 1)) (=> (= x__ARRAY__155 1) BOOL____01209)) (and (=> (not BOOL____01209) (= x__ARRAY__155 0)) (=> (= x__ARRAY__155 0) (not BOOL____01209)))))
(assert (and (and (=> BOOL____01211 (= x__ARRAY__156 1)) (=> (= x__ARRAY__156 1) BOOL____01211)) (and (=> (not BOOL____01211) (= x__ARRAY__156 0)) (=> (= x__ARRAY__156 0) (not BOOL____01211)))))
(assert (and (and (=> BOOL____01213 (= x__ARRAY__157 1)) (=> (= x__ARRAY__157 1) BOOL____01213)) (and (=> (not BOOL____01213) (= x__ARRAY__157 0)) (=> (= x__ARRAY__157 0) (not BOOL____01213)))))
(assert (and (and (=> BOOL____01215 (= x__ARRAY__158 1)) (=> (= x__ARRAY__158 1) BOOL____01215)) (and (=> (not BOOL____01215) (= x__ARRAY__158 0)) (=> (= x__ARRAY__158 0) (not BOOL____01215)))))
(assert (and (and (=> BOOL____01217 (= x__ARRAY__159 1)) (=> (= x__ARRAY__159 1) BOOL____01217)) (and (=> (not BOOL____01217) (= x__ARRAY__159 0)) (=> (= x__ARRAY__159 0) (not BOOL____01217)))))
(assert (and (and (=> BOOL____01219 (= x__ARRAY__160 1)) (=> (= x__ARRAY__160 1) BOOL____01219)) (and (=> (not BOOL____01219) (= x__ARRAY__160 0)) (=> (= x__ARRAY__160 0) (not BOOL____01219)))))
(assert (and (and (=> BOOL____01221 (= x__ARRAY__161 1)) (=> (= x__ARRAY__161 1) BOOL____01221)) (and (=> (not BOOL____01221) (= x__ARRAY__161 0)) (=> (= x__ARRAY__161 0) (not BOOL____01221)))))
(assert (and (and (=> BOOL____01223 (= x__ARRAY__162 1)) (=> (= x__ARRAY__162 1) BOOL____01223)) (and (=> (not BOOL____01223) (= x__ARRAY__162 0)) (=> (= x__ARRAY__162 0) (not BOOL____01223)))))
(assert (and (and (=> BOOL____01225 (= x__ARRAY__163 1)) (=> (= x__ARRAY__163 1) BOOL____01225)) (and (=> (not BOOL____01225) (= x__ARRAY__163 0)) (=> (= x__ARRAY__163 0) (not BOOL____01225)))))
(assert (and (and (=> BOOL____01227 (= x__ARRAY__164 1)) (=> (= x__ARRAY__164 1) BOOL____01227)) (and (=> (not BOOL____01227) (= x__ARRAY__164 0)) (=> (= x__ARRAY__164 0) (not BOOL____01227)))))
(assert (and (and (=> BOOL____01229 (= x__ARRAY__165 1)) (=> (= x__ARRAY__165 1) BOOL____01229)) (and (=> (not BOOL____01229) (= x__ARRAY__165 0)) (=> (= x__ARRAY__165 0) (not BOOL____01229)))))
(assert (and (and (=> BOOL____01231 (= x__ARRAY__166 1)) (=> (= x__ARRAY__166 1) BOOL____01231)) (and (=> (not BOOL____01231) (= x__ARRAY__166 0)) (=> (= x__ARRAY__166 0) (not BOOL____01231)))))
(assert (and (and (=> BOOL____01233 (= x__ARRAY__167 1)) (=> (= x__ARRAY__167 1) BOOL____01233)) (and (=> (not BOOL____01233) (= x__ARRAY__167 0)) (=> (= x__ARRAY__167 0) (not BOOL____01233)))))
(assert (and (and (=> BOOL____01235 (= x__ARRAY__168 1)) (=> (= x__ARRAY__168 1) BOOL____01235)) (and (=> (not BOOL____01235) (= x__ARRAY__168 0)) (=> (= x__ARRAY__168 0) (not BOOL____01235)))))
(assert (and (and (=> BOOL____01237 (= x__ARRAY__169 1)) (=> (= x__ARRAY__169 1) BOOL____01237)) (and (=> (not BOOL____01237) (= x__ARRAY__169 0)) (=> (= x__ARRAY__169 0) (not BOOL____01237)))))
(assert (and (and (=> BOOL____01239 (= x__ARRAY__170 1)) (=> (= x__ARRAY__170 1) BOOL____01239)) (and (=> (not BOOL____01239) (= x__ARRAY__170 0)) (=> (= x__ARRAY__170 0) (not BOOL____01239)))))
(assert (and (and (=> BOOL____01241 (= x__ARRAY__171 1)) (=> (= x__ARRAY__171 1) BOOL____01241)) (and (=> (not BOOL____01241) (= x__ARRAY__171 0)) (=> (= x__ARRAY__171 0) (not BOOL____01241)))))
(assert (and (and (=> BOOL____01243 (= x__ARRAY__172 1)) (=> (= x__ARRAY__172 1) BOOL____01243)) (and (=> (not BOOL____01243) (= x__ARRAY__172 0)) (=> (= x__ARRAY__172 0) (not BOOL____01243)))))
(assert (and (and (=> BOOL____01245 (= x__ARRAY__173 1)) (=> (= x__ARRAY__173 1) BOOL____01245)) (and (=> (not BOOL____01245) (= x__ARRAY__173 0)) (=> (= x__ARRAY__173 0) (not BOOL____01245)))))
(assert (and (and (=> BOOL____01247 (= x__ARRAY__174 1)) (=> (= x__ARRAY__174 1) BOOL____01247)) (and (=> (not BOOL____01247) (= x__ARRAY__174 0)) (=> (= x__ARRAY__174 0) (not BOOL____01247)))))
(assert (and (and (=> BOOL____01249 (= x__ARRAY__175 1)) (=> (= x__ARRAY__175 1) BOOL____01249)) (and (=> (not BOOL____01249) (= x__ARRAY__175 0)) (=> (= x__ARRAY__175 0) (not BOOL____01249)))))
(assert (and (and (=> BOOL____01251 (= x__ARRAY__176 1)) (=> (= x__ARRAY__176 1) BOOL____01251)) (and (=> (not BOOL____01251) (= x__ARRAY__176 0)) (=> (= x__ARRAY__176 0) (not BOOL____01251)))))
(assert (and (and (=> BOOL____01253 (= x__ARRAY__177 1)) (=> (= x__ARRAY__177 1) BOOL____01253)) (and (=> (not BOOL____01253) (= x__ARRAY__177 0)) (=> (= x__ARRAY__177 0) (not BOOL____01253)))))
(assert (and (and (=> BOOL____01255 (= x__ARRAY__178 1)) (=> (= x__ARRAY__178 1) BOOL____01255)) (and (=> (not BOOL____01255) (= x__ARRAY__178 0)) (=> (= x__ARRAY__178 0) (not BOOL____01255)))))
(assert (and (and (=> BOOL____01257 (= x__ARRAY__179 1)) (=> (= x__ARRAY__179 1) BOOL____01257)) (and (=> (not BOOL____01257) (= x__ARRAY__179 0)) (=> (= x__ARRAY__179 0) (not BOOL____01257)))))
(assert (and (and (=> BOOL____01259 (= x__ARRAY__180 1)) (=> (= x__ARRAY__180 1) BOOL____01259)) (and (=> (not BOOL____01259) (= x__ARRAY__180 0)) (=> (= x__ARRAY__180 0) (not BOOL____01259)))))
(assert (and (and (=> BOOL____01261 (= x__ARRAY__181 1)) (=> (= x__ARRAY__181 1) BOOL____01261)) (and (=> (not BOOL____01261) (= x__ARRAY__181 0)) (=> (= x__ARRAY__181 0) (not BOOL____01261)))))
(assert (and (and (=> BOOL____01263 (= x__ARRAY__182 1)) (=> (= x__ARRAY__182 1) BOOL____01263)) (and (=> (not BOOL____01263) (= x__ARRAY__182 0)) (=> (= x__ARRAY__182 0) (not BOOL____01263)))))
(assert (and (and (=> BOOL____01265 (= x__ARRAY__183 1)) (=> (= x__ARRAY__183 1) BOOL____01265)) (and (=> (not BOOL____01265) (= x__ARRAY__183 0)) (=> (= x__ARRAY__183 0) (not BOOL____01265)))))
(assert (and (and (=> BOOL____01267 (= x__ARRAY__184 1)) (=> (= x__ARRAY__184 1) BOOL____01267)) (and (=> (not BOOL____01267) (= x__ARRAY__184 0)) (=> (= x__ARRAY__184 0) (not BOOL____01267)))))
(assert (and (and (=> BOOL____01269 (= x__ARRAY__185 1)) (=> (= x__ARRAY__185 1) BOOL____01269)) (and (=> (not BOOL____01269) (= x__ARRAY__185 0)) (=> (= x__ARRAY__185 0) (not BOOL____01269)))))
(assert (and (and (=> BOOL____01271 (= x__ARRAY__186 1)) (=> (= x__ARRAY__186 1) BOOL____01271)) (and (=> (not BOOL____01271) (= x__ARRAY__186 0)) (=> (= x__ARRAY__186 0) (not BOOL____01271)))))
(assert (and (and (=> BOOL____01273 (= x__ARRAY__187 1)) (=> (= x__ARRAY__187 1) BOOL____01273)) (and (=> (not BOOL____01273) (= x__ARRAY__187 0)) (=> (= x__ARRAY__187 0) (not BOOL____01273)))))
(assert (and (and (=> BOOL____01275 (= x__ARRAY__188 1)) (=> (= x__ARRAY__188 1) BOOL____01275)) (and (=> (not BOOL____01275) (= x__ARRAY__188 0)) (=> (= x__ARRAY__188 0) (not BOOL____01275)))))
(assert (and (and (=> BOOL____01277 (= x__ARRAY__189 1)) (=> (= x__ARRAY__189 1) BOOL____01277)) (and (=> (not BOOL____01277) (= x__ARRAY__189 0)) (=> (= x__ARRAY__189 0) (not BOOL____01277)))))
(assert (and (and (=> BOOL____01279 (= x__ARRAY__190 1)) (=> (= x__ARRAY__190 1) BOOL____01279)) (and (=> (not BOOL____01279) (= x__ARRAY__190 0)) (=> (= x__ARRAY__190 0) (not BOOL____01279)))))
(assert (and (and (=> BOOL____01281 (= x__ARRAY__191 1)) (=> (= x__ARRAY__191 1) BOOL____01281)) (and (=> (not BOOL____01281) (= x__ARRAY__191 0)) (=> (= x__ARRAY__191 0) (not BOOL____01281)))))
(assert (and (and (=> BOOL____01283 (= x__ARRAY__192 1)) (=> (= x__ARRAY__192 1) BOOL____01283)) (and (=> (not BOOL____01283) (= x__ARRAY__192 0)) (=> (= x__ARRAY__192 0) (not BOOL____01283)))))
(assert (and (and (=> BOOL____01285 (= x__ARRAY__193 1)) (=> (= x__ARRAY__193 1) BOOL____01285)) (and (=> (not BOOL____01285) (= x__ARRAY__193 0)) (=> (= x__ARRAY__193 0) (not BOOL____01285)))))
(assert (and (and (=> BOOL____01287 (= x__ARRAY__194 1)) (=> (= x__ARRAY__194 1) BOOL____01287)) (and (=> (not BOOL____01287) (= x__ARRAY__194 0)) (=> (= x__ARRAY__194 0) (not BOOL____01287)))))
(assert (and (and (=> BOOL____01289 (= x__ARRAY__195 1)) (=> (= x__ARRAY__195 1) BOOL____01289)) (and (=> (not BOOL____01289) (= x__ARRAY__195 0)) (=> (= x__ARRAY__195 0) (not BOOL____01289)))))
(assert (and (and (=> BOOL____01291 (= x__ARRAY__196 1)) (=> (= x__ARRAY__196 1) BOOL____01291)) (and (=> (not BOOL____01291) (= x__ARRAY__196 0)) (=> (= x__ARRAY__196 0) (not BOOL____01291)))))
(assert (and (and (=> BOOL____01293 (= x__ARRAY__197 1)) (=> (= x__ARRAY__197 1) BOOL____01293)) (and (=> (not BOOL____01293) (= x__ARRAY__197 0)) (=> (= x__ARRAY__197 0) (not BOOL____01293)))))
(assert (and (and (=> BOOL____01295 (= x__ARRAY__198 1)) (=> (= x__ARRAY__198 1) BOOL____01295)) (and (=> (not BOOL____01295) (= x__ARRAY__198 0)) (=> (= x__ARRAY__198 0) (not BOOL____01295)))))
(assert (and (and (=> BOOL____01297 (= x__ARRAY__199 1)) (=> (= x__ARRAY__199 1) BOOL____01297)) (and (=> (not BOOL____01297) (= x__ARRAY__199 0)) (=> (= x__ARRAY__199 0) (not BOOL____01297)))))
(assert (and (and (=> BOOL____01299 (= x__ARRAY__200 1)) (=> (= x__ARRAY__200 1) BOOL____01299)) (and (=> (not BOOL____01299) (= x__ARRAY__200 0)) (=> (= x__ARRAY__200 0) (not BOOL____01299)))))
(assert (and (and (=> BOOL____01301 (= x__ARRAY__201 1)) (=> (= x__ARRAY__201 1) BOOL____01301)) (and (=> (not BOOL____01301) (= x__ARRAY__201 0)) (=> (= x__ARRAY__201 0) (not BOOL____01301)))))
(assert (and (and (=> BOOL____01303 (= x__ARRAY__202 1)) (=> (= x__ARRAY__202 1) BOOL____01303)) (and (=> (not BOOL____01303) (= x__ARRAY__202 0)) (=> (= x__ARRAY__202 0) (not BOOL____01303)))))
(assert (and (and (=> BOOL____01305 (= x__ARRAY__203 1)) (=> (= x__ARRAY__203 1) BOOL____01305)) (and (=> (not BOOL____01305) (= x__ARRAY__203 0)) (=> (= x__ARRAY__203 0) (not BOOL____01305)))))
(assert (and (and (=> BOOL____01307 (= x__ARRAY__204 1)) (=> (= x__ARRAY__204 1) BOOL____01307)) (and (=> (not BOOL____01307) (= x__ARRAY__204 0)) (=> (= x__ARRAY__204 0) (not BOOL____01307)))))
(assert (and (and (=> BOOL____01309 (= x__ARRAY__205 1)) (=> (= x__ARRAY__205 1) BOOL____01309)) (and (=> (not BOOL____01309) (= x__ARRAY__205 0)) (=> (= x__ARRAY__205 0) (not BOOL____01309)))))
(assert (and (and (=> BOOL____01311 (= x__ARRAY__206 1)) (=> (= x__ARRAY__206 1) BOOL____01311)) (and (=> (not BOOL____01311) (= x__ARRAY__206 0)) (=> (= x__ARRAY__206 0) (not BOOL____01311)))))
(assert (and (and (=> BOOL____01313 (= x__ARRAY__207 1)) (=> (= x__ARRAY__207 1) BOOL____01313)) (and (=> (not BOOL____01313) (= x__ARRAY__207 0)) (=> (= x__ARRAY__207 0) (not BOOL____01313)))))
(assert (and (and (=> BOOL____01315 (= x__ARRAY__208 1)) (=> (= x__ARRAY__208 1) BOOL____01315)) (and (=> (not BOOL____01315) (= x__ARRAY__208 0)) (=> (= x__ARRAY__208 0) (not BOOL____01315)))))
(assert (and (and (=> BOOL____01317 (= x__ARRAY__209 1)) (=> (= x__ARRAY__209 1) BOOL____01317)) (and (=> (not BOOL____01317) (= x__ARRAY__209 0)) (=> (= x__ARRAY__209 0) (not BOOL____01317)))))
(assert (and (and (=> BOOL____01319 (= x__ARRAY__210 1)) (=> (= x__ARRAY__210 1) BOOL____01319)) (and (=> (not BOOL____01319) (= x__ARRAY__210 0)) (=> (= x__ARRAY__210 0) (not BOOL____01319)))))
(assert (and (and (=> BOOL____01321 (= x__ARRAY__211 1)) (=> (= x__ARRAY__211 1) BOOL____01321)) (and (=> (not BOOL____01321) (= x__ARRAY__211 0)) (=> (= x__ARRAY__211 0) (not BOOL____01321)))))
(assert (and (and (=> BOOL____01323 (= x__ARRAY__212 1)) (=> (= x__ARRAY__212 1) BOOL____01323)) (and (=> (not BOOL____01323) (= x__ARRAY__212 0)) (=> (= x__ARRAY__212 0) (not BOOL____01323)))))
(assert (and (and (=> BOOL____01325 (= x__ARRAY__213 1)) (=> (= x__ARRAY__213 1) BOOL____01325)) (and (=> (not BOOL____01325) (= x__ARRAY__213 0)) (=> (= x__ARRAY__213 0) (not BOOL____01325)))))
(assert (and (and (=> BOOL____01327 (= x__ARRAY__214 1)) (=> (= x__ARRAY__214 1) BOOL____01327)) (and (=> (not BOOL____01327) (= x__ARRAY__214 0)) (=> (= x__ARRAY__214 0) (not BOOL____01327)))))
(assert (and (and (=> BOOL____01329 (= x__ARRAY__215 1)) (=> (= x__ARRAY__215 1) BOOL____01329)) (and (=> (not BOOL____01329) (= x__ARRAY__215 0)) (=> (= x__ARRAY__215 0) (not BOOL____01329)))))
(assert (and (and (=> BOOL____01331 (= x__ARRAY__216 1)) (=> (= x__ARRAY__216 1) BOOL____01331)) (and (=> (not BOOL____01331) (= x__ARRAY__216 0)) (=> (= x__ARRAY__216 0) (not BOOL____01331)))))
(assert (and (and (=> BOOL____01333 (= x__ARRAY__217 1)) (=> (= x__ARRAY__217 1) BOOL____01333)) (and (=> (not BOOL____01333) (= x__ARRAY__217 0)) (=> (= x__ARRAY__217 0) (not BOOL____01333)))))
(assert (and (and (=> BOOL____01335 (= x__ARRAY__218 1)) (=> (= x__ARRAY__218 1) BOOL____01335)) (and (=> (not BOOL____01335) (= x__ARRAY__218 0)) (=> (= x__ARRAY__218 0) (not BOOL____01335)))))
(assert (and (and (=> BOOL____01337 (= x__ARRAY__219 1)) (=> (= x__ARRAY__219 1) BOOL____01337)) (and (=> (not BOOL____01337) (= x__ARRAY__219 0)) (=> (= x__ARRAY__219 0) (not BOOL____01337)))))
(assert (and (and (=> BOOL____01339 (= x__ARRAY__220 1)) (=> (= x__ARRAY__220 1) BOOL____01339)) (and (=> (not BOOL____01339) (= x__ARRAY__220 0)) (=> (= x__ARRAY__220 0) (not BOOL____01339)))))
(assert (and (and (=> BOOL____01341 (= x__ARRAY__221 1)) (=> (= x__ARRAY__221 1) BOOL____01341)) (and (=> (not BOOL____01341) (= x__ARRAY__221 0)) (=> (= x__ARRAY__221 0) (not BOOL____01341)))))
(assert (and (and (=> BOOL____01343 (= x__ARRAY__222 1)) (=> (= x__ARRAY__222 1) BOOL____01343)) (and (=> (not BOOL____01343) (= x__ARRAY__222 0)) (=> (= x__ARRAY__222 0) (not BOOL____01343)))))
(assert (and (and (=> BOOL____01345 (= x__ARRAY__223 1)) (=> (= x__ARRAY__223 1) BOOL____01345)) (and (=> (not BOOL____01345) (= x__ARRAY__223 0)) (=> (= x__ARRAY__223 0) (not BOOL____01345)))))
(assert (and (and (=> BOOL____01347 (= x__ARRAY__224 1)) (=> (= x__ARRAY__224 1) BOOL____01347)) (and (=> (not BOOL____01347) (= x__ARRAY__224 0)) (=> (= x__ARRAY__224 0) (not BOOL____01347)))))
(assert (and (and (=> BOOL____01349 (= x__ARRAY__225 1)) (=> (= x__ARRAY__225 1) BOOL____01349)) (and (=> (not BOOL____01349) (= x__ARRAY__225 0)) (=> (= x__ARRAY__225 0) (not BOOL____01349)))))
(assert (and (and (=> BOOL____01351 (= x__ARRAY__226 1)) (=> (= x__ARRAY__226 1) BOOL____01351)) (and (=> (not BOOL____01351) (= x__ARRAY__226 0)) (=> (= x__ARRAY__226 0) (not BOOL____01351)))))
(assert (and (and (=> BOOL____01353 (= x__ARRAY__227 1)) (=> (= x__ARRAY__227 1) BOOL____01353)) (and (=> (not BOOL____01353) (= x__ARRAY__227 0)) (=> (= x__ARRAY__227 0) (not BOOL____01353)))))
(assert (and (and (=> BOOL____01355 (= x__ARRAY__228 1)) (=> (= x__ARRAY__228 1) BOOL____01355)) (and (=> (not BOOL____01355) (= x__ARRAY__228 0)) (=> (= x__ARRAY__228 0) (not BOOL____01355)))))
(assert (and (and (=> BOOL____01357 (= x__ARRAY__229 1)) (=> (= x__ARRAY__229 1) BOOL____01357)) (and (=> (not BOOL____01357) (= x__ARRAY__229 0)) (=> (= x__ARRAY__229 0) (not BOOL____01357)))))
(assert (and (and (=> BOOL____01359 (= x__ARRAY__230 1)) (=> (= x__ARRAY__230 1) BOOL____01359)) (and (=> (not BOOL____01359) (= x__ARRAY__230 0)) (=> (= x__ARRAY__230 0) (not BOOL____01359)))))
(assert (and (and (=> BOOL____01361 (= x__ARRAY__231 1)) (=> (= x__ARRAY__231 1) BOOL____01361)) (and (=> (not BOOL____01361) (= x__ARRAY__231 0)) (=> (= x__ARRAY__231 0) (not BOOL____01361)))))
(assert (and (and (=> BOOL____01363 (= x__ARRAY__232 1)) (=> (= x__ARRAY__232 1) BOOL____01363)) (and (=> (not BOOL____01363) (= x__ARRAY__232 0)) (=> (= x__ARRAY__232 0) (not BOOL____01363)))))
(assert (and (and (=> BOOL____01365 (= x__ARRAY__233 1)) (=> (= x__ARRAY__233 1) BOOL____01365)) (and (=> (not BOOL____01365) (= x__ARRAY__233 0)) (=> (= x__ARRAY__233 0) (not BOOL____01365)))))
(assert (and (and (=> BOOL____01367 (= x__ARRAY__234 1)) (=> (= x__ARRAY__234 1) BOOL____01367)) (and (=> (not BOOL____01367) (= x__ARRAY__234 0)) (=> (= x__ARRAY__234 0) (not BOOL____01367)))))
(assert (and (and (=> BOOL____01369 (= x__ARRAY__235 1)) (=> (= x__ARRAY__235 1) BOOL____01369)) (and (=> (not BOOL____01369) (= x__ARRAY__235 0)) (=> (= x__ARRAY__235 0) (not BOOL____01369)))))
(assert (and (and (=> BOOL____01371 (= x__ARRAY__236 1)) (=> (= x__ARRAY__236 1) BOOL____01371)) (and (=> (not BOOL____01371) (= x__ARRAY__236 0)) (=> (= x__ARRAY__236 0) (not BOOL____01371)))))
(assert (and (and (=> BOOL____01373 (= x__ARRAY__237 1)) (=> (= x__ARRAY__237 1) BOOL____01373)) (and (=> (not BOOL____01373) (= x__ARRAY__237 0)) (=> (= x__ARRAY__237 0) (not BOOL____01373)))))
(assert (and (and (=> BOOL____01375 (= x__ARRAY__238 1)) (=> (= x__ARRAY__238 1) BOOL____01375)) (and (=> (not BOOL____01375) (= x__ARRAY__238 0)) (=> (= x__ARRAY__238 0) (not BOOL____01375)))))
(assert (and (and (=> BOOL____01377 (= x__ARRAY__239 1)) (=> (= x__ARRAY__239 1) BOOL____01377)) (and (=> (not BOOL____01377) (= x__ARRAY__239 0)) (=> (= x__ARRAY__239 0) (not BOOL____01377)))))
(assert (and (and (=> BOOL____01379 (= x__ARRAY__240 1)) (=> (= x__ARRAY__240 1) BOOL____01379)) (and (=> (not BOOL____01379) (= x__ARRAY__240 0)) (=> (= x__ARRAY__240 0) (not BOOL____01379)))))
(assert (and (and (=> BOOL____01381 (= x__ARRAY__241 1)) (=> (= x__ARRAY__241 1) BOOL____01381)) (and (=> (not BOOL____01381) (= x__ARRAY__241 0)) (=> (= x__ARRAY__241 0) (not BOOL____01381)))))
(assert (and (and (=> BOOL____01383 (= x__ARRAY__242 1)) (=> (= x__ARRAY__242 1) BOOL____01383)) (and (=> (not BOOL____01383) (= x__ARRAY__242 0)) (=> (= x__ARRAY__242 0) (not BOOL____01383)))))
(assert (and (and (=> BOOL____01385 (= x__ARRAY__243 1)) (=> (= x__ARRAY__243 1) BOOL____01385)) (and (=> (not BOOL____01385) (= x__ARRAY__243 0)) (=> (= x__ARRAY__243 0) (not BOOL____01385)))))
(assert (and (and (=> BOOL____01387 (= x__ARRAY__244 1)) (=> (= x__ARRAY__244 1) BOOL____01387)) (and (=> (not BOOL____01387) (= x__ARRAY__244 0)) (=> (= x__ARRAY__244 0) (not BOOL____01387)))))
(assert (and (and (=> BOOL____01389 (= x__ARRAY__245 1)) (=> (= x__ARRAY__245 1) BOOL____01389)) (and (=> (not BOOL____01389) (= x__ARRAY__245 0)) (=> (= x__ARRAY__245 0) (not BOOL____01389)))))
(assert (and (and (=> BOOL____01391 (= x__ARRAY__246 1)) (=> (= x__ARRAY__246 1) BOOL____01391)) (and (=> (not BOOL____01391) (= x__ARRAY__246 0)) (=> (= x__ARRAY__246 0) (not BOOL____01391)))))
(assert (and (and (=> BOOL____01393 (= x__ARRAY__247 1)) (=> (= x__ARRAY__247 1) BOOL____01393)) (and (=> (not BOOL____01393) (= x__ARRAY__247 0)) (=> (= x__ARRAY__247 0) (not BOOL____01393)))))
(assert (and (and (=> BOOL____01395 (= x__ARRAY__248 1)) (=> (= x__ARRAY__248 1) BOOL____01395)) (and (=> (not BOOL____01395) (= x__ARRAY__248 0)) (=> (= x__ARRAY__248 0) (not BOOL____01395)))))
(assert (and (and (=> BOOL____01397 (= x__ARRAY__249 1)) (=> (= x__ARRAY__249 1) BOOL____01397)) (and (=> (not BOOL____01397) (= x__ARRAY__249 0)) (=> (= x__ARRAY__249 0) (not BOOL____01397)))))
(assert (and (and (=> BOOL____01399 (= x__ARRAY__250 1)) (=> (= x__ARRAY__250 1) BOOL____01399)) (and (=> (not BOOL____01399) (= x__ARRAY__250 0)) (=> (= x__ARRAY__250 0) (not BOOL____01399)))))
(assert (and (and (=> BOOL____01401 (= x__ARRAY__251 1)) (=> (= x__ARRAY__251 1) BOOL____01401)) (and (=> (not BOOL____01401) (= x__ARRAY__251 0)) (=> (= x__ARRAY__251 0) (not BOOL____01401)))))
(assert (and (and (=> BOOL____01403 (= x__ARRAY__252 1)) (=> (= x__ARRAY__252 1) BOOL____01403)) (and (=> (not BOOL____01403) (= x__ARRAY__252 0)) (=> (= x__ARRAY__252 0) (not BOOL____01403)))))
(assert (and (and (=> BOOL____01405 (= x__ARRAY__253 1)) (=> (= x__ARRAY__253 1) BOOL____01405)) (and (=> (not BOOL____01405) (= x__ARRAY__253 0)) (=> (= x__ARRAY__253 0) (not BOOL____01405)))))
(assert (and (and (=> BOOL____01407 (= x__ARRAY__254 1)) (=> (= x__ARRAY__254 1) BOOL____01407)) (and (=> (not BOOL____01407) (= x__ARRAY__254 0)) (=> (= x__ARRAY__254 0) (not BOOL____01407)))))
(assert (and (and (=> BOOL____01409 (= x__ARRAY__255 1)) (=> (= x__ARRAY__255 1) BOOL____01409)) (and (=> (not BOOL____01409) (= x__ARRAY__255 0)) (=> (= x__ARRAY__255 0) (not BOOL____01409)))))
(assert (and (and (=> BOOL____01411 (= x__ARRAY__256 1)) (=> (= x__ARRAY__256 1) BOOL____01411)) (and (=> (not BOOL____01411) (= x__ARRAY__256 0)) (=> (= x__ARRAY__256 0) (not BOOL____01411)))))
(assert (and (and (=> BOOL____01413 (= x__ARRAY__257 1)) (=> (= x__ARRAY__257 1) BOOL____01413)) (and (=> (not BOOL____01413) (= x__ARRAY__257 0)) (=> (= x__ARRAY__257 0) (not BOOL____01413)))))
(assert (and (and (=> BOOL____01415 (= x__ARRAY__258 1)) (=> (= x__ARRAY__258 1) BOOL____01415)) (and (=> (not BOOL____01415) (= x__ARRAY__258 0)) (=> (= x__ARRAY__258 0) (not BOOL____01415)))))
(assert (and (and (=> BOOL____01417 (= x__ARRAY__259 1)) (=> (= x__ARRAY__259 1) BOOL____01417)) (and (=> (not BOOL____01417) (= x__ARRAY__259 0)) (=> (= x__ARRAY__259 0) (not BOOL____01417)))))
(assert (and (and (=> BOOL____01419 (= x__ARRAY__260 1)) (=> (= x__ARRAY__260 1) BOOL____01419)) (and (=> (not BOOL____01419) (= x__ARRAY__260 0)) (=> (= x__ARRAY__260 0) (not BOOL____01419)))))
(assert (and (and (=> BOOL____01421 (= x__ARRAY__261 1)) (=> (= x__ARRAY__261 1) BOOL____01421)) (and (=> (not BOOL____01421) (= x__ARRAY__261 0)) (=> (= x__ARRAY__261 0) (not BOOL____01421)))))
(assert (and (and (=> BOOL____01423 (= x__ARRAY__262 1)) (=> (= x__ARRAY__262 1) BOOL____01423)) (and (=> (not BOOL____01423) (= x__ARRAY__262 0)) (=> (= x__ARRAY__262 0) (not BOOL____01423)))))
(assert (and (and (=> BOOL____01425 (= x__ARRAY__263 1)) (=> (= x__ARRAY__263 1) BOOL____01425)) (and (=> (not BOOL____01425) (= x__ARRAY__263 0)) (=> (= x__ARRAY__263 0) (not BOOL____01425)))))
(assert (and (and (=> BOOL____01427 (= x__ARRAY__264 1)) (=> (= x__ARRAY__264 1) BOOL____01427)) (and (=> (not BOOL____01427) (= x__ARRAY__264 0)) (=> (= x__ARRAY__264 0) (not BOOL____01427)))))
(assert (and (and (=> BOOL____01429 (= x__ARRAY__265 1)) (=> (= x__ARRAY__265 1) BOOL____01429)) (and (=> (not BOOL____01429) (= x__ARRAY__265 0)) (=> (= x__ARRAY__265 0) (not BOOL____01429)))))
(assert (and (and (=> BOOL____01431 (= x__ARRAY__266 1)) (=> (= x__ARRAY__266 1) BOOL____01431)) (and (=> (not BOOL____01431) (= x__ARRAY__266 0)) (=> (= x__ARRAY__266 0) (not BOOL____01431)))))
(assert (and (and (=> BOOL____01433 (= x__ARRAY__267 1)) (=> (= x__ARRAY__267 1) BOOL____01433)) (and (=> (not BOOL____01433) (= x__ARRAY__267 0)) (=> (= x__ARRAY__267 0) (not BOOL____01433)))))
(assert (and (and (=> BOOL____01435 (= x__ARRAY__268 1)) (=> (= x__ARRAY__268 1) BOOL____01435)) (and (=> (not BOOL____01435) (= x__ARRAY__268 0)) (=> (= x__ARRAY__268 0) (not BOOL____01435)))))
(assert (and (and (=> BOOL____01437 (= x__ARRAY__269 1)) (=> (= x__ARRAY__269 1) BOOL____01437)) (and (=> (not BOOL____01437) (= x__ARRAY__269 0)) (=> (= x__ARRAY__269 0) (not BOOL____01437)))))
(assert (and (and (=> BOOL____01439 (= x__ARRAY__270 1)) (=> (= x__ARRAY__270 1) BOOL____01439)) (and (=> (not BOOL____01439) (= x__ARRAY__270 0)) (=> (= x__ARRAY__270 0) (not BOOL____01439)))))
(assert (and (and (=> BOOL____01441 (= x__ARRAY__271 1)) (=> (= x__ARRAY__271 1) BOOL____01441)) (and (=> (not BOOL____01441) (= x__ARRAY__271 0)) (=> (= x__ARRAY__271 0) (not BOOL____01441)))))
(assert (and (and (=> BOOL____01443 (= x__ARRAY__272 1)) (=> (= x__ARRAY__272 1) BOOL____01443)) (and (=> (not BOOL____01443) (= x__ARRAY__272 0)) (=> (= x__ARRAY__272 0) (not BOOL____01443)))))
(assert (and (and (=> BOOL____01445 (= x__ARRAY__273 1)) (=> (= x__ARRAY__273 1) BOOL____01445)) (and (=> (not BOOL____01445) (= x__ARRAY__273 0)) (=> (= x__ARRAY__273 0) (not BOOL____01445)))))
(assert (and (and (=> BOOL____01447 (= x__ARRAY__274 1)) (=> (= x__ARRAY__274 1) BOOL____01447)) (and (=> (not BOOL____01447) (= x__ARRAY__274 0)) (=> (= x__ARRAY__274 0) (not BOOL____01447)))))
(assert (and (and (=> BOOL____01449 (= x__ARRAY__275 1)) (=> (= x__ARRAY__275 1) BOOL____01449)) (and (=> (not BOOL____01449) (= x__ARRAY__275 0)) (=> (= x__ARRAY__275 0) (not BOOL____01449)))))
(assert (and (and (=> BOOL____01451 (= x__ARRAY__276 1)) (=> (= x__ARRAY__276 1) BOOL____01451)) (and (=> (not BOOL____01451) (= x__ARRAY__276 0)) (=> (= x__ARRAY__276 0) (not BOOL____01451)))))
(assert (and (and (=> BOOL____01453 (= x__ARRAY__277 1)) (=> (= x__ARRAY__277 1) BOOL____01453)) (and (=> (not BOOL____01453) (= x__ARRAY__277 0)) (=> (= x__ARRAY__277 0) (not BOOL____01453)))))
(assert (and (and (=> BOOL____01455 (= x__ARRAY__278 1)) (=> (= x__ARRAY__278 1) BOOL____01455)) (and (=> (not BOOL____01455) (= x__ARRAY__278 0)) (=> (= x__ARRAY__278 0) (not BOOL____01455)))))
(assert (and (and (=> BOOL____01457 (= x__ARRAY__279 1)) (=> (= x__ARRAY__279 1) BOOL____01457)) (and (=> (not BOOL____01457) (= x__ARRAY__279 0)) (=> (= x__ARRAY__279 0) (not BOOL____01457)))))
(assert (and (and (=> BOOL____01459 (= x__ARRAY__280 1)) (=> (= x__ARRAY__280 1) BOOL____01459)) (and (=> (not BOOL____01459) (= x__ARRAY__280 0)) (=> (= x__ARRAY__280 0) (not BOOL____01459)))))
(assert (and (and (=> BOOL____01461 (= x__ARRAY__281 1)) (=> (= x__ARRAY__281 1) BOOL____01461)) (and (=> (not BOOL____01461) (= x__ARRAY__281 0)) (=> (= x__ARRAY__281 0) (not BOOL____01461)))))
(assert (and (and (=> BOOL____01463 (= x__ARRAY__282 1)) (=> (= x__ARRAY__282 1) BOOL____01463)) (and (=> (not BOOL____01463) (= x__ARRAY__282 0)) (=> (= x__ARRAY__282 0) (not BOOL____01463)))))
(assert (and (and (=> BOOL____01465 (= x__ARRAY__283 1)) (=> (= x__ARRAY__283 1) BOOL____01465)) (and (=> (not BOOL____01465) (= x__ARRAY__283 0)) (=> (= x__ARRAY__283 0) (not BOOL____01465)))))
(assert (and (and (=> BOOL____01467 (= x__ARRAY__284 1)) (=> (= x__ARRAY__284 1) BOOL____01467)) (and (=> (not BOOL____01467) (= x__ARRAY__284 0)) (=> (= x__ARRAY__284 0) (not BOOL____01467)))))
(assert (and (and (=> BOOL____01469 (= x__ARRAY__285 1)) (=> (= x__ARRAY__285 1) BOOL____01469)) (and (=> (not BOOL____01469) (= x__ARRAY__285 0)) (=> (= x__ARRAY__285 0) (not BOOL____01469)))))
(assert (and (and (=> BOOL____01471 (= x__ARRAY__286 1)) (=> (= x__ARRAY__286 1) BOOL____01471)) (and (=> (not BOOL____01471) (= x__ARRAY__286 0)) (=> (= x__ARRAY__286 0) (not BOOL____01471)))))
(assert (and (and (=> BOOL____01473 (= x__ARRAY__287 1)) (=> (= x__ARRAY__287 1) BOOL____01473)) (and (=> (not BOOL____01473) (= x__ARRAY__287 0)) (=> (= x__ARRAY__287 0) (not BOOL____01473)))))
(assert (and (and (=> BOOL____01475 (= x__ARRAY__288 1)) (=> (= x__ARRAY__288 1) BOOL____01475)) (and (=> (not BOOL____01475) (= x__ARRAY__288 0)) (=> (= x__ARRAY__288 0) (not BOOL____01475)))))
(assert (and (and (=> BOOL____01477 (= x__ARRAY__289 1)) (=> (= x__ARRAY__289 1) BOOL____01477)) (and (=> (not BOOL____01477) (= x__ARRAY__289 0)) (=> (= x__ARRAY__289 0) (not BOOL____01477)))))
(assert (and (and (=> BOOL____01479 (= x__ARRAY__290 1)) (=> (= x__ARRAY__290 1) BOOL____01479)) (and (=> (not BOOL____01479) (= x__ARRAY__290 0)) (=> (= x__ARRAY__290 0) (not BOOL____01479)))))
(assert (and (and (=> BOOL____01481 (= x__ARRAY__291 1)) (=> (= x__ARRAY__291 1) BOOL____01481)) (and (=> (not BOOL____01481) (= x__ARRAY__291 0)) (=> (= x__ARRAY__291 0) (not BOOL____01481)))))
(assert (and (and (=> BOOL____01483 (= x__ARRAY__292 1)) (=> (= x__ARRAY__292 1) BOOL____01483)) (and (=> (not BOOL____01483) (= x__ARRAY__292 0)) (=> (= x__ARRAY__292 0) (not BOOL____01483)))))
(assert (and (and (=> BOOL____01485 (= x__ARRAY__293 1)) (=> (= x__ARRAY__293 1) BOOL____01485)) (and (=> (not BOOL____01485) (= x__ARRAY__293 0)) (=> (= x__ARRAY__293 0) (not BOOL____01485)))))
(assert (and (and (=> BOOL____01487 (= x__ARRAY__294 1)) (=> (= x__ARRAY__294 1) BOOL____01487)) (and (=> (not BOOL____01487) (= x__ARRAY__294 0)) (=> (= x__ARRAY__294 0) (not BOOL____01487)))))
(assert (and (and (=> BOOL____01489 (= x__ARRAY__295 1)) (=> (= x__ARRAY__295 1) BOOL____01489)) (and (=> (not BOOL____01489) (= x__ARRAY__295 0)) (=> (= x__ARRAY__295 0) (not BOOL____01489)))))
(assert (and (and (=> BOOL____01491 (= x__ARRAY__296 1)) (=> (= x__ARRAY__296 1) BOOL____01491)) (and (=> (not BOOL____01491) (= x__ARRAY__296 0)) (=> (= x__ARRAY__296 0) (not BOOL____01491)))))
(assert (and (and (=> BOOL____01493 (= x__ARRAY__297 1)) (=> (= x__ARRAY__297 1) BOOL____01493)) (and (=> (not BOOL____01493) (= x__ARRAY__297 0)) (=> (= x__ARRAY__297 0) (not BOOL____01493)))))
(assert (and (and (=> BOOL____01495 (= x__ARRAY__298 1)) (=> (= x__ARRAY__298 1) BOOL____01495)) (and (=> (not BOOL____01495) (= x__ARRAY__298 0)) (=> (= x__ARRAY__298 0) (not BOOL____01495)))))
(assert (and (and (=> BOOL____01497 (= x__ARRAY__299 1)) (=> (= x__ARRAY__299 1) BOOL____01497)) (and (=> (not BOOL____01497) (= x__ARRAY__299 0)) (=> (= x__ARRAY__299 0) (not BOOL____01497)))))
(assert (and (and (=> BOOL____01499 (= x__ARRAY__300 1)) (=> (= x__ARRAY__300 1) BOOL____01499)) (and (=> (not BOOL____01499) (= x__ARRAY__300 0)) (=> (= x__ARRAY__300 0) (not BOOL____01499)))))
(assert (and (and (=> BOOL____01501 (= x__ARRAY__301 1)) (=> (= x__ARRAY__301 1) BOOL____01501)) (and (=> (not BOOL____01501) (= x__ARRAY__301 0)) (=> (= x__ARRAY__301 0) (not BOOL____01501)))))
(assert (and (and (=> BOOL____01503 (= x__ARRAY__302 1)) (=> (= x__ARRAY__302 1) BOOL____01503)) (and (=> (not BOOL____01503) (= x__ARRAY__302 0)) (=> (= x__ARRAY__302 0) (not BOOL____01503)))))
(assert (and (and (=> BOOL____01505 (= x__ARRAY__303 1)) (=> (= x__ARRAY__303 1) BOOL____01505)) (and (=> (not BOOL____01505) (= x__ARRAY__303 0)) (=> (= x__ARRAY__303 0) (not BOOL____01505)))))
(assert (and (and (=> BOOL____01507 (= x__ARRAY__304 1)) (=> (= x__ARRAY__304 1) BOOL____01507)) (and (=> (not BOOL____01507) (= x__ARRAY__304 0)) (=> (= x__ARRAY__304 0) (not BOOL____01507)))))
(assert (and (and (=> BOOL____01509 (= x__ARRAY__305 1)) (=> (= x__ARRAY__305 1) BOOL____01509)) (and (=> (not BOOL____01509) (= x__ARRAY__305 0)) (=> (= x__ARRAY__305 0) (not BOOL____01509)))))
(assert (and (and (=> BOOL____01511 (= x__ARRAY__306 1)) (=> (= x__ARRAY__306 1) BOOL____01511)) (and (=> (not BOOL____01511) (= x__ARRAY__306 0)) (=> (= x__ARRAY__306 0) (not BOOL____01511)))))
(assert (and (and (=> BOOL____01513 (= x__ARRAY__307 1)) (=> (= x__ARRAY__307 1) BOOL____01513)) (and (=> (not BOOL____01513) (= x__ARRAY__307 0)) (=> (= x__ARRAY__307 0) (not BOOL____01513)))))
(assert (and (and (=> BOOL____01515 (= x__ARRAY__308 1)) (=> (= x__ARRAY__308 1) BOOL____01515)) (and (=> (not BOOL____01515) (= x__ARRAY__308 0)) (=> (= x__ARRAY__308 0) (not BOOL____01515)))))
(assert (and (and (=> BOOL____01517 (= x__ARRAY__309 1)) (=> (= x__ARRAY__309 1) BOOL____01517)) (and (=> (not BOOL____01517) (= x__ARRAY__309 0)) (=> (= x__ARRAY__309 0) (not BOOL____01517)))))
(assert (and (and (=> BOOL____01519 (= x__ARRAY__310 1)) (=> (= x__ARRAY__310 1) BOOL____01519)) (and (=> (not BOOL____01519) (= x__ARRAY__310 0)) (=> (= x__ARRAY__310 0) (not BOOL____01519)))))
(assert (and (and (=> BOOL____01521 (= x__ARRAY__311 1)) (=> (= x__ARRAY__311 1) BOOL____01521)) (and (=> (not BOOL____01521) (= x__ARRAY__311 0)) (=> (= x__ARRAY__311 0) (not BOOL____01521)))))
(assert (and (and (=> BOOL____01523 (= x__ARRAY__312 1)) (=> (= x__ARRAY__312 1) BOOL____01523)) (and (=> (not BOOL____01523) (= x__ARRAY__312 0)) (=> (= x__ARRAY__312 0) (not BOOL____01523)))))
(assert (and (and (=> BOOL____01525 (= x__ARRAY__313 1)) (=> (= x__ARRAY__313 1) BOOL____01525)) (and (=> (not BOOL____01525) (= x__ARRAY__313 0)) (=> (= x__ARRAY__313 0) (not BOOL____01525)))))
(assert (and (and (=> BOOL____01527 (= x__ARRAY__314 1)) (=> (= x__ARRAY__314 1) BOOL____01527)) (and (=> (not BOOL____01527) (= x__ARRAY__314 0)) (=> (= x__ARRAY__314 0) (not BOOL____01527)))))
(assert (and (and (=> BOOL____01529 (= x__ARRAY__315 1)) (=> (= x__ARRAY__315 1) BOOL____01529)) (and (=> (not BOOL____01529) (= x__ARRAY__315 0)) (=> (= x__ARRAY__315 0) (not BOOL____01529)))))
(assert (and (and (=> BOOL____01531 (= x__ARRAY__316 1)) (=> (= x__ARRAY__316 1) BOOL____01531)) (and (=> (not BOOL____01531) (= x__ARRAY__316 0)) (=> (= x__ARRAY__316 0) (not BOOL____01531)))))
(assert (and (and (=> BOOL____01533 (= x__ARRAY__317 1)) (=> (= x__ARRAY__317 1) BOOL____01533)) (and (=> (not BOOL____01533) (= x__ARRAY__317 0)) (=> (= x__ARRAY__317 0) (not BOOL____01533)))))
(assert (and (and (=> BOOL____01535 (= x__ARRAY__318 1)) (=> (= x__ARRAY__318 1) BOOL____01535)) (and (=> (not BOOL____01535) (= x__ARRAY__318 0)) (=> (= x__ARRAY__318 0) (not BOOL____01535)))))
(assert (and (and (=> BOOL____01537 (= x__ARRAY__319 1)) (=> (= x__ARRAY__319 1) BOOL____01537)) (and (=> (not BOOL____01537) (= x__ARRAY__319 0)) (=> (= x__ARRAY__319 0) (not BOOL____01537)))))
(assert (and (and (=> BOOL____01539 (= x__ARRAY__320 1)) (=> (= x__ARRAY__320 1) BOOL____01539)) (and (=> (not BOOL____01539) (= x__ARRAY__320 0)) (=> (= x__ARRAY__320 0) (not BOOL____01539)))))
(assert (and (and (=> BOOL____01541 (= x__ARRAY__321 1)) (=> (= x__ARRAY__321 1) BOOL____01541)) (and (=> (not BOOL____01541) (= x__ARRAY__321 0)) (=> (= x__ARRAY__321 0) (not BOOL____01541)))))
(assert (and (and (=> BOOL____01543 (= x__ARRAY__322 1)) (=> (= x__ARRAY__322 1) BOOL____01543)) (and (=> (not BOOL____01543) (= x__ARRAY__322 0)) (=> (= x__ARRAY__322 0) (not BOOL____01543)))))
(assert (and (and (=> BOOL____01545 (= x__ARRAY__323 1)) (=> (= x__ARRAY__323 1) BOOL____01545)) (and (=> (not BOOL____01545) (= x__ARRAY__323 0)) (=> (= x__ARRAY__323 0) (not BOOL____01545)))))
(assert (and (and (=> BOOL____01547 (= x__ARRAY__324 1)) (=> (= x__ARRAY__324 1) BOOL____01547)) (and (=> (not BOOL____01547) (= x__ARRAY__324 0)) (=> (= x__ARRAY__324 0) (not BOOL____01547)))))
(assert (and (and (=> BOOL____01549 (= x__ARRAY__325 1)) (=> (= x__ARRAY__325 1) BOOL____01549)) (and (=> (not BOOL____01549) (= x__ARRAY__325 0)) (=> (= x__ARRAY__325 0) (not BOOL____01549)))))
(assert (and (and (=> BOOL____01551 (= x__ARRAY__326 1)) (=> (= x__ARRAY__326 1) BOOL____01551)) (and (=> (not BOOL____01551) (= x__ARRAY__326 0)) (=> (= x__ARRAY__326 0) (not BOOL____01551)))))
(assert (and (and (=> BOOL____01553 (= x__ARRAY__327 1)) (=> (= x__ARRAY__327 1) BOOL____01553)) (and (=> (not BOOL____01553) (= x__ARRAY__327 0)) (=> (= x__ARRAY__327 0) (not BOOL____01553)))))
(assert (and (and (=> BOOL____01555 (= x__ARRAY__328 1)) (=> (= x__ARRAY__328 1) BOOL____01555)) (and (=> (not BOOL____01555) (= x__ARRAY__328 0)) (=> (= x__ARRAY__328 0) (not BOOL____01555)))))
(assert (and (and (=> BOOL____01557 (= x__ARRAY__329 1)) (=> (= x__ARRAY__329 1) BOOL____01557)) (and (=> (not BOOL____01557) (= x__ARRAY__329 0)) (=> (= x__ARRAY__329 0) (not BOOL____01557)))))
(assert (and (and (=> BOOL____01559 (= x__ARRAY__330 1)) (=> (= x__ARRAY__330 1) BOOL____01559)) (and (=> (not BOOL____01559) (= x__ARRAY__330 0)) (=> (= x__ARRAY__330 0) (not BOOL____01559)))))
(assert (and (and (=> BOOL____01561 (= x__ARRAY__331 1)) (=> (= x__ARRAY__331 1) BOOL____01561)) (and (=> (not BOOL____01561) (= x__ARRAY__331 0)) (=> (= x__ARRAY__331 0) (not BOOL____01561)))))
(assert (and (and (=> BOOL____01563 (= x__ARRAY__332 1)) (=> (= x__ARRAY__332 1) BOOL____01563)) (and (=> (not BOOL____01563) (= x__ARRAY__332 0)) (=> (= x__ARRAY__332 0) (not BOOL____01563)))))
(assert (and (and (=> BOOL____01565 (= x__ARRAY__333 1)) (=> (= x__ARRAY__333 1) BOOL____01565)) (and (=> (not BOOL____01565) (= x__ARRAY__333 0)) (=> (= x__ARRAY__333 0) (not BOOL____01565)))))
(assert (and (and (=> BOOL____01567 (= x__ARRAY__334 1)) (=> (= x__ARRAY__334 1) BOOL____01567)) (and (=> (not BOOL____01567) (= x__ARRAY__334 0)) (=> (= x__ARRAY__334 0) (not BOOL____01567)))))
(assert (and (and (=> BOOL____01569 (= x__ARRAY__335 1)) (=> (= x__ARRAY__335 1) BOOL____01569)) (and (=> (not BOOL____01569) (= x__ARRAY__335 0)) (=> (= x__ARRAY__335 0) (not BOOL____01569)))))
(assert (and (and (=> BOOL____01571 (= x__ARRAY__336 1)) (=> (= x__ARRAY__336 1) BOOL____01571)) (and (=> (not BOOL____01571) (= x__ARRAY__336 0)) (=> (= x__ARRAY__336 0) (not BOOL____01571)))))
(assert (and (and (=> BOOL____01573 (= x__ARRAY__337 1)) (=> (= x__ARRAY__337 1) BOOL____01573)) (and (=> (not BOOL____01573) (= x__ARRAY__337 0)) (=> (= x__ARRAY__337 0) (not BOOL____01573)))))
(assert (and (and (=> BOOL____01575 (= x__ARRAY__338 1)) (=> (= x__ARRAY__338 1) BOOL____01575)) (and (=> (not BOOL____01575) (= x__ARRAY__338 0)) (=> (= x__ARRAY__338 0) (not BOOL____01575)))))
(assert (and (and (=> BOOL____01577 (= x__ARRAY__339 1)) (=> (= x__ARRAY__339 1) BOOL____01577)) (and (=> (not BOOL____01577) (= x__ARRAY__339 0)) (=> (= x__ARRAY__339 0) (not BOOL____01577)))))
(assert (and (and (=> BOOL____01579 (= x__ARRAY__340 1)) (=> (= x__ARRAY__340 1) BOOL____01579)) (and (=> (not BOOL____01579) (= x__ARRAY__340 0)) (=> (= x__ARRAY__340 0) (not BOOL____01579)))))
(assert (and (and (=> BOOL____01581 (= x__ARRAY__341 1)) (=> (= x__ARRAY__341 1) BOOL____01581)) (and (=> (not BOOL____01581) (= x__ARRAY__341 0)) (=> (= x__ARRAY__341 0) (not BOOL____01581)))))
(assert (and (and (=> BOOL____01583 (= x__ARRAY__342 1)) (=> (= x__ARRAY__342 1) BOOL____01583)) (and (=> (not BOOL____01583) (= x__ARRAY__342 0)) (=> (= x__ARRAY__342 0) (not BOOL____01583)))))
(assert (and (and (=> BOOL____01585 (= x__ARRAY__343 1)) (=> (= x__ARRAY__343 1) BOOL____01585)) (and (=> (not BOOL____01585) (= x__ARRAY__343 0)) (=> (= x__ARRAY__343 0) (not BOOL____01585)))))
(assert (and (and (=> BOOL____01587 (= x__ARRAY__344 1)) (=> (= x__ARRAY__344 1) BOOL____01587)) (and (=> (not BOOL____01587) (= x__ARRAY__344 0)) (=> (= x__ARRAY__344 0) (not BOOL____01587)))))
(assert (and (and (=> BOOL____01589 (= x__ARRAY__345 1)) (=> (= x__ARRAY__345 1) BOOL____01589)) (and (=> (not BOOL____01589) (= x__ARRAY__345 0)) (=> (= x__ARRAY__345 0) (not BOOL____01589)))))
(assert (and (and (=> BOOL____01591 (= x__ARRAY__346 1)) (=> (= x__ARRAY__346 1) BOOL____01591)) (and (=> (not BOOL____01591) (= x__ARRAY__346 0)) (=> (= x__ARRAY__346 0) (not BOOL____01591)))))
(assert (and (and (=> BOOL____01593 (= x__ARRAY__347 1)) (=> (= x__ARRAY__347 1) BOOL____01593)) (and (=> (not BOOL____01593) (= x__ARRAY__347 0)) (=> (= x__ARRAY__347 0) (not BOOL____01593)))))
(assert (and (and (=> BOOL____01595 (= x__ARRAY__348 1)) (=> (= x__ARRAY__348 1) BOOL____01595)) (and (=> (not BOOL____01595) (= x__ARRAY__348 0)) (=> (= x__ARRAY__348 0) (not BOOL____01595)))))
(assert (and (and (=> BOOL____01597 (= x__ARRAY__349 1)) (=> (= x__ARRAY__349 1) BOOL____01597)) (and (=> (not BOOL____01597) (= x__ARRAY__349 0)) (=> (= x__ARRAY__349 0) (not BOOL____01597)))))
(assert (and (and (=> BOOL____01599 (= x__ARRAY__350 1)) (=> (= x__ARRAY__350 1) BOOL____01599)) (and (=> (not BOOL____01599) (= x__ARRAY__350 0)) (=> (= x__ARRAY__350 0) (not BOOL____01599)))))
(assert (and (and (=> BOOL____01601 (= x__ARRAY__351 1)) (=> (= x__ARRAY__351 1) BOOL____01601)) (and (=> (not BOOL____01601) (= x__ARRAY__351 0)) (=> (= x__ARRAY__351 0) (not BOOL____01601)))))
(assert (and (and (=> BOOL____01603 (= x__ARRAY__352 1)) (=> (= x__ARRAY__352 1) BOOL____01603)) (and (=> (not BOOL____01603) (= x__ARRAY__352 0)) (=> (= x__ARRAY__352 0) (not BOOL____01603)))))
(assert (and (and (=> BOOL____01605 (= x__ARRAY__353 1)) (=> (= x__ARRAY__353 1) BOOL____01605)) (and (=> (not BOOL____01605) (= x__ARRAY__353 0)) (=> (= x__ARRAY__353 0) (not BOOL____01605)))))
(assert (and (and (=> BOOL____01607 (= x__ARRAY__354 1)) (=> (= x__ARRAY__354 1) BOOL____01607)) (and (=> (not BOOL____01607) (= x__ARRAY__354 0)) (=> (= x__ARRAY__354 0) (not BOOL____01607)))))
(assert (and (and (=> BOOL____01609 (= x__ARRAY__355 1)) (=> (= x__ARRAY__355 1) BOOL____01609)) (and (=> (not BOOL____01609) (= x__ARRAY__355 0)) (=> (= x__ARRAY__355 0) (not BOOL____01609)))))
(assert (and (and (=> BOOL____01611 (= x__ARRAY__356 1)) (=> (= x__ARRAY__356 1) BOOL____01611)) (and (=> (not BOOL____01611) (= x__ARRAY__356 0)) (=> (= x__ARRAY__356 0) (not BOOL____01611)))))
(assert (and (and (=> BOOL____01613 (= x__ARRAY__357 1)) (=> (= x__ARRAY__357 1) BOOL____01613)) (and (=> (not BOOL____01613) (= x__ARRAY__357 0)) (=> (= x__ARRAY__357 0) (not BOOL____01613)))))
(assert (and (and (=> BOOL____01615 (= x__ARRAY__358 1)) (=> (= x__ARRAY__358 1) BOOL____01615)) (and (=> (not BOOL____01615) (= x__ARRAY__358 0)) (=> (= x__ARRAY__358 0) (not BOOL____01615)))))
(assert (and (and (=> BOOL____01617 (= x__ARRAY__359 1)) (=> (= x__ARRAY__359 1) BOOL____01617)) (and (=> (not BOOL____01617) (= x__ARRAY__359 0)) (=> (= x__ARRAY__359 0) (not BOOL____01617)))))
(assert (and (and (=> BOOL____01619 (= x__ARRAY__360 1)) (=> (= x__ARRAY__360 1) BOOL____01619)) (and (=> (not BOOL____01619) (= x__ARRAY__360 0)) (=> (= x__ARRAY__360 0) (not BOOL____01619)))))
(assert (and (and (=> BOOL____01621 (= x__ARRAY__361 1)) (=> (= x__ARRAY__361 1) BOOL____01621)) (and (=> (not BOOL____01621) (= x__ARRAY__361 0)) (=> (= x__ARRAY__361 0) (not BOOL____01621)))))
(assert (and (and (=> BOOL____01623 (= x__ARRAY__362 1)) (=> (= x__ARRAY__362 1) BOOL____01623)) (and (=> (not BOOL____01623) (= x__ARRAY__362 0)) (=> (= x__ARRAY__362 0) (not BOOL____01623)))))
(assert (and (and (=> BOOL____01625 (= x__ARRAY__363 1)) (=> (= x__ARRAY__363 1) BOOL____01625)) (and (=> (not BOOL____01625) (= x__ARRAY__363 0)) (=> (= x__ARRAY__363 0) (not BOOL____01625)))))
(assert (and (and (=> BOOL____01627 (= x__ARRAY__364 1)) (=> (= x__ARRAY__364 1) BOOL____01627)) (and (=> (not BOOL____01627) (= x__ARRAY__364 0)) (=> (= x__ARRAY__364 0) (not BOOL____01627)))))
(assert (and (and (=> BOOL____01629 (= x__ARRAY__365 1)) (=> (= x__ARRAY__365 1) BOOL____01629)) (and (=> (not BOOL____01629) (= x__ARRAY__365 0)) (=> (= x__ARRAY__365 0) (not BOOL____01629)))))
(assert (and (and (=> BOOL____01631 (= x__ARRAY__366 1)) (=> (= x__ARRAY__366 1) BOOL____01631)) (and (=> (not BOOL____01631) (= x__ARRAY__366 0)) (=> (= x__ARRAY__366 0) (not BOOL____01631)))))
(assert (and (and (=> BOOL____01633 (= x__ARRAY__367 1)) (=> (= x__ARRAY__367 1) BOOL____01633)) (and (=> (not BOOL____01633) (= x__ARRAY__367 0)) (=> (= x__ARRAY__367 0) (not BOOL____01633)))))
(assert (and (and (=> BOOL____01635 (= x__ARRAY__368 1)) (=> (= x__ARRAY__368 1) BOOL____01635)) (and (=> (not BOOL____01635) (= x__ARRAY__368 0)) (=> (= x__ARRAY__368 0) (not BOOL____01635)))))
(assert (and (and (=> BOOL____01637 (= x__ARRAY__369 1)) (=> (= x__ARRAY__369 1) BOOL____01637)) (and (=> (not BOOL____01637) (= x__ARRAY__369 0)) (=> (= x__ARRAY__369 0) (not BOOL____01637)))))
(assert (and (and (=> BOOL____01639 (= x__ARRAY__370 1)) (=> (= x__ARRAY__370 1) BOOL____01639)) (and (=> (not BOOL____01639) (= x__ARRAY__370 0)) (=> (= x__ARRAY__370 0) (not BOOL____01639)))))
(assert (and (and (=> BOOL____01641 (= x__ARRAY__371 1)) (=> (= x__ARRAY__371 1) BOOL____01641)) (and (=> (not BOOL____01641) (= x__ARRAY__371 0)) (=> (= x__ARRAY__371 0) (not BOOL____01641)))))
(assert (and (and (=> BOOL____01643 (= x__ARRAY__372 1)) (=> (= x__ARRAY__372 1) BOOL____01643)) (and (=> (not BOOL____01643) (= x__ARRAY__372 0)) (=> (= x__ARRAY__372 0) (not BOOL____01643)))))
(assert (and (and (=> BOOL____01645 (= x__ARRAY__373 1)) (=> (= x__ARRAY__373 1) BOOL____01645)) (and (=> (not BOOL____01645) (= x__ARRAY__373 0)) (=> (= x__ARRAY__373 0) (not BOOL____01645)))))
(assert (and (and (=> BOOL____01647 (= x__ARRAY__374 1)) (=> (= x__ARRAY__374 1) BOOL____01647)) (and (=> (not BOOL____01647) (= x__ARRAY__374 0)) (=> (= x__ARRAY__374 0) (not BOOL____01647)))))
(assert (and (and (=> BOOL____01649 (= x__ARRAY__375 1)) (=> (= x__ARRAY__375 1) BOOL____01649)) (and (=> (not BOOL____01649) (= x__ARRAY__375 0)) (=> (= x__ARRAY__375 0) (not BOOL____01649)))))
(assert (and (and (=> BOOL____01651 (= x__ARRAY__376 1)) (=> (= x__ARRAY__376 1) BOOL____01651)) (and (=> (not BOOL____01651) (= x__ARRAY__376 0)) (=> (= x__ARRAY__376 0) (not BOOL____01651)))))
(assert (and (and (=> BOOL____01653 (= x__ARRAY__377 1)) (=> (= x__ARRAY__377 1) BOOL____01653)) (and (=> (not BOOL____01653) (= x__ARRAY__377 0)) (=> (= x__ARRAY__377 0) (not BOOL____01653)))))
(assert (and (and (=> BOOL____01655 (= x__ARRAY__378 1)) (=> (= x__ARRAY__378 1) BOOL____01655)) (and (=> (not BOOL____01655) (= x__ARRAY__378 0)) (=> (= x__ARRAY__378 0) (not BOOL____01655)))))
(assert (and (and (=> BOOL____01657 (= x__ARRAY__379 1)) (=> (= x__ARRAY__379 1) BOOL____01657)) (and (=> (not BOOL____01657) (= x__ARRAY__379 0)) (=> (= x__ARRAY__379 0) (not BOOL____01657)))))
(assert (and (and (=> BOOL____01659 (= x__ARRAY__380 1)) (=> (= x__ARRAY__380 1) BOOL____01659)) (and (=> (not BOOL____01659) (= x__ARRAY__380 0)) (=> (= x__ARRAY__380 0) (not BOOL____01659)))))
(assert (and (and (=> BOOL____01661 (= x__ARRAY__381 1)) (=> (= x__ARRAY__381 1) BOOL____01661)) (and (=> (not BOOL____01661) (= x__ARRAY__381 0)) (=> (= x__ARRAY__381 0) (not BOOL____01661)))))
(assert (and (and (=> BOOL____01663 (= x__ARRAY__382 1)) (=> (= x__ARRAY__382 1) BOOL____01663)) (and (=> (not BOOL____01663) (= x__ARRAY__382 0)) (=> (= x__ARRAY__382 0) (not BOOL____01663)))))
(assert (and (and (=> BOOL____01665 (= x__ARRAY__383 1)) (=> (= x__ARRAY__383 1) BOOL____01665)) (and (=> (not BOOL____01665) (= x__ARRAY__383 0)) (=> (= x__ARRAY__383 0) (not BOOL____01665)))))
(assert (and (and (=> BOOL____01667 (= x__ARRAY__384 1)) (=> (= x__ARRAY__384 1) BOOL____01667)) (and (=> (not BOOL____01667) (= x__ARRAY__384 0)) (=> (= x__ARRAY__384 0) (not BOOL____01667)))))
(assert (and (and (=> BOOL____01669 (= x__ARRAY__385 1)) (=> (= x__ARRAY__385 1) BOOL____01669)) (and (=> (not BOOL____01669) (= x__ARRAY__385 0)) (=> (= x__ARRAY__385 0) (not BOOL____01669)))))
(assert (and (and (=> BOOL____01671 (= x__ARRAY__386 1)) (=> (= x__ARRAY__386 1) BOOL____01671)) (and (=> (not BOOL____01671) (= x__ARRAY__386 0)) (=> (= x__ARRAY__386 0) (not BOOL____01671)))))
(assert (and (and (=> BOOL____01673 (= x__ARRAY__387 1)) (=> (= x__ARRAY__387 1) BOOL____01673)) (and (=> (not BOOL____01673) (= x__ARRAY__387 0)) (=> (= x__ARRAY__387 0) (not BOOL____01673)))))
(assert (and (and (=> BOOL____01675 (= x__ARRAY__388 1)) (=> (= x__ARRAY__388 1) BOOL____01675)) (and (=> (not BOOL____01675) (= x__ARRAY__388 0)) (=> (= x__ARRAY__388 0) (not BOOL____01675)))))
(assert (and (and (=> BOOL____01677 (= x__ARRAY__389 1)) (=> (= x__ARRAY__389 1) BOOL____01677)) (and (=> (not BOOL____01677) (= x__ARRAY__389 0)) (=> (= x__ARRAY__389 0) (not BOOL____01677)))))
(assert (and (and (=> BOOL____01679 (= x__ARRAY__390 1)) (=> (= x__ARRAY__390 1) BOOL____01679)) (and (=> (not BOOL____01679) (= x__ARRAY__390 0)) (=> (= x__ARRAY__390 0) (not BOOL____01679)))))
(assert (and (and (=> BOOL____01681 (= x__ARRAY__391 1)) (=> (= x__ARRAY__391 1) BOOL____01681)) (and (=> (not BOOL____01681) (= x__ARRAY__391 0)) (=> (= x__ARRAY__391 0) (not BOOL____01681)))))
(assert (and (and (=> BOOL____01683 (= x__ARRAY__392 1)) (=> (= x__ARRAY__392 1) BOOL____01683)) (and (=> (not BOOL____01683) (= x__ARRAY__392 0)) (=> (= x__ARRAY__392 0) (not BOOL____01683)))))
(assert (and (and (=> BOOL____01685 (= x__ARRAY__393 1)) (=> (= x__ARRAY__393 1) BOOL____01685)) (and (=> (not BOOL____01685) (= x__ARRAY__393 0)) (=> (= x__ARRAY__393 0) (not BOOL____01685)))))
(assert (and (and (=> BOOL____01687 (= x__ARRAY__394 1)) (=> (= x__ARRAY__394 1) BOOL____01687)) (and (=> (not BOOL____01687) (= x__ARRAY__394 0)) (=> (= x__ARRAY__394 0) (not BOOL____01687)))))
(assert (and (and (=> BOOL____01689 (= x__ARRAY__395 1)) (=> (= x__ARRAY__395 1) BOOL____01689)) (and (=> (not BOOL____01689) (= x__ARRAY__395 0)) (=> (= x__ARRAY__395 0) (not BOOL____01689)))))
(assert (and (and (=> BOOL____01691 (= x__ARRAY__396 1)) (=> (= x__ARRAY__396 1) BOOL____01691)) (and (=> (not BOOL____01691) (= x__ARRAY__396 0)) (=> (= x__ARRAY__396 0) (not BOOL____01691)))))
(assert (and (and (=> BOOL____01693 (= x__ARRAY__397 1)) (=> (= x__ARRAY__397 1) BOOL____01693)) (and (=> (not BOOL____01693) (= x__ARRAY__397 0)) (=> (= x__ARRAY__397 0) (not BOOL____01693)))))
(assert (and (and (=> BOOL____01695 (= x__ARRAY__398 1)) (=> (= x__ARRAY__398 1) BOOL____01695)) (and (=> (not BOOL____01695) (= x__ARRAY__398 0)) (=> (= x__ARRAY__398 0) (not BOOL____01695)))))
(assert (and (and (=> BOOL____01697 (= x__ARRAY__399 1)) (=> (= x__ARRAY__399 1) BOOL____01697)) (and (=> (not BOOL____01697) (= x__ARRAY__399 0)) (=> (= x__ARRAY__399 0) (not BOOL____01697)))))
(assert (and (and (=> BOOL____01699 (= x__ARRAY__400 1)) (=> (= x__ARRAY__400 1) BOOL____01699)) (and (=> (not BOOL____01699) (= x__ARRAY__400 0)) (=> (= x__ARRAY__400 0) (not BOOL____01699)))))
(assert (and (and (=> BOOL____01701 (= x__ARRAY__401 1)) (=> (= x__ARRAY__401 1) BOOL____01701)) (and (=> (not BOOL____01701) (= x__ARRAY__401 0)) (=> (= x__ARRAY__401 0) (not BOOL____01701)))))
(assert (and (and (=> BOOL____01703 (= x__ARRAY__402 1)) (=> (= x__ARRAY__402 1) BOOL____01703)) (and (=> (not BOOL____01703) (= x__ARRAY__402 0)) (=> (= x__ARRAY__402 0) (not BOOL____01703)))))
(assert (and (and (=> BOOL____01705 (= x__ARRAY__403 1)) (=> (= x__ARRAY__403 1) BOOL____01705)) (and (=> (not BOOL____01705) (= x__ARRAY__403 0)) (=> (= x__ARRAY__403 0) (not BOOL____01705)))))
(assert (and (and (=> BOOL____01707 (= x__ARRAY__404 1)) (=> (= x__ARRAY__404 1) BOOL____01707)) (and (=> (not BOOL____01707) (= x__ARRAY__404 0)) (=> (= x__ARRAY__404 0) (not BOOL____01707)))))
(assert (and (and (=> BOOL____01709 (= x__ARRAY__405 1)) (=> (= x__ARRAY__405 1) BOOL____01709)) (and (=> (not BOOL____01709) (= x__ARRAY__405 0)) (=> (= x__ARRAY__405 0) (not BOOL____01709)))))
(assert (and (and (=> BOOL____01711 (= x__ARRAY__406 1)) (=> (= x__ARRAY__406 1) BOOL____01711)) (and (=> (not BOOL____01711) (= x__ARRAY__406 0)) (=> (= x__ARRAY__406 0) (not BOOL____01711)))))
(assert (and (and (=> BOOL____01713 (= x__ARRAY__407 1)) (=> (= x__ARRAY__407 1) BOOL____01713)) (and (=> (not BOOL____01713) (= x__ARRAY__407 0)) (=> (= x__ARRAY__407 0) (not BOOL____01713)))))
(assert (and (and (=> BOOL____01715 (= x__ARRAY__408 1)) (=> (= x__ARRAY__408 1) BOOL____01715)) (and (=> (not BOOL____01715) (= x__ARRAY__408 0)) (=> (= x__ARRAY__408 0) (not BOOL____01715)))))
(assert (and (and (=> BOOL____01717 (= x__ARRAY__409 1)) (=> (= x__ARRAY__409 1) BOOL____01717)) (and (=> (not BOOL____01717) (= x__ARRAY__409 0)) (=> (= x__ARRAY__409 0) (not BOOL____01717)))))
(assert (and (and (=> BOOL____01719 (= x__ARRAY__410 1)) (=> (= x__ARRAY__410 1) BOOL____01719)) (and (=> (not BOOL____01719) (= x__ARRAY__410 0)) (=> (= x__ARRAY__410 0) (not BOOL____01719)))))
(assert (and (and (=> BOOL____01721 (= x__ARRAY__411 1)) (=> (= x__ARRAY__411 1) BOOL____01721)) (and (=> (not BOOL____01721) (= x__ARRAY__411 0)) (=> (= x__ARRAY__411 0) (not BOOL____01721)))))
(assert (and (and (=> BOOL____01723 (= x__ARRAY__412 1)) (=> (= x__ARRAY__412 1) BOOL____01723)) (and (=> (not BOOL____01723) (= x__ARRAY__412 0)) (=> (= x__ARRAY__412 0) (not BOOL____01723)))))
(assert (and (and (=> BOOL____01725 (= x__ARRAY__413 1)) (=> (= x__ARRAY__413 1) BOOL____01725)) (and (=> (not BOOL____01725) (= x__ARRAY__413 0)) (=> (= x__ARRAY__413 0) (not BOOL____01725)))))
(assert (and (and (=> BOOL____01727 (= x__ARRAY__414 1)) (=> (= x__ARRAY__414 1) BOOL____01727)) (and (=> (not BOOL____01727) (= x__ARRAY__414 0)) (=> (= x__ARRAY__414 0) (not BOOL____01727)))))
(assert (and (and (=> BOOL____01729 (= x__ARRAY__415 1)) (=> (= x__ARRAY__415 1) BOOL____01729)) (and (=> (not BOOL____01729) (= x__ARRAY__415 0)) (=> (= x__ARRAY__415 0) (not BOOL____01729)))))
(assert (and (and (=> BOOL____01731 (= x__ARRAY__416 1)) (=> (= x__ARRAY__416 1) BOOL____01731)) (and (=> (not BOOL____01731) (= x__ARRAY__416 0)) (=> (= x__ARRAY__416 0) (not BOOL____01731)))))
(assert (and (and (=> BOOL____01733 (= x__ARRAY__417 1)) (=> (= x__ARRAY__417 1) BOOL____01733)) (and (=> (not BOOL____01733) (= x__ARRAY__417 0)) (=> (= x__ARRAY__417 0) (not BOOL____01733)))))
(assert (and (and (=> BOOL____01735 (= x__ARRAY__418 1)) (=> (= x__ARRAY__418 1) BOOL____01735)) (and (=> (not BOOL____01735) (= x__ARRAY__418 0)) (=> (= x__ARRAY__418 0) (not BOOL____01735)))))
(assert (and (and (=> BOOL____01737 (= x__ARRAY__419 1)) (=> (= x__ARRAY__419 1) BOOL____01737)) (and (=> (not BOOL____01737) (= x__ARRAY__419 0)) (=> (= x__ARRAY__419 0) (not BOOL____01737)))))
(assert (and (and (=> BOOL____01739 (= x__ARRAY__420 1)) (=> (= x__ARRAY__420 1) BOOL____01739)) (and (=> (not BOOL____01739) (= x__ARRAY__420 0)) (=> (= x__ARRAY__420 0) (not BOOL____01739)))))
(assert (and (and (=> BOOL____01741 (= x__ARRAY__421 1)) (=> (= x__ARRAY__421 1) BOOL____01741)) (and (=> (not BOOL____01741) (= x__ARRAY__421 0)) (=> (= x__ARRAY__421 0) (not BOOL____01741)))))
(assert (and (and (=> BOOL____01743 (= x__ARRAY__422 1)) (=> (= x__ARRAY__422 1) BOOL____01743)) (and (=> (not BOOL____01743) (= x__ARRAY__422 0)) (=> (= x__ARRAY__422 0) (not BOOL____01743)))))
(assert (and (and (=> BOOL____01745 (= x__ARRAY__423 1)) (=> (= x__ARRAY__423 1) BOOL____01745)) (and (=> (not BOOL____01745) (= x__ARRAY__423 0)) (=> (= x__ARRAY__423 0) (not BOOL____01745)))))
(assert (and (and (=> BOOL____01747 (= x__ARRAY__424 1)) (=> (= x__ARRAY__424 1) BOOL____01747)) (and (=> (not BOOL____01747) (= x__ARRAY__424 0)) (=> (= x__ARRAY__424 0) (not BOOL____01747)))))
(assert (and (and (=> BOOL____01749 (= x__ARRAY__425 1)) (=> (= x__ARRAY__425 1) BOOL____01749)) (and (=> (not BOOL____01749) (= x__ARRAY__425 0)) (=> (= x__ARRAY__425 0) (not BOOL____01749)))))
(assert (and (and (=> BOOL____01751 (= x__ARRAY__426 1)) (=> (= x__ARRAY__426 1) BOOL____01751)) (and (=> (not BOOL____01751) (= x__ARRAY__426 0)) (=> (= x__ARRAY__426 0) (not BOOL____01751)))))
(assert (and (and (=> BOOL____01753 (= x__ARRAY__427 1)) (=> (= x__ARRAY__427 1) BOOL____01753)) (and (=> (not BOOL____01753) (= x__ARRAY__427 0)) (=> (= x__ARRAY__427 0) (not BOOL____01753)))))
(assert (and (and (=> BOOL____01755 (= x__ARRAY__428 1)) (=> (= x__ARRAY__428 1) BOOL____01755)) (and (=> (not BOOL____01755) (= x__ARRAY__428 0)) (=> (= x__ARRAY__428 0) (not BOOL____01755)))))
(assert (and (and (=> BOOL____01757 (= x__ARRAY__429 1)) (=> (= x__ARRAY__429 1) BOOL____01757)) (and (=> (not BOOL____01757) (= x__ARRAY__429 0)) (=> (= x__ARRAY__429 0) (not BOOL____01757)))))
(assert (and (and (=> BOOL____01759 (= x__ARRAY__430 1)) (=> (= x__ARRAY__430 1) BOOL____01759)) (and (=> (not BOOL____01759) (= x__ARRAY__430 0)) (=> (= x__ARRAY__430 0) (not BOOL____01759)))))
(assert (and (and (=> BOOL____01761 (= x__ARRAY__431 1)) (=> (= x__ARRAY__431 1) BOOL____01761)) (and (=> (not BOOL____01761) (= x__ARRAY__431 0)) (=> (= x__ARRAY__431 0) (not BOOL____01761)))))
(assert (and (and (=> BOOL____01763 (= x__ARRAY__432 1)) (=> (= x__ARRAY__432 1) BOOL____01763)) (and (=> (not BOOL____01763) (= x__ARRAY__432 0)) (=> (= x__ARRAY__432 0) (not BOOL____01763)))))
(assert (and (and (=> BOOL____01765 (= x__ARRAY__433 1)) (=> (= x__ARRAY__433 1) BOOL____01765)) (and (=> (not BOOL____01765) (= x__ARRAY__433 0)) (=> (= x__ARRAY__433 0) (not BOOL____01765)))))
(assert (and (and (=> BOOL____01767 (= x__ARRAY__434 1)) (=> (= x__ARRAY__434 1) BOOL____01767)) (and (=> (not BOOL____01767) (= x__ARRAY__434 0)) (=> (= x__ARRAY__434 0) (not BOOL____01767)))))
(assert (and (and (=> BOOL____01769 (= x__ARRAY__435 1)) (=> (= x__ARRAY__435 1) BOOL____01769)) (and (=> (not BOOL____01769) (= x__ARRAY__435 0)) (=> (= x__ARRAY__435 0) (not BOOL____01769)))))
(assert (and (and (=> BOOL____01771 (= x__ARRAY__436 1)) (=> (= x__ARRAY__436 1) BOOL____01771)) (and (=> (not BOOL____01771) (= x__ARRAY__436 0)) (=> (= x__ARRAY__436 0) (not BOOL____01771)))))
(assert (and (and (=> BOOL____01773 (= x__ARRAY__437 1)) (=> (= x__ARRAY__437 1) BOOL____01773)) (and (=> (not BOOL____01773) (= x__ARRAY__437 0)) (=> (= x__ARRAY__437 0) (not BOOL____01773)))))
(assert (and (and (=> BOOL____01775 (= x__ARRAY__438 1)) (=> (= x__ARRAY__438 1) BOOL____01775)) (and (=> (not BOOL____01775) (= x__ARRAY__438 0)) (=> (= x__ARRAY__438 0) (not BOOL____01775)))))
(assert (and (and (=> BOOL____01777 (= x__ARRAY__439 1)) (=> (= x__ARRAY__439 1) BOOL____01777)) (and (=> (not BOOL____01777) (= x__ARRAY__439 0)) (=> (= x__ARRAY__439 0) (not BOOL____01777)))))
(assert (and (and (=> BOOL____01779 (= x__ARRAY__440 1)) (=> (= x__ARRAY__440 1) BOOL____01779)) (and (=> (not BOOL____01779) (= x__ARRAY__440 0)) (=> (= x__ARRAY__440 0) (not BOOL____01779)))))
(assert (and (and (=> BOOL____01781 (= x__ARRAY__441 1)) (=> (= x__ARRAY__441 1) BOOL____01781)) (and (=> (not BOOL____01781) (= x__ARRAY__441 0)) (=> (= x__ARRAY__441 0) (not BOOL____01781)))))
(assert (and (and (=> BOOL____01783 (= x__ARRAY__442 1)) (=> (= x__ARRAY__442 1) BOOL____01783)) (and (=> (not BOOL____01783) (= x__ARRAY__442 0)) (=> (= x__ARRAY__442 0) (not BOOL____01783)))))
(assert (and (and (=> BOOL____01785 (= x__ARRAY__443 1)) (=> (= x__ARRAY__443 1) BOOL____01785)) (and (=> (not BOOL____01785) (= x__ARRAY__443 0)) (=> (= x__ARRAY__443 0) (not BOOL____01785)))))
(assert (and (and (=> BOOL____01787 (= x__ARRAY__444 1)) (=> (= x__ARRAY__444 1) BOOL____01787)) (and (=> (not BOOL____01787) (= x__ARRAY__444 0)) (=> (= x__ARRAY__444 0) (not BOOL____01787)))))
(assert (and (and (=> BOOL____01789 (= x__ARRAY__445 1)) (=> (= x__ARRAY__445 1) BOOL____01789)) (and (=> (not BOOL____01789) (= x__ARRAY__445 0)) (=> (= x__ARRAY__445 0) (not BOOL____01789)))))
(assert (and (and (=> BOOL____01791 (= x__ARRAY__446 1)) (=> (= x__ARRAY__446 1) BOOL____01791)) (and (=> (not BOOL____01791) (= x__ARRAY__446 0)) (=> (= x__ARRAY__446 0) (not BOOL____01791)))))
(assert (and (and (=> BOOL____01793 (= x__ARRAY__447 1)) (=> (= x__ARRAY__447 1) BOOL____01793)) (and (=> (not BOOL____01793) (= x__ARRAY__447 0)) (=> (= x__ARRAY__447 0) (not BOOL____01793)))))
(assert (and (and (=> BOOL____01795 (= x__ARRAY__448 1)) (=> (= x__ARRAY__448 1) BOOL____01795)) (and (=> (not BOOL____01795) (= x__ARRAY__448 0)) (=> (= x__ARRAY__448 0) (not BOOL____01795)))))
(assert (and (and (=> BOOL____01797 (= x__ARRAY__449 1)) (=> (= x__ARRAY__449 1) BOOL____01797)) (and (=> (not BOOL____01797) (= x__ARRAY__449 0)) (=> (= x__ARRAY__449 0) (not BOOL____01797)))))
(assert (and (and (=> BOOL____01799 (= x__ARRAY__450 1)) (=> (= x__ARRAY__450 1) BOOL____01799)) (and (=> (not BOOL____01799) (= x__ARRAY__450 0)) (=> (= x__ARRAY__450 0) (not BOOL____01799)))))
(assert (and (and (=> BOOL____01801 (= x__ARRAY__451 1)) (=> (= x__ARRAY__451 1) BOOL____01801)) (and (=> (not BOOL____01801) (= x__ARRAY__451 0)) (=> (= x__ARRAY__451 0) (not BOOL____01801)))))
(assert (and (and (=> BOOL____01803 (= x__ARRAY__452 1)) (=> (= x__ARRAY__452 1) BOOL____01803)) (and (=> (not BOOL____01803) (= x__ARRAY__452 0)) (=> (= x__ARRAY__452 0) (not BOOL____01803)))))
(assert (and (and (=> BOOL____01805 (= x__ARRAY__453 1)) (=> (= x__ARRAY__453 1) BOOL____01805)) (and (=> (not BOOL____01805) (= x__ARRAY__453 0)) (=> (= x__ARRAY__453 0) (not BOOL____01805)))))
(assert (and (and (=> BOOL____01807 (= x__ARRAY__454 1)) (=> (= x__ARRAY__454 1) BOOL____01807)) (and (=> (not BOOL____01807) (= x__ARRAY__454 0)) (=> (= x__ARRAY__454 0) (not BOOL____01807)))))
(assert (and (and (=> BOOL____01809 (= x__ARRAY__455 1)) (=> (= x__ARRAY__455 1) BOOL____01809)) (and (=> (not BOOL____01809) (= x__ARRAY__455 0)) (=> (= x__ARRAY__455 0) (not BOOL____01809)))))
(assert (and (and (=> BOOL____01811 (= x__ARRAY__456 1)) (=> (= x__ARRAY__456 1) BOOL____01811)) (and (=> (not BOOL____01811) (= x__ARRAY__456 0)) (=> (= x__ARRAY__456 0) (not BOOL____01811)))))
(assert (and (and (=> BOOL____01813 (= x__ARRAY__457 1)) (=> (= x__ARRAY__457 1) BOOL____01813)) (and (=> (not BOOL____01813) (= x__ARRAY__457 0)) (=> (= x__ARRAY__457 0) (not BOOL____01813)))))
(assert (and (and (=> BOOL____01815 (= x__ARRAY__458 1)) (=> (= x__ARRAY__458 1) BOOL____01815)) (and (=> (not BOOL____01815) (= x__ARRAY__458 0)) (=> (= x__ARRAY__458 0) (not BOOL____01815)))))
(assert (and (and (=> BOOL____01817 (= x__ARRAY__459 1)) (=> (= x__ARRAY__459 1) BOOL____01817)) (and (=> (not BOOL____01817) (= x__ARRAY__459 0)) (=> (= x__ARRAY__459 0) (not BOOL____01817)))))
(assert (and (and (=> BOOL____01819 (= x__ARRAY__460 1)) (=> (= x__ARRAY__460 1) BOOL____01819)) (and (=> (not BOOL____01819) (= x__ARRAY__460 0)) (=> (= x__ARRAY__460 0) (not BOOL____01819)))))
(assert (and (and (=> BOOL____01821 (= x__ARRAY__461 1)) (=> (= x__ARRAY__461 1) BOOL____01821)) (and (=> (not BOOL____01821) (= x__ARRAY__461 0)) (=> (= x__ARRAY__461 0) (not BOOL____01821)))))
(assert (and (and (=> BOOL____01823 (= x__ARRAY__462 1)) (=> (= x__ARRAY__462 1) BOOL____01823)) (and (=> (not BOOL____01823) (= x__ARRAY__462 0)) (=> (= x__ARRAY__462 0) (not BOOL____01823)))))
(assert (and (and (=> BOOL____01825 (= x__ARRAY__463 1)) (=> (= x__ARRAY__463 1) BOOL____01825)) (and (=> (not BOOL____01825) (= x__ARRAY__463 0)) (=> (= x__ARRAY__463 0) (not BOOL____01825)))))
(assert (and (and (=> BOOL____01827 (= x__ARRAY__464 1)) (=> (= x__ARRAY__464 1) BOOL____01827)) (and (=> (not BOOL____01827) (= x__ARRAY__464 0)) (=> (= x__ARRAY__464 0) (not BOOL____01827)))))
(assert (and (and (=> BOOL____01829 (= x__ARRAY__465 1)) (=> (= x__ARRAY__465 1) BOOL____01829)) (and (=> (not BOOL____01829) (= x__ARRAY__465 0)) (=> (= x__ARRAY__465 0) (not BOOL____01829)))))
(assert (and (and (=> BOOL____01831 (= x__ARRAY__466 1)) (=> (= x__ARRAY__466 1) BOOL____01831)) (and (=> (not BOOL____01831) (= x__ARRAY__466 0)) (=> (= x__ARRAY__466 0) (not BOOL____01831)))))
(assert (and (and (=> BOOL____01833 (= x__ARRAY__467 1)) (=> (= x__ARRAY__467 1) BOOL____01833)) (and (=> (not BOOL____01833) (= x__ARRAY__467 0)) (=> (= x__ARRAY__467 0) (not BOOL____01833)))))
(assert (and (and (=> BOOL____01835 (= x__ARRAY__468 1)) (=> (= x__ARRAY__468 1) BOOL____01835)) (and (=> (not BOOL____01835) (= x__ARRAY__468 0)) (=> (= x__ARRAY__468 0) (not BOOL____01835)))))
(assert (and (and (=> BOOL____01837 (= x__ARRAY__469 1)) (=> (= x__ARRAY__469 1) BOOL____01837)) (and (=> (not BOOL____01837) (= x__ARRAY__469 0)) (=> (= x__ARRAY__469 0) (not BOOL____01837)))))
(assert (and (and (=> BOOL____01839 (= x__ARRAY__470 1)) (=> (= x__ARRAY__470 1) BOOL____01839)) (and (=> (not BOOL____01839) (= x__ARRAY__470 0)) (=> (= x__ARRAY__470 0) (not BOOL____01839)))))
(assert (and (and (=> BOOL____01841 (= x__ARRAY__471 1)) (=> (= x__ARRAY__471 1) BOOL____01841)) (and (=> (not BOOL____01841) (= x__ARRAY__471 0)) (=> (= x__ARRAY__471 0) (not BOOL____01841)))))
(assert (and (and (=> BOOL____01843 (= x__ARRAY__472 1)) (=> (= x__ARRAY__472 1) BOOL____01843)) (and (=> (not BOOL____01843) (= x__ARRAY__472 0)) (=> (= x__ARRAY__472 0) (not BOOL____01843)))))
(assert (and (and (=> BOOL____01845 (= x__ARRAY__473 1)) (=> (= x__ARRAY__473 1) BOOL____01845)) (and (=> (not BOOL____01845) (= x__ARRAY__473 0)) (=> (= x__ARRAY__473 0) (not BOOL____01845)))))
(assert (and (and (=> BOOL____01847 (= x__ARRAY__474 1)) (=> (= x__ARRAY__474 1) BOOL____01847)) (and (=> (not BOOL____01847) (= x__ARRAY__474 0)) (=> (= x__ARRAY__474 0) (not BOOL____01847)))))
(assert (and (and (=> BOOL____01849 (= x__ARRAY__475 1)) (=> (= x__ARRAY__475 1) BOOL____01849)) (and (=> (not BOOL____01849) (= x__ARRAY__475 0)) (=> (= x__ARRAY__475 0) (not BOOL____01849)))))
(assert (and (and (=> BOOL____01851 (= x__ARRAY__476 1)) (=> (= x__ARRAY__476 1) BOOL____01851)) (and (=> (not BOOL____01851) (= x__ARRAY__476 0)) (=> (= x__ARRAY__476 0) (not BOOL____01851)))))
(assert (and (and (=> BOOL____01853 (= x__ARRAY__477 1)) (=> (= x__ARRAY__477 1) BOOL____01853)) (and (=> (not BOOL____01853) (= x__ARRAY__477 0)) (=> (= x__ARRAY__477 0) (not BOOL____01853)))))
(assert (and (and (=> BOOL____01855 (= x__ARRAY__478 1)) (=> (= x__ARRAY__478 1) BOOL____01855)) (and (=> (not BOOL____01855) (= x__ARRAY__478 0)) (=> (= x__ARRAY__478 0) (not BOOL____01855)))))
(assert (and (and (=> BOOL____01857 (= x__ARRAY__479 1)) (=> (= x__ARRAY__479 1) BOOL____01857)) (and (=> (not BOOL____01857) (= x__ARRAY__479 0)) (=> (= x__ARRAY__479 0) (not BOOL____01857)))))
(assert (and (and (=> BOOL____01859 (= x__ARRAY__480 1)) (=> (= x__ARRAY__480 1) BOOL____01859)) (and (=> (not BOOL____01859) (= x__ARRAY__480 0)) (=> (= x__ARRAY__480 0) (not BOOL____01859)))))
(assert (and (and (=> BOOL____01861 (= x__ARRAY__481 1)) (=> (= x__ARRAY__481 1) BOOL____01861)) (and (=> (not BOOL____01861) (= x__ARRAY__481 0)) (=> (= x__ARRAY__481 0) (not BOOL____01861)))))
(assert (and (and (=> BOOL____01863 (= x__ARRAY__482 1)) (=> (= x__ARRAY__482 1) BOOL____01863)) (and (=> (not BOOL____01863) (= x__ARRAY__482 0)) (=> (= x__ARRAY__482 0) (not BOOL____01863)))))
(assert (and (and (=> BOOL____01865 (= x__ARRAY__483 1)) (=> (= x__ARRAY__483 1) BOOL____01865)) (and (=> (not BOOL____01865) (= x__ARRAY__483 0)) (=> (= x__ARRAY__483 0) (not BOOL____01865)))))
(assert (and (and (=> BOOL____01867 (= x__ARRAY__484 1)) (=> (= x__ARRAY__484 1) BOOL____01867)) (and (=> (not BOOL____01867) (= x__ARRAY__484 0)) (=> (= x__ARRAY__484 0) (not BOOL____01867)))))
(assert (and (and (=> BOOL____01869 (= x__ARRAY__485 1)) (=> (= x__ARRAY__485 1) BOOL____01869)) (and (=> (not BOOL____01869) (= x__ARRAY__485 0)) (=> (= x__ARRAY__485 0) (not BOOL____01869)))))
(assert (and (and (=> BOOL____01871 (= x__ARRAY__486 1)) (=> (= x__ARRAY__486 1) BOOL____01871)) (and (=> (not BOOL____01871) (= x__ARRAY__486 0)) (=> (= x__ARRAY__486 0) (not BOOL____01871)))))
(assert (and (and (=> BOOL____01873 (= x__ARRAY__487 1)) (=> (= x__ARRAY__487 1) BOOL____01873)) (and (=> (not BOOL____01873) (= x__ARRAY__487 0)) (=> (= x__ARRAY__487 0) (not BOOL____01873)))))
(assert (and (and (=> BOOL____01875 (= x__ARRAY__488 1)) (=> (= x__ARRAY__488 1) BOOL____01875)) (and (=> (not BOOL____01875) (= x__ARRAY__488 0)) (=> (= x__ARRAY__488 0) (not BOOL____01875)))))
(assert (and (and (=> BOOL____01877 (= x__ARRAY__489 1)) (=> (= x__ARRAY__489 1) BOOL____01877)) (and (=> (not BOOL____01877) (= x__ARRAY__489 0)) (=> (= x__ARRAY__489 0) (not BOOL____01877)))))
(assert (and (and (=> BOOL____01879 (= x__ARRAY__490 1)) (=> (= x__ARRAY__490 1) BOOL____01879)) (and (=> (not BOOL____01879) (= x__ARRAY__490 0)) (=> (= x__ARRAY__490 0) (not BOOL____01879)))))
(assert (and (and (=> BOOL____01881 (= x__ARRAY__491 1)) (=> (= x__ARRAY__491 1) BOOL____01881)) (and (=> (not BOOL____01881) (= x__ARRAY__491 0)) (=> (= x__ARRAY__491 0) (not BOOL____01881)))))
(assert (and (and (=> BOOL____01883 (= x__ARRAY__492 1)) (=> (= x__ARRAY__492 1) BOOL____01883)) (and (=> (not BOOL____01883) (= x__ARRAY__492 0)) (=> (= x__ARRAY__492 0) (not BOOL____01883)))))
(assert (and (and (=> BOOL____01885 (= x__ARRAY__493 1)) (=> (= x__ARRAY__493 1) BOOL____01885)) (and (=> (not BOOL____01885) (= x__ARRAY__493 0)) (=> (= x__ARRAY__493 0) (not BOOL____01885)))))
(assert (and (and (=> BOOL____01887 (= x__ARRAY__494 1)) (=> (= x__ARRAY__494 1) BOOL____01887)) (and (=> (not BOOL____01887) (= x__ARRAY__494 0)) (=> (= x__ARRAY__494 0) (not BOOL____01887)))))
(assert (and (and (=> BOOL____01889 (= x__ARRAY__495 1)) (=> (= x__ARRAY__495 1) BOOL____01889)) (and (=> (not BOOL____01889) (= x__ARRAY__495 0)) (=> (= x__ARRAY__495 0) (not BOOL____01889)))))
(assert (and (and (=> BOOL____01891 (= x__ARRAY__496 1)) (=> (= x__ARRAY__496 1) BOOL____01891)) (and (=> (not BOOL____01891) (= x__ARRAY__496 0)) (=> (= x__ARRAY__496 0) (not BOOL____01891)))))
(assert (and (and (=> BOOL____01893 (= x__ARRAY__497 1)) (=> (= x__ARRAY__497 1) BOOL____01893)) (and (=> (not BOOL____01893) (= x__ARRAY__497 0)) (=> (= x__ARRAY__497 0) (not BOOL____01893)))))
(assert (and (and (=> BOOL____01895 (= x__ARRAY__498 1)) (=> (= x__ARRAY__498 1) BOOL____01895)) (and (=> (not BOOL____01895) (= x__ARRAY__498 0)) (=> (= x__ARRAY__498 0) (not BOOL____01895)))))
(assert (and (and (=> BOOL____01897 (= x__ARRAY__499 1)) (=> (= x__ARRAY__499 1) BOOL____01897)) (and (=> (not BOOL____01897) (= x__ARRAY__499 0)) (=> (= x__ARRAY__499 0) (not BOOL____01897)))))
(assert (and (and (=> BOOL____01899 (= x__ARRAY__500 1)) (=> (= x__ARRAY__500 1) BOOL____01899)) (and (=> (not BOOL____01899) (= x__ARRAY__500 0)) (=> (= x__ARRAY__500 0) (not BOOL____01899)))))
(assert (and (=> (= course_period__ARRAY__1 1) BOOL____00901) (=> BOOL____00901 (= course_period__ARRAY__1 1))))
(assert (and (=> (= course_period__ARRAY__1 2) BOOL____01001) (=> BOOL____01001 (= course_period__ARRAY__1 2))))
(assert (and (=> (= course_period__ARRAY__1 3) BOOL____01101) (=> BOOL____01101 (= course_period__ARRAY__1 3))))
(assert (and (=> (= course_period__ARRAY__1 4) BOOL____01201) (=> BOOL____01201 (= course_period__ARRAY__1 4))))
(assert (and (=> (= course_period__ARRAY__1 5) BOOL____01301) (=> BOOL____01301 (= course_period__ARRAY__1 5))))
(assert (and (=> (= course_period__ARRAY__1 6) BOOL____01401) (=> BOOL____01401 (= course_period__ARRAY__1 6))))
(assert (and (=> (= course_period__ARRAY__1 7) BOOL____01501) (=> BOOL____01501 (= course_period__ARRAY__1 7))))
(assert (and (=> (= course_period__ARRAY__1 8) BOOL____01601) (=> BOOL____01601 (= course_period__ARRAY__1 8))))
(assert (and (=> (= course_period__ARRAY__1 9) BOOL____01701) (=> BOOL____01701 (= course_period__ARRAY__1 9))))
(assert (and (=> (= course_period__ARRAY__1 10) BOOL____01801) (=> BOOL____01801 (= course_period__ARRAY__1 10))))
(assert (and (=> (= course_period__ARRAY__2 1) BOOL____00903) (=> BOOL____00903 (= course_period__ARRAY__2 1))))
(assert (and (=> (= course_period__ARRAY__2 2) BOOL____01003) (=> BOOL____01003 (= course_period__ARRAY__2 2))))
(assert (and (=> (= course_period__ARRAY__2 3) BOOL____01103) (=> BOOL____01103 (= course_period__ARRAY__2 3))))
(assert (and (=> (= course_period__ARRAY__2 4) BOOL____01203) (=> BOOL____01203 (= course_period__ARRAY__2 4))))
(assert (and (=> (= course_period__ARRAY__2 5) BOOL____01303) (=> BOOL____01303 (= course_period__ARRAY__2 5))))
(assert (and (=> (= course_period__ARRAY__2 6) BOOL____01403) (=> BOOL____01403 (= course_period__ARRAY__2 6))))
(assert (and (=> (= course_period__ARRAY__2 7) BOOL____01503) (=> BOOL____01503 (= course_period__ARRAY__2 7))))
(assert (and (=> (= course_period__ARRAY__2 8) BOOL____01603) (=> BOOL____01603 (= course_period__ARRAY__2 8))))
(assert (and (=> (= course_period__ARRAY__2 9) BOOL____01703) (=> BOOL____01703 (= course_period__ARRAY__2 9))))
(assert (and (=> (= course_period__ARRAY__2 10) BOOL____01803) (=> BOOL____01803 (= course_period__ARRAY__2 10))))
(assert (and (=> (= course_period__ARRAY__3 1) BOOL____00905) (=> BOOL____00905 (= course_period__ARRAY__3 1))))
(assert (and (=> (= course_period__ARRAY__3 2) BOOL____01005) (=> BOOL____01005 (= course_period__ARRAY__3 2))))
(assert (and (=> (= course_period__ARRAY__3 3) BOOL____01105) (=> BOOL____01105 (= course_period__ARRAY__3 3))))
(assert (and (=> (= course_period__ARRAY__3 4) BOOL____01205) (=> BOOL____01205 (= course_period__ARRAY__3 4))))
(assert (and (=> (= course_period__ARRAY__3 5) BOOL____01305) (=> BOOL____01305 (= course_period__ARRAY__3 5))))
(assert (and (=> (= course_period__ARRAY__3 6) BOOL____01405) (=> BOOL____01405 (= course_period__ARRAY__3 6))))
(assert (and (=> (= course_period__ARRAY__3 7) BOOL____01505) (=> BOOL____01505 (= course_period__ARRAY__3 7))))
(assert (and (=> (= course_period__ARRAY__3 8) BOOL____01605) (=> BOOL____01605 (= course_period__ARRAY__3 8))))
(assert (and (=> (= course_period__ARRAY__3 9) BOOL____01705) (=> BOOL____01705 (= course_period__ARRAY__3 9))))
(assert (and (=> (= course_period__ARRAY__3 10) BOOL____01805) (=> BOOL____01805 (= course_period__ARRAY__3 10))))
(assert (and (=> (= course_period__ARRAY__4 1) BOOL____00907) (=> BOOL____00907 (= course_period__ARRAY__4 1))))
(assert (and (=> (= course_period__ARRAY__4 2) BOOL____01007) (=> BOOL____01007 (= course_period__ARRAY__4 2))))
(assert (and (=> (= course_period__ARRAY__4 3) BOOL____01107) (=> BOOL____01107 (= course_period__ARRAY__4 3))))
(assert (and (=> (= course_period__ARRAY__4 4) BOOL____01207) (=> BOOL____01207 (= course_period__ARRAY__4 4))))
(assert (and (=> (= course_period__ARRAY__4 5) BOOL____01307) (=> BOOL____01307 (= course_period__ARRAY__4 5))))
(assert (and (=> (= course_period__ARRAY__4 6) BOOL____01407) (=> BOOL____01407 (= course_period__ARRAY__4 6))))
(assert (and (=> (= course_period__ARRAY__4 7) BOOL____01507) (=> BOOL____01507 (= course_period__ARRAY__4 7))))
(assert (and (=> (= course_period__ARRAY__4 8) BOOL____01607) (=> BOOL____01607 (= course_period__ARRAY__4 8))))
(assert (and (=> (= course_period__ARRAY__4 9) BOOL____01707) (=> BOOL____01707 (= course_period__ARRAY__4 9))))
(assert (and (=> (= course_period__ARRAY__4 10) BOOL____01807) (=> BOOL____01807 (= course_period__ARRAY__4 10))))
(assert (and (=> (= course_period__ARRAY__5 1) BOOL____00909) (=> BOOL____00909 (= course_period__ARRAY__5 1))))
(assert (and (=> (= course_period__ARRAY__5 2) BOOL____01009) (=> BOOL____01009 (= course_period__ARRAY__5 2))))
(assert (and (=> (= course_period__ARRAY__5 3) BOOL____01109) (=> BOOL____01109 (= course_period__ARRAY__5 3))))
(assert (and (=> (= course_period__ARRAY__5 4) BOOL____01209) (=> BOOL____01209 (= course_period__ARRAY__5 4))))
(assert (and (=> (= course_period__ARRAY__5 5) BOOL____01309) (=> BOOL____01309 (= course_period__ARRAY__5 5))))
(assert (and (=> (= course_period__ARRAY__5 6) BOOL____01409) (=> BOOL____01409 (= course_period__ARRAY__5 6))))
(assert (and (=> (= course_period__ARRAY__5 7) BOOL____01509) (=> BOOL____01509 (= course_period__ARRAY__5 7))))
(assert (and (=> (= course_period__ARRAY__5 8) BOOL____01609) (=> BOOL____01609 (= course_period__ARRAY__5 8))))
(assert (and (=> (= course_period__ARRAY__5 9) BOOL____01709) (=> BOOL____01709 (= course_period__ARRAY__5 9))))
(assert (and (=> (= course_period__ARRAY__5 10) BOOL____01809) (=> BOOL____01809 (= course_period__ARRAY__5 10))))
(assert (and (=> (= course_period__ARRAY__6 1) BOOL____00911) (=> BOOL____00911 (= course_period__ARRAY__6 1))))
(assert (and (=> (= course_period__ARRAY__6 2) BOOL____01011) (=> BOOL____01011 (= course_period__ARRAY__6 2))))
(assert (and (=> (= course_period__ARRAY__6 3) BOOL____01111) (=> BOOL____01111 (= course_period__ARRAY__6 3))))
(assert (and (=> (= course_period__ARRAY__6 4) BOOL____01211) (=> BOOL____01211 (= course_period__ARRAY__6 4))))
(assert (and (=> (= course_period__ARRAY__6 5) BOOL____01311) (=> BOOL____01311 (= course_period__ARRAY__6 5))))
(assert (and (=> (= course_period__ARRAY__6 6) BOOL____01411) (=> BOOL____01411 (= course_period__ARRAY__6 6))))
(assert (and (=> (= course_period__ARRAY__6 7) BOOL____01511) (=> BOOL____01511 (= course_period__ARRAY__6 7))))
(assert (and (=> (= course_period__ARRAY__6 8) BOOL____01611) (=> BOOL____01611 (= course_period__ARRAY__6 8))))
(assert (and (=> (= course_period__ARRAY__6 9) BOOL____01711) (=> BOOL____01711 (= course_period__ARRAY__6 9))))
(assert (and (=> (= course_period__ARRAY__6 10) BOOL____01811) (=> BOOL____01811 (= course_period__ARRAY__6 10))))
(assert (and (=> (= course_period__ARRAY__7 1) BOOL____00913) (=> BOOL____00913 (= course_period__ARRAY__7 1))))
(assert (and (=> (= course_period__ARRAY__7 2) BOOL____01013) (=> BOOL____01013 (= course_period__ARRAY__7 2))))
(assert (and (=> (= course_period__ARRAY__7 3) BOOL____01113) (=> BOOL____01113 (= course_period__ARRAY__7 3))))
(assert (and (=> (= course_period__ARRAY__7 4) BOOL____01213) (=> BOOL____01213 (= course_period__ARRAY__7 4))))
(assert (and (=> (= course_period__ARRAY__7 5) BOOL____01313) (=> BOOL____01313 (= course_period__ARRAY__7 5))))
(assert (and (=> (= course_period__ARRAY__7 6) BOOL____01413) (=> BOOL____01413 (= course_period__ARRAY__7 6))))
(assert (and (=> (= course_period__ARRAY__7 7) BOOL____01513) (=> BOOL____01513 (= course_period__ARRAY__7 7))))
(assert (and (=> (= course_period__ARRAY__7 8) BOOL____01613) (=> BOOL____01613 (= course_period__ARRAY__7 8))))
(assert (and (=> (= course_period__ARRAY__7 9) BOOL____01713) (=> BOOL____01713 (= course_period__ARRAY__7 9))))
(assert (and (=> (= course_period__ARRAY__7 10) BOOL____01813) (=> BOOL____01813 (= course_period__ARRAY__7 10))))
(assert (and (=> (= course_period__ARRAY__8 1) BOOL____00915) (=> BOOL____00915 (= course_period__ARRAY__8 1))))
(assert (and (=> (= course_period__ARRAY__8 2) BOOL____01015) (=> BOOL____01015 (= course_period__ARRAY__8 2))))
(assert (and (=> (= course_period__ARRAY__8 3) BOOL____01115) (=> BOOL____01115 (= course_period__ARRAY__8 3))))
(assert (and (=> (= course_period__ARRAY__8 4) BOOL____01215) (=> BOOL____01215 (= course_period__ARRAY__8 4))))
(assert (and (=> (= course_period__ARRAY__8 5) BOOL____01315) (=> BOOL____01315 (= course_period__ARRAY__8 5))))
(assert (and (=> (= course_period__ARRAY__8 6) BOOL____01415) (=> BOOL____01415 (= course_period__ARRAY__8 6))))
(assert (and (=> (= course_period__ARRAY__8 7) BOOL____01515) (=> BOOL____01515 (= course_period__ARRAY__8 7))))
(assert (and (=> (= course_period__ARRAY__8 8) BOOL____01615) (=> BOOL____01615 (= course_period__ARRAY__8 8))))
(assert (and (=> (= course_period__ARRAY__8 9) BOOL____01715) (=> BOOL____01715 (= course_period__ARRAY__8 9))))
(assert (and (=> (= course_period__ARRAY__8 10) BOOL____01815) (=> BOOL____01815 (= course_period__ARRAY__8 10))))
(assert (and (=> (= course_period__ARRAY__9 1) BOOL____00917) (=> BOOL____00917 (= course_period__ARRAY__9 1))))
(assert (and (=> (= course_period__ARRAY__9 2) BOOL____01017) (=> BOOL____01017 (= course_period__ARRAY__9 2))))
(assert (and (=> (= course_period__ARRAY__9 3) BOOL____01117) (=> BOOL____01117 (= course_period__ARRAY__9 3))))
(assert (and (=> (= course_period__ARRAY__9 4) BOOL____01217) (=> BOOL____01217 (= course_period__ARRAY__9 4))))
(assert (and (=> (= course_period__ARRAY__9 5) BOOL____01317) (=> BOOL____01317 (= course_period__ARRAY__9 5))))
(assert (and (=> (= course_period__ARRAY__9 6) BOOL____01417) (=> BOOL____01417 (= course_period__ARRAY__9 6))))
(assert (and (=> (= course_period__ARRAY__9 7) BOOL____01517) (=> BOOL____01517 (= course_period__ARRAY__9 7))))
(assert (and (=> (= course_period__ARRAY__9 8) BOOL____01617) (=> BOOL____01617 (= course_period__ARRAY__9 8))))
(assert (and (=> (= course_period__ARRAY__9 9) BOOL____01717) (=> BOOL____01717 (= course_period__ARRAY__9 9))))
(assert (and (=> (= course_period__ARRAY__9 10) BOOL____01817) (=> BOOL____01817 (= course_period__ARRAY__9 10))))
(assert (and (=> (= course_period__ARRAY__10 1) BOOL____00919) (=> BOOL____00919 (= course_period__ARRAY__10 1))))
(assert (and (=> (= course_period__ARRAY__10 2) BOOL____01019) (=> BOOL____01019 (= course_period__ARRAY__10 2))))
(assert (and (=> (= course_period__ARRAY__10 3) BOOL____01119) (=> BOOL____01119 (= course_period__ARRAY__10 3))))
(assert (and (=> (= course_period__ARRAY__10 4) BOOL____01219) (=> BOOL____01219 (= course_period__ARRAY__10 4))))
(assert (and (=> (= course_period__ARRAY__10 5) BOOL____01319) (=> BOOL____01319 (= course_period__ARRAY__10 5))))
(assert (and (=> (= course_period__ARRAY__10 6) BOOL____01419) (=> BOOL____01419 (= course_period__ARRAY__10 6))))
(assert (and (=> (= course_period__ARRAY__10 7) BOOL____01519) (=> BOOL____01519 (= course_period__ARRAY__10 7))))
(assert (and (=> (= course_period__ARRAY__10 8) BOOL____01619) (=> BOOL____01619 (= course_period__ARRAY__10 8))))
(assert (and (=> (= course_period__ARRAY__10 9) BOOL____01719) (=> BOOL____01719 (= course_period__ARRAY__10 9))))
(assert (and (=> (= course_period__ARRAY__10 10) BOOL____01819) (=> BOOL____01819 (= course_period__ARRAY__10 10))))
(assert (and (=> (= course_period__ARRAY__11 1) BOOL____00921) (=> BOOL____00921 (= course_period__ARRAY__11 1))))
(assert (and (=> (= course_period__ARRAY__11 2) BOOL____01021) (=> BOOL____01021 (= course_period__ARRAY__11 2))))
(assert (and (=> (= course_period__ARRAY__11 3) BOOL____01121) (=> BOOL____01121 (= course_period__ARRAY__11 3))))
(assert (and (=> (= course_period__ARRAY__11 4) BOOL____01221) (=> BOOL____01221 (= course_period__ARRAY__11 4))))
(assert (and (=> (= course_period__ARRAY__11 5) BOOL____01321) (=> BOOL____01321 (= course_period__ARRAY__11 5))))
(assert (and (=> (= course_period__ARRAY__11 6) BOOL____01421) (=> BOOL____01421 (= course_period__ARRAY__11 6))))
(assert (and (=> (= course_period__ARRAY__11 7) BOOL____01521) (=> BOOL____01521 (= course_period__ARRAY__11 7))))
(assert (and (=> (= course_period__ARRAY__11 8) BOOL____01621) (=> BOOL____01621 (= course_period__ARRAY__11 8))))
(assert (and (=> (= course_period__ARRAY__11 9) BOOL____01721) (=> BOOL____01721 (= course_period__ARRAY__11 9))))
(assert (and (=> (= course_period__ARRAY__11 10) BOOL____01821) (=> BOOL____01821 (= course_period__ARRAY__11 10))))
(assert (and (=> (= course_period__ARRAY__12 1) BOOL____00923) (=> BOOL____00923 (= course_period__ARRAY__12 1))))
(assert (and (=> (= course_period__ARRAY__12 2) BOOL____01023) (=> BOOL____01023 (= course_period__ARRAY__12 2))))
(assert (and (=> (= course_period__ARRAY__12 3) BOOL____01123) (=> BOOL____01123 (= course_period__ARRAY__12 3))))
(assert (and (=> (= course_period__ARRAY__12 4) BOOL____01223) (=> BOOL____01223 (= course_period__ARRAY__12 4))))
(assert (and (=> (= course_period__ARRAY__12 5) BOOL____01323) (=> BOOL____01323 (= course_period__ARRAY__12 5))))
(assert (and (=> (= course_period__ARRAY__12 6) BOOL____01423) (=> BOOL____01423 (= course_period__ARRAY__12 6))))
(assert (and (=> (= course_period__ARRAY__12 7) BOOL____01523) (=> BOOL____01523 (= course_period__ARRAY__12 7))))
(assert (and (=> (= course_period__ARRAY__12 8) BOOL____01623) (=> BOOL____01623 (= course_period__ARRAY__12 8))))
(assert (and (=> (= course_period__ARRAY__12 9) BOOL____01723) (=> BOOL____01723 (= course_period__ARRAY__12 9))))
(assert (and (=> (= course_period__ARRAY__12 10) BOOL____01823) (=> BOOL____01823 (= course_period__ARRAY__12 10))))
(assert (and (=> (= course_period__ARRAY__13 1) BOOL____00925) (=> BOOL____00925 (= course_period__ARRAY__13 1))))
(assert (and (=> (= course_period__ARRAY__13 2) BOOL____01025) (=> BOOL____01025 (= course_period__ARRAY__13 2))))
(assert (and (=> (= course_period__ARRAY__13 3) BOOL____01125) (=> BOOL____01125 (= course_period__ARRAY__13 3))))
(assert (and (=> (= course_period__ARRAY__13 4) BOOL____01225) (=> BOOL____01225 (= course_period__ARRAY__13 4))))
(assert (and (=> (= course_period__ARRAY__13 5) BOOL____01325) (=> BOOL____01325 (= course_period__ARRAY__13 5))))
(assert (and (=> (= course_period__ARRAY__13 6) BOOL____01425) (=> BOOL____01425 (= course_period__ARRAY__13 6))))
(assert (and (=> (= course_period__ARRAY__13 7) BOOL____01525) (=> BOOL____01525 (= course_period__ARRAY__13 7))))
(assert (and (=> (= course_period__ARRAY__13 8) BOOL____01625) (=> BOOL____01625 (= course_period__ARRAY__13 8))))
(assert (and (=> (= course_period__ARRAY__13 9) BOOL____01725) (=> BOOL____01725 (= course_period__ARRAY__13 9))))
(assert (and (=> (= course_period__ARRAY__13 10) BOOL____01825) (=> BOOL____01825 (= course_period__ARRAY__13 10))))
(assert (and (=> (= course_period__ARRAY__14 1) BOOL____00927) (=> BOOL____00927 (= course_period__ARRAY__14 1))))
(assert (and (=> (= course_period__ARRAY__14 2) BOOL____01027) (=> BOOL____01027 (= course_period__ARRAY__14 2))))
(assert (and (=> (= course_period__ARRAY__14 3) BOOL____01127) (=> BOOL____01127 (= course_period__ARRAY__14 3))))
(assert (and (=> (= course_period__ARRAY__14 4) BOOL____01227) (=> BOOL____01227 (= course_period__ARRAY__14 4))))
(assert (and (=> (= course_period__ARRAY__14 5) BOOL____01327) (=> BOOL____01327 (= course_period__ARRAY__14 5))))
(assert (and (=> (= course_period__ARRAY__14 6) BOOL____01427) (=> BOOL____01427 (= course_period__ARRAY__14 6))))
(assert (and (=> (= course_period__ARRAY__14 7) BOOL____01527) (=> BOOL____01527 (= course_period__ARRAY__14 7))))
(assert (and (=> (= course_period__ARRAY__14 8) BOOL____01627) (=> BOOL____01627 (= course_period__ARRAY__14 8))))
(assert (and (=> (= course_period__ARRAY__14 9) BOOL____01727) (=> BOOL____01727 (= course_period__ARRAY__14 9))))
(assert (and (=> (= course_period__ARRAY__14 10) BOOL____01827) (=> BOOL____01827 (= course_period__ARRAY__14 10))))
(assert (and (=> (= course_period__ARRAY__15 1) BOOL____00929) (=> BOOL____00929 (= course_period__ARRAY__15 1))))
(assert (and (=> (= course_period__ARRAY__15 2) BOOL____01029) (=> BOOL____01029 (= course_period__ARRAY__15 2))))
(assert (and (=> (= course_period__ARRAY__15 3) BOOL____01129) (=> BOOL____01129 (= course_period__ARRAY__15 3))))
(assert (and (=> (= course_period__ARRAY__15 4) BOOL____01229) (=> BOOL____01229 (= course_period__ARRAY__15 4))))
(assert (and (=> (= course_period__ARRAY__15 5) BOOL____01329) (=> BOOL____01329 (= course_period__ARRAY__15 5))))
(assert (and (=> (= course_period__ARRAY__15 6) BOOL____01429) (=> BOOL____01429 (= course_period__ARRAY__15 6))))
(assert (and (=> (= course_period__ARRAY__15 7) BOOL____01529) (=> BOOL____01529 (= course_period__ARRAY__15 7))))
(assert (and (=> (= course_period__ARRAY__15 8) BOOL____01629) (=> BOOL____01629 (= course_period__ARRAY__15 8))))
(assert (and (=> (= course_period__ARRAY__15 9) BOOL____01729) (=> BOOL____01729 (= course_period__ARRAY__15 9))))
(assert (and (=> (= course_period__ARRAY__15 10) BOOL____01829) (=> BOOL____01829 (= course_period__ARRAY__15 10))))
(assert (and (=> (= course_period__ARRAY__16 1) BOOL____00931) (=> BOOL____00931 (= course_period__ARRAY__16 1))))
(assert (and (=> (= course_period__ARRAY__16 2) BOOL____01031) (=> BOOL____01031 (= course_period__ARRAY__16 2))))
(assert (and (=> (= course_period__ARRAY__16 3) BOOL____01131) (=> BOOL____01131 (= course_period__ARRAY__16 3))))
(assert (and (=> (= course_period__ARRAY__16 4) BOOL____01231) (=> BOOL____01231 (= course_period__ARRAY__16 4))))
(assert (and (=> (= course_period__ARRAY__16 5) BOOL____01331) (=> BOOL____01331 (= course_period__ARRAY__16 5))))
(assert (and (=> (= course_period__ARRAY__16 6) BOOL____01431) (=> BOOL____01431 (= course_period__ARRAY__16 6))))
(assert (and (=> (= course_period__ARRAY__16 7) BOOL____01531) (=> BOOL____01531 (= course_period__ARRAY__16 7))))
(assert (and (=> (= course_period__ARRAY__16 8) BOOL____01631) (=> BOOL____01631 (= course_period__ARRAY__16 8))))
(assert (and (=> (= course_period__ARRAY__16 9) BOOL____01731) (=> BOOL____01731 (= course_period__ARRAY__16 9))))
(assert (and (=> (= course_period__ARRAY__16 10) BOOL____01831) (=> BOOL____01831 (= course_period__ARRAY__16 10))))
(assert (and (=> (= course_period__ARRAY__17 1) BOOL____00933) (=> BOOL____00933 (= course_period__ARRAY__17 1))))
(assert (and (=> (= course_period__ARRAY__17 2) BOOL____01033) (=> BOOL____01033 (= course_period__ARRAY__17 2))))
(assert (and (=> (= course_period__ARRAY__17 3) BOOL____01133) (=> BOOL____01133 (= course_period__ARRAY__17 3))))
(assert (and (=> (= course_period__ARRAY__17 4) BOOL____01233) (=> BOOL____01233 (= course_period__ARRAY__17 4))))
(assert (and (=> (= course_period__ARRAY__17 5) BOOL____01333) (=> BOOL____01333 (= course_period__ARRAY__17 5))))
(assert (and (=> (= course_period__ARRAY__17 6) BOOL____01433) (=> BOOL____01433 (= course_period__ARRAY__17 6))))
(assert (and (=> (= course_period__ARRAY__17 7) BOOL____01533) (=> BOOL____01533 (= course_period__ARRAY__17 7))))
(assert (and (=> (= course_period__ARRAY__17 8) BOOL____01633) (=> BOOL____01633 (= course_period__ARRAY__17 8))))
(assert (and (=> (= course_period__ARRAY__17 9) BOOL____01733) (=> BOOL____01733 (= course_period__ARRAY__17 9))))
(assert (and (=> (= course_period__ARRAY__17 10) BOOL____01833) (=> BOOL____01833 (= course_period__ARRAY__17 10))))
(assert (and (=> (= course_period__ARRAY__18 1) BOOL____00935) (=> BOOL____00935 (= course_period__ARRAY__18 1))))
(assert (and (=> (= course_period__ARRAY__18 2) BOOL____01035) (=> BOOL____01035 (= course_period__ARRAY__18 2))))
(assert (and (=> (= course_period__ARRAY__18 3) BOOL____01135) (=> BOOL____01135 (= course_period__ARRAY__18 3))))
(assert (and (=> (= course_period__ARRAY__18 4) BOOL____01235) (=> BOOL____01235 (= course_period__ARRAY__18 4))))
(assert (and (=> (= course_period__ARRAY__18 5) BOOL____01335) (=> BOOL____01335 (= course_period__ARRAY__18 5))))
(assert (and (=> (= course_period__ARRAY__18 6) BOOL____01435) (=> BOOL____01435 (= course_period__ARRAY__18 6))))
(assert (and (=> (= course_period__ARRAY__18 7) BOOL____01535) (=> BOOL____01535 (= course_period__ARRAY__18 7))))
(assert (and (=> (= course_period__ARRAY__18 8) BOOL____01635) (=> BOOL____01635 (= course_period__ARRAY__18 8))))
(assert (and (=> (= course_period__ARRAY__18 9) BOOL____01735) (=> BOOL____01735 (= course_period__ARRAY__18 9))))
(assert (and (=> (= course_period__ARRAY__18 10) BOOL____01835) (=> BOOL____01835 (= course_period__ARRAY__18 10))))
(assert (and (=> (= course_period__ARRAY__19 1) BOOL____00937) (=> BOOL____00937 (= course_period__ARRAY__19 1))))
(assert (and (=> (= course_period__ARRAY__19 2) BOOL____01037) (=> BOOL____01037 (= course_period__ARRAY__19 2))))
(assert (and (=> (= course_period__ARRAY__19 3) BOOL____01137) (=> BOOL____01137 (= course_period__ARRAY__19 3))))
(assert (and (=> (= course_period__ARRAY__19 4) BOOL____01237) (=> BOOL____01237 (= course_period__ARRAY__19 4))))
(assert (and (=> (= course_period__ARRAY__19 5) BOOL____01337) (=> BOOL____01337 (= course_period__ARRAY__19 5))))
(assert (and (=> (= course_period__ARRAY__19 6) BOOL____01437) (=> BOOL____01437 (= course_period__ARRAY__19 6))))
(assert (and (=> (= course_period__ARRAY__19 7) BOOL____01537) (=> BOOL____01537 (= course_period__ARRAY__19 7))))
(assert (and (=> (= course_period__ARRAY__19 8) BOOL____01637) (=> BOOL____01637 (= course_period__ARRAY__19 8))))
(assert (and (=> (= course_period__ARRAY__19 9) BOOL____01737) (=> BOOL____01737 (= course_period__ARRAY__19 9))))
(assert (and (=> (= course_period__ARRAY__19 10) BOOL____01837) (=> BOOL____01837 (= course_period__ARRAY__19 10))))
(assert (and (=> (= course_period__ARRAY__20 1) BOOL____00939) (=> BOOL____00939 (= course_period__ARRAY__20 1))))
(assert (and (=> (= course_period__ARRAY__20 2) BOOL____01039) (=> BOOL____01039 (= course_period__ARRAY__20 2))))
(assert (and (=> (= course_period__ARRAY__20 3) BOOL____01139) (=> BOOL____01139 (= course_period__ARRAY__20 3))))
(assert (and (=> (= course_period__ARRAY__20 4) BOOL____01239) (=> BOOL____01239 (= course_period__ARRAY__20 4))))
(assert (and (=> (= course_period__ARRAY__20 5) BOOL____01339) (=> BOOL____01339 (= course_period__ARRAY__20 5))))
(assert (and (=> (= course_period__ARRAY__20 6) BOOL____01439) (=> BOOL____01439 (= course_period__ARRAY__20 6))))
(assert (and (=> (= course_period__ARRAY__20 7) BOOL____01539) (=> BOOL____01539 (= course_period__ARRAY__20 7))))
(assert (and (=> (= course_period__ARRAY__20 8) BOOL____01639) (=> BOOL____01639 (= course_period__ARRAY__20 8))))
(assert (and (=> (= course_period__ARRAY__20 9) BOOL____01739) (=> BOOL____01739 (= course_period__ARRAY__20 9))))
(assert (and (=> (= course_period__ARRAY__20 10) BOOL____01839) (=> BOOL____01839 (= course_period__ARRAY__20 10))))
(assert (and (=> (= course_period__ARRAY__21 1) BOOL____00941) (=> BOOL____00941 (= course_period__ARRAY__21 1))))
(assert (and (=> (= course_period__ARRAY__21 2) BOOL____01041) (=> BOOL____01041 (= course_period__ARRAY__21 2))))
(assert (and (=> (= course_period__ARRAY__21 3) BOOL____01141) (=> BOOL____01141 (= course_period__ARRAY__21 3))))
(assert (and (=> (= course_period__ARRAY__21 4) BOOL____01241) (=> BOOL____01241 (= course_period__ARRAY__21 4))))
(assert (and (=> (= course_period__ARRAY__21 5) BOOL____01341) (=> BOOL____01341 (= course_period__ARRAY__21 5))))
(assert (and (=> (= course_period__ARRAY__21 6) BOOL____01441) (=> BOOL____01441 (= course_period__ARRAY__21 6))))
(assert (and (=> (= course_period__ARRAY__21 7) BOOL____01541) (=> BOOL____01541 (= course_period__ARRAY__21 7))))
(assert (and (=> (= course_period__ARRAY__21 8) BOOL____01641) (=> BOOL____01641 (= course_period__ARRAY__21 8))))
(assert (and (=> (= course_period__ARRAY__21 9) BOOL____01741) (=> BOOL____01741 (= course_period__ARRAY__21 9))))
(assert (and (=> (= course_period__ARRAY__21 10) BOOL____01841) (=> BOOL____01841 (= course_period__ARRAY__21 10))))
(assert (and (=> (= course_period__ARRAY__22 1) BOOL____00943) (=> BOOL____00943 (= course_period__ARRAY__22 1))))
(assert (and (=> (= course_period__ARRAY__22 2) BOOL____01043) (=> BOOL____01043 (= course_period__ARRAY__22 2))))
(assert (and (=> (= course_period__ARRAY__22 3) BOOL____01143) (=> BOOL____01143 (= course_period__ARRAY__22 3))))
(assert (and (=> (= course_period__ARRAY__22 4) BOOL____01243) (=> BOOL____01243 (= course_period__ARRAY__22 4))))
(assert (and (=> (= course_period__ARRAY__22 5) BOOL____01343) (=> BOOL____01343 (= course_period__ARRAY__22 5))))
(assert (and (=> (= course_period__ARRAY__22 6) BOOL____01443) (=> BOOL____01443 (= course_period__ARRAY__22 6))))
(assert (and (=> (= course_period__ARRAY__22 7) BOOL____01543) (=> BOOL____01543 (= course_period__ARRAY__22 7))))
(assert (and (=> (= course_period__ARRAY__22 8) BOOL____01643) (=> BOOL____01643 (= course_period__ARRAY__22 8))))
(assert (and (=> (= course_period__ARRAY__22 9) BOOL____01743) (=> BOOL____01743 (= course_period__ARRAY__22 9))))
(assert (and (=> (= course_period__ARRAY__22 10) BOOL____01843) (=> BOOL____01843 (= course_period__ARRAY__22 10))))
(assert (and (=> (= course_period__ARRAY__23 1) BOOL____00945) (=> BOOL____00945 (= course_period__ARRAY__23 1))))
(assert (and (=> (= course_period__ARRAY__23 2) BOOL____01045) (=> BOOL____01045 (= course_period__ARRAY__23 2))))
(assert (and (=> (= course_period__ARRAY__23 3) BOOL____01145) (=> BOOL____01145 (= course_period__ARRAY__23 3))))
(assert (and (=> (= course_period__ARRAY__23 4) BOOL____01245) (=> BOOL____01245 (= course_period__ARRAY__23 4))))
(assert (and (=> (= course_period__ARRAY__23 5) BOOL____01345) (=> BOOL____01345 (= course_period__ARRAY__23 5))))
(assert (and (=> (= course_period__ARRAY__23 6) BOOL____01445) (=> BOOL____01445 (= course_period__ARRAY__23 6))))
(assert (and (=> (= course_period__ARRAY__23 7) BOOL____01545) (=> BOOL____01545 (= course_period__ARRAY__23 7))))
(assert (and (=> (= course_period__ARRAY__23 8) BOOL____01645) (=> BOOL____01645 (= course_period__ARRAY__23 8))))
(assert (and (=> (= course_period__ARRAY__23 9) BOOL____01745) (=> BOOL____01745 (= course_period__ARRAY__23 9))))
(assert (and (=> (= course_period__ARRAY__23 10) BOOL____01845) (=> BOOL____01845 (= course_period__ARRAY__23 10))))
(assert (and (=> (= course_period__ARRAY__24 1) BOOL____00947) (=> BOOL____00947 (= course_period__ARRAY__24 1))))
(assert (and (=> (= course_period__ARRAY__24 2) BOOL____01047) (=> BOOL____01047 (= course_period__ARRAY__24 2))))
(assert (and (=> (= course_period__ARRAY__24 3) BOOL____01147) (=> BOOL____01147 (= course_period__ARRAY__24 3))))
(assert (and (=> (= course_period__ARRAY__24 4) BOOL____01247) (=> BOOL____01247 (= course_period__ARRAY__24 4))))
(assert (and (=> (= course_period__ARRAY__24 5) BOOL____01347) (=> BOOL____01347 (= course_period__ARRAY__24 5))))
(assert (and (=> (= course_period__ARRAY__24 6) BOOL____01447) (=> BOOL____01447 (= course_period__ARRAY__24 6))))
(assert (and (=> (= course_period__ARRAY__24 7) BOOL____01547) (=> BOOL____01547 (= course_period__ARRAY__24 7))))
(assert (and (=> (= course_period__ARRAY__24 8) BOOL____01647) (=> BOOL____01647 (= course_period__ARRAY__24 8))))
(assert (and (=> (= course_period__ARRAY__24 9) BOOL____01747) (=> BOOL____01747 (= course_period__ARRAY__24 9))))
(assert (and (=> (= course_period__ARRAY__24 10) BOOL____01847) (=> BOOL____01847 (= course_period__ARRAY__24 10))))
(assert (and (=> (= course_period__ARRAY__25 1) BOOL____00949) (=> BOOL____00949 (= course_period__ARRAY__25 1))))
(assert (and (=> (= course_period__ARRAY__25 2) BOOL____01049) (=> BOOL____01049 (= course_period__ARRAY__25 2))))
(assert (and (=> (= course_period__ARRAY__25 3) BOOL____01149) (=> BOOL____01149 (= course_period__ARRAY__25 3))))
(assert (and (=> (= course_period__ARRAY__25 4) BOOL____01249) (=> BOOL____01249 (= course_period__ARRAY__25 4))))
(assert (and (=> (= course_period__ARRAY__25 5) BOOL____01349) (=> BOOL____01349 (= course_period__ARRAY__25 5))))
(assert (and (=> (= course_period__ARRAY__25 6) BOOL____01449) (=> BOOL____01449 (= course_period__ARRAY__25 6))))
(assert (and (=> (= course_period__ARRAY__25 7) BOOL____01549) (=> BOOL____01549 (= course_period__ARRAY__25 7))))
(assert (and (=> (= course_period__ARRAY__25 8) BOOL____01649) (=> BOOL____01649 (= course_period__ARRAY__25 8))))
(assert (and (=> (= course_period__ARRAY__25 9) BOOL____01749) (=> BOOL____01749 (= course_period__ARRAY__25 9))))
(assert (and (=> (= course_period__ARRAY__25 10) BOOL____01849) (=> BOOL____01849 (= course_period__ARRAY__25 10))))
(assert (and (=> (= course_period__ARRAY__26 1) BOOL____00951) (=> BOOL____00951 (= course_period__ARRAY__26 1))))
(assert (and (=> (= course_period__ARRAY__26 2) BOOL____01051) (=> BOOL____01051 (= course_period__ARRAY__26 2))))
(assert (and (=> (= course_period__ARRAY__26 3) BOOL____01151) (=> BOOL____01151 (= course_period__ARRAY__26 3))))
(assert (and (=> (= course_period__ARRAY__26 4) BOOL____01251) (=> BOOL____01251 (= course_period__ARRAY__26 4))))
(assert (and (=> (= course_period__ARRAY__26 5) BOOL____01351) (=> BOOL____01351 (= course_period__ARRAY__26 5))))
(assert (and (=> (= course_period__ARRAY__26 6) BOOL____01451) (=> BOOL____01451 (= course_period__ARRAY__26 6))))
(assert (and (=> (= course_period__ARRAY__26 7) BOOL____01551) (=> BOOL____01551 (= course_period__ARRAY__26 7))))
(assert (and (=> (= course_period__ARRAY__26 8) BOOL____01651) (=> BOOL____01651 (= course_period__ARRAY__26 8))))
(assert (and (=> (= course_period__ARRAY__26 9) BOOL____01751) (=> BOOL____01751 (= course_period__ARRAY__26 9))))
(assert (and (=> (= course_period__ARRAY__26 10) BOOL____01851) (=> BOOL____01851 (= course_period__ARRAY__26 10))))
(assert (and (=> (= course_period__ARRAY__27 1) BOOL____00953) (=> BOOL____00953 (= course_period__ARRAY__27 1))))
(assert (and (=> (= course_period__ARRAY__27 2) BOOL____01053) (=> BOOL____01053 (= course_period__ARRAY__27 2))))
(assert (and (=> (= course_period__ARRAY__27 3) BOOL____01153) (=> BOOL____01153 (= course_period__ARRAY__27 3))))
(assert (and (=> (= course_period__ARRAY__27 4) BOOL____01253) (=> BOOL____01253 (= course_period__ARRAY__27 4))))
(assert (and (=> (= course_period__ARRAY__27 5) BOOL____01353) (=> BOOL____01353 (= course_period__ARRAY__27 5))))
(assert (and (=> (= course_period__ARRAY__27 6) BOOL____01453) (=> BOOL____01453 (= course_period__ARRAY__27 6))))
(assert (and (=> (= course_period__ARRAY__27 7) BOOL____01553) (=> BOOL____01553 (= course_period__ARRAY__27 7))))
(assert (and (=> (= course_period__ARRAY__27 8) BOOL____01653) (=> BOOL____01653 (= course_period__ARRAY__27 8))))
(assert (and (=> (= course_period__ARRAY__27 9) BOOL____01753) (=> BOOL____01753 (= course_period__ARRAY__27 9))))
(assert (and (=> (= course_period__ARRAY__27 10) BOOL____01853) (=> BOOL____01853 (= course_period__ARRAY__27 10))))
(assert (and (=> (= course_period__ARRAY__28 1) BOOL____00955) (=> BOOL____00955 (= course_period__ARRAY__28 1))))
(assert (and (=> (= course_period__ARRAY__28 2) BOOL____01055) (=> BOOL____01055 (= course_period__ARRAY__28 2))))
(assert (and (=> (= course_period__ARRAY__28 3) BOOL____01155) (=> BOOL____01155 (= course_period__ARRAY__28 3))))
(assert (and (=> (= course_period__ARRAY__28 4) BOOL____01255) (=> BOOL____01255 (= course_period__ARRAY__28 4))))
(assert (and (=> (= course_period__ARRAY__28 5) BOOL____01355) (=> BOOL____01355 (= course_period__ARRAY__28 5))))
(assert (and (=> (= course_period__ARRAY__28 6) BOOL____01455) (=> BOOL____01455 (= course_period__ARRAY__28 6))))
(assert (and (=> (= course_period__ARRAY__28 7) BOOL____01555) (=> BOOL____01555 (= course_period__ARRAY__28 7))))
(assert (and (=> (= course_period__ARRAY__28 8) BOOL____01655) (=> BOOL____01655 (= course_period__ARRAY__28 8))))
(assert (and (=> (= course_period__ARRAY__28 9) BOOL____01755) (=> BOOL____01755 (= course_period__ARRAY__28 9))))
(assert (and (=> (= course_period__ARRAY__28 10) BOOL____01855) (=> BOOL____01855 (= course_period__ARRAY__28 10))))
(assert (and (=> (= course_period__ARRAY__29 1) BOOL____00957) (=> BOOL____00957 (= course_period__ARRAY__29 1))))
(assert (and (=> (= course_period__ARRAY__29 2) BOOL____01057) (=> BOOL____01057 (= course_period__ARRAY__29 2))))
(assert (and (=> (= course_period__ARRAY__29 3) BOOL____01157) (=> BOOL____01157 (= course_period__ARRAY__29 3))))
(assert (and (=> (= course_period__ARRAY__29 4) BOOL____01257) (=> BOOL____01257 (= course_period__ARRAY__29 4))))
(assert (and (=> (= course_period__ARRAY__29 5) BOOL____01357) (=> BOOL____01357 (= course_period__ARRAY__29 5))))
(assert (and (=> (= course_period__ARRAY__29 6) BOOL____01457) (=> BOOL____01457 (= course_period__ARRAY__29 6))))
(assert (and (=> (= course_period__ARRAY__29 7) BOOL____01557) (=> BOOL____01557 (= course_period__ARRAY__29 7))))
(assert (and (=> (= course_period__ARRAY__29 8) BOOL____01657) (=> BOOL____01657 (= course_period__ARRAY__29 8))))
(assert (and (=> (= course_period__ARRAY__29 9) BOOL____01757) (=> BOOL____01757 (= course_period__ARRAY__29 9))))
(assert (and (=> (= course_period__ARRAY__29 10) BOOL____01857) (=> BOOL____01857 (= course_period__ARRAY__29 10))))
(assert (and (=> (= course_period__ARRAY__30 1) BOOL____00959) (=> BOOL____00959 (= course_period__ARRAY__30 1))))
(assert (and (=> (= course_period__ARRAY__30 2) BOOL____01059) (=> BOOL____01059 (= course_period__ARRAY__30 2))))
(assert (and (=> (= course_period__ARRAY__30 3) BOOL____01159) (=> BOOL____01159 (= course_period__ARRAY__30 3))))
(assert (and (=> (= course_period__ARRAY__30 4) BOOL____01259) (=> BOOL____01259 (= course_period__ARRAY__30 4))))
(assert (and (=> (= course_period__ARRAY__30 5) BOOL____01359) (=> BOOL____01359 (= course_period__ARRAY__30 5))))
(assert (and (=> (= course_period__ARRAY__30 6) BOOL____01459) (=> BOOL____01459 (= course_period__ARRAY__30 6))))
(assert (and (=> (= course_period__ARRAY__30 7) BOOL____01559) (=> BOOL____01559 (= course_period__ARRAY__30 7))))
(assert (and (=> (= course_period__ARRAY__30 8) BOOL____01659) (=> BOOL____01659 (= course_period__ARRAY__30 8))))
(assert (and (=> (= course_period__ARRAY__30 9) BOOL____01759) (=> BOOL____01759 (= course_period__ARRAY__30 9))))
(assert (and (=> (= course_period__ARRAY__30 10) BOOL____01859) (=> BOOL____01859 (= course_period__ARRAY__30 10))))
(assert (and (=> (= course_period__ARRAY__31 1) BOOL____00961) (=> BOOL____00961 (= course_period__ARRAY__31 1))))
(assert (and (=> (= course_period__ARRAY__31 2) BOOL____01061) (=> BOOL____01061 (= course_period__ARRAY__31 2))))
(assert (and (=> (= course_period__ARRAY__31 3) BOOL____01161) (=> BOOL____01161 (= course_period__ARRAY__31 3))))
(assert (and (=> (= course_period__ARRAY__31 4) BOOL____01261) (=> BOOL____01261 (= course_period__ARRAY__31 4))))
(assert (and (=> (= course_period__ARRAY__31 5) BOOL____01361) (=> BOOL____01361 (= course_period__ARRAY__31 5))))
(assert (and (=> (= course_period__ARRAY__31 6) BOOL____01461) (=> BOOL____01461 (= course_period__ARRAY__31 6))))
(assert (and (=> (= course_period__ARRAY__31 7) BOOL____01561) (=> BOOL____01561 (= course_period__ARRAY__31 7))))
(assert (and (=> (= course_period__ARRAY__31 8) BOOL____01661) (=> BOOL____01661 (= course_period__ARRAY__31 8))))
(assert (and (=> (= course_period__ARRAY__31 9) BOOL____01761) (=> BOOL____01761 (= course_period__ARRAY__31 9))))
(assert (and (=> (= course_period__ARRAY__31 10) BOOL____01861) (=> BOOL____01861 (= course_period__ARRAY__31 10))))
(assert (and (=> (= course_period__ARRAY__32 1) BOOL____00963) (=> BOOL____00963 (= course_period__ARRAY__32 1))))
(assert (and (=> (= course_period__ARRAY__32 2) BOOL____01063) (=> BOOL____01063 (= course_period__ARRAY__32 2))))
(assert (and (=> (= course_period__ARRAY__32 3) BOOL____01163) (=> BOOL____01163 (= course_period__ARRAY__32 3))))
(assert (and (=> (= course_period__ARRAY__32 4) BOOL____01263) (=> BOOL____01263 (= course_period__ARRAY__32 4))))
(assert (and (=> (= course_period__ARRAY__32 5) BOOL____01363) (=> BOOL____01363 (= course_period__ARRAY__32 5))))
(assert (and (=> (= course_period__ARRAY__32 6) BOOL____01463) (=> BOOL____01463 (= course_period__ARRAY__32 6))))
(assert (and (=> (= course_period__ARRAY__32 7) BOOL____01563) (=> BOOL____01563 (= course_period__ARRAY__32 7))))
(assert (and (=> (= course_period__ARRAY__32 8) BOOL____01663) (=> BOOL____01663 (= course_period__ARRAY__32 8))))
(assert (and (=> (= course_period__ARRAY__32 9) BOOL____01763) (=> BOOL____01763 (= course_period__ARRAY__32 9))))
(assert (and (=> (= course_period__ARRAY__32 10) BOOL____01863) (=> BOOL____01863 (= course_period__ARRAY__32 10))))
(assert (and (=> (= course_period__ARRAY__33 1) BOOL____00965) (=> BOOL____00965 (= course_period__ARRAY__33 1))))
(assert (and (=> (= course_period__ARRAY__33 2) BOOL____01065) (=> BOOL____01065 (= course_period__ARRAY__33 2))))
(assert (and (=> (= course_period__ARRAY__33 3) BOOL____01165) (=> BOOL____01165 (= course_period__ARRAY__33 3))))
(assert (and (=> (= course_period__ARRAY__33 4) BOOL____01265) (=> BOOL____01265 (= course_period__ARRAY__33 4))))
(assert (and (=> (= course_period__ARRAY__33 5) BOOL____01365) (=> BOOL____01365 (= course_period__ARRAY__33 5))))
(assert (and (=> (= course_period__ARRAY__33 6) BOOL____01465) (=> BOOL____01465 (= course_period__ARRAY__33 6))))
(assert (and (=> (= course_period__ARRAY__33 7) BOOL____01565) (=> BOOL____01565 (= course_period__ARRAY__33 7))))
(assert (and (=> (= course_period__ARRAY__33 8) BOOL____01665) (=> BOOL____01665 (= course_period__ARRAY__33 8))))
(assert (and (=> (= course_period__ARRAY__33 9) BOOL____01765) (=> BOOL____01765 (= course_period__ARRAY__33 9))))
(assert (and (=> (= course_period__ARRAY__33 10) BOOL____01865) (=> BOOL____01865 (= course_period__ARRAY__33 10))))
(assert (and (=> (= course_period__ARRAY__34 1) BOOL____00967) (=> BOOL____00967 (= course_period__ARRAY__34 1))))
(assert (and (=> (= course_period__ARRAY__34 2) BOOL____01067) (=> BOOL____01067 (= course_period__ARRAY__34 2))))
(assert (and (=> (= course_period__ARRAY__34 3) BOOL____01167) (=> BOOL____01167 (= course_period__ARRAY__34 3))))
(assert (and (=> (= course_period__ARRAY__34 4) BOOL____01267) (=> BOOL____01267 (= course_period__ARRAY__34 4))))
(assert (and (=> (= course_period__ARRAY__34 5) BOOL____01367) (=> BOOL____01367 (= course_period__ARRAY__34 5))))
(assert (and (=> (= course_period__ARRAY__34 6) BOOL____01467) (=> BOOL____01467 (= course_period__ARRAY__34 6))))
(assert (and (=> (= course_period__ARRAY__34 7) BOOL____01567) (=> BOOL____01567 (= course_period__ARRAY__34 7))))
(assert (and (=> (= course_period__ARRAY__34 8) BOOL____01667) (=> BOOL____01667 (= course_period__ARRAY__34 8))))
(assert (and (=> (= course_period__ARRAY__34 9) BOOL____01767) (=> BOOL____01767 (= course_period__ARRAY__34 9))))
(assert (and (=> (= course_period__ARRAY__34 10) BOOL____01867) (=> BOOL____01867 (= course_period__ARRAY__34 10))))
(assert (and (=> (= course_period__ARRAY__35 1) BOOL____00969) (=> BOOL____00969 (= course_period__ARRAY__35 1))))
(assert (and (=> (= course_period__ARRAY__35 2) BOOL____01069) (=> BOOL____01069 (= course_period__ARRAY__35 2))))
(assert (and (=> (= course_period__ARRAY__35 3) BOOL____01169) (=> BOOL____01169 (= course_period__ARRAY__35 3))))
(assert (and (=> (= course_period__ARRAY__35 4) BOOL____01269) (=> BOOL____01269 (= course_period__ARRAY__35 4))))
(assert (and (=> (= course_period__ARRAY__35 5) BOOL____01369) (=> BOOL____01369 (= course_period__ARRAY__35 5))))
(assert (and (=> (= course_period__ARRAY__35 6) BOOL____01469) (=> BOOL____01469 (= course_period__ARRAY__35 6))))
(assert (and (=> (= course_period__ARRAY__35 7) BOOL____01569) (=> BOOL____01569 (= course_period__ARRAY__35 7))))
(assert (and (=> (= course_period__ARRAY__35 8) BOOL____01669) (=> BOOL____01669 (= course_period__ARRAY__35 8))))
(assert (and (=> (= course_period__ARRAY__35 9) BOOL____01769) (=> BOOL____01769 (= course_period__ARRAY__35 9))))
(assert (and (=> (= course_period__ARRAY__35 10) BOOL____01869) (=> BOOL____01869 (= course_period__ARRAY__35 10))))
(assert (and (=> (= course_period__ARRAY__36 1) BOOL____00971) (=> BOOL____00971 (= course_period__ARRAY__36 1))))
(assert (and (=> (= course_period__ARRAY__36 2) BOOL____01071) (=> BOOL____01071 (= course_period__ARRAY__36 2))))
(assert (and (=> (= course_period__ARRAY__36 3) BOOL____01171) (=> BOOL____01171 (= course_period__ARRAY__36 3))))
(assert (and (=> (= course_period__ARRAY__36 4) BOOL____01271) (=> BOOL____01271 (= course_period__ARRAY__36 4))))
(assert (and (=> (= course_period__ARRAY__36 5) BOOL____01371) (=> BOOL____01371 (= course_period__ARRAY__36 5))))
(assert (and (=> (= course_period__ARRAY__36 6) BOOL____01471) (=> BOOL____01471 (= course_period__ARRAY__36 6))))
(assert (and (=> (= course_period__ARRAY__36 7) BOOL____01571) (=> BOOL____01571 (= course_period__ARRAY__36 7))))
(assert (and (=> (= course_period__ARRAY__36 8) BOOL____01671) (=> BOOL____01671 (= course_period__ARRAY__36 8))))
(assert (and (=> (= course_period__ARRAY__36 9) BOOL____01771) (=> BOOL____01771 (= course_period__ARRAY__36 9))))
(assert (and (=> (= course_period__ARRAY__36 10) BOOL____01871) (=> BOOL____01871 (= course_period__ARRAY__36 10))))
(assert (and (=> (= course_period__ARRAY__37 1) BOOL____00973) (=> BOOL____00973 (= course_period__ARRAY__37 1))))
(assert (and (=> (= course_period__ARRAY__37 2) BOOL____01073) (=> BOOL____01073 (= course_period__ARRAY__37 2))))
(assert (and (=> (= course_period__ARRAY__37 3) BOOL____01173) (=> BOOL____01173 (= course_period__ARRAY__37 3))))
(assert (and (=> (= course_period__ARRAY__37 4) BOOL____01273) (=> BOOL____01273 (= course_period__ARRAY__37 4))))
(assert (and (=> (= course_period__ARRAY__37 5) BOOL____01373) (=> BOOL____01373 (= course_period__ARRAY__37 5))))
(assert (and (=> (= course_period__ARRAY__37 6) BOOL____01473) (=> BOOL____01473 (= course_period__ARRAY__37 6))))
(assert (and (=> (= course_period__ARRAY__37 7) BOOL____01573) (=> BOOL____01573 (= course_period__ARRAY__37 7))))
(assert (and (=> (= course_period__ARRAY__37 8) BOOL____01673) (=> BOOL____01673 (= course_period__ARRAY__37 8))))
(assert (and (=> (= course_period__ARRAY__37 9) BOOL____01773) (=> BOOL____01773 (= course_period__ARRAY__37 9))))
(assert (and (=> (= course_period__ARRAY__37 10) BOOL____01873) (=> BOOL____01873 (= course_period__ARRAY__37 10))))
(assert (and (=> (= course_period__ARRAY__38 1) BOOL____00975) (=> BOOL____00975 (= course_period__ARRAY__38 1))))
(assert (and (=> (= course_period__ARRAY__38 2) BOOL____01075) (=> BOOL____01075 (= course_period__ARRAY__38 2))))
(assert (and (=> (= course_period__ARRAY__38 3) BOOL____01175) (=> BOOL____01175 (= course_period__ARRAY__38 3))))
(assert (and (=> (= course_period__ARRAY__38 4) BOOL____01275) (=> BOOL____01275 (= course_period__ARRAY__38 4))))
(assert (and (=> (= course_period__ARRAY__38 5) BOOL____01375) (=> BOOL____01375 (= course_period__ARRAY__38 5))))
(assert (and (=> (= course_period__ARRAY__38 6) BOOL____01475) (=> BOOL____01475 (= course_period__ARRAY__38 6))))
(assert (and (=> (= course_period__ARRAY__38 7) BOOL____01575) (=> BOOL____01575 (= course_period__ARRAY__38 7))))
(assert (and (=> (= course_period__ARRAY__38 8) BOOL____01675) (=> BOOL____01675 (= course_period__ARRAY__38 8))))
(assert (and (=> (= course_period__ARRAY__38 9) BOOL____01775) (=> BOOL____01775 (= course_period__ARRAY__38 9))))
(assert (and (=> (= course_period__ARRAY__38 10) BOOL____01875) (=> BOOL____01875 (= course_period__ARRAY__38 10))))
(assert (and (=> (= course_period__ARRAY__39 1) BOOL____00977) (=> BOOL____00977 (= course_period__ARRAY__39 1))))
(assert (and (=> (= course_period__ARRAY__39 2) BOOL____01077) (=> BOOL____01077 (= course_period__ARRAY__39 2))))
(assert (and (=> (= course_period__ARRAY__39 3) BOOL____01177) (=> BOOL____01177 (= course_period__ARRAY__39 3))))
(assert (and (=> (= course_period__ARRAY__39 4) BOOL____01277) (=> BOOL____01277 (= course_period__ARRAY__39 4))))
(assert (and (=> (= course_period__ARRAY__39 5) BOOL____01377) (=> BOOL____01377 (= course_period__ARRAY__39 5))))
(assert (and (=> (= course_period__ARRAY__39 6) BOOL____01477) (=> BOOL____01477 (= course_period__ARRAY__39 6))))
(assert (and (=> (= course_period__ARRAY__39 7) BOOL____01577) (=> BOOL____01577 (= course_period__ARRAY__39 7))))
(assert (and (=> (= course_period__ARRAY__39 8) BOOL____01677) (=> BOOL____01677 (= course_period__ARRAY__39 8))))
(assert (and (=> (= course_period__ARRAY__39 9) BOOL____01777) (=> BOOL____01777 (= course_period__ARRAY__39 9))))
(assert (and (=> (= course_period__ARRAY__39 10) BOOL____01877) (=> BOOL____01877 (= course_period__ARRAY__39 10))))
(assert (and (=> (= course_period__ARRAY__40 1) BOOL____00979) (=> BOOL____00979 (= course_period__ARRAY__40 1))))
(assert (and (=> (= course_period__ARRAY__40 2) BOOL____01079) (=> BOOL____01079 (= course_period__ARRAY__40 2))))
(assert (and (=> (= course_period__ARRAY__40 3) BOOL____01179) (=> BOOL____01179 (= course_period__ARRAY__40 3))))
(assert (and (=> (= course_period__ARRAY__40 4) BOOL____01279) (=> BOOL____01279 (= course_period__ARRAY__40 4))))
(assert (and (=> (= course_period__ARRAY__40 5) BOOL____01379) (=> BOOL____01379 (= course_period__ARRAY__40 5))))
(assert (and (=> (= course_period__ARRAY__40 6) BOOL____01479) (=> BOOL____01479 (= course_period__ARRAY__40 6))))
(assert (and (=> (= course_period__ARRAY__40 7) BOOL____01579) (=> BOOL____01579 (= course_period__ARRAY__40 7))))
(assert (and (=> (= course_period__ARRAY__40 8) BOOL____01679) (=> BOOL____01679 (= course_period__ARRAY__40 8))))
(assert (and (=> (= course_period__ARRAY__40 9) BOOL____01779) (=> BOOL____01779 (= course_period__ARRAY__40 9))))
(assert (and (=> (= course_period__ARRAY__40 10) BOOL____01879) (=> BOOL____01879 (= course_period__ARRAY__40 10))))
(assert (and (=> (= course_period__ARRAY__41 1) BOOL____00981) (=> BOOL____00981 (= course_period__ARRAY__41 1))))
(assert (and (=> (= course_period__ARRAY__41 2) BOOL____01081) (=> BOOL____01081 (= course_period__ARRAY__41 2))))
(assert (and (=> (= course_period__ARRAY__41 3) BOOL____01181) (=> BOOL____01181 (= course_period__ARRAY__41 3))))
(assert (and (=> (= course_period__ARRAY__41 4) BOOL____01281) (=> BOOL____01281 (= course_period__ARRAY__41 4))))
(assert (and (=> (= course_period__ARRAY__41 5) BOOL____01381) (=> BOOL____01381 (= course_period__ARRAY__41 5))))
(assert (and (=> (= course_period__ARRAY__41 6) BOOL____01481) (=> BOOL____01481 (= course_period__ARRAY__41 6))))
(assert (and (=> (= course_period__ARRAY__41 7) BOOL____01581) (=> BOOL____01581 (= course_period__ARRAY__41 7))))
(assert (and (=> (= course_period__ARRAY__41 8) BOOL____01681) (=> BOOL____01681 (= course_period__ARRAY__41 8))))
(assert (and (=> (= course_period__ARRAY__41 9) BOOL____01781) (=> BOOL____01781 (= course_period__ARRAY__41 9))))
(assert (and (=> (= course_period__ARRAY__41 10) BOOL____01881) (=> BOOL____01881 (= course_period__ARRAY__41 10))))
(assert (and (=> (= course_period__ARRAY__42 1) BOOL____00983) (=> BOOL____00983 (= course_period__ARRAY__42 1))))
(assert (and (=> (= course_period__ARRAY__42 2) BOOL____01083) (=> BOOL____01083 (= course_period__ARRAY__42 2))))
(assert (and (=> (= course_period__ARRAY__42 3) BOOL____01183) (=> BOOL____01183 (= course_period__ARRAY__42 3))))
(assert (and (=> (= course_period__ARRAY__42 4) BOOL____01283) (=> BOOL____01283 (= course_period__ARRAY__42 4))))
(assert (and (=> (= course_period__ARRAY__42 5) BOOL____01383) (=> BOOL____01383 (= course_period__ARRAY__42 5))))
(assert (and (=> (= course_period__ARRAY__42 6) BOOL____01483) (=> BOOL____01483 (= course_period__ARRAY__42 6))))
(assert (and (=> (= course_period__ARRAY__42 7) BOOL____01583) (=> BOOL____01583 (= course_period__ARRAY__42 7))))
(assert (and (=> (= course_period__ARRAY__42 8) BOOL____01683) (=> BOOL____01683 (= course_period__ARRAY__42 8))))
(assert (and (=> (= course_period__ARRAY__42 9) BOOL____01783) (=> BOOL____01783 (= course_period__ARRAY__42 9))))
(assert (and (=> (= course_period__ARRAY__42 10) BOOL____01883) (=> BOOL____01883 (= course_period__ARRAY__42 10))))
(assert (and (=> (= course_period__ARRAY__43 1) BOOL____00985) (=> BOOL____00985 (= course_period__ARRAY__43 1))))
(assert (and (=> (= course_period__ARRAY__43 2) BOOL____01085) (=> BOOL____01085 (= course_period__ARRAY__43 2))))
(assert (and (=> (= course_period__ARRAY__43 3) BOOL____01185) (=> BOOL____01185 (= course_period__ARRAY__43 3))))
(assert (and (=> (= course_period__ARRAY__43 4) BOOL____01285) (=> BOOL____01285 (= course_period__ARRAY__43 4))))
(assert (and (=> (= course_period__ARRAY__43 5) BOOL____01385) (=> BOOL____01385 (= course_period__ARRAY__43 5))))
(assert (and (=> (= course_period__ARRAY__43 6) BOOL____01485) (=> BOOL____01485 (= course_period__ARRAY__43 6))))
(assert (and (=> (= course_period__ARRAY__43 7) BOOL____01585) (=> BOOL____01585 (= course_period__ARRAY__43 7))))
(assert (and (=> (= course_period__ARRAY__43 8) BOOL____01685) (=> BOOL____01685 (= course_period__ARRAY__43 8))))
(assert (and (=> (= course_period__ARRAY__43 9) BOOL____01785) (=> BOOL____01785 (= course_period__ARRAY__43 9))))
(assert (and (=> (= course_period__ARRAY__43 10) BOOL____01885) (=> BOOL____01885 (= course_period__ARRAY__43 10))))
(assert (and (=> (= course_period__ARRAY__44 1) BOOL____00987) (=> BOOL____00987 (= course_period__ARRAY__44 1))))
(assert (and (=> (= course_period__ARRAY__44 2) BOOL____01087) (=> BOOL____01087 (= course_period__ARRAY__44 2))))
(assert (and (=> (= course_period__ARRAY__44 3) BOOL____01187) (=> BOOL____01187 (= course_period__ARRAY__44 3))))
(assert (and (=> (= course_period__ARRAY__44 4) BOOL____01287) (=> BOOL____01287 (= course_period__ARRAY__44 4))))
(assert (and (=> (= course_period__ARRAY__44 5) BOOL____01387) (=> BOOL____01387 (= course_period__ARRAY__44 5))))
(assert (and (=> (= course_period__ARRAY__44 6) BOOL____01487) (=> BOOL____01487 (= course_period__ARRAY__44 6))))
(assert (and (=> (= course_period__ARRAY__44 7) BOOL____01587) (=> BOOL____01587 (= course_period__ARRAY__44 7))))
(assert (and (=> (= course_period__ARRAY__44 8) BOOL____01687) (=> BOOL____01687 (= course_period__ARRAY__44 8))))
(assert (and (=> (= course_period__ARRAY__44 9) BOOL____01787) (=> BOOL____01787 (= course_period__ARRAY__44 9))))
(assert (and (=> (= course_period__ARRAY__44 10) BOOL____01887) (=> BOOL____01887 (= course_period__ARRAY__44 10))))
(assert (and (=> (= course_period__ARRAY__45 1) BOOL____00989) (=> BOOL____00989 (= course_period__ARRAY__45 1))))
(assert (and (=> (= course_period__ARRAY__45 2) BOOL____01089) (=> BOOL____01089 (= course_period__ARRAY__45 2))))
(assert (and (=> (= course_period__ARRAY__45 3) BOOL____01189) (=> BOOL____01189 (= course_period__ARRAY__45 3))))
(assert (and (=> (= course_period__ARRAY__45 4) BOOL____01289) (=> BOOL____01289 (= course_period__ARRAY__45 4))))
(assert (and (=> (= course_period__ARRAY__45 5) BOOL____01389) (=> BOOL____01389 (= course_period__ARRAY__45 5))))
(assert (and (=> (= course_period__ARRAY__45 6) BOOL____01489) (=> BOOL____01489 (= course_period__ARRAY__45 6))))
(assert (and (=> (= course_period__ARRAY__45 7) BOOL____01589) (=> BOOL____01589 (= course_period__ARRAY__45 7))))
(assert (and (=> (= course_period__ARRAY__45 8) BOOL____01689) (=> BOOL____01689 (= course_period__ARRAY__45 8))))
(assert (and (=> (= course_period__ARRAY__45 9) BOOL____01789) (=> BOOL____01789 (= course_period__ARRAY__45 9))))
(assert (and (=> (= course_period__ARRAY__45 10) BOOL____01889) (=> BOOL____01889 (= course_period__ARRAY__45 10))))
(assert (and (=> (= course_period__ARRAY__46 1) BOOL____00991) (=> BOOL____00991 (= course_period__ARRAY__46 1))))
(assert (and (=> (= course_period__ARRAY__46 2) BOOL____01091) (=> BOOL____01091 (= course_period__ARRAY__46 2))))
(assert (and (=> (= course_period__ARRAY__46 3) BOOL____01191) (=> BOOL____01191 (= course_period__ARRAY__46 3))))
(assert (and (=> (= course_period__ARRAY__46 4) BOOL____01291) (=> BOOL____01291 (= course_period__ARRAY__46 4))))
(assert (and (=> (= course_period__ARRAY__46 5) BOOL____01391) (=> BOOL____01391 (= course_period__ARRAY__46 5))))
(assert (and (=> (= course_period__ARRAY__46 6) BOOL____01491) (=> BOOL____01491 (= course_period__ARRAY__46 6))))
(assert (and (=> (= course_period__ARRAY__46 7) BOOL____01591) (=> BOOL____01591 (= course_period__ARRAY__46 7))))
(assert (and (=> (= course_period__ARRAY__46 8) BOOL____01691) (=> BOOL____01691 (= course_period__ARRAY__46 8))))
(assert (and (=> (= course_period__ARRAY__46 9) BOOL____01791) (=> BOOL____01791 (= course_period__ARRAY__46 9))))
(assert (and (=> (= course_period__ARRAY__46 10) BOOL____01891) (=> BOOL____01891 (= course_period__ARRAY__46 10))))
(assert (and (=> (= course_period__ARRAY__47 1) BOOL____00993) (=> BOOL____00993 (= course_period__ARRAY__47 1))))
(assert (and (=> (= course_period__ARRAY__47 2) BOOL____01093) (=> BOOL____01093 (= course_period__ARRAY__47 2))))
(assert (and (=> (= course_period__ARRAY__47 3) BOOL____01193) (=> BOOL____01193 (= course_period__ARRAY__47 3))))
(assert (and (=> (= course_period__ARRAY__47 4) BOOL____01293) (=> BOOL____01293 (= course_period__ARRAY__47 4))))
(assert (and (=> (= course_period__ARRAY__47 5) BOOL____01393) (=> BOOL____01393 (= course_period__ARRAY__47 5))))
(assert (and (=> (= course_period__ARRAY__47 6) BOOL____01493) (=> BOOL____01493 (= course_period__ARRAY__47 6))))
(assert (and (=> (= course_period__ARRAY__47 7) BOOL____01593) (=> BOOL____01593 (= course_period__ARRAY__47 7))))
(assert (and (=> (= course_period__ARRAY__47 8) BOOL____01693) (=> BOOL____01693 (= course_period__ARRAY__47 8))))
(assert (and (=> (= course_period__ARRAY__47 9) BOOL____01793) (=> BOOL____01793 (= course_period__ARRAY__47 9))))
(assert (and (=> (= course_period__ARRAY__47 10) BOOL____01893) (=> BOOL____01893 (= course_period__ARRAY__47 10))))
(assert (and (=> (= course_period__ARRAY__48 1) BOOL____00995) (=> BOOL____00995 (= course_period__ARRAY__48 1))))
(assert (and (=> (= course_period__ARRAY__48 2) BOOL____01095) (=> BOOL____01095 (= course_period__ARRAY__48 2))))
(assert (and (=> (= course_period__ARRAY__48 3) BOOL____01195) (=> BOOL____01195 (= course_period__ARRAY__48 3))))
(assert (and (=> (= course_period__ARRAY__48 4) BOOL____01295) (=> BOOL____01295 (= course_period__ARRAY__48 4))))
(assert (and (=> (= course_period__ARRAY__48 5) BOOL____01395) (=> BOOL____01395 (= course_period__ARRAY__48 5))))
(assert (and (=> (= course_period__ARRAY__48 6) BOOL____01495) (=> BOOL____01495 (= course_period__ARRAY__48 6))))
(assert (and (=> (= course_period__ARRAY__48 7) BOOL____01595) (=> BOOL____01595 (= course_period__ARRAY__48 7))))
(assert (and (=> (= course_period__ARRAY__48 8) BOOL____01695) (=> BOOL____01695 (= course_period__ARRAY__48 8))))
(assert (and (=> (= course_period__ARRAY__48 9) BOOL____01795) (=> BOOL____01795 (= course_period__ARRAY__48 9))))
(assert (and (=> (= course_period__ARRAY__48 10) BOOL____01895) (=> BOOL____01895 (= course_period__ARRAY__48 10))))
(assert (and (=> (= course_period__ARRAY__49 1) BOOL____00997) (=> BOOL____00997 (= course_period__ARRAY__49 1))))
(assert (and (=> (= course_period__ARRAY__49 2) BOOL____01097) (=> BOOL____01097 (= course_period__ARRAY__49 2))))
(assert (and (=> (= course_period__ARRAY__49 3) BOOL____01197) (=> BOOL____01197 (= course_period__ARRAY__49 3))))
(assert (and (=> (= course_period__ARRAY__49 4) BOOL____01297) (=> BOOL____01297 (= course_period__ARRAY__49 4))))
(assert (and (=> (= course_period__ARRAY__49 5) BOOL____01397) (=> BOOL____01397 (= course_period__ARRAY__49 5))))
(assert (and (=> (= course_period__ARRAY__49 6) BOOL____01497) (=> BOOL____01497 (= course_period__ARRAY__49 6))))
(assert (and (=> (= course_period__ARRAY__49 7) BOOL____01597) (=> BOOL____01597 (= course_period__ARRAY__49 7))))
(assert (and (=> (= course_period__ARRAY__49 8) BOOL____01697) (=> BOOL____01697 (= course_period__ARRAY__49 8))))
(assert (and (=> (= course_period__ARRAY__49 9) BOOL____01797) (=> BOOL____01797 (= course_period__ARRAY__49 9))))
(assert (and (=> (= course_period__ARRAY__49 10) BOOL____01897) (=> BOOL____01897 (= course_period__ARRAY__49 10))))
(assert (and (=> (= course_period__ARRAY__50 1) BOOL____00999) (=> BOOL____00999 (= course_period__ARRAY__50 1))))
(assert (and (=> (= course_period__ARRAY__50 2) BOOL____01099) (=> BOOL____01099 (= course_period__ARRAY__50 2))))
(assert (and (=> (= course_period__ARRAY__50 3) BOOL____01199) (=> BOOL____01199 (= course_period__ARRAY__50 3))))
(assert (and (=> (= course_period__ARRAY__50 4) BOOL____01299) (=> BOOL____01299 (= course_period__ARRAY__50 4))))
(assert (and (=> (= course_period__ARRAY__50 5) BOOL____01399) (=> BOOL____01399 (= course_period__ARRAY__50 5))))
(assert (and (=> (= course_period__ARRAY__50 6) BOOL____01499) (=> BOOL____01499 (= course_period__ARRAY__50 6))))
(assert (and (=> (= course_period__ARRAY__50 7) BOOL____01599) (=> BOOL____01599 (= course_period__ARRAY__50 7))))
(assert (and (=> (= course_period__ARRAY__50 8) BOOL____01699) (=> BOOL____01699 (= course_period__ARRAY__50 8))))
(assert (and (=> (= course_period__ARRAY__50 9) BOOL____01799) (=> BOOL____01799 (= course_period__ARRAY__50 9))))
(assert (and (=> (= course_period__ARRAY__50 10) BOOL____01899) (=> BOOL____01899 (= course_period__ARRAY__50 10))))
(assert (<= load__ARRAY__1 objective))
(assert (<= load__ARRAY__2 objective))
(assert (<= load__ARRAY__3 objective))
(assert (<= load__ARRAY__4 objective))
(assert (<= load__ARRAY__5 objective))
(assert (<= load__ARRAY__6 objective))
(assert (<= load__ARRAY__7 objective))
(assert (<= load__ARRAY__8 objective))
(assert (<= load__ARRAY__9 objective))
(assert (<= load__ARRAY__10 objective))
(assert (= (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (* load__ARRAY__1 1) (* x__ARRAY__1 (- 9))) (* x__ARRAY__2 (- 1))) (* x__ARRAY__3 (- 2))) (* x__ARRAY__4 (- 10))) (* x__ARRAY__5 (- 3))) (* x__ARRAY__6 (- 4))) (* x__ARRAY__7 (- 1))) (* x__ARRAY__8 (- 10))) (* x__ARRAY__9 (- 4))) (* x__ARRAY__10 (- 1))) (* x__ARRAY__11 (- 1))) (* x__ARRAY__12 (- 5))) (* x__ARRAY__13 (- 10))) (* x__ARRAY__14 (- 9))) (* x__ARRAY__15 (- 1))) (* x__ARRAY__16 (- 8))) (* x__ARRAY__17 (- 5))) (* x__ARRAY__18 (- 2))) (* x__ARRAY__19 (- 8))) (* x__ARRAY__20 (- 6))) (* x__ARRAY__21 (- 9))) (* x__ARRAY__22 (- 5))) (* x__ARRAY__23 (- 10))) (* x__ARRAY__24 (- 7))) (* x__ARRAY__25 (- 8))) (* x__ARRAY__26 (- 3))) (* x__ARRAY__27 (- 7))) (* x__ARRAY__28 (- 4))) (* x__ARRAY__29 (- 10))) (* x__ARRAY__30 (- 3))) (* x__ARRAY__31 (- 1))) (* x__ARRAY__32 (- 10))) (* x__ARRAY__33 (- 6))) (* x__ARRAY__34 (- 2))) (* x__ARRAY__35 (- 9))) (* x__ARRAY__36 (- 8))) (* x__ARRAY__37 (- 7))) (* x__ARRAY__38 (- 2))) (* x__ARRAY__39 (- 7))) (* x__ARRAY__40 (- 3))) (* x__ARRAY__41 (- 4))) (* x__ARRAY__42 (- 8))) (* x__ARRAY__43 (- 9))) (* x__ARRAY__44 (- 3))) (* x__ARRAY__45 (- 8))) (* x__ARRAY__46 (- 1))) (* x__ARRAY__47 (- 2))) (* x__ARRAY__48 (- 3))) (* x__ARRAY__49 (- 2))) (* x__ARRAY__50 (- 9))) 0))
(assert (= (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (* load__ARRAY__2 1) (* x__ARRAY__51 (- 9))) (* x__ARRAY__52 (- 1))) (* x__ARRAY__53 (- 2))) (* x__ARRAY__54 (- 10))) (* x__ARRAY__55 (- 3))) (* x__ARRAY__56 (- 4))) (* x__ARRAY__57 (- 1))) (* x__ARRAY__58 (- 10))) (* x__ARRAY__59 (- 4))) (* x__ARRAY__60 (- 1))) (* x__ARRAY__61 (- 1))) (* x__ARRAY__62 (- 5))) (* x__ARRAY__63 (- 10))) (* x__ARRAY__64 (- 9))) (* x__ARRAY__65 (- 1))) (* x__ARRAY__66 (- 8))) (* x__ARRAY__67 (- 5))) (* x__ARRAY__68 (- 2))) (* x__ARRAY__69 (- 8))) (* x__ARRAY__70 (- 6))) (* x__ARRAY__71 (- 9))) (* x__ARRAY__72 (- 5))) (* x__ARRAY__73 (- 10))) (* x__ARRAY__74 (- 7))) (* x__ARRAY__75 (- 8))) (* x__ARRAY__76 (- 3))) (* x__ARRAY__77 (- 7))) (* x__ARRAY__78 (- 4))) (* x__ARRAY__79 (- 10))) (* x__ARRAY__80 (- 3))) (* x__ARRAY__81 (- 1))) (* x__ARRAY__82 (- 10))) (* x__ARRAY__83 (- 6))) (* x__ARRAY__84 (- 2))) (* x__ARRAY__85 (- 9))) (* x__ARRAY__86 (- 8))) (* x__ARRAY__87 (- 7))) (* x__ARRAY__88 (- 2))) (* x__ARRAY__89 (- 7))) (* x__ARRAY__90 (- 3))) (* x__ARRAY__91 (- 4))) (* x__ARRAY__92 (- 8))) (* x__ARRAY__93 (- 9))) (* x__ARRAY__94 (- 3))) (* x__ARRAY__95 (- 8))) (* x__ARRAY__96 (- 1))) (* x__ARRAY__97 (- 2))) (* x__ARRAY__98 (- 3))) (* x__ARRAY__99 (- 2))) (* x__ARRAY__100 (- 9))) 0))
(assert (= (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (* load__ARRAY__3 1) (* x__ARRAY__101 (- 9))) (* x__ARRAY__102 (- 1))) (* x__ARRAY__103 (- 2))) (* x__ARRAY__104 (- 10))) (* x__ARRAY__105 (- 3))) (* x__ARRAY__106 (- 4))) (* x__ARRAY__107 (- 1))) (* x__ARRAY__108 (- 10))) (* x__ARRAY__109 (- 4))) (* x__ARRAY__110 (- 1))) (* x__ARRAY__111 (- 1))) (* x__ARRAY__112 (- 5))) (* x__ARRAY__113 (- 10))) (* x__ARRAY__114 (- 9))) (* x__ARRAY__115 (- 1))) (* x__ARRAY__116 (- 8))) (* x__ARRAY__117 (- 5))) (* x__ARRAY__118 (- 2))) (* x__ARRAY__119 (- 8))) (* x__ARRAY__120 (- 6))) (* x__ARRAY__121 (- 9))) (* x__ARRAY__122 (- 5))) (* x__ARRAY__123 (- 10))) (* x__ARRAY__124 (- 7))) (* x__ARRAY__125 (- 8))) (* x__ARRAY__126 (- 3))) (* x__ARRAY__127 (- 7))) (* x__ARRAY__128 (- 4))) (* x__ARRAY__129 (- 10))) (* x__ARRAY__130 (- 3))) (* x__ARRAY__131 (- 1))) (* x__ARRAY__132 (- 10))) (* x__ARRAY__133 (- 6))) (* x__ARRAY__134 (- 2))) (* x__ARRAY__135 (- 9))) (* x__ARRAY__136 (- 8))) (* x__ARRAY__137 (- 7))) (* x__ARRAY__138 (- 2))) (* x__ARRAY__139 (- 7))) (* x__ARRAY__140 (- 3))) (* x__ARRAY__141 (- 4))) (* x__ARRAY__142 (- 8))) (* x__ARRAY__143 (- 9))) (* x__ARRAY__144 (- 3))) (* x__ARRAY__145 (- 8))) (* x__ARRAY__146 (- 1))) (* x__ARRAY__147 (- 2))) (* x__ARRAY__148 (- 3))) (* x__ARRAY__149 (- 2))) (* x__ARRAY__150 (- 9))) 0))
(assert (= (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (* load__ARRAY__4 1) (* x__ARRAY__151 (- 9))) (* x__ARRAY__152 (- 1))) (* x__ARRAY__153 (- 2))) (* x__ARRAY__154 (- 10))) (* x__ARRAY__155 (- 3))) (* x__ARRAY__156 (- 4))) (* x__ARRAY__157 (- 1))) (* x__ARRAY__158 (- 10))) (* x__ARRAY__159 (- 4))) (* x__ARRAY__160 (- 1))) (* x__ARRAY__161 (- 1))) (* x__ARRAY__162 (- 5))) (* x__ARRAY__163 (- 10))) (* x__ARRAY__164 (- 9))) (* x__ARRAY__165 (- 1))) (* x__ARRAY__166 (- 8))) (* x__ARRAY__167 (- 5))) (* x__ARRAY__168 (- 2))) (* x__ARRAY__169 (- 8))) (* x__ARRAY__170 (- 6))) (* x__ARRAY__171 (- 9))) (* x__ARRAY__172 (- 5))) (* x__ARRAY__173 (- 10))) (* x__ARRAY__174 (- 7))) (* x__ARRAY__175 (- 8))) (* x__ARRAY__176 (- 3))) (* x__ARRAY__177 (- 7))) (* x__ARRAY__178 (- 4))) (* x__ARRAY__179 (- 10))) (* x__ARRAY__180 (- 3))) (* x__ARRAY__181 (- 1))) (* x__ARRAY__182 (- 10))) (* x__ARRAY__183 (- 6))) (* x__ARRAY__184 (- 2))) (* x__ARRAY__185 (- 9))) (* x__ARRAY__186 (- 8))) (* x__ARRAY__187 (- 7))) (* x__ARRAY__188 (- 2))) (* x__ARRAY__189 (- 7))) (* x__ARRAY__190 (- 3))) (* x__ARRAY__191 (- 4))) (* x__ARRAY__192 (- 8))) (* x__ARRAY__193 (- 9))) (* x__ARRAY__194 (- 3))) (* x__ARRAY__195 (- 8))) (* x__ARRAY__196 (- 1))) (* x__ARRAY__197 (- 2))) (* x__ARRAY__198 (- 3))) (* x__ARRAY__199 (- 2))) (* x__ARRAY__200 (- 9))) 0))
(assert (= (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (* load__ARRAY__5 1) (* x__ARRAY__201 (- 9))) (* x__ARRAY__202 (- 1))) (* x__ARRAY__203 (- 2))) (* x__ARRAY__204 (- 10))) (* x__ARRAY__205 (- 3))) (* x__ARRAY__206 (- 4))) (* x__ARRAY__207 (- 1))) (* x__ARRAY__208 (- 10))) (* x__ARRAY__209 (- 4))) (* x__ARRAY__210 (- 1))) (* x__ARRAY__211 (- 1))) (* x__ARRAY__212 (- 5))) (* x__ARRAY__213 (- 10))) (* x__ARRAY__214 (- 9))) (* x__ARRAY__215 (- 1))) (* x__ARRAY__216 (- 8))) (* x__ARRAY__217 (- 5))) (* x__ARRAY__218 (- 2))) (* x__ARRAY__219 (- 8))) (* x__ARRAY__220 (- 6))) (* x__ARRAY__221 (- 9))) (* x__ARRAY__222 (- 5))) (* x__ARRAY__223 (- 10))) (* x__ARRAY__224 (- 7))) (* x__ARRAY__225 (- 8))) (* x__ARRAY__226 (- 3))) (* x__ARRAY__227 (- 7))) (* x__ARRAY__228 (- 4))) (* x__ARRAY__229 (- 10))) (* x__ARRAY__230 (- 3))) (* x__ARRAY__231 (- 1))) (* x__ARRAY__232 (- 10))) (* x__ARRAY__233 (- 6))) (* x__ARRAY__234 (- 2))) (* x__ARRAY__235 (- 9))) (* x__ARRAY__236 (- 8))) (* x__ARRAY__237 (- 7))) (* x__ARRAY__238 (- 2))) (* x__ARRAY__239 (- 7))) (* x__ARRAY__240 (- 3))) (* x__ARRAY__241 (- 4))) (* x__ARRAY__242 (- 8))) (* x__ARRAY__243 (- 9))) (* x__ARRAY__244 (- 3))) (* x__ARRAY__245 (- 8))) (* x__ARRAY__246 (- 1))) (* x__ARRAY__247 (- 2))) (* x__ARRAY__248 (- 3))) (* x__ARRAY__249 (- 2))) (* x__ARRAY__250 (- 9))) 0))
(assert (= (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (* load__ARRAY__6 1) (* x__ARRAY__251 (- 9))) (* x__ARRAY__252 (- 1))) (* x__ARRAY__253 (- 2))) (* x__ARRAY__254 (- 10))) (* x__ARRAY__255 (- 3))) (* x__ARRAY__256 (- 4))) (* x__ARRAY__257 (- 1))) (* x__ARRAY__258 (- 10))) (* x__ARRAY__259 (- 4))) (* x__ARRAY__260 (- 1))) (* x__ARRAY__261 (- 1))) (* x__ARRAY__262 (- 5))) (* x__ARRAY__263 (- 10))) (* x__ARRAY__264 (- 9))) (* x__ARRAY__265 (- 1))) (* x__ARRAY__266 (- 8))) (* x__ARRAY__267 (- 5))) (* x__ARRAY__268 (- 2))) (* x__ARRAY__269 (- 8))) (* x__ARRAY__270 (- 6))) (* x__ARRAY__271 (- 9))) (* x__ARRAY__272 (- 5))) (* x__ARRAY__273 (- 10))) (* x__ARRAY__274 (- 7))) (* x__ARRAY__275 (- 8))) (* x__ARRAY__276 (- 3))) (* x__ARRAY__277 (- 7))) (* x__ARRAY__278 (- 4))) (* x__ARRAY__279 (- 10))) (* x__ARRAY__280 (- 3))) (* x__ARRAY__281 (- 1))) (* x__ARRAY__282 (- 10))) (* x__ARRAY__283 (- 6))) (* x__ARRAY__284 (- 2))) (* x__ARRAY__285 (- 9))) (* x__ARRAY__286 (- 8))) (* x__ARRAY__287 (- 7))) (* x__ARRAY__288 (- 2))) (* x__ARRAY__289 (- 7))) (* x__ARRAY__290 (- 3))) (* x__ARRAY__291 (- 4))) (* x__ARRAY__292 (- 8))) (* x__ARRAY__293 (- 9))) (* x__ARRAY__294 (- 3))) (* x__ARRAY__295 (- 8))) (* x__ARRAY__296 (- 1))) (* x__ARRAY__297 (- 2))) (* x__ARRAY__298 (- 3))) (* x__ARRAY__299 (- 2))) (* x__ARRAY__300 (- 9))) 0))
(assert (= (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (* load__ARRAY__7 1) (* x__ARRAY__301 (- 9))) (* x__ARRAY__302 (- 1))) (* x__ARRAY__303 (- 2))) (* x__ARRAY__304 (- 10))) (* x__ARRAY__305 (- 3))) (* x__ARRAY__306 (- 4))) (* x__ARRAY__307 (- 1))) (* x__ARRAY__308 (- 10))) (* x__ARRAY__309 (- 4))) (* x__ARRAY__310 (- 1))) (* x__ARRAY__311 (- 1))) (* x__ARRAY__312 (- 5))) (* x__ARRAY__313 (- 10))) (* x__ARRAY__314 (- 9))) (* x__ARRAY__315 (- 1))) (* x__ARRAY__316 (- 8))) (* x__ARRAY__317 (- 5))) (* x__ARRAY__318 (- 2))) (* x__ARRAY__319 (- 8))) (* x__ARRAY__320 (- 6))) (* x__ARRAY__321 (- 9))) (* x__ARRAY__322 (- 5))) (* x__ARRAY__323 (- 10))) (* x__ARRAY__324 (- 7))) (* x__ARRAY__325 (- 8))) (* x__ARRAY__326 (- 3))) (* x__ARRAY__327 (- 7))) (* x__ARRAY__328 (- 4))) (* x__ARRAY__329 (- 10))) (* x__ARRAY__330 (- 3))) (* x__ARRAY__331 (- 1))) (* x__ARRAY__332 (- 10))) (* x__ARRAY__333 (- 6))) (* x__ARRAY__334 (- 2))) (* x__ARRAY__335 (- 9))) (* x__ARRAY__336 (- 8))) (* x__ARRAY__337 (- 7))) (* x__ARRAY__338 (- 2))) (* x__ARRAY__339 (- 7))) (* x__ARRAY__340 (- 3))) (* x__ARRAY__341 (- 4))) (* x__ARRAY__342 (- 8))) (* x__ARRAY__343 (- 9))) (* x__ARRAY__344 (- 3))) (* x__ARRAY__345 (- 8))) (* x__ARRAY__346 (- 1))) (* x__ARRAY__347 (- 2))) (* x__ARRAY__348 (- 3))) (* x__ARRAY__349 (- 2))) (* x__ARRAY__350 (- 9))) 0))
(assert (= (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (* load__ARRAY__8 1) (* x__ARRAY__351 (- 9))) (* x__ARRAY__352 (- 1))) (* x__ARRAY__353 (- 2))) (* x__ARRAY__354 (- 10))) (* x__ARRAY__355 (- 3))) (* x__ARRAY__356 (- 4))) (* x__ARRAY__357 (- 1))) (* x__ARRAY__358 (- 10))) (* x__ARRAY__359 (- 4))) (* x__ARRAY__360 (- 1))) (* x__ARRAY__361 (- 1))) (* x__ARRAY__362 (- 5))) (* x__ARRAY__363 (- 10))) (* x__ARRAY__364 (- 9))) (* x__ARRAY__365 (- 1))) (* x__ARRAY__366 (- 8))) (* x__ARRAY__367 (- 5))) (* x__ARRAY__368 (- 2))) (* x__ARRAY__369 (- 8))) (* x__ARRAY__370 (- 6))) (* x__ARRAY__371 (- 9))) (* x__ARRAY__372 (- 5))) (* x__ARRAY__373 (- 10))) (* x__ARRAY__374 (- 7))) (* x__ARRAY__375 (- 8))) (* x__ARRAY__376 (- 3))) (* x__ARRAY__377 (- 7))) (* x__ARRAY__378 (- 4))) (* x__ARRAY__379 (- 10))) (* x__ARRAY__380 (- 3))) (* x__ARRAY__381 (- 1))) (* x__ARRAY__382 (- 10))) (* x__ARRAY__383 (- 6))) (* x__ARRAY__384 (- 2))) (* x__ARRAY__385 (- 9))) (* x__ARRAY__386 (- 8))) (* x__ARRAY__387 (- 7))) (* x__ARRAY__388 (- 2))) (* x__ARRAY__389 (- 7))) (* x__ARRAY__390 (- 3))) (* x__ARRAY__391 (- 4))) (* x__ARRAY__392 (- 8))) (* x__ARRAY__393 (- 9))) (* x__ARRAY__394 (- 3))) (* x__ARRAY__395 (- 8))) (* x__ARRAY__396 (- 1))) (* x__ARRAY__397 (- 2))) (* x__ARRAY__398 (- 3))) (* x__ARRAY__399 (- 2))) (* x__ARRAY__400 (- 9))) 0))
(assert (= (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (* load__ARRAY__9 1) (* x__ARRAY__401 (- 9))) (* x__ARRAY__402 (- 1))) (* x__ARRAY__403 (- 2))) (* x__ARRAY__404 (- 10))) (* x__ARRAY__405 (- 3))) (* x__ARRAY__406 (- 4))) (* x__ARRAY__407 (- 1))) (* x__ARRAY__408 (- 10))) (* x__ARRAY__409 (- 4))) (* x__ARRAY__410 (- 1))) (* x__ARRAY__411 (- 1))) (* x__ARRAY__412 (- 5))) (* x__ARRAY__413 (- 10))) (* x__ARRAY__414 (- 9))) (* x__ARRAY__415 (- 1))) (* x__ARRAY__416 (- 8))) (* x__ARRAY__417 (- 5))) (* x__ARRAY__418 (- 2))) (* x__ARRAY__419 (- 8))) (* x__ARRAY__420 (- 6))) (* x__ARRAY__421 (- 9))) (* x__ARRAY__422 (- 5))) (* x__ARRAY__423 (- 10))) (* x__ARRAY__424 (- 7))) (* x__ARRAY__425 (- 8))) (* x__ARRAY__426 (- 3))) (* x__ARRAY__427 (- 7))) (* x__ARRAY__428 (- 4))) (* x__ARRAY__429 (- 10))) (* x__ARRAY__430 (- 3))) (* x__ARRAY__431 (- 1))) (* x__ARRAY__432 (- 10))) (* x__ARRAY__433 (- 6))) (* x__ARRAY__434 (- 2))) (* x__ARRAY__435 (- 9))) (* x__ARRAY__436 (- 8))) (* x__ARRAY__437 (- 7))) (* x__ARRAY__438 (- 2))) (* x__ARRAY__439 (- 7))) (* x__ARRAY__440 (- 3))) (* x__ARRAY__441 (- 4))) (* x__ARRAY__442 (- 8))) (* x__ARRAY__443 (- 9))) (* x__ARRAY__444 (- 3))) (* x__ARRAY__445 (- 8))) (* x__ARRAY__446 (- 1))) (* x__ARRAY__447 (- 2))) (* x__ARRAY__448 (- 3))) (* x__ARRAY__449 (- 2))) (* x__ARRAY__450 (- 9))) 0))
(assert (= (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (* load__ARRAY__10 1) (* x__ARRAY__451 (- 9))) (* x__ARRAY__452 (- 1))) (* x__ARRAY__453 (- 2))) (* x__ARRAY__454 (- 10))) (* x__ARRAY__455 (- 3))) (* x__ARRAY__456 (- 4))) (* x__ARRAY__457 (- 1))) (* x__ARRAY__458 (- 10))) (* x__ARRAY__459 (- 4))) (* x__ARRAY__460 (- 1))) (* x__ARRAY__461 (- 1))) (* x__ARRAY__462 (- 5))) (* x__ARRAY__463 (- 10))) (* x__ARRAY__464 (- 9))) (* x__ARRAY__465 (- 1))) (* x__ARRAY__466 (- 8))) (* x__ARRAY__467 (- 5))) (* x__ARRAY__468 (- 2))) (* x__ARRAY__469 (- 8))) (* x__ARRAY__470 (- 6))) (* x__ARRAY__471 (- 9))) (* x__ARRAY__472 (- 5))) (* x__ARRAY__473 (- 10))) (* x__ARRAY__474 (- 7))) (* x__ARRAY__475 (- 8))) (* x__ARRAY__476 (- 3))) (* x__ARRAY__477 (- 7))) (* x__ARRAY__478 (- 4))) (* x__ARRAY__479 (- 10))) (* x__ARRAY__480 (- 3))) (* x__ARRAY__481 (- 1))) (* x__ARRAY__482 (- 10))) (* x__ARRAY__483 (- 6))) (* x__ARRAY__484 (- 2))) (* x__ARRAY__485 (- 9))) (* x__ARRAY__486 (- 8))) (* x__ARRAY__487 (- 7))) (* x__ARRAY__488 (- 2))) (* x__ARRAY__489 (- 7))) (* x__ARRAY__490 (- 3))) (* x__ARRAY__491 (- 4))) (* x__ARRAY__492 (- 8))) (* x__ARRAY__493 (- 9))) (* x__ARRAY__494 (- 3))) (* x__ARRAY__495 (- 8))) (* x__ARRAY__496 (- 1))) (* x__ARRAY__497 (- 2))) (* x__ARRAY__498 (- 3))) (* x__ARRAY__499 (- 2))) (* x__ARRAY__500 (- 9))) 0))
(assert (<= (* objective (- 10)) (- 270)))
(assert (<= (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (* Real____00002 (- 9)) (* Real____00004 (- 1))) (* Real____00006 (- 2))) (* Real____00008 (- 10))) (* Real____00010 (- 3))) (* Real____00012 (- 4))) (* Real____00014 (- 1))) (* Real____00016 (- 10))) (* Real____00018 (- 4))) (* Real____00020 (- 1))) (* Real____00022 (- 1))) (* Real____00024 (- 5))) (* Real____00026 (- 10))) (* Real____00028 (- 9))) (* Real____00030 (- 1))) (* Real____00032 (- 8))) (* Real____00034 (- 5))) (* Real____00036 (- 2))) (* Real____00038 (- 8))) (* Real____00040 (- 6))) (* Real____00042 (- 9))) (* Real____00044 (- 5))) (* Real____00046 (- 10))) (* Real____00048 (- 7))) (* Real____00050 (- 8))) (* Real____00052 (- 3))) (* Real____00054 (- 7))) (* Real____00056 (- 4))) (* Real____00058 (- 10))) (* Real____00060 (- 3))) (* Real____00062 (- 1))) (* Real____00064 (- 10))) (* Real____00066 (- 6))) (* Real____00068 (- 2))) (* Real____00070 (- 9))) (* Real____00072 (- 8))) (* Real____00074 (- 7))) (* Real____00076 (- 2))) (* Real____00078 (- 7))) (* Real____00080 (- 3))) (* Real____00082 (- 4))) (* Real____00084 (- 8))) (* Real____00086 (- 9))) (* Real____00088 (- 3))) (* Real____00090 (- 8))) (* Real____00092 (- 1))) (* Real____00094 (- 2))) (* Real____00096 (- 3))) (* Real____00098 (- 2))) (* Real____00100 (- 9))) (- 18)))
(assert (<= (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (* Real____00102 (- 9)) (* Real____00104 (- 1))) (* Real____00106 (- 2))) (* Real____00108 (- 10))) (* Real____00110 (- 3))) (* Real____00112 (- 4))) (* Real____00114 (- 1))) (* Real____00116 (- 10))) (* Real____00118 (- 4))) (* Real____00120 (- 1))) (* Real____00122 (- 1))) (* Real____00124 (- 5))) (* Real____00126 (- 10))) (* Real____00128 (- 9))) (* Real____00130 (- 1))) (* Real____00132 (- 8))) (* Real____00134 (- 5))) (* Real____00136 (- 2))) (* Real____00138 (- 8))) (* Real____00140 (- 6))) (* Real____00142 (- 9))) (* Real____00144 (- 5))) (* Real____00146 (- 10))) (* Real____00148 (- 7))) (* Real____00150 (- 8))) (* Real____00152 (- 3))) (* Real____00154 (- 7))) (* Real____00156 (- 4))) (* Real____00158 (- 10))) (* Real____00160 (- 3))) (* Real____00162 (- 1))) (* Real____00164 (- 10))) (* Real____00166 (- 6))) (* Real____00168 (- 2))) (* Real____00170 (- 9))) (* Real____00172 (- 8))) (* Real____00174 (- 7))) (* Real____00176 (- 2))) (* Real____00178 (- 7))) (* Real____00180 (- 3))) (* Real____00182 (- 4))) (* Real____00184 (- 8))) (* Real____00186 (- 9))) (* Real____00188 (- 3))) (* Real____00190 (- 8))) (* Real____00192 (- 1))) (* Real____00194 (- 2))) (* Real____00196 (- 3))) (* Real____00198 (- 2))) (* Real____00200 (- 9))) (- 16)))
(assert (<= (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (* Real____00202 (- 9)) (* Real____00204 (- 1))) (* Real____00206 (- 2))) (* Real____00208 (- 10))) (* Real____00210 (- 3))) (* Real____00212 (- 4))) (* Real____00214 (- 1))) (* Real____00216 (- 10))) (* Real____00218 (- 4))) (* Real____00220 (- 1))) (* Real____00222 (- 1))) (* Real____00224 (- 5))) (* Real____00226 (- 10))) (* Real____00228 (- 9))) (* Real____00230 (- 1))) (* Real____00232 (- 8))) (* Real____00234 (- 5))) (* Real____00236 (- 2))) (* Real____00238 (- 8))) (* Real____00240 (- 6))) (* Real____00242 (- 9))) (* Real____00244 (- 5))) (* Real____00246 (- 10))) (* Real____00248 (- 7))) (* Real____00250 (- 8))) (* Real____00252 (- 3))) (* Real____00254 (- 7))) (* Real____00256 (- 4))) (* Real____00258 (- 10))) (* Real____00260 (- 3))) (* Real____00262 (- 1))) (* Real____00264 (- 10))) (* Real____00266 (- 6))) (* Real____00268 (- 2))) (* Real____00270 (- 9))) (* Real____00272 (- 8))) (* Real____00274 (- 7))) (* Real____00276 (- 2))) (* Real____00278 (- 7))) (* Real____00280 (- 3))) (* Real____00282 (- 4))) (* Real____00284 (- 8))) (* Real____00286 (- 9))) (* Real____00288 (- 3))) (* Real____00290 (- 8))) (* Real____00292 (- 1))) (* Real____00294 (- 2))) (* Real____00296 (- 3))) (* Real____00298 (- 2))) (* Real____00300 (- 9))) (- 14)))
(assert (<= (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (* Real____00302 (- 9)) (* Real____00304 (- 1))) (* Real____00306 (- 2))) (* Real____00308 (- 10))) (* Real____00310 (- 3))) (* Real____00312 (- 4))) (* Real____00314 (- 1))) (* Real____00316 (- 10))) (* Real____00318 (- 4))) (* Real____00320 (- 1))) (* Real____00322 (- 1))) (* Real____00324 (- 5))) (* Real____00326 (- 10))) (* Real____00328 (- 9))) (* Real____00330 (- 1))) (* Real____00332 (- 8))) (* Real____00334 (- 5))) (* Real____00336 (- 2))) (* Real____00338 (- 8))) (* Real____00340 (- 6))) (* Real____00342 (- 9))) (* Real____00344 (- 5))) (* Real____00346 (- 10))) (* Real____00348 (- 7))) (* Real____00350 (- 8))) (* Real____00352 (- 3))) (* Real____00354 (- 7))) (* Real____00356 (- 4))) (* Real____00358 (- 10))) (* Real____00360 (- 3))) (* Real____00362 (- 1))) (* Real____00364 (- 10))) (* Real____00366 (- 6))) (* Real____00368 (- 2))) (* Real____00370 (- 9))) (* Real____00372 (- 8))) (* Real____00374 (- 7))) (* Real____00376 (- 2))) (* Real____00378 (- 7))) (* Real____00380 (- 3))) (* Real____00382 (- 4))) (* Real____00384 (- 8))) (* Real____00386 (- 9))) (* Real____00388 (- 3))) (* Real____00390 (- 8))) (* Real____00392 (- 1))) (* Real____00394 (- 2))) (* Real____00396 (- 3))) (* Real____00398 (- 2))) (* Real____00400 (- 9))) (- 12)))
(assert (<= (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (* Real____00402 (- 9)) (* Real____00404 (- 1))) (* Real____00406 (- 2))) (* Real____00408 (- 10))) (* Real____00410 (- 3))) (* Real____00412 (- 4))) (* Real____00414 (- 1))) (* Real____00416 (- 10))) (* Real____00418 (- 4))) (* Real____00420 (- 1))) (* Real____00422 (- 1))) (* Real____00424 (- 5))) (* Real____00426 (- 10))) (* Real____00428 (- 9))) (* Real____00430 (- 1))) (* Real____00432 (- 8))) (* Real____00434 (- 5))) (* Real____00436 (- 2))) (* Real____00438 (- 8))) (* Real____00440 (- 6))) (* Real____00442 (- 9))) (* Real____00444 (- 5))) (* Real____00446 (- 10))) (* Real____00448 (- 7))) (* Real____00450 (- 8))) (* Real____00452 (- 3))) (* Real____00454 (- 7))) (* Real____00456 (- 4))) (* Real____00458 (- 10))) (* Real____00460 (- 3))) (* Real____00462 (- 1))) (* Real____00464 (- 10))) (* Real____00466 (- 6))) (* Real____00468 (- 2))) (* Real____00470 (- 9))) (* Real____00472 (- 8))) (* Real____00474 (- 7))) (* Real____00476 (- 2))) (* Real____00478 (- 7))) (* Real____00480 (- 3))) (* Real____00482 (- 4))) (* Real____00484 (- 8))) (* Real____00486 (- 9))) (* Real____00488 (- 3))) (* Real____00490 (- 8))) (* Real____00492 (- 1))) (* Real____00494 (- 2))) (* Real____00496 (- 3))) (* Real____00498 (- 2))) (* Real____00500 (- 9))) (- 10)))
(assert (<= (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (* Real____00502 (- 9)) (* Real____00504 (- 1))) (* Real____00506 (- 2))) (* Real____00508 (- 10))) (* Real____00510 (- 3))) (* Real____00512 (- 4))) (* Real____00514 (- 1))) (* Real____00516 (- 10))) (* Real____00518 (- 4))) (* Real____00520 (- 1))) (* Real____00522 (- 1))) (* Real____00524 (- 5))) (* Real____00526 (- 10))) (* Real____00528 (- 9))) (* Real____00530 (- 1))) (* Real____00532 (- 8))) (* Real____00534 (- 5))) (* Real____00536 (- 2))) (* Real____00538 (- 8))) (* Real____00540 (- 6))) (* Real____00542 (- 9))) (* Real____00544 (- 5))) (* Real____00546 (- 10))) (* Real____00548 (- 7))) (* Real____00550 (- 8))) (* Real____00552 (- 3))) (* Real____00554 (- 7))) (* Real____00556 (- 4))) (* Real____00558 (- 10))) (* Real____00560 (- 3))) (* Real____00562 (- 1))) (* Real____00564 (- 10))) (* Real____00566 (- 6))) (* Real____00568 (- 2))) (* Real____00570 (- 9))) (* Real____00572 (- 8))) (* Real____00574 (- 7))) (* Real____00576 (- 2))) (* Real____00578 (- 7))) (* Real____00580 (- 3))) (* Real____00582 (- 4))) (* Real____00584 (- 8))) (* Real____00586 (- 9))) (* Real____00588 (- 3))) (* Real____00590 (- 8))) (* Real____00592 (- 1))) (* Real____00594 (- 2))) (* Real____00596 (- 3))) (* Real____00598 (- 2))) (* Real____00600 (- 9))) (- 8)))
(assert (<= (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (* Real____00602 (- 9)) (* Real____00604 (- 1))) (* Real____00606 (- 2))) (* Real____00608 (- 10))) (* Real____00610 (- 3))) (* Real____00612 (- 4))) (* Real____00614 (- 1))) (* Real____00616 (- 10))) (* Real____00618 (- 4))) (* Real____00620 (- 1))) (* Real____00622 (- 1))) (* Real____00624 (- 5))) (* Real____00626 (- 10))) (* Real____00628 (- 9))) (* Real____00630 (- 1))) (* Real____00632 (- 8))) (* Real____00634 (- 5))) (* Real____00636 (- 2))) (* Real____00638 (- 8))) (* Real____00640 (- 6))) (* Real____00642 (- 9))) (* Real____00644 (- 5))) (* Real____00646 (- 10))) (* Real____00648 (- 7))) (* Real____00650 (- 8))) (* Real____00652 (- 3))) (* Real____00654 (- 7))) (* Real____00656 (- 4))) (* Real____00658 (- 10))) (* Real____00660 (- 3))) (* Real____00662 (- 1))) (* Real____00664 (- 10))) (* Real____00666 (- 6))) (* Real____00668 (- 2))) (* Real____00670 (- 9))) (* Real____00672 (- 8))) (* Real____00674 (- 7))) (* Real____00676 (- 2))) (* Real____00678 (- 7))) (* Real____00680 (- 3))) (* Real____00682 (- 4))) (* Real____00684 (- 8))) (* Real____00686 (- 9))) (* Real____00688 (- 3))) (* Real____00690 (- 8))) (* Real____00692 (- 1))) (* Real____00694 (- 2))) (* Real____00696 (- 3))) (* Real____00698 (- 2))) (* Real____00700 (- 9))) (- 6)))
(assert (<= (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (* Real____00702 (- 9)) (* Real____00704 (- 1))) (* Real____00706 (- 2))) (* Real____00708 (- 10))) (* Real____00710 (- 3))) (* Real____00712 (- 4))) (* Real____00714 (- 1))) (* Real____00716 (- 10))) (* Real____00718 (- 4))) (* Real____00720 (- 1))) (* Real____00722 (- 1))) (* Real____00724 (- 5))) (* Real____00726 (- 10))) (* Real____00728 (- 9))) (* Real____00730 (- 1))) (* Real____00732 (- 8))) (* Real____00734 (- 5))) (* Real____00736 (- 2))) (* Real____00738 (- 8))) (* Real____00740 (- 6))) (* Real____00742 (- 9))) (* Real____00744 (- 5))) (* Real____00746 (- 10))) (* Real____00748 (- 7))) (* Real____00750 (- 8))) (* Real____00752 (- 3))) (* Real____00754 (- 7))) (* Real____00756 (- 4))) (* Real____00758 (- 10))) (* Real____00760 (- 3))) (* Real____00762 (- 1))) (* Real____00764 (- 10))) (* Real____00766 (- 6))) (* Real____00768 (- 2))) (* Real____00770 (- 9))) (* Real____00772 (- 8))) (* Real____00774 (- 7))) (* Real____00776 (- 2))) (* Real____00778 (- 7))) (* Real____00780 (- 3))) (* Real____00782 (- 4))) (* Real____00784 (- 8))) (* Real____00786 (- 9))) (* Real____00788 (- 3))) (* Real____00790 (- 8))) (* Real____00792 (- 1))) (* Real____00794 (- 2))) (* Real____00796 (- 3))) (* Real____00798 (- 2))) (* Real____00800 (- 9))) (- 4)))
(assert (<= (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (* Real____00802 (- 9)) (* Real____00804 (- 1))) (* Real____00806 (- 2))) (* Real____00808 (- 10))) (* Real____00810 (- 3))) (* Real____00812 (- 4))) (* Real____00814 (- 1))) (* Real____00816 (- 10))) (* Real____00818 (- 4))) (* Real____00820 (- 1))) (* Real____00822 (- 1))) (* Real____00824 (- 5))) (* Real____00826 (- 10))) (* Real____00828 (- 9))) (* Real____00830 (- 1))) (* Real____00832 (- 8))) (* Real____00834 (- 5))) (* Real____00836 (- 2))) (* Real____00838 (- 8))) (* Real____00840 (- 6))) (* Real____00842 (- 9))) (* Real____00844 (- 5))) (* Real____00846 (- 10))) (* Real____00848 (- 7))) (* Real____00850 (- 8))) (* Real____00852 (- 3))) (* Real____00854 (- 7))) (* Real____00856 (- 4))) (* Real____00858 (- 10))) (* Real____00860 (- 3))) (* Real____00862 (- 1))) (* Real____00864 (- 10))) (* Real____00866 (- 6))) (* Real____00868 (- 2))) (* Real____00870 (- 9))) (* Real____00872 (- 8))) (* Real____00874 (- 7))) (* Real____00876 (- 2))) (* Real____00878 (- 7))) (* Real____00880 (- 3))) (* Real____00882 (- 4))) (* Real____00884 (- 8))) (* Real____00886 (- 9))) (* Real____00888 (- 3))) (* Real____00890 (- 8))) (* Real____00892 (- 1))) (* Real____00894 (- 2))) (* Real____00896 (- 3))) (* Real____00898 (- 2))) (* Real____00900 (- 9))) (- 2)))
(assert (<= (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (* x__ARRAY__1 (- 1)) (* x__ARRAY__2 (- 1))) (* x__ARRAY__3 (- 1))) (* x__ARRAY__4 (- 1))) (* x__ARRAY__5 (- 1))) (* x__ARRAY__6 (- 1))) (* x__ARRAY__7 (- 1))) (* x__ARRAY__8 (- 1))) (* x__ARRAY__9 (- 1))) (* x__ARRAY__10 (- 1))) (* x__ARRAY__11 (- 1))) (* x__ARRAY__12 (- 1))) (* x__ARRAY__13 (- 1))) (* x__ARRAY__14 (- 1))) (* x__ARRAY__15 (- 1))) (* x__ARRAY__16 (- 1))) (* x__ARRAY__17 (- 1))) (* x__ARRAY__18 (- 1))) (* x__ARRAY__19 (- 1))) (* x__ARRAY__20 (- 1))) (* x__ARRAY__21 (- 1))) (* x__ARRAY__22 (- 1))) (* x__ARRAY__23 (- 1))) (* x__ARRAY__24 (- 1))) (* x__ARRAY__25 (- 1))) (* x__ARRAY__26 (- 1))) (* x__ARRAY__27 (- 1))) (* x__ARRAY__28 (- 1))) (* x__ARRAY__29 (- 1))) (* x__ARRAY__30 (- 1))) (* x__ARRAY__31 (- 1))) (* x__ARRAY__32 (- 1))) (* x__ARRAY__33 (- 1))) (* x__ARRAY__34 (- 1))) (* x__ARRAY__35 (- 1))) (* x__ARRAY__36 (- 1))) (* x__ARRAY__37 (- 1))) (* x__ARRAY__38 (- 1))) (* x__ARRAY__39 (- 1))) (* x__ARRAY__40 (- 1))) (* x__ARRAY__41 (- 1))) (* x__ARRAY__42 (- 1))) (* x__ARRAY__43 (- 1))) (* x__ARRAY__44 (- 1))) (* x__ARRAY__45 (- 1))) (* x__ARRAY__46 (- 1))) (* x__ARRAY__47 (- 1))) (* x__ARRAY__48 (- 1))) (* x__ARRAY__49 (- 1))) (* x__ARRAY__50 (- 1))) (- 2)))
(assert (<= (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (* x__ARRAY__51 (- 1)) (* x__ARRAY__52 (- 1))) (* x__ARRAY__53 (- 1))) (* x__ARRAY__54 (- 1))) (* x__ARRAY__55 (- 1))) (* x__ARRAY__56 (- 1))) (* x__ARRAY__57 (- 1))) (* x__ARRAY__58 (- 1))) (* x__ARRAY__59 (- 1))) (* x__ARRAY__60 (- 1))) (* x__ARRAY__61 (- 1))) (* x__ARRAY__62 (- 1))) (* x__ARRAY__63 (- 1))) (* x__ARRAY__64 (- 1))) (* x__ARRAY__65 (- 1))) (* x__ARRAY__66 (- 1))) (* x__ARRAY__67 (- 1))) (* x__ARRAY__68 (- 1))) (* x__ARRAY__69 (- 1))) (* x__ARRAY__70 (- 1))) (* x__ARRAY__71 (- 1))) (* x__ARRAY__72 (- 1))) (* x__ARRAY__73 (- 1))) (* x__ARRAY__74 (- 1))) (* x__ARRAY__75 (- 1))) (* x__ARRAY__76 (- 1))) (* x__ARRAY__77 (- 1))) (* x__ARRAY__78 (- 1))) (* x__ARRAY__79 (- 1))) (* x__ARRAY__80 (- 1))) (* x__ARRAY__81 (- 1))) (* x__ARRAY__82 (- 1))) (* x__ARRAY__83 (- 1))) (* x__ARRAY__84 (- 1))) (* x__ARRAY__85 (- 1))) (* x__ARRAY__86 (- 1))) (* x__ARRAY__87 (- 1))) (* x__ARRAY__88 (- 1))) (* x__ARRAY__89 (- 1))) (* x__ARRAY__90 (- 1))) (* x__ARRAY__91 (- 1))) (* x__ARRAY__92 (- 1))) (* x__ARRAY__93 (- 1))) (* x__ARRAY__94 (- 1))) (* x__ARRAY__95 (- 1))) (* x__ARRAY__96 (- 1))) (* x__ARRAY__97 (- 1))) (* x__ARRAY__98 (- 1))) (* x__ARRAY__99 (- 1))) (* x__ARRAY__100 (- 1))) (- 2)))
(assert (<= (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (* x__ARRAY__101 (- 1)) (* x__ARRAY__102 (- 1))) (* x__ARRAY__103 (- 1))) (* x__ARRAY__104 (- 1))) (* x__ARRAY__105 (- 1))) (* x__ARRAY__106 (- 1))) (* x__ARRAY__107 (- 1))) (* x__ARRAY__108 (- 1))) (* x__ARRAY__109 (- 1))) (* x__ARRAY__110 (- 1))) (* x__ARRAY__111 (- 1))) (* x__ARRAY__112 (- 1))) (* x__ARRAY__113 (- 1))) (* x__ARRAY__114 (- 1))) (* x__ARRAY__115 (- 1))) (* x__ARRAY__116 (- 1))) (* x__ARRAY__117 (- 1))) (* x__ARRAY__118 (- 1))) (* x__ARRAY__119 (- 1))) (* x__ARRAY__120 (- 1))) (* x__ARRAY__121 (- 1))) (* x__ARRAY__122 (- 1))) (* x__ARRAY__123 (- 1))) (* x__ARRAY__124 (- 1))) (* x__ARRAY__125 (- 1))) (* x__ARRAY__126 (- 1))) (* x__ARRAY__127 (- 1))) (* x__ARRAY__128 (- 1))) (* x__ARRAY__129 (- 1))) (* x__ARRAY__130 (- 1))) (* x__ARRAY__131 (- 1))) (* x__ARRAY__132 (- 1))) (* x__ARRAY__133 (- 1))) (* x__ARRAY__134 (- 1))) (* x__ARRAY__135 (- 1))) (* x__ARRAY__136 (- 1))) (* x__ARRAY__137 (- 1))) (* x__ARRAY__138 (- 1))) (* x__ARRAY__139 (- 1))) (* x__ARRAY__140 (- 1))) (* x__ARRAY__141 (- 1))) (* x__ARRAY__142 (- 1))) (* x__ARRAY__143 (- 1))) (* x__ARRAY__144 (- 1))) (* x__ARRAY__145 (- 1))) (* x__ARRAY__146 (- 1))) (* x__ARRAY__147 (- 1))) (* x__ARRAY__148 (- 1))) (* x__ARRAY__149 (- 1))) (* x__ARRAY__150 (- 1))) (- 2)))
(assert (<= (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (* x__ARRAY__151 (- 1)) (* x__ARRAY__152 (- 1))) (* x__ARRAY__153 (- 1))) (* x__ARRAY__154 (- 1))) (* x__ARRAY__155 (- 1))) (* x__ARRAY__156 (- 1))) (* x__ARRAY__157 (- 1))) (* x__ARRAY__158 (- 1))) (* x__ARRAY__159 (- 1))) (* x__ARRAY__160 (- 1))) (* x__ARRAY__161 (- 1))) (* x__ARRAY__162 (- 1))) (* x__ARRAY__163 (- 1))) (* x__ARRAY__164 (- 1))) (* x__ARRAY__165 (- 1))) (* x__ARRAY__166 (- 1))) (* x__ARRAY__167 (- 1))) (* x__ARRAY__168 (- 1))) (* x__ARRAY__169 (- 1))) (* x__ARRAY__170 (- 1))) (* x__ARRAY__171 (- 1))) (* x__ARRAY__172 (- 1))) (* x__ARRAY__173 (- 1))) (* x__ARRAY__174 (- 1))) (* x__ARRAY__175 (- 1))) (* x__ARRAY__176 (- 1))) (* x__ARRAY__177 (- 1))) (* x__ARRAY__178 (- 1))) (* x__ARRAY__179 (- 1))) (* x__ARRAY__180 (- 1))) (* x__ARRAY__181 (- 1))) (* x__ARRAY__182 (- 1))) (* x__ARRAY__183 (- 1))) (* x__ARRAY__184 (- 1))) (* x__ARRAY__185 (- 1))) (* x__ARRAY__186 (- 1))) (* x__ARRAY__187 (- 1))) (* x__ARRAY__188 (- 1))) (* x__ARRAY__189 (- 1))) (* x__ARRAY__190 (- 1))) (* x__ARRAY__191 (- 1))) (* x__ARRAY__192 (- 1))) (* x__ARRAY__193 (- 1))) (* x__ARRAY__194 (- 1))) (* x__ARRAY__195 (- 1))) (* x__ARRAY__196 (- 1))) (* x__ARRAY__197 (- 1))) (* x__ARRAY__198 (- 1))) (* x__ARRAY__199 (- 1))) (* x__ARRAY__200 (- 1))) (- 2)))
(assert (<= (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (* x__ARRAY__201 (- 1)) (* x__ARRAY__202 (- 1))) (* x__ARRAY__203 (- 1))) (* x__ARRAY__204 (- 1))) (* x__ARRAY__205 (- 1))) (* x__ARRAY__206 (- 1))) (* x__ARRAY__207 (- 1))) (* x__ARRAY__208 (- 1))) (* x__ARRAY__209 (- 1))) (* x__ARRAY__210 (- 1))) (* x__ARRAY__211 (- 1))) (* x__ARRAY__212 (- 1))) (* x__ARRAY__213 (- 1))) (* x__ARRAY__214 (- 1))) (* x__ARRAY__215 (- 1))) (* x__ARRAY__216 (- 1))) (* x__ARRAY__217 (- 1))) (* x__ARRAY__218 (- 1))) (* x__ARRAY__219 (- 1))) (* x__ARRAY__220 (- 1))) (* x__ARRAY__221 (- 1))) (* x__ARRAY__222 (- 1))) (* x__ARRAY__223 (- 1))) (* x__ARRAY__224 (- 1))) (* x__ARRAY__225 (- 1))) (* x__ARRAY__226 (- 1))) (* x__ARRAY__227 (- 1))) (* x__ARRAY__228 (- 1))) (* x__ARRAY__229 (- 1))) (* x__ARRAY__230 (- 1))) (* x__ARRAY__231 (- 1))) (* x__ARRAY__232 (- 1))) (* x__ARRAY__233 (- 1))) (* x__ARRAY__234 (- 1))) (* x__ARRAY__235 (- 1))) (* x__ARRAY__236 (- 1))) (* x__ARRAY__237 (- 1))) (* x__ARRAY__238 (- 1))) (* x__ARRAY__239 (- 1))) (* x__ARRAY__240 (- 1))) (* x__ARRAY__241 (- 1))) (* x__ARRAY__242 (- 1))) (* x__ARRAY__243 (- 1))) (* x__ARRAY__244 (- 1))) (* x__ARRAY__245 (- 1))) (* x__ARRAY__246 (- 1))) (* x__ARRAY__247 (- 1))) (* x__ARRAY__248 (- 1))) (* x__ARRAY__249 (- 1))) (* x__ARRAY__250 (- 1))) (- 2)))
(assert (<= (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (* x__ARRAY__251 (- 1)) (* x__ARRAY__252 (- 1))) (* x__ARRAY__253 (- 1))) (* x__ARRAY__254 (- 1))) (* x__ARRAY__255 (- 1))) (* x__ARRAY__256 (- 1))) (* x__ARRAY__257 (- 1))) (* x__ARRAY__258 (- 1))) (* x__ARRAY__259 (- 1))) (* x__ARRAY__260 (- 1))) (* x__ARRAY__261 (- 1))) (* x__ARRAY__262 (- 1))) (* x__ARRAY__263 (- 1))) (* x__ARRAY__264 (- 1))) (* x__ARRAY__265 (- 1))) (* x__ARRAY__266 (- 1))) (* x__ARRAY__267 (- 1))) (* x__ARRAY__268 (- 1))) (* x__ARRAY__269 (- 1))) (* x__ARRAY__270 (- 1))) (* x__ARRAY__271 (- 1))) (* x__ARRAY__272 (- 1))) (* x__ARRAY__273 (- 1))) (* x__ARRAY__274 (- 1))) (* x__ARRAY__275 (- 1))) (* x__ARRAY__276 (- 1))) (* x__ARRAY__277 (- 1))) (* x__ARRAY__278 (- 1))) (* x__ARRAY__279 (- 1))) (* x__ARRAY__280 (- 1))) (* x__ARRAY__281 (- 1))) (* x__ARRAY__282 (- 1))) (* x__ARRAY__283 (- 1))) (* x__ARRAY__284 (- 1))) (* x__ARRAY__285 (- 1))) (* x__ARRAY__286 (- 1))) (* x__ARRAY__287 (- 1))) (* x__ARRAY__288 (- 1))) (* x__ARRAY__289 (- 1))) (* x__ARRAY__290 (- 1))) (* x__ARRAY__291 (- 1))) (* x__ARRAY__292 (- 1))) (* x__ARRAY__293 (- 1))) (* x__ARRAY__294 (- 1))) (* x__ARRAY__295 (- 1))) (* x__ARRAY__296 (- 1))) (* x__ARRAY__297 (- 1))) (* x__ARRAY__298 (- 1))) (* x__ARRAY__299 (- 1))) (* x__ARRAY__300 (- 1))) (- 2)))
(assert (<= (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (* x__ARRAY__301 (- 1)) (* x__ARRAY__302 (- 1))) (* x__ARRAY__303 (- 1))) (* x__ARRAY__304 (- 1))) (* x__ARRAY__305 (- 1))) (* x__ARRAY__306 (- 1))) (* x__ARRAY__307 (- 1))) (* x__ARRAY__308 (- 1))) (* x__ARRAY__309 (- 1))) (* x__ARRAY__310 (- 1))) (* x__ARRAY__311 (- 1))) (* x__ARRAY__312 (- 1))) (* x__ARRAY__313 (- 1))) (* x__ARRAY__314 (- 1))) (* x__ARRAY__315 (- 1))) (* x__ARRAY__316 (- 1))) (* x__ARRAY__317 (- 1))) (* x__ARRAY__318 (- 1))) (* x__ARRAY__319 (- 1))) (* x__ARRAY__320 (- 1))) (* x__ARRAY__321 (- 1))) (* x__ARRAY__322 (- 1))) (* x__ARRAY__323 (- 1))) (* x__ARRAY__324 (- 1))) (* x__ARRAY__325 (- 1))) (* x__ARRAY__326 (- 1))) (* x__ARRAY__327 (- 1))) (* x__ARRAY__328 (- 1))) (* x__ARRAY__329 (- 1))) (* x__ARRAY__330 (- 1))) (* x__ARRAY__331 (- 1))) (* x__ARRAY__332 (- 1))) (* x__ARRAY__333 (- 1))) (* x__ARRAY__334 (- 1))) (* x__ARRAY__335 (- 1))) (* x__ARRAY__336 (- 1))) (* x__ARRAY__337 (- 1))) (* x__ARRAY__338 (- 1))) (* x__ARRAY__339 (- 1))) (* x__ARRAY__340 (- 1))) (* x__ARRAY__341 (- 1))) (* x__ARRAY__342 (- 1))) (* x__ARRAY__343 (- 1))) (* x__ARRAY__344 (- 1))) (* x__ARRAY__345 (- 1))) (* x__ARRAY__346 (- 1))) (* x__ARRAY__347 (- 1))) (* x__ARRAY__348 (- 1))) (* x__ARRAY__349 (- 1))) (* x__ARRAY__350 (- 1))) (- 2)))
(assert (<= (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (* x__ARRAY__351 (- 1)) (* x__ARRAY__352 (- 1))) (* x__ARRAY__353 (- 1))) (* x__ARRAY__354 (- 1))) (* x__ARRAY__355 (- 1))) (* x__ARRAY__356 (- 1))) (* x__ARRAY__357 (- 1))) (* x__ARRAY__358 (- 1))) (* x__ARRAY__359 (- 1))) (* x__ARRAY__360 (- 1))) (* x__ARRAY__361 (- 1))) (* x__ARRAY__362 (- 1))) (* x__ARRAY__363 (- 1))) (* x__ARRAY__364 (- 1))) (* x__ARRAY__365 (- 1))) (* x__ARRAY__366 (- 1))) (* x__ARRAY__367 (- 1))) (* x__ARRAY__368 (- 1))) (* x__ARRAY__369 (- 1))) (* x__ARRAY__370 (- 1))) (* x__ARRAY__371 (- 1))) (* x__ARRAY__372 (- 1))) (* x__ARRAY__373 (- 1))) (* x__ARRAY__374 (- 1))) (* x__ARRAY__375 (- 1))) (* x__ARRAY__376 (- 1))) (* x__ARRAY__377 (- 1))) (* x__ARRAY__378 (- 1))) (* x__ARRAY__379 (- 1))) (* x__ARRAY__380 (- 1))) (* x__ARRAY__381 (- 1))) (* x__ARRAY__382 (- 1))) (* x__ARRAY__383 (- 1))) (* x__ARRAY__384 (- 1))) (* x__ARRAY__385 (- 1))) (* x__ARRAY__386 (- 1))) (* x__ARRAY__387 (- 1))) (* x__ARRAY__388 (- 1))) (* x__ARRAY__389 (- 1))) (* x__ARRAY__390 (- 1))) (* x__ARRAY__391 (- 1))) (* x__ARRAY__392 (- 1))) (* x__ARRAY__393 (- 1))) (* x__ARRAY__394 (- 1))) (* x__ARRAY__395 (- 1))) (* x__ARRAY__396 (- 1))) (* x__ARRAY__397 (- 1))) (* x__ARRAY__398 (- 1))) (* x__ARRAY__399 (- 1))) (* x__ARRAY__400 (- 1))) (- 2)))
(assert (<= (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (* x__ARRAY__401 (- 1)) (* x__ARRAY__402 (- 1))) (* x__ARRAY__403 (- 1))) (* x__ARRAY__404 (- 1))) (* x__ARRAY__405 (- 1))) (* x__ARRAY__406 (- 1))) (* x__ARRAY__407 (- 1))) (* x__ARRAY__408 (- 1))) (* x__ARRAY__409 (- 1))) (* x__ARRAY__410 (- 1))) (* x__ARRAY__411 (- 1))) (* x__ARRAY__412 (- 1))) (* x__ARRAY__413 (- 1))) (* x__ARRAY__414 (- 1))) (* x__ARRAY__415 (- 1))) (* x__ARRAY__416 (- 1))) (* x__ARRAY__417 (- 1))) (* x__ARRAY__418 (- 1))) (* x__ARRAY__419 (- 1))) (* x__ARRAY__420 (- 1))) (* x__ARRAY__421 (- 1))) (* x__ARRAY__422 (- 1))) (* x__ARRAY__423 (- 1))) (* x__ARRAY__424 (- 1))) (* x__ARRAY__425 (- 1))) (* x__ARRAY__426 (- 1))) (* x__ARRAY__427 (- 1))) (* x__ARRAY__428 (- 1))) (* x__ARRAY__429 (- 1))) (* x__ARRAY__430 (- 1))) (* x__ARRAY__431 (- 1))) (* x__ARRAY__432 (- 1))) (* x__ARRAY__433 (- 1))) (* x__ARRAY__434 (- 1))) (* x__ARRAY__435 (- 1))) (* x__ARRAY__436 (- 1))) (* x__ARRAY__437 (- 1))) (* x__ARRAY__438 (- 1))) (* x__ARRAY__439 (- 1))) (* x__ARRAY__440 (- 1))) (* x__ARRAY__441 (- 1))) (* x__ARRAY__442 (- 1))) (* x__ARRAY__443 (- 1))) (* x__ARRAY__444 (- 1))) (* x__ARRAY__445 (- 1))) (* x__ARRAY__446 (- 1))) (* x__ARRAY__447 (- 1))) (* x__ARRAY__448 (- 1))) (* x__ARRAY__449 (- 1))) (* x__ARRAY__450 (- 1))) (- 2)))
(assert (<= (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (* x__ARRAY__451 (- 1)) (* x__ARRAY__452 (- 1))) (* x__ARRAY__453 (- 1))) (* x__ARRAY__454 (- 1))) (* x__ARRAY__455 (- 1))) (* x__ARRAY__456 (- 1))) (* x__ARRAY__457 (- 1))) (* x__ARRAY__458 (- 1))) (* x__ARRAY__459 (- 1))) (* x__ARRAY__460 (- 1))) (* x__ARRAY__461 (- 1))) (* x__ARRAY__462 (- 1))) (* x__ARRAY__463 (- 1))) (* x__ARRAY__464 (- 1))) (* x__ARRAY__465 (- 1))) (* x__ARRAY__466 (- 1))) (* x__ARRAY__467 (- 1))) (* x__ARRAY__468 (- 1))) (* x__ARRAY__469 (- 1))) (* x__ARRAY__470 (- 1))) (* x__ARRAY__471 (- 1))) (* x__ARRAY__472 (- 1))) (* x__ARRAY__473 (- 1))) (* x__ARRAY__474 (- 1))) (* x__ARRAY__475 (- 1))) (* x__ARRAY__476 (- 1))) (* x__ARRAY__477 (- 1))) (* x__ARRAY__478 (- 1))) (* x__ARRAY__479 (- 1))) (* x__ARRAY__480 (- 1))) (* x__ARRAY__481 (- 1))) (* x__ARRAY__482 (- 1))) (* x__ARRAY__483 (- 1))) (* x__ARRAY__484 (- 1))) (* x__ARRAY__485 (- 1))) (* x__ARRAY__486 (- 1))) (* x__ARRAY__487 (- 1))) (* x__ARRAY__488 (- 1))) (* x__ARRAY__489 (- 1))) (* x__ARRAY__490 (- 1))) (* x__ARRAY__491 (- 1))) (* x__ARRAY__492 (- 1))) (* x__ARRAY__493 (- 1))) (* x__ARRAY__494 (- 1))) (* x__ARRAY__495 (- 1))) (* x__ARRAY__496 (- 1))) (* x__ARRAY__497 (- 1))) (* x__ARRAY__498 (- 1))) (* x__ARRAY__499 (- 1))) (* x__ARRAY__500 (- 1))) (- 2)))
(assert (<= (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (* x__ARRAY__1 1) (* x__ARRAY__2 1)) (* x__ARRAY__3 1)) (* x__ARRAY__4 1)) (* x__ARRAY__5 1)) (* x__ARRAY__6 1)) (* x__ARRAY__7 1)) (* x__ARRAY__8 1)) (* x__ARRAY__9 1)) (* x__ARRAY__10 1)) (* x__ARRAY__11 1)) (* x__ARRAY__12 1)) (* x__ARRAY__13 1)) (* x__ARRAY__14 1)) (* x__ARRAY__15 1)) (* x__ARRAY__16 1)) (* x__ARRAY__17 1)) (* x__ARRAY__18 1)) (* x__ARRAY__19 1)) (* x__ARRAY__20 1)) (* x__ARRAY__21 1)) (* x__ARRAY__22 1)) (* x__ARRAY__23 1)) (* x__ARRAY__24 1)) (* x__ARRAY__25 1)) (* x__ARRAY__26 1)) (* x__ARRAY__27 1)) (* x__ARRAY__28 1)) (* x__ARRAY__29 1)) (* x__ARRAY__30 1)) (* x__ARRAY__31 1)) (* x__ARRAY__32 1)) (* x__ARRAY__33 1)) (* x__ARRAY__34 1)) (* x__ARRAY__35 1)) (* x__ARRAY__36 1)) (* x__ARRAY__37 1)) (* x__ARRAY__38 1)) (* x__ARRAY__39 1)) (* x__ARRAY__40 1)) (* x__ARRAY__41 1)) (* x__ARRAY__42 1)) (* x__ARRAY__43 1)) (* x__ARRAY__44 1)) (* x__ARRAY__45 1)) (* x__ARRAY__46 1)) (* x__ARRAY__47 1)) (* x__ARRAY__48 1)) (* x__ARRAY__49 1)) (* x__ARRAY__50 1)) 10))
(assert (<= (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (* x__ARRAY__51 1) (* x__ARRAY__52 1)) (* x__ARRAY__53 1)) (* x__ARRAY__54 1)) (* x__ARRAY__55 1)) (* x__ARRAY__56 1)) (* x__ARRAY__57 1)) (* x__ARRAY__58 1)) (* x__ARRAY__59 1)) (* x__ARRAY__60 1)) (* x__ARRAY__61 1)) (* x__ARRAY__62 1)) (* x__ARRAY__63 1)) (* x__ARRAY__64 1)) (* x__ARRAY__65 1)) (* x__ARRAY__66 1)) (* x__ARRAY__67 1)) (* x__ARRAY__68 1)) (* x__ARRAY__69 1)) (* x__ARRAY__70 1)) (* x__ARRAY__71 1)) (* x__ARRAY__72 1)) (* x__ARRAY__73 1)) (* x__ARRAY__74 1)) (* x__ARRAY__75 1)) (* x__ARRAY__76 1)) (* x__ARRAY__77 1)) (* x__ARRAY__78 1)) (* x__ARRAY__79 1)) (* x__ARRAY__80 1)) (* x__ARRAY__81 1)) (* x__ARRAY__82 1)) (* x__ARRAY__83 1)) (* x__ARRAY__84 1)) (* x__ARRAY__85 1)) (* x__ARRAY__86 1)) (* x__ARRAY__87 1)) (* x__ARRAY__88 1)) (* x__ARRAY__89 1)) (* x__ARRAY__90 1)) (* x__ARRAY__91 1)) (* x__ARRAY__92 1)) (* x__ARRAY__93 1)) (* x__ARRAY__94 1)) (* x__ARRAY__95 1)) (* x__ARRAY__96 1)) (* x__ARRAY__97 1)) (* x__ARRAY__98 1)) (* x__ARRAY__99 1)) (* x__ARRAY__100 1)) 10))
(assert (<= (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (* x__ARRAY__101 1) (* x__ARRAY__102 1)) (* x__ARRAY__103 1)) (* x__ARRAY__104 1)) (* x__ARRAY__105 1)) (* x__ARRAY__106 1)) (* x__ARRAY__107 1)) (* x__ARRAY__108 1)) (* x__ARRAY__109 1)) (* x__ARRAY__110 1)) (* x__ARRAY__111 1)) (* x__ARRAY__112 1)) (* x__ARRAY__113 1)) (* x__ARRAY__114 1)) (* x__ARRAY__115 1)) (* x__ARRAY__116 1)) (* x__ARRAY__117 1)) (* x__ARRAY__118 1)) (* x__ARRAY__119 1)) (* x__ARRAY__120 1)) (* x__ARRAY__121 1)) (* x__ARRAY__122 1)) (* x__ARRAY__123 1)) (* x__ARRAY__124 1)) (* x__ARRAY__125 1)) (* x__ARRAY__126 1)) (* x__ARRAY__127 1)) (* x__ARRAY__128 1)) (* x__ARRAY__129 1)) (* x__ARRAY__130 1)) (* x__ARRAY__131 1)) (* x__ARRAY__132 1)) (* x__ARRAY__133 1)) (* x__ARRAY__134 1)) (* x__ARRAY__135 1)) (* x__ARRAY__136 1)) (* x__ARRAY__137 1)) (* x__ARRAY__138 1)) (* x__ARRAY__139 1)) (* x__ARRAY__140 1)) (* x__ARRAY__141 1)) (* x__ARRAY__142 1)) (* x__ARRAY__143 1)) (* x__ARRAY__144 1)) (* x__ARRAY__145 1)) (* x__ARRAY__146 1)) (* x__ARRAY__147 1)) (* x__ARRAY__148 1)) (* x__ARRAY__149 1)) (* x__ARRAY__150 1)) 10))
(assert (<= (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (* x__ARRAY__151 1) (* x__ARRAY__152 1)) (* x__ARRAY__153 1)) (* x__ARRAY__154 1)) (* x__ARRAY__155 1)) (* x__ARRAY__156 1)) (* x__ARRAY__157 1)) (* x__ARRAY__158 1)) (* x__ARRAY__159 1)) (* x__ARRAY__160 1)) (* x__ARRAY__161 1)) (* x__ARRAY__162 1)) (* x__ARRAY__163 1)) (* x__ARRAY__164 1)) (* x__ARRAY__165 1)) (* x__ARRAY__166 1)) (* x__ARRAY__167 1)) (* x__ARRAY__168 1)) (* x__ARRAY__169 1)) (* x__ARRAY__170 1)) (* x__ARRAY__171 1)) (* x__ARRAY__172 1)) (* x__ARRAY__173 1)) (* x__ARRAY__174 1)) (* x__ARRAY__175 1)) (* x__ARRAY__176 1)) (* x__ARRAY__177 1)) (* x__ARRAY__178 1)) (* x__ARRAY__179 1)) (* x__ARRAY__180 1)) (* x__ARRAY__181 1)) (* x__ARRAY__182 1)) (* x__ARRAY__183 1)) (* x__ARRAY__184 1)) (* x__ARRAY__185 1)) (* x__ARRAY__186 1)) (* x__ARRAY__187 1)) (* x__ARRAY__188 1)) (* x__ARRAY__189 1)) (* x__ARRAY__190 1)) (* x__ARRAY__191 1)) (* x__ARRAY__192 1)) (* x__ARRAY__193 1)) (* x__ARRAY__194 1)) (* x__ARRAY__195 1)) (* x__ARRAY__196 1)) (* x__ARRAY__197 1)) (* x__ARRAY__198 1)) (* x__ARRAY__199 1)) (* x__ARRAY__200 1)) 10))
(assert (<= (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (* x__ARRAY__201 1) (* x__ARRAY__202 1)) (* x__ARRAY__203 1)) (* x__ARRAY__204 1)) (* x__ARRAY__205 1)) (* x__ARRAY__206 1)) (* x__ARRAY__207 1)) (* x__ARRAY__208 1)) (* x__ARRAY__209 1)) (* x__ARRAY__210 1)) (* x__ARRAY__211 1)) (* x__ARRAY__212 1)) (* x__ARRAY__213 1)) (* x__ARRAY__214 1)) (* x__ARRAY__215 1)) (* x__ARRAY__216 1)) (* x__ARRAY__217 1)) (* x__ARRAY__218 1)) (* x__ARRAY__219 1)) (* x__ARRAY__220 1)) (* x__ARRAY__221 1)) (* x__ARRAY__222 1)) (* x__ARRAY__223 1)) (* x__ARRAY__224 1)) (* x__ARRAY__225 1)) (* x__ARRAY__226 1)) (* x__ARRAY__227 1)) (* x__ARRAY__228 1)) (* x__ARRAY__229 1)) (* x__ARRAY__230 1)) (* x__ARRAY__231 1)) (* x__ARRAY__232 1)) (* x__ARRAY__233 1)) (* x__ARRAY__234 1)) (* x__ARRAY__235 1)) (* x__ARRAY__236 1)) (* x__ARRAY__237 1)) (* x__ARRAY__238 1)) (* x__ARRAY__239 1)) (* x__ARRAY__240 1)) (* x__ARRAY__241 1)) (* x__ARRAY__242 1)) (* x__ARRAY__243 1)) (* x__ARRAY__244 1)) (* x__ARRAY__245 1)) (* x__ARRAY__246 1)) (* x__ARRAY__247 1)) (* x__ARRAY__248 1)) (* x__ARRAY__249 1)) (* x__ARRAY__250 1)) 10))
(assert (<= (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (* x__ARRAY__251 1) (* x__ARRAY__252 1)) (* x__ARRAY__253 1)) (* x__ARRAY__254 1)) (* x__ARRAY__255 1)) (* x__ARRAY__256 1)) (* x__ARRAY__257 1)) (* x__ARRAY__258 1)) (* x__ARRAY__259 1)) (* x__ARRAY__260 1)) (* x__ARRAY__261 1)) (* x__ARRAY__262 1)) (* x__ARRAY__263 1)) (* x__ARRAY__264 1)) (* x__ARRAY__265 1)) (* x__ARRAY__266 1)) (* x__ARRAY__267 1)) (* x__ARRAY__268 1)) (* x__ARRAY__269 1)) (* x__ARRAY__270 1)) (* x__ARRAY__271 1)) (* x__ARRAY__272 1)) (* x__ARRAY__273 1)) (* x__ARRAY__274 1)) (* x__ARRAY__275 1)) (* x__ARRAY__276 1)) (* x__ARRAY__277 1)) (* x__ARRAY__278 1)) (* x__ARRAY__279 1)) (* x__ARRAY__280 1)) (* x__ARRAY__281 1)) (* x__ARRAY__282 1)) (* x__ARRAY__283 1)) (* x__ARRAY__284 1)) (* x__ARRAY__285 1)) (* x__ARRAY__286 1)) (* x__ARRAY__287 1)) (* x__ARRAY__288 1)) (* x__ARRAY__289 1)) (* x__ARRAY__290 1)) (* x__ARRAY__291 1)) (* x__ARRAY__292 1)) (* x__ARRAY__293 1)) (* x__ARRAY__294 1)) (* x__ARRAY__295 1)) (* x__ARRAY__296 1)) (* x__ARRAY__297 1)) (* x__ARRAY__298 1)) (* x__ARRAY__299 1)) (* x__ARRAY__300 1)) 10))
(assert (<= (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (* x__ARRAY__301 1) (* x__ARRAY__302 1)) (* x__ARRAY__303 1)) (* x__ARRAY__304 1)) (* x__ARRAY__305 1)) (* x__ARRAY__306 1)) (* x__ARRAY__307 1)) (* x__ARRAY__308 1)) (* x__ARRAY__309 1)) (* x__ARRAY__310 1)) (* x__ARRAY__311 1)) (* x__ARRAY__312 1)) (* x__ARRAY__313 1)) (* x__ARRAY__314 1)) (* x__ARRAY__315 1)) (* x__ARRAY__316 1)) (* x__ARRAY__317 1)) (* x__ARRAY__318 1)) (* x__ARRAY__319 1)) (* x__ARRAY__320 1)) (* x__ARRAY__321 1)) (* x__ARRAY__322 1)) (* x__ARRAY__323 1)) (* x__ARRAY__324 1)) (* x__ARRAY__325 1)) (* x__ARRAY__326 1)) (* x__ARRAY__327 1)) (* x__ARRAY__328 1)) (* x__ARRAY__329 1)) (* x__ARRAY__330 1)) (* x__ARRAY__331 1)) (* x__ARRAY__332 1)) (* x__ARRAY__333 1)) (* x__ARRAY__334 1)) (* x__ARRAY__335 1)) (* x__ARRAY__336 1)) (* x__ARRAY__337 1)) (* x__ARRAY__338 1)) (* x__ARRAY__339 1)) (* x__ARRAY__340 1)) (* x__ARRAY__341 1)) (* x__ARRAY__342 1)) (* x__ARRAY__343 1)) (* x__ARRAY__344 1)) (* x__ARRAY__345 1)) (* x__ARRAY__346 1)) (* x__ARRAY__347 1)) (* x__ARRAY__348 1)) (* x__ARRAY__349 1)) (* x__ARRAY__350 1)) 10))
(assert (<= (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (* x__ARRAY__351 1) (* x__ARRAY__352 1)) (* x__ARRAY__353 1)) (* x__ARRAY__354 1)) (* x__ARRAY__355 1)) (* x__ARRAY__356 1)) (* x__ARRAY__357 1)) (* x__ARRAY__358 1)) (* x__ARRAY__359 1)) (* x__ARRAY__360 1)) (* x__ARRAY__361 1)) (* x__ARRAY__362 1)) (* x__ARRAY__363 1)) (* x__ARRAY__364 1)) (* x__ARRAY__365 1)) (* x__ARRAY__366 1)) (* x__ARRAY__367 1)) (* x__ARRAY__368 1)) (* x__ARRAY__369 1)) (* x__ARRAY__370 1)) (* x__ARRAY__371 1)) (* x__ARRAY__372 1)) (* x__ARRAY__373 1)) (* x__ARRAY__374 1)) (* x__ARRAY__375 1)) (* x__ARRAY__376 1)) (* x__ARRAY__377 1)) (* x__ARRAY__378 1)) (* x__ARRAY__379 1)) (* x__ARRAY__380 1)) (* x__ARRAY__381 1)) (* x__ARRAY__382 1)) (* x__ARRAY__383 1)) (* x__ARRAY__384 1)) (* x__ARRAY__385 1)) (* x__ARRAY__386 1)) (* x__ARRAY__387 1)) (* x__ARRAY__388 1)) (* x__ARRAY__389 1)) (* x__ARRAY__390 1)) (* x__ARRAY__391 1)) (* x__ARRAY__392 1)) (* x__ARRAY__393 1)) (* x__ARRAY__394 1)) (* x__ARRAY__395 1)) (* x__ARRAY__396 1)) (* x__ARRAY__397 1)) (* x__ARRAY__398 1)) (* x__ARRAY__399 1)) (* x__ARRAY__400 1)) 10))
(assert (<= (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (* x__ARRAY__401 1) (* x__ARRAY__402 1)) (* x__ARRAY__403 1)) (* x__ARRAY__404 1)) (* x__ARRAY__405 1)) (* x__ARRAY__406 1)) (* x__ARRAY__407 1)) (* x__ARRAY__408 1)) (* x__ARRAY__409 1)) (* x__ARRAY__410 1)) (* x__ARRAY__411 1)) (* x__ARRAY__412 1)) (* x__ARRAY__413 1)) (* x__ARRAY__414 1)) (* x__ARRAY__415 1)) (* x__ARRAY__416 1)) (* x__ARRAY__417 1)) (* x__ARRAY__418 1)) (* x__ARRAY__419 1)) (* x__ARRAY__420 1)) (* x__ARRAY__421 1)) (* x__ARRAY__422 1)) (* x__ARRAY__423 1)) (* x__ARRAY__424 1)) (* x__ARRAY__425 1)) (* x__ARRAY__426 1)) (* x__ARRAY__427 1)) (* x__ARRAY__428 1)) (* x__ARRAY__429 1)) (* x__ARRAY__430 1)) (* x__ARRAY__431 1)) (* x__ARRAY__432 1)) (* x__ARRAY__433 1)) (* x__ARRAY__434 1)) (* x__ARRAY__435 1)) (* x__ARRAY__436 1)) (* x__ARRAY__437 1)) (* x__ARRAY__438 1)) (* x__ARRAY__439 1)) (* x__ARRAY__440 1)) (* x__ARRAY__441 1)) (* x__ARRAY__442 1)) (* x__ARRAY__443 1)) (* x__ARRAY__444 1)) (* x__ARRAY__445 1)) (* x__ARRAY__446 1)) (* x__ARRAY__447 1)) (* x__ARRAY__448 1)) (* x__ARRAY__449 1)) (* x__ARRAY__450 1)) 10))
(assert (<= (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (* x__ARRAY__451 1) (* x__ARRAY__452 1)) (* x__ARRAY__453 1)) (* x__ARRAY__454 1)) (* x__ARRAY__455 1)) (* x__ARRAY__456 1)) (* x__ARRAY__457 1)) (* x__ARRAY__458 1)) (* x__ARRAY__459 1)) (* x__ARRAY__460 1)) (* x__ARRAY__461 1)) (* x__ARRAY__462 1)) (* x__ARRAY__463 1)) (* x__ARRAY__464 1)) (* x__ARRAY__465 1)) (* x__ARRAY__466 1)) (* x__ARRAY__467 1)) (* x__ARRAY__468 1)) (* x__ARRAY__469 1)) (* x__ARRAY__470 1)) (* x__ARRAY__471 1)) (* x__ARRAY__472 1)) (* x__ARRAY__473 1)) (* x__ARRAY__474 1)) (* x__ARRAY__475 1)) (* x__ARRAY__476 1)) (* x__ARRAY__477 1)) (* x__ARRAY__478 1)) (* x__ARRAY__479 1)) (* x__ARRAY__480 1)) (* x__ARRAY__481 1)) (* x__ARRAY__482 1)) (* x__ARRAY__483 1)) (* x__ARRAY__484 1)) (* x__ARRAY__485 1)) (* x__ARRAY__486 1)) (* x__ARRAY__487 1)) (* x__ARRAY__488 1)) (* x__ARRAY__489 1)) (* x__ARRAY__490 1)) (* x__ARRAY__491 1)) (* x__ARRAY__492 1)) (* x__ARRAY__493 1)) (* x__ARRAY__494 1)) (* x__ARRAY__495 1)) (* x__ARRAY__496 1)) (* x__ARRAY__497 1)) (* x__ARRAY__498 1)) (* x__ARRAY__499 1)) (* x__ARRAY__500 1)) 10))
(assert (<= (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (* Real____00002 9) (* Real____00004 1)) (* Real____00006 2)) (* Real____00008 10)) (* Real____00010 3)) (* Real____00012 4)) (* Real____00014 1)) (* Real____00016 10)) (* Real____00018 4)) (* Real____00020 1)) (* Real____00022 1)) (* Real____00024 5)) (* Real____00026 10)) (* Real____00028 9)) (* Real____00030 1)) (* Real____00032 8)) (* Real____00034 5)) (* Real____00036 2)) (* Real____00038 8)) (* Real____00040 6)) (* Real____00042 9)) (* Real____00044 5)) (* Real____00046 10)) (* Real____00048 7)) (* Real____00050 8)) (* Real____00052 3)) (* Real____00054 7)) (* Real____00056 4)) (* Real____00058 10)) (* Real____00060 3)) (* Real____00062 1)) (* Real____00064 10)) (* Real____00066 6)) (* Real____00068 2)) (* Real____00070 9)) (* Real____00072 8)) (* Real____00074 7)) (* Real____00076 2)) (* Real____00078 7)) (* Real____00080 3)) (* Real____00082 4)) (* Real____00084 8)) (* Real____00086 9)) (* Real____00088 3)) (* Real____00090 8)) (* Real____00092 1)) (* Real____00094 2)) (* Real____00096 3)) (* Real____00098 2)) (* Real____00100 9)) (* objective (- 9))) 0))
(assert (<= (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (* Real____00102 9) (* Real____00104 1)) (* Real____00106 2)) (* Real____00108 10)) (* Real____00110 3)) (* Real____00112 4)) (* Real____00114 1)) (* Real____00116 10)) (* Real____00118 4)) (* Real____00120 1)) (* Real____00122 1)) (* Real____00124 5)) (* Real____00126 10)) (* Real____00128 9)) (* Real____00130 1)) (* Real____00132 8)) (* Real____00134 5)) (* Real____00136 2)) (* Real____00138 8)) (* Real____00140 6)) (* Real____00142 9)) (* Real____00144 5)) (* Real____00146 10)) (* Real____00148 7)) (* Real____00150 8)) (* Real____00152 3)) (* Real____00154 7)) (* Real____00156 4)) (* Real____00158 10)) (* Real____00160 3)) (* Real____00162 1)) (* Real____00164 10)) (* Real____00166 6)) (* Real____00168 2)) (* Real____00170 9)) (* Real____00172 8)) (* Real____00174 7)) (* Real____00176 2)) (* Real____00178 7)) (* Real____00180 3)) (* Real____00182 4)) (* Real____00184 8)) (* Real____00186 9)) (* Real____00188 3)) (* Real____00190 8)) (* Real____00192 1)) (* Real____00194 2)) (* Real____00196 3)) (* Real____00198 2)) (* Real____00200 9)) (* objective (- 8))) 0))
(assert (<= (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (* Real____00202 9) (* Real____00204 1)) (* Real____00206 2)) (* Real____00208 10)) (* Real____00210 3)) (* Real____00212 4)) (* Real____00214 1)) (* Real____00216 10)) (* Real____00218 4)) (* Real____00220 1)) (* Real____00222 1)) (* Real____00224 5)) (* Real____00226 10)) (* Real____00228 9)) (* Real____00230 1)) (* Real____00232 8)) (* Real____00234 5)) (* Real____00236 2)) (* Real____00238 8)) (* Real____00240 6)) (* Real____00242 9)) (* Real____00244 5)) (* Real____00246 10)) (* Real____00248 7)) (* Real____00250 8)) (* Real____00252 3)) (* Real____00254 7)) (* Real____00256 4)) (* Real____00258 10)) (* Real____00260 3)) (* Real____00262 1)) (* Real____00264 10)) (* Real____00266 6)) (* Real____00268 2)) (* Real____00270 9)) (* Real____00272 8)) (* Real____00274 7)) (* Real____00276 2)) (* Real____00278 7)) (* Real____00280 3)) (* Real____00282 4)) (* Real____00284 8)) (* Real____00286 9)) (* Real____00288 3)) (* Real____00290 8)) (* Real____00292 1)) (* Real____00294 2)) (* Real____00296 3)) (* Real____00298 2)) (* Real____00300 9)) (* objective (- 7))) 0))
(assert (<= (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (* Real____00302 9) (* Real____00304 1)) (* Real____00306 2)) (* Real____00308 10)) (* Real____00310 3)) (* Real____00312 4)) (* Real____00314 1)) (* Real____00316 10)) (* Real____00318 4)) (* Real____00320 1)) (* Real____00322 1)) (* Real____00324 5)) (* Real____00326 10)) (* Real____00328 9)) (* Real____00330 1)) (* Real____00332 8)) (* Real____00334 5)) (* Real____00336 2)) (* Real____00338 8)) (* Real____00340 6)) (* Real____00342 9)) (* Real____00344 5)) (* Real____00346 10)) (* Real____00348 7)) (* Real____00350 8)) (* Real____00352 3)) (* Real____00354 7)) (* Real____00356 4)) (* Real____00358 10)) (* Real____00360 3)) (* Real____00362 1)) (* Real____00364 10)) (* Real____00366 6)) (* Real____00368 2)) (* Real____00370 9)) (* Real____00372 8)) (* Real____00374 7)) (* Real____00376 2)) (* Real____00378 7)) (* Real____00380 3)) (* Real____00382 4)) (* Real____00384 8)) (* Real____00386 9)) (* Real____00388 3)) (* Real____00390 8)) (* Real____00392 1)) (* Real____00394 2)) (* Real____00396 3)) (* Real____00398 2)) (* Real____00400 9)) (* objective (- 6))) 0))
(assert (<= (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (* Real____00402 9) (* Real____00404 1)) (* Real____00406 2)) (* Real____00408 10)) (* Real____00410 3)) (* Real____00412 4)) (* Real____00414 1)) (* Real____00416 10)) (* Real____00418 4)) (* Real____00420 1)) (* Real____00422 1)) (* Real____00424 5)) (* Real____00426 10)) (* Real____00428 9)) (* Real____00430 1)) (* Real____00432 8)) (* Real____00434 5)) (* Real____00436 2)) (* Real____00438 8)) (* Real____00440 6)) (* Real____00442 9)) (* Real____00444 5)) (* Real____00446 10)) (* Real____00448 7)) (* Real____00450 8)) (* Real____00452 3)) (* Real____00454 7)) (* Real____00456 4)) (* Real____00458 10)) (* Real____00460 3)) (* Real____00462 1)) (* Real____00464 10)) (* Real____00466 6)) (* Real____00468 2)) (* Real____00470 9)) (* Real____00472 8)) (* Real____00474 7)) (* Real____00476 2)) (* Real____00478 7)) (* Real____00480 3)) (* Real____00482 4)) (* Real____00484 8)) (* Real____00486 9)) (* Real____00488 3)) (* Real____00490 8)) (* Real____00492 1)) (* Real____00494 2)) (* Real____00496 3)) (* Real____00498 2)) (* Real____00500 9)) (* objective (- 5))) 0))
(assert (<= (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (* Real____00502 9) (* Real____00504 1)) (* Real____00506 2)) (* Real____00508 10)) (* Real____00510 3)) (* Real____00512 4)) (* Real____00514 1)) (* Real____00516 10)) (* Real____00518 4)) (* Real____00520 1)) (* Real____00522 1)) (* Real____00524 5)) (* Real____00526 10)) (* Real____00528 9)) (* Real____00530 1)) (* Real____00532 8)) (* Real____00534 5)) (* Real____00536 2)) (* Real____00538 8)) (* Real____00540 6)) (* Real____00542 9)) (* Real____00544 5)) (* Real____00546 10)) (* Real____00548 7)) (* Real____00550 8)) (* Real____00552 3)) (* Real____00554 7)) (* Real____00556 4)) (* Real____00558 10)) (* Real____00560 3)) (* Real____00562 1)) (* Real____00564 10)) (* Real____00566 6)) (* Real____00568 2)) (* Real____00570 9)) (* Real____00572 8)) (* Real____00574 7)) (* Real____00576 2)) (* Real____00578 7)) (* Real____00580 3)) (* Real____00582 4)) (* Real____00584 8)) (* Real____00586 9)) (* Real____00588 3)) (* Real____00590 8)) (* Real____00592 1)) (* Real____00594 2)) (* Real____00596 3)) (* Real____00598 2)) (* Real____00600 9)) (* objective (- 4))) 0))
(assert (<= (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (* Real____00602 9) (* Real____00604 1)) (* Real____00606 2)) (* Real____00608 10)) (* Real____00610 3)) (* Real____00612 4)) (* Real____00614 1)) (* Real____00616 10)) (* Real____00618 4)) (* Real____00620 1)) (* Real____00622 1)) (* Real____00624 5)) (* Real____00626 10)) (* Real____00628 9)) (* Real____00630 1)) (* Real____00632 8)) (* Real____00634 5)) (* Real____00636 2)) (* Real____00638 8)) (* Real____00640 6)) (* Real____00642 9)) (* Real____00644 5)) (* Real____00646 10)) (* Real____00648 7)) (* Real____00650 8)) (* Real____00652 3)) (* Real____00654 7)) (* Real____00656 4)) (* Real____00658 10)) (* Real____00660 3)) (* Real____00662 1)) (* Real____00664 10)) (* Real____00666 6)) (* Real____00668 2)) (* Real____00670 9)) (* Real____00672 8)) (* Real____00674 7)) (* Real____00676 2)) (* Real____00678 7)) (* Real____00680 3)) (* Real____00682 4)) (* Real____00684 8)) (* Real____00686 9)) (* Real____00688 3)) (* Real____00690 8)) (* Real____00692 1)) (* Real____00694 2)) (* Real____00696 3)) (* Real____00698 2)) (* Real____00700 9)) (* objective (- 3))) 0))
(assert (<= (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (* Real____00702 9) (* Real____00704 1)) (* Real____00706 2)) (* Real____00708 10)) (* Real____00710 3)) (* Real____00712 4)) (* Real____00714 1)) (* Real____00716 10)) (* Real____00718 4)) (* Real____00720 1)) (* Real____00722 1)) (* Real____00724 5)) (* Real____00726 10)) (* Real____00728 9)) (* Real____00730 1)) (* Real____00732 8)) (* Real____00734 5)) (* Real____00736 2)) (* Real____00738 8)) (* Real____00740 6)) (* Real____00742 9)) (* Real____00744 5)) (* Real____00746 10)) (* Real____00748 7)) (* Real____00750 8)) (* Real____00752 3)) (* Real____00754 7)) (* Real____00756 4)) (* Real____00758 10)) (* Real____00760 3)) (* Real____00762 1)) (* Real____00764 10)) (* Real____00766 6)) (* Real____00768 2)) (* Real____00770 9)) (* Real____00772 8)) (* Real____00774 7)) (* Real____00776 2)) (* Real____00778 7)) (* Real____00780 3)) (* Real____00782 4)) (* Real____00784 8)) (* Real____00786 9)) (* Real____00788 3)) (* Real____00790 8)) (* Real____00792 1)) (* Real____00794 2)) (* Real____00796 3)) (* Real____00798 2)) (* Real____00800 9)) (* objective (- 2))) 0))
(assert (<= (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (+ (* Real____00802 9) (* Real____00804 1)) (* Real____00806 2)) (* Real____00808 10)) (* Real____00810 3)) (* Real____00812 4)) (* Real____00814 1)) (* Real____00816 10)) (* Real____00818 4)) (* Real____00820 1)) (* Real____00822 1)) (* Real____00824 5)) (* Real____00826 10)) (* Real____00828 9)) (* Real____00830 1)) (* Real____00832 8)) (* Real____00834 5)) (* Real____00836 2)) (* Real____00838 8)) (* Real____00840 6)) (* Real____00842 9)) (* Real____00844 5)) (* Real____00846 10)) (* Real____00848 7)) (* Real____00850 8)) (* Real____00852 3)) (* Real____00854 7)) (* Real____00856 4)) (* Real____00858 10)) (* Real____00860 3)) (* Real____00862 1)) (* Real____00864 10)) (* Real____00866 6)) (* Real____00868 2)) (* Real____00870 9)) (* Real____00872 8)) (* Real____00874 7)) (* Real____00876 2)) (* Real____00878 7)) (* Real____00880 3)) (* Real____00882 4)) (* Real____00884 8)) (* Real____00886 9)) (* Real____00888 3)) (* Real____00890 8)) (* Real____00892 1)) (* Real____00894 2)) (* Real____00896 3)) (* Real____00898 2)) (* Real____00900 9)) (* objective (- 1))) 0))
(assert (< course_period__ARRAY__1 course_period__ARRAY__8))
(assert (< course_period__ARRAY__1 course_period__ARRAY__9))
(assert (< course_period__ARRAY__2 course_period__ARRAY__7))
(assert (< course_period__ARRAY__3 course_period__ARRAY__6))
(assert (< course_period__ARRAY__3 course_period__ARRAY__9))
(assert (< course_period__ARRAY__4 course_period__ARRAY__8))
(assert (< course_period__ARRAY__6 course_period__ARRAY__10))
(assert (< course_period__ARRAY__8 course_period__ARRAY__10))
(assert (< course_period__ARRAY__9 course_period__ARRAY__10))
(assert (< course_period__ARRAY__9 course_period__ARRAY__11))
(assert (< course_period__ARRAY__9 course_period__ARRAY__12))
(assert (< course_period__ARRAY__10 course_period__ARRAY__14))
(assert (< course_period__ARRAY__10 course_period__ARRAY__15))
(assert (< course_period__ARRAY__11 course_period__ARRAY__14))
(assert (< course_period__ARRAY__12 course_period__ARRAY__15))
(assert (< course_period__ARRAY__14 course_period__ARRAY__20))
(assert (< course_period__ARRAY__14 course_period__ARRAY__21))
(assert (< course_period__ARRAY__14 course_period__ARRAY__24))
(assert (< course_period__ARRAY__15 course_period__ARRAY__19))
(assert (< course_period__ARRAY__15 course_period__ARRAY__20))
(assert (< course_period__ARRAY__15 course_period__ARRAY__23))
(assert (< course_period__ARRAY__16 course_period__ARRAY__20))
(assert (< course_period__ARRAY__16 course_period__ARRAY__22))
(assert (< course_period__ARRAY__16 course_period__ARRAY__25))
(assert (< course_period__ARRAY__17 course_period__ARRAY__20))
(assert (< course_period__ARRAY__17 course_period__ARRAY__23))
(assert (< course_period__ARRAY__17 course_period__ARRAY__24))
(assert (< course_period__ARRAY__18 course_period__ARRAY__27))
(assert (< course_period__ARRAY__19 course_period__ARRAY__27))
(assert (< course_period__ARRAY__21 course_period__ARRAY__26))
(assert (< course_period__ARRAY__21 course_period__ARRAY__27))
(assert (< course_period__ARRAY__22 course_period__ARRAY__27))
(assert (< course_period__ARRAY__25 course_period__ARRAY__26))
(assert (< course_period__ARRAY__26 course_period__ARRAY__29))
(assert (< course_period__ARRAY__27 course_period__ARRAY__34))
(assert (< course_period__ARRAY__31 course_period__ARRAY__36))
(assert (< course_period__ARRAY__31 course_period__ARRAY__37))
(assert (< course_period__ARRAY__32 course_period__ARRAY__36))
(assert (< course_period__ARRAY__33 course_period__ARRAY__35))
(assert (< course_period__ARRAY__33 course_period__ARRAY__36))
(assert (< course_period__ARRAY__34 course_period__ARRAY__35))
(assert (< course_period__ARRAY__34 course_period__ARRAY__38))
(assert (< course_period__ARRAY__34 course_period__ARRAY__39))
(assert (< course_period__ARRAY__35 course_period__ARRAY__40))
(assert (< course_period__ARRAY__35 course_period__ARRAY__42))
(assert (< course_period__ARRAY__36 course_period__ARRAY__41))
(assert (< course_period__ARRAY__36 course_period__ARRAY__43))
(assert (< course_period__ARRAY__38 course_period__ARRAY__40))
(assert (< course_period__ARRAY__39 course_period__ARRAY__40))
(assert (< course_period__ARRAY__39 course_period__ARRAY__41))
(assert (< course_period__ARRAY__39 course_period__ARRAY__43))
(assert (< course_period__ARRAY__41 course_period__ARRAY__47))
(assert (< course_period__ARRAY__41 course_period__ARRAY__48))
(assert (< course_period__ARRAY__41 course_period__ARRAY__49))
(assert (< course_period__ARRAY__41 course_period__ARRAY__50))
(assert (< course_period__ARRAY__42 course_period__ARRAY__45))
(assert (< course_period__ARRAY__43 course_period__ARRAY__45))
(assert (< course_period__ARRAY__43 course_period__ARRAY__46))
(assert (< course_period__ARRAY__43 course_period__ARRAY__47))
(assert (< course_period__ARRAY__43 course_period__ARRAY__48))
(assert (< course_period__ARRAY__43 course_period__ARRAY__49))
(assert (< course_period__ARRAY__44 course_period__ARRAY__45))
(assert (< course_period__ARRAY__44 course_period__ARRAY__49))
(assert (< course_period__ARRAY__44 course_period__ARRAY__50))
(assert (and (=> (< 1 course_period__ARRAY__1) BOOL____00001) (=> BOOL____00001 (< 1 course_period__ARRAY__1))))
(assert (and (=> (< 1 course_period__ARRAY__2) BOOL____00003) (=> BOOL____00003 (< 1 course_period__ARRAY__2))))
(assert (and (=> (< 1 course_period__ARRAY__3) BOOL____00005) (=> BOOL____00005 (< 1 course_period__ARRAY__3))))
(assert (and (=> (< 1 course_period__ARRAY__4) BOOL____00007) (=> BOOL____00007 (< 1 course_period__ARRAY__4))))
(assert (and (=> (< 1 course_period__ARRAY__5) BOOL____00009) (=> BOOL____00009 (< 1 course_period__ARRAY__5))))
(assert (and (=> (< 1 course_period__ARRAY__6) BOOL____00011) (=> BOOL____00011 (< 1 course_period__ARRAY__6))))
(assert (and (=> (< 1 course_period__ARRAY__7) BOOL____00013) (=> BOOL____00013 (< 1 course_period__ARRAY__7))))
(assert (and (=> (< 1 course_period__ARRAY__8) BOOL____00015) (=> BOOL____00015 (< 1 course_period__ARRAY__8))))
(assert (and (=> (< 1 course_period__ARRAY__9) BOOL____00017) (=> BOOL____00017 (< 1 course_period__ARRAY__9))))
(assert (and (=> (< 1 course_period__ARRAY__10) BOOL____00019) (=> BOOL____00019 (< 1 course_period__ARRAY__10))))
(assert (and (=> (< 1 course_period__ARRAY__11) BOOL____00021) (=> BOOL____00021 (< 1 course_period__ARRAY__11))))
(assert (and (=> (< 1 course_period__ARRAY__12) BOOL____00023) (=> BOOL____00023 (< 1 course_period__ARRAY__12))))
(assert (and (=> (< 1 course_period__ARRAY__13) BOOL____00025) (=> BOOL____00025 (< 1 course_period__ARRAY__13))))
(assert (and (=> (< 1 course_period__ARRAY__14) BOOL____00027) (=> BOOL____00027 (< 1 course_period__ARRAY__14))))
(assert (and (=> (< 1 course_period__ARRAY__15) BOOL____00029) (=> BOOL____00029 (< 1 course_period__ARRAY__15))))
(assert (and (=> (< 1 course_period__ARRAY__16) BOOL____00031) (=> BOOL____00031 (< 1 course_period__ARRAY__16))))
(assert (and (=> (< 1 course_period__ARRAY__17) BOOL____00033) (=> BOOL____00033 (< 1 course_period__ARRAY__17))))
(assert (and (=> (< 1 course_period__ARRAY__18) BOOL____00035) (=> BOOL____00035 (< 1 course_period__ARRAY__18))))
(assert (and (=> (< 1 course_period__ARRAY__19) BOOL____00037) (=> BOOL____00037 (< 1 course_period__ARRAY__19))))
(assert (and (=> (< 1 course_period__ARRAY__20) BOOL____00039) (=> BOOL____00039 (< 1 course_period__ARRAY__20))))
(assert (and (=> (< 1 course_period__ARRAY__21) BOOL____00041) (=> BOOL____00041 (< 1 course_period__ARRAY__21))))
(assert (and (=> (< 1 course_period__ARRAY__22) BOOL____00043) (=> BOOL____00043 (< 1 course_period__ARRAY__22))))
(assert (and (=> (< 1 course_period__ARRAY__23) BOOL____00045) (=> BOOL____00045 (< 1 course_period__ARRAY__23))))
(assert (and (=> (< 1 course_period__ARRAY__24) BOOL____00047) (=> BOOL____00047 (< 1 course_period__ARRAY__24))))
(assert (and (=> (< 1 course_period__ARRAY__25) BOOL____00049) (=> BOOL____00049 (< 1 course_period__ARRAY__25))))
(assert (and (=> (< 1 course_period__ARRAY__26) BOOL____00051) (=> BOOL____00051 (< 1 course_period__ARRAY__26))))
(assert (and (=> (< 1 course_period__ARRAY__27) BOOL____00053) (=> BOOL____00053 (< 1 course_period__ARRAY__27))))
(assert (and (=> (< 1 course_period__ARRAY__28) BOOL____00055) (=> BOOL____00055 (< 1 course_period__ARRAY__28))))
(assert (and (=> (< 1 course_period__ARRAY__29) BOOL____00057) (=> BOOL____00057 (< 1 course_period__ARRAY__29))))
(assert (and (=> (< 1 course_period__ARRAY__30) BOOL____00059) (=> BOOL____00059 (< 1 course_period__ARRAY__30))))
(assert (and (=> (< 1 course_period__ARRAY__31) BOOL____00061) (=> BOOL____00061 (< 1 course_period__ARRAY__31))))
(assert (and (=> (< 1 course_period__ARRAY__32) BOOL____00063) (=> BOOL____00063 (< 1 course_period__ARRAY__32))))
(assert (and (=> (< 1 course_period__ARRAY__33) BOOL____00065) (=> BOOL____00065 (< 1 course_period__ARRAY__33))))
(assert (and (=> (< 1 course_period__ARRAY__34) BOOL____00067) (=> BOOL____00067 (< 1 course_period__ARRAY__34))))
(assert (and (=> (< 1 course_period__ARRAY__35) BOOL____00069) (=> BOOL____00069 (< 1 course_period__ARRAY__35))))
(assert (and (=> (< 1 course_period__ARRAY__36) BOOL____00071) (=> BOOL____00071 (< 1 course_period__ARRAY__36))))
(assert (and (=> (< 1 course_period__ARRAY__37) BOOL____00073) (=> BOOL____00073 (< 1 course_period__ARRAY__37))))
(assert (and (=> (< 1 course_period__ARRAY__38) BOOL____00075) (=> BOOL____00075 (< 1 course_period__ARRAY__38))))
(assert (and (=> (< 1 course_period__ARRAY__39) BOOL____00077) (=> BOOL____00077 (< 1 course_period__ARRAY__39))))
(assert (and (=> (< 1 course_period__ARRAY__40) BOOL____00079) (=> BOOL____00079 (< 1 course_period__ARRAY__40))))
(assert (and (=> (< 1 course_period__ARRAY__41) BOOL____00081) (=> BOOL____00081 (< 1 course_period__ARRAY__41))))
(assert (and (=> (< 1 course_period__ARRAY__42) BOOL____00083) (=> BOOL____00083 (< 1 course_period__ARRAY__42))))
(assert (and (=> (< 1 course_period__ARRAY__43) BOOL____00085) (=> BOOL____00085 (< 1 course_period__ARRAY__43))))
(assert (and (=> (< 1 course_period__ARRAY__44) BOOL____00087) (=> BOOL____00087 (< 1 course_period__ARRAY__44))))
(assert (and (=> (< 1 course_period__ARRAY__45) BOOL____00089) (=> BOOL____00089 (< 1 course_period__ARRAY__45))))
(assert (and (=> (< 1 course_period__ARRAY__46) BOOL____00091) (=> BOOL____00091 (< 1 course_period__ARRAY__46))))
(assert (and (=> (< 1 course_period__ARRAY__47) BOOL____00093) (=> BOOL____00093 (< 1 course_period__ARRAY__47))))
(assert (and (=> (< 1 course_period__ARRAY__48) BOOL____00095) (=> BOOL____00095 (< 1 course_period__ARRAY__48))))
(assert (and (=> (< 1 course_period__ARRAY__49) BOOL____00097) (=> BOOL____00097 (< 1 course_period__ARRAY__49))))
(assert (and (=> (< 1 course_period__ARRAY__50) BOOL____00099) (=> BOOL____00099 (< 1 course_period__ARRAY__50))))
(assert (and (=> (< 2 course_period__ARRAY__1) BOOL____00101) (=> BOOL____00101 (< 2 course_period__ARRAY__1))))
(assert (and (=> (< 2 course_period__ARRAY__2) BOOL____00103) (=> BOOL____00103 (< 2 course_period__ARRAY__2))))
(assert (and (=> (< 2 course_period__ARRAY__3) BOOL____00105) (=> BOOL____00105 (< 2 course_period__ARRAY__3))))
(assert (and (=> (< 2 course_period__ARRAY__4) BOOL____00107) (=> BOOL____00107 (< 2 course_period__ARRAY__4))))
(assert (and (=> (< 2 course_period__ARRAY__5) BOOL____00109) (=> BOOL____00109 (< 2 course_period__ARRAY__5))))
(assert (and (=> (< 2 course_period__ARRAY__6) BOOL____00111) (=> BOOL____00111 (< 2 course_period__ARRAY__6))))
(assert (and (=> (< 2 course_period__ARRAY__7) BOOL____00113) (=> BOOL____00113 (< 2 course_period__ARRAY__7))))
(assert (and (=> (< 2 course_period__ARRAY__8) BOOL____00115) (=> BOOL____00115 (< 2 course_period__ARRAY__8))))
(assert (and (=> (< 2 course_period__ARRAY__9) BOOL____00117) (=> BOOL____00117 (< 2 course_period__ARRAY__9))))
(assert (and (=> (< 2 course_period__ARRAY__10) BOOL____00119) (=> BOOL____00119 (< 2 course_period__ARRAY__10))))
(assert (and (=> (< 2 course_period__ARRAY__11) BOOL____00121) (=> BOOL____00121 (< 2 course_period__ARRAY__11))))
(assert (and (=> (< 2 course_period__ARRAY__12) BOOL____00123) (=> BOOL____00123 (< 2 course_period__ARRAY__12))))
(assert (and (=> (< 2 course_period__ARRAY__13) BOOL____00125) (=> BOOL____00125 (< 2 course_period__ARRAY__13))))
(assert (and (=> (< 2 course_period__ARRAY__14) BOOL____00127) (=> BOOL____00127 (< 2 course_period__ARRAY__14))))
(assert (and (=> (< 2 course_period__ARRAY__15) BOOL____00129) (=> BOOL____00129 (< 2 course_period__ARRAY__15))))
(assert (and (=> (< 2 course_period__ARRAY__16) BOOL____00131) (=> BOOL____00131 (< 2 course_period__ARRAY__16))))
(assert (and (=> (< 2 course_period__ARRAY__17) BOOL____00133) (=> BOOL____00133 (< 2 course_period__ARRAY__17))))
(assert (and (=> (< 2 course_period__ARRAY__18) BOOL____00135) (=> BOOL____00135 (< 2 course_period__ARRAY__18))))
(assert (and (=> (< 2 course_period__ARRAY__19) BOOL____00137) (=> BOOL____00137 (< 2 course_period__ARRAY__19))))
(assert (and (=> (< 2 course_period__ARRAY__20) BOOL____00139) (=> BOOL____00139 (< 2 course_period__ARRAY__20))))
(assert (and (=> (< 2 course_period__ARRAY__21) BOOL____00141) (=> BOOL____00141 (< 2 course_period__ARRAY__21))))
(assert (and (=> (< 2 course_period__ARRAY__22) BOOL____00143) (=> BOOL____00143 (< 2 course_period__ARRAY__22))))
(assert (and (=> (< 2 course_period__ARRAY__23) BOOL____00145) (=> BOOL____00145 (< 2 course_period__ARRAY__23))))
(assert (and (=> (< 2 course_period__ARRAY__24) BOOL____00147) (=> BOOL____00147 (< 2 course_period__ARRAY__24))))
(assert (and (=> (< 2 course_period__ARRAY__25) BOOL____00149) (=> BOOL____00149 (< 2 course_period__ARRAY__25))))
(assert (and (=> (< 2 course_period__ARRAY__26) BOOL____00151) (=> BOOL____00151 (< 2 course_period__ARRAY__26))))
(assert (and (=> (< 2 course_period__ARRAY__27) BOOL____00153) (=> BOOL____00153 (< 2 course_period__ARRAY__27))))
(assert (and (=> (< 2 course_period__ARRAY__28) BOOL____00155) (=> BOOL____00155 (< 2 course_period__ARRAY__28))))
(assert (and (=> (< 2 course_period__ARRAY__29) BOOL____00157) (=> BOOL____00157 (< 2 course_period__ARRAY__29))))
(assert (and (=> (< 2 course_period__ARRAY__30) BOOL____00159) (=> BOOL____00159 (< 2 course_period__ARRAY__30))))
(assert (and (=> (< 2 course_period__ARRAY__31) BOOL____00161) (=> BOOL____00161 (< 2 course_period__ARRAY__31))))
(assert (and (=> (< 2 course_period__ARRAY__32) BOOL____00163) (=> BOOL____00163 (< 2 course_period__ARRAY__32))))
(assert (and (=> (< 2 course_period__ARRAY__33) BOOL____00165) (=> BOOL____00165 (< 2 course_period__ARRAY__33))))
(assert (and (=> (< 2 course_period__ARRAY__34) BOOL____00167) (=> BOOL____00167 (< 2 course_period__ARRAY__34))))
(assert (and (=> (< 2 course_period__ARRAY__35) BOOL____00169) (=> BOOL____00169 (< 2 course_period__ARRAY__35))))
(assert (and (=> (< 2 course_period__ARRAY__36) BOOL____00171) (=> BOOL____00171 (< 2 course_period__ARRAY__36))))
(assert (and (=> (< 2 course_period__ARRAY__37) BOOL____00173) (=> BOOL____00173 (< 2 course_period__ARRAY__37))))
(assert (and (=> (< 2 course_period__ARRAY__38) BOOL____00175) (=> BOOL____00175 (< 2 course_period__ARRAY__38))))
(assert (and (=> (< 2 course_period__ARRAY__39) BOOL____00177) (=> BOOL____00177 (< 2 course_period__ARRAY__39))))
(assert (and (=> (< 2 course_period__ARRAY__40) BOOL____00179) (=> BOOL____00179 (< 2 course_period__ARRAY__40))))
(assert (and (=> (< 2 course_period__ARRAY__41) BOOL____00181) (=> BOOL____00181 (< 2 course_period__ARRAY__41))))
(assert (and (=> (< 2 course_period__ARRAY__42) BOOL____00183) (=> BOOL____00183 (< 2 course_period__ARRAY__42))))
(assert (and (=> (< 2 course_period__ARRAY__43) BOOL____00185) (=> BOOL____00185 (< 2 course_period__ARRAY__43))))
(assert (and (=> (< 2 course_period__ARRAY__44) BOOL____00187) (=> BOOL____00187 (< 2 course_period__ARRAY__44))))
(assert (and (=> (< 2 course_period__ARRAY__45) BOOL____00189) (=> BOOL____00189 (< 2 course_period__ARRAY__45))))
(assert (and (=> (< 2 course_period__ARRAY__46) BOOL____00191) (=> BOOL____00191 (< 2 course_period__ARRAY__46))))
(assert (and (=> (< 2 course_period__ARRAY__47) BOOL____00193) (=> BOOL____00193 (< 2 course_period__ARRAY__47))))
(assert (and (=> (< 2 course_period__ARRAY__48) BOOL____00195) (=> BOOL____00195 (< 2 course_period__ARRAY__48))))
(assert (and (=> (< 2 course_period__ARRAY__49) BOOL____00197) (=> BOOL____00197 (< 2 course_period__ARRAY__49))))
(assert (and (=> (< 2 course_period__ARRAY__50) BOOL____00199) (=> BOOL____00199 (< 2 course_period__ARRAY__50))))
(assert (and (=> (< 3 course_period__ARRAY__1) BOOL____00201) (=> BOOL____00201 (< 3 course_period__ARRAY__1))))
(assert (and (=> (< 3 course_period__ARRAY__2) BOOL____00203) (=> BOOL____00203 (< 3 course_period__ARRAY__2))))
(assert (and (=> (< 3 course_period__ARRAY__3) BOOL____00205) (=> BOOL____00205 (< 3 course_period__ARRAY__3))))
(assert (and (=> (< 3 course_period__ARRAY__4) BOOL____00207) (=> BOOL____00207 (< 3 course_period__ARRAY__4))))
(assert (and (=> (< 3 course_period__ARRAY__5) BOOL____00209) (=> BOOL____00209 (< 3 course_period__ARRAY__5))))
(assert (and (=> (< 3 course_period__ARRAY__6) BOOL____00211) (=> BOOL____00211 (< 3 course_period__ARRAY__6))))
(assert (and (=> (< 3 course_period__ARRAY__7) BOOL____00213) (=> BOOL____00213 (< 3 course_period__ARRAY__7))))
(assert (and (=> (< 3 course_period__ARRAY__8) BOOL____00215) (=> BOOL____00215 (< 3 course_period__ARRAY__8))))
(assert (and (=> (< 3 course_period__ARRAY__9) BOOL____00217) (=> BOOL____00217 (< 3 course_period__ARRAY__9))))
(assert (and (=> (< 3 course_period__ARRAY__10) BOOL____00219) (=> BOOL____00219 (< 3 course_period__ARRAY__10))))
(assert (and (=> (< 3 course_period__ARRAY__11) BOOL____00221) (=> BOOL____00221 (< 3 course_period__ARRAY__11))))
(assert (and (=> (< 3 course_period__ARRAY__12) BOOL____00223) (=> BOOL____00223 (< 3 course_period__ARRAY__12))))
(assert (and (=> (< 3 course_period__ARRAY__13) BOOL____00225) (=> BOOL____00225 (< 3 course_period__ARRAY__13))))
(assert (and (=> (< 3 course_period__ARRAY__14) BOOL____00227) (=> BOOL____00227 (< 3 course_period__ARRAY__14))))
(assert (and (=> (< 3 course_period__ARRAY__15) BOOL____00229) (=> BOOL____00229 (< 3 course_period__ARRAY__15))))
(assert (and (=> (< 3 course_period__ARRAY__16) BOOL____00231) (=> BOOL____00231 (< 3 course_period__ARRAY__16))))
(assert (and (=> (< 3 course_period__ARRAY__17) BOOL____00233) (=> BOOL____00233 (< 3 course_period__ARRAY__17))))
(assert (and (=> (< 3 course_period__ARRAY__18) BOOL____00235) (=> BOOL____00235 (< 3 course_period__ARRAY__18))))
(assert (and (=> (< 3 course_period__ARRAY__19) BOOL____00237) (=> BOOL____00237 (< 3 course_period__ARRAY__19))))
(assert (and (=> (< 3 course_period__ARRAY__20) BOOL____00239) (=> BOOL____00239 (< 3 course_period__ARRAY__20))))
(assert (and (=> (< 3 course_period__ARRAY__21) BOOL____00241) (=> BOOL____00241 (< 3 course_period__ARRAY__21))))
(assert (and (=> (< 3 course_period__ARRAY__22) BOOL____00243) (=> BOOL____00243 (< 3 course_period__ARRAY__22))))
(assert (and (=> (< 3 course_period__ARRAY__23) BOOL____00245) (=> BOOL____00245 (< 3 course_period__ARRAY__23))))
(assert (and (=> (< 3 course_period__ARRAY__24) BOOL____00247) (=> BOOL____00247 (< 3 course_period__ARRAY__24))))
(assert (and (=> (< 3 course_period__ARRAY__25) BOOL____00249) (=> BOOL____00249 (< 3 course_period__ARRAY__25))))
(assert (and (=> (< 3 course_period__ARRAY__26) BOOL____00251) (=> BOOL____00251 (< 3 course_period__ARRAY__26))))
(assert (and (=> (< 3 course_period__ARRAY__27) BOOL____00253) (=> BOOL____00253 (< 3 course_period__ARRAY__27))))
(assert (and (=> (< 3 course_period__ARRAY__28) BOOL____00255) (=> BOOL____00255 (< 3 course_period__ARRAY__28))))
(assert (and (=> (< 3 course_period__ARRAY__29) BOOL____00257) (=> BOOL____00257 (< 3 course_period__ARRAY__29))))
(assert (and (=> (< 3 course_period__ARRAY__30) BOOL____00259) (=> BOOL____00259 (< 3 course_period__ARRAY__30))))
(assert (and (=> (< 3 course_period__ARRAY__31) BOOL____00261) (=> BOOL____00261 (< 3 course_period__ARRAY__31))))
(assert (and (=> (< 3 course_period__ARRAY__32) BOOL____00263) (=> BOOL____00263 (< 3 course_period__ARRAY__32))))
(assert (and (=> (< 3 course_period__ARRAY__33) BOOL____00265) (=> BOOL____00265 (< 3 course_period__ARRAY__33))))
(assert (and (=> (< 3 course_period__ARRAY__34) BOOL____00267) (=> BOOL____00267 (< 3 course_period__ARRAY__34))))
(assert (and (=> (< 3 course_period__ARRAY__35) BOOL____00269) (=> BOOL____00269 (< 3 course_period__ARRAY__35))))
(assert (and (=> (< 3 course_period__ARRAY__36) BOOL____00271) (=> BOOL____00271 (< 3 course_period__ARRAY__36))))
(assert (and (=> (< 3 course_period__ARRAY__37) BOOL____00273) (=> BOOL____00273 (< 3 course_period__ARRAY__37))))
(assert (and (=> (< 3 course_period__ARRAY__38) BOOL____00275) (=> BOOL____00275 (< 3 course_period__ARRAY__38))))
(assert (and (=> (< 3 course_period__ARRAY__39) BOOL____00277) (=> BOOL____00277 (< 3 course_period__ARRAY__39))))
(assert (and (=> (< 3 course_period__ARRAY__40) BOOL____00279) (=> BOOL____00279 (< 3 course_period__ARRAY__40))))
(assert (and (=> (< 3 course_period__ARRAY__41) BOOL____00281) (=> BOOL____00281 (< 3 course_period__ARRAY__41))))
(assert (and (=> (< 3 course_period__ARRAY__42) BOOL____00283) (=> BOOL____00283 (< 3 course_period__ARRAY__42))))
(assert (and (=> (< 3 course_period__ARRAY__43) BOOL____00285) (=> BOOL____00285 (< 3 course_period__ARRAY__43))))
(assert (and (=> (< 3 course_period__ARRAY__44) BOOL____00287) (=> BOOL____00287 (< 3 course_period__ARRAY__44))))
(assert (and (=> (< 3 course_period__ARRAY__45) BOOL____00289) (=> BOOL____00289 (< 3 course_period__ARRAY__45))))
(assert (and (=> (< 3 course_period__ARRAY__46) BOOL____00291) (=> BOOL____00291 (< 3 course_period__ARRAY__46))))
(assert (and (=> (< 3 course_period__ARRAY__47) BOOL____00293) (=> BOOL____00293 (< 3 course_period__ARRAY__47))))
(assert (and (=> (< 3 course_period__ARRAY__48) BOOL____00295) (=> BOOL____00295 (< 3 course_period__ARRAY__48))))
(assert (and (=> (< 3 course_period__ARRAY__49) BOOL____00297) (=> BOOL____00297 (< 3 course_period__ARRAY__49))))
(assert (and (=> (< 3 course_period__ARRAY__50) BOOL____00299) (=> BOOL____00299 (< 3 course_period__ARRAY__50))))
(assert (and (=> (< 4 course_period__ARRAY__1) BOOL____00301) (=> BOOL____00301 (< 4 course_period__ARRAY__1))))
(assert (and (=> (< 4 course_period__ARRAY__2) BOOL____00303) (=> BOOL____00303 (< 4 course_period__ARRAY__2))))
(assert (and (=> (< 4 course_period__ARRAY__3) BOOL____00305) (=> BOOL____00305 (< 4 course_period__ARRAY__3))))
(assert (and (=> (< 4 course_period__ARRAY__4) BOOL____00307) (=> BOOL____00307 (< 4 course_period__ARRAY__4))))
(assert (and (=> (< 4 course_period__ARRAY__5) BOOL____00309) (=> BOOL____00309 (< 4 course_period__ARRAY__5))))
(assert (and (=> (< 4 course_period__ARRAY__6) BOOL____00311) (=> BOOL____00311 (< 4 course_period__ARRAY__6))))
(assert (and (=> (< 4 course_period__ARRAY__7) BOOL____00313) (=> BOOL____00313 (< 4 course_period__ARRAY__7))))
(assert (and (=> (< 4 course_period__ARRAY__8) BOOL____00315) (=> BOOL____00315 (< 4 course_period__ARRAY__8))))
(assert (and (=> (< 4 course_period__ARRAY__9) BOOL____00317) (=> BOOL____00317 (< 4 course_period__ARRAY__9))))
(assert (and (=> (< 4 course_period__ARRAY__10) BOOL____00319) (=> BOOL____00319 (< 4 course_period__ARRAY__10))))
(assert (and (=> (< 4 course_period__ARRAY__11) BOOL____00321) (=> BOOL____00321 (< 4 course_period__ARRAY__11))))
(assert (and (=> (< 4 course_period__ARRAY__12) BOOL____00323) (=> BOOL____00323 (< 4 course_period__ARRAY__12))))
(assert (and (=> (< 4 course_period__ARRAY__13) BOOL____00325) (=> BOOL____00325 (< 4 course_period__ARRAY__13))))
(assert (and (=> (< 4 course_period__ARRAY__14) BOOL____00327) (=> BOOL____00327 (< 4 course_period__ARRAY__14))))
(assert (and (=> (< 4 course_period__ARRAY__15) BOOL____00329) (=> BOOL____00329 (< 4 course_period__ARRAY__15))))
(assert (and (=> (< 4 course_period__ARRAY__16) BOOL____00331) (=> BOOL____00331 (< 4 course_period__ARRAY__16))))
(assert (and (=> (< 4 course_period__ARRAY__17) BOOL____00333) (=> BOOL____00333 (< 4 course_period__ARRAY__17))))
(assert (and (=> (< 4 course_period__ARRAY__18) BOOL____00335) (=> BOOL____00335 (< 4 course_period__ARRAY__18))))
(assert (and (=> (< 4 course_period__ARRAY__19) BOOL____00337) (=> BOOL____00337 (< 4 course_period__ARRAY__19))))
(assert (and (=> (< 4 course_period__ARRAY__20) BOOL____00339) (=> BOOL____00339 (< 4 course_period__ARRAY__20))))
(assert (and (=> (< 4 course_period__ARRAY__21) BOOL____00341) (=> BOOL____00341 (< 4 course_period__ARRAY__21))))
(assert (and (=> (< 4 course_period__ARRAY__22) BOOL____00343) (=> BOOL____00343 (< 4 course_period__ARRAY__22))))
(assert (and (=> (< 4 course_period__ARRAY__23) BOOL____00345) (=> BOOL____00345 (< 4 course_period__ARRAY__23))))
(assert (and (=> (< 4 course_period__ARRAY__24) BOOL____00347) (=> BOOL____00347 (< 4 course_period__ARRAY__24))))
(assert (and (=> (< 4 course_period__ARRAY__25) BOOL____00349) (=> BOOL____00349 (< 4 course_period__ARRAY__25))))
(assert (and (=> (< 4 course_period__ARRAY__26) BOOL____00351) (=> BOOL____00351 (< 4 course_period__ARRAY__26))))
(assert (and (=> (< 4 course_period__ARRAY__27) BOOL____00353) (=> BOOL____00353 (< 4 course_period__ARRAY__27))))
(assert (and (=> (< 4 course_period__ARRAY__28) BOOL____00355) (=> BOOL____00355 (< 4 course_period__ARRAY__28))))
(assert (and (=> (< 4 course_period__ARRAY__29) BOOL____00357) (=> BOOL____00357 (< 4 course_period__ARRAY__29))))
(assert (and (=> (< 4 course_period__ARRAY__30) BOOL____00359) (=> BOOL____00359 (< 4 course_period__ARRAY__30))))
(assert (and (=> (< 4 course_period__ARRAY__31) BOOL____00361) (=> BOOL____00361 (< 4 course_period__ARRAY__31))))
(assert (and (=> (< 4 course_period__ARRAY__32) BOOL____00363) (=> BOOL____00363 (< 4 course_period__ARRAY__32))))
(assert (and (=> (< 4 course_period__ARRAY__33) BOOL____00365) (=> BOOL____00365 (< 4 course_period__ARRAY__33))))
(assert (and (=> (< 4 course_period__ARRAY__34) BOOL____00367) (=> BOOL____00367 (< 4 course_period__ARRAY__34))))
(assert (and (=> (< 4 course_period__ARRAY__35) BOOL____00369) (=> BOOL____00369 (< 4 course_period__ARRAY__35))))
(assert (and (=> (< 4 course_period__ARRAY__36) BOOL____00371) (=> BOOL____00371 (< 4 course_period__ARRAY__36))))
(assert (and (=> (< 4 course_period__ARRAY__37) BOOL____00373) (=> BOOL____00373 (< 4 course_period__ARRAY__37))))
(assert (and (=> (< 4 course_period__ARRAY__38) BOOL____00375) (=> BOOL____00375 (< 4 course_period__ARRAY__38))))
(assert (and (=> (< 4 course_period__ARRAY__39) BOOL____00377) (=> BOOL____00377 (< 4 course_period__ARRAY__39))))
(assert (and (=> (< 4 course_period__ARRAY__40) BOOL____00379) (=> BOOL____00379 (< 4 course_period__ARRAY__40))))
(assert (and (=> (< 4 course_period__ARRAY__41) BOOL____00381) (=> BOOL____00381 (< 4 course_period__ARRAY__41))))
(assert (and (=> (< 4 course_period__ARRAY__42) BOOL____00383) (=> BOOL____00383 (< 4 course_period__ARRAY__42))))
(assert (and (=> (< 4 course_period__ARRAY__43) BOOL____00385) (=> BOOL____00385 (< 4 course_period__ARRAY__43))))
(assert (and (=> (< 4 course_period__ARRAY__44) BOOL____00387) (=> BOOL____00387 (< 4 course_period__ARRAY__44))))
(assert (and (=> (< 4 course_period__ARRAY__45) BOOL____00389) (=> BOOL____00389 (< 4 course_period__ARRAY__45))))
(assert (and (=> (< 4 course_period__ARRAY__46) BOOL____00391) (=> BOOL____00391 (< 4 course_period__ARRAY__46))))
(assert (and (=> (< 4 course_period__ARRAY__47) BOOL____00393) (=> BOOL____00393 (< 4 course_period__ARRAY__47))))
(assert (and (=> (< 4 course_period__ARRAY__48) BOOL____00395) (=> BOOL____00395 (< 4 course_period__ARRAY__48))))
(assert (and (=> (< 4 course_period__ARRAY__49) BOOL____00397) (=> BOOL____00397 (< 4 course_period__ARRAY__49))))
(assert (and (=> (< 4 course_period__ARRAY__50) BOOL____00399) (=> BOOL____00399 (< 4 course_period__ARRAY__50))))
(assert (and (=> (< 5 course_period__ARRAY__1) BOOL____00401) (=> BOOL____00401 (< 5 course_period__ARRAY__1))))
(assert (and (=> (< 5 course_period__ARRAY__2) BOOL____00403) (=> BOOL____00403 (< 5 course_period__ARRAY__2))))
(assert (and (=> (< 5 course_period__ARRAY__3) BOOL____00405) (=> BOOL____00405 (< 5 course_period__ARRAY__3))))
(assert (and (=> (< 5 course_period__ARRAY__4) BOOL____00407) (=> BOOL____00407 (< 5 course_period__ARRAY__4))))
(assert (and (=> (< 5 course_period__ARRAY__5) BOOL____00409) (=> BOOL____00409 (< 5 course_period__ARRAY__5))))
(assert (and (=> (< 5 course_period__ARRAY__6) BOOL____00411) (=> BOOL____00411 (< 5 course_period__ARRAY__6))))
(assert (and (=> (< 5 course_period__ARRAY__7) BOOL____00413) (=> BOOL____00413 (< 5 course_period__ARRAY__7))))
(assert (and (=> (< 5 course_period__ARRAY__8) BOOL____00415) (=> BOOL____00415 (< 5 course_period__ARRAY__8))))
(assert (and (=> (< 5 course_period__ARRAY__9) BOOL____00417) (=> BOOL____00417 (< 5 course_period__ARRAY__9))))
(assert (and (=> (< 5 course_period__ARRAY__10) BOOL____00419) (=> BOOL____00419 (< 5 course_period__ARRAY__10))))
(assert (and (=> (< 5 course_period__ARRAY__11) BOOL____00421) (=> BOOL____00421 (< 5 course_period__ARRAY__11))))
(assert (and (=> (< 5 course_period__ARRAY__12) BOOL____00423) (=> BOOL____00423 (< 5 course_period__ARRAY__12))))
(assert (and (=> (< 5 course_period__ARRAY__13) BOOL____00425) (=> BOOL____00425 (< 5 course_period__ARRAY__13))))
(assert (and (=> (< 5 course_period__ARRAY__14) BOOL____00427) (=> BOOL____00427 (< 5 course_period__ARRAY__14))))
(assert (and (=> (< 5 course_period__ARRAY__15) BOOL____00429) (=> BOOL____00429 (< 5 course_period__ARRAY__15))))
(assert (and (=> (< 5 course_period__ARRAY__16) BOOL____00431) (=> BOOL____00431 (< 5 course_period__ARRAY__16))))
(assert (and (=> (< 5 course_period__ARRAY__17) BOOL____00433) (=> BOOL____00433 (< 5 course_period__ARRAY__17))))
(assert (and (=> (< 5 course_period__ARRAY__18) BOOL____00435) (=> BOOL____00435 (< 5 course_period__ARRAY__18))))
(assert (and (=> (< 5 course_period__ARRAY__19) BOOL____00437) (=> BOOL____00437 (< 5 course_period__ARRAY__19))))
(assert (and (=> (< 5 course_period__ARRAY__20) BOOL____00439) (=> BOOL____00439 (< 5 course_period__ARRAY__20))))
(assert (and (=> (< 5 course_period__ARRAY__21) BOOL____00441) (=> BOOL____00441 (< 5 course_period__ARRAY__21))))
(assert (and (=> (< 5 course_period__ARRAY__22) BOOL____00443) (=> BOOL____00443 (< 5 course_period__ARRAY__22))))
(assert (and (=> (< 5 course_period__ARRAY__23) BOOL____00445) (=> BOOL____00445 (< 5 course_period__ARRAY__23))))
(assert (and (=> (< 5 course_period__ARRAY__24) BOOL____00447) (=> BOOL____00447 (< 5 course_period__ARRAY__24))))
(assert (and (=> (< 5 course_period__ARRAY__25) BOOL____00449) (=> BOOL____00449 (< 5 course_period__ARRAY__25))))
(assert (and (=> (< 5 course_period__ARRAY__26) BOOL____00451) (=> BOOL____00451 (< 5 course_period__ARRAY__26))))
(assert (and (=> (< 5 course_period__ARRAY__27) BOOL____00453) (=> BOOL____00453 (< 5 course_period__ARRAY__27))))
(assert (and (=> (< 5 course_period__ARRAY__28) BOOL____00455) (=> BOOL____00455 (< 5 course_period__ARRAY__28))))
(assert (and (=> (< 5 course_period__ARRAY__29) BOOL____00457) (=> BOOL____00457 (< 5 course_period__ARRAY__29))))
(assert (and (=> (< 5 course_period__ARRAY__30) BOOL____00459) (=> BOOL____00459 (< 5 course_period__ARRAY__30))))
(assert (and (=> (< 5 course_period__ARRAY__31) BOOL____00461) (=> BOOL____00461 (< 5 course_period__ARRAY__31))))
(assert (and (=> (< 5 course_period__ARRAY__32) BOOL____00463) (=> BOOL____00463 (< 5 course_period__ARRAY__32))))
(assert (and (=> (< 5 course_period__ARRAY__33) BOOL____00465) (=> BOOL____00465 (< 5 course_period__ARRAY__33))))
(assert (and (=> (< 5 course_period__ARRAY__34) BOOL____00467) (=> BOOL____00467 (< 5 course_period__ARRAY__34))))
(assert (and (=> (< 5 course_period__ARRAY__35) BOOL____00469) (=> BOOL____00469 (< 5 course_period__ARRAY__35))))
(assert (and (=> (< 5 course_period__ARRAY__36) BOOL____00471) (=> BOOL____00471 (< 5 course_period__ARRAY__36))))
(assert (and (=> (< 5 course_period__ARRAY__37) BOOL____00473) (=> BOOL____00473 (< 5 course_period__ARRAY__37))))
(assert (and (=> (< 5 course_period__ARRAY__38) BOOL____00475) (=> BOOL____00475 (< 5 course_period__ARRAY__38))))
(assert (and (=> (< 5 course_period__ARRAY__39) BOOL____00477) (=> BOOL____00477 (< 5 course_period__ARRAY__39))))
(assert (and (=> (< 5 course_period__ARRAY__40) BOOL____00479) (=> BOOL____00479 (< 5 course_period__ARRAY__40))))
(assert (and (=> (< 5 course_period__ARRAY__41) BOOL____00481) (=> BOOL____00481 (< 5 course_period__ARRAY__41))))
(assert (and (=> (< 5 course_period__ARRAY__42) BOOL____00483) (=> BOOL____00483 (< 5 course_period__ARRAY__42))))
(assert (and (=> (< 5 course_period__ARRAY__43) BOOL____00485) (=> BOOL____00485 (< 5 course_period__ARRAY__43))))
(assert (and (=> (< 5 course_period__ARRAY__44) BOOL____00487) (=> BOOL____00487 (< 5 course_period__ARRAY__44))))
(assert (and (=> (< 5 course_period__ARRAY__45) BOOL____00489) (=> BOOL____00489 (< 5 course_period__ARRAY__45))))
(assert (and (=> (< 5 course_period__ARRAY__46) BOOL____00491) (=> BOOL____00491 (< 5 course_period__ARRAY__46))))
(assert (and (=> (< 5 course_period__ARRAY__47) BOOL____00493) (=> BOOL____00493 (< 5 course_period__ARRAY__47))))
(assert (and (=> (< 5 course_period__ARRAY__48) BOOL____00495) (=> BOOL____00495 (< 5 course_period__ARRAY__48))))
(assert (and (=> (< 5 course_period__ARRAY__49) BOOL____00497) (=> BOOL____00497 (< 5 course_period__ARRAY__49))))
(assert (and (=> (< 5 course_period__ARRAY__50) BOOL____00499) (=> BOOL____00499 (< 5 course_period__ARRAY__50))))
(assert (and (=> (< 6 course_period__ARRAY__1) BOOL____00501) (=> BOOL____00501 (< 6 course_period__ARRAY__1))))
(assert (and (=> (< 6 course_period__ARRAY__2) BOOL____00503) (=> BOOL____00503 (< 6 course_period__ARRAY__2))))
(assert (and (=> (< 6 course_period__ARRAY__3) BOOL____00505) (=> BOOL____00505 (< 6 course_period__ARRAY__3))))
(assert (and (=> (< 6 course_period__ARRAY__4) BOOL____00507) (=> BOOL____00507 (< 6 course_period__ARRAY__4))))
(assert (and (=> (< 6 course_period__ARRAY__5) BOOL____00509) (=> BOOL____00509 (< 6 course_period__ARRAY__5))))
(assert (and (=> (< 6 course_period__ARRAY__6) BOOL____00511) (=> BOOL____00511 (< 6 course_period__ARRAY__6))))
(assert (and (=> (< 6 course_period__ARRAY__7) BOOL____00513) (=> BOOL____00513 (< 6 course_period__ARRAY__7))))
(assert (and (=> (< 6 course_period__ARRAY__8) BOOL____00515) (=> BOOL____00515 (< 6 course_period__ARRAY__8))))
(assert (and (=> (< 6 course_period__ARRAY__9) BOOL____00517) (=> BOOL____00517 (< 6 course_period__ARRAY__9))))
(assert (and (=> (< 6 course_period__ARRAY__10) BOOL____00519) (=> BOOL____00519 (< 6 course_period__ARRAY__10))))
(assert (and (=> (< 6 course_period__ARRAY__11) BOOL____00521) (=> BOOL____00521 (< 6 course_period__ARRAY__11))))
(assert (and (=> (< 6 course_period__ARRAY__12) BOOL____00523) (=> BOOL____00523 (< 6 course_period__ARRAY__12))))
(assert (and (=> (< 6 course_period__ARRAY__13) BOOL____00525) (=> BOOL____00525 (< 6 course_period__ARRAY__13))))
(assert (and (=> (< 6 course_period__ARRAY__14) BOOL____00527) (=> BOOL____00527 (< 6 course_period__ARRAY__14))))
(assert (and (=> (< 6 course_period__ARRAY__15) BOOL____00529) (=> BOOL____00529 (< 6 course_period__ARRAY__15))))
(assert (and (=> (< 6 course_period__ARRAY__16) BOOL____00531) (=> BOOL____00531 (< 6 course_period__ARRAY__16))))
(assert (and (=> (< 6 course_period__ARRAY__17) BOOL____00533) (=> BOOL____00533 (< 6 course_period__ARRAY__17))))
(assert (and (=> (< 6 course_period__ARRAY__18) BOOL____00535) (=> BOOL____00535 (< 6 course_period__ARRAY__18))))
(assert (and (=> (< 6 course_period__ARRAY__19) BOOL____00537) (=> BOOL____00537 (< 6 course_period__ARRAY__19))))
(assert (and (=> (< 6 course_period__ARRAY__20) BOOL____00539) (=> BOOL____00539 (< 6 course_period__ARRAY__20))))
(assert (and (=> (< 6 course_period__ARRAY__21) BOOL____00541) (=> BOOL____00541 (< 6 course_period__ARRAY__21))))
(assert (and (=> (< 6 course_period__ARRAY__22) BOOL____00543) (=> BOOL____00543 (< 6 course_period__ARRAY__22))))
(assert (and (=> (< 6 course_period__ARRAY__23) BOOL____00545) (=> BOOL____00545 (< 6 course_period__ARRAY__23))))
(assert (and (=> (< 6 course_period__ARRAY__24) BOOL____00547) (=> BOOL____00547 (< 6 course_period__ARRAY__24))))
(assert (and (=> (< 6 course_period__ARRAY__25) BOOL____00549) (=> BOOL____00549 (< 6 course_period__ARRAY__25))))
(assert (and (=> (< 6 course_period__ARRAY__26) BOOL____00551) (=> BOOL____00551 (< 6 course_period__ARRAY__26))))
(assert (and (=> (< 6 course_period__ARRAY__27) BOOL____00553) (=> BOOL____00553 (< 6 course_period__ARRAY__27))))
(assert (and (=> (< 6 course_period__ARRAY__28) BOOL____00555) (=> BOOL____00555 (< 6 course_period__ARRAY__28))))
(assert (and (=> (< 6 course_period__ARRAY__29) BOOL____00557) (=> BOOL____00557 (< 6 course_period__ARRAY__29))))
(assert (and (=> (< 6 course_period__ARRAY__30) BOOL____00559) (=> BOOL____00559 (< 6 course_period__ARRAY__30))))
(assert (and (=> (< 6 course_period__ARRAY__31) BOOL____00561) (=> BOOL____00561 (< 6 course_period__ARRAY__31))))
(assert (and (=> (< 6 course_period__ARRAY__32) BOOL____00563) (=> BOOL____00563 (< 6 course_period__ARRAY__32))))
(assert (and (=> (< 6 course_period__ARRAY__33) BOOL____00565) (=> BOOL____00565 (< 6 course_period__ARRAY__33))))
(assert (and (=> (< 6 course_period__ARRAY__34) BOOL____00567) (=> BOOL____00567 (< 6 course_period__ARRAY__34))))
(assert (and (=> (< 6 course_period__ARRAY__35) BOOL____00569) (=> BOOL____00569 (< 6 course_period__ARRAY__35))))
(assert (and (=> (< 6 course_period__ARRAY__36) BOOL____00571) (=> BOOL____00571 (< 6 course_period__ARRAY__36))))
(assert (and (=> (< 6 course_period__ARRAY__37) BOOL____00573) (=> BOOL____00573 (< 6 course_period__ARRAY__37))))
(assert (and (=> (< 6 course_period__ARRAY__38) BOOL____00575) (=> BOOL____00575 (< 6 course_period__ARRAY__38))))
(assert (and (=> (< 6 course_period__ARRAY__39) BOOL____00577) (=> BOOL____00577 (< 6 course_period__ARRAY__39))))
(assert (and (=> (< 6 course_period__ARRAY__40) BOOL____00579) (=> BOOL____00579 (< 6 course_period__ARRAY__40))))
(assert (and (=> (< 6 course_period__ARRAY__41) BOOL____00581) (=> BOOL____00581 (< 6 course_period__ARRAY__41))))
(assert (and (=> (< 6 course_period__ARRAY__42) BOOL____00583) (=> BOOL____00583 (< 6 course_period__ARRAY__42))))
(assert (and (=> (< 6 course_period__ARRAY__43) BOOL____00585) (=> BOOL____00585 (< 6 course_period__ARRAY__43))))
(assert (and (=> (< 6 course_period__ARRAY__44) BOOL____00587) (=> BOOL____00587 (< 6 course_period__ARRAY__44))))
(assert (and (=> (< 6 course_period__ARRAY__45) BOOL____00589) (=> BOOL____00589 (< 6 course_period__ARRAY__45))))
(assert (and (=> (< 6 course_period__ARRAY__46) BOOL____00591) (=> BOOL____00591 (< 6 course_period__ARRAY__46))))
(assert (and (=> (< 6 course_period__ARRAY__47) BOOL____00593) (=> BOOL____00593 (< 6 course_period__ARRAY__47))))
(assert (and (=> (< 6 course_period__ARRAY__48) BOOL____00595) (=> BOOL____00595 (< 6 course_period__ARRAY__48))))
(assert (and (=> (< 6 course_period__ARRAY__49) BOOL____00597) (=> BOOL____00597 (< 6 course_period__ARRAY__49))))
(assert (and (=> (< 6 course_period__ARRAY__50) BOOL____00599) (=> BOOL____00599 (< 6 course_period__ARRAY__50))))
(assert (and (=> (< 7 course_period__ARRAY__1) BOOL____00601) (=> BOOL____00601 (< 7 course_period__ARRAY__1))))
(assert (and (=> (< 7 course_period__ARRAY__2) BOOL____00603) (=> BOOL____00603 (< 7 course_period__ARRAY__2))))
(assert (and (=> (< 7 course_period__ARRAY__3) BOOL____00605) (=> BOOL____00605 (< 7 course_period__ARRAY__3))))
(assert (and (=> (< 7 course_period__ARRAY__4) BOOL____00607) (=> BOOL____00607 (< 7 course_period__ARRAY__4))))
(assert (and (=> (< 7 course_period__ARRAY__5) BOOL____00609) (=> BOOL____00609 (< 7 course_period__ARRAY__5))))
(assert (and (=> (< 7 course_period__ARRAY__6) BOOL____00611) (=> BOOL____00611 (< 7 course_period__ARRAY__6))))
(assert (and (=> (< 7 course_period__ARRAY__7) BOOL____00613) (=> BOOL____00613 (< 7 course_period__ARRAY__7))))
(assert (and (=> (< 7 course_period__ARRAY__8) BOOL____00615) (=> BOOL____00615 (< 7 course_period__ARRAY__8))))
(assert (and (=> (< 7 course_period__ARRAY__9) BOOL____00617) (=> BOOL____00617 (< 7 course_period__ARRAY__9))))
(assert (and (=> (< 7 course_period__ARRAY__10) BOOL____00619) (=> BOOL____00619 (< 7 course_period__ARRAY__10))))
(assert (and (=> (< 7 course_period__ARRAY__11) BOOL____00621) (=> BOOL____00621 (< 7 course_period__ARRAY__11))))
(assert (and (=> (< 7 course_period__ARRAY__12) BOOL____00623) (=> BOOL____00623 (< 7 course_period__ARRAY__12))))
(assert (and (=> (< 7 course_period__ARRAY__13) BOOL____00625) (=> BOOL____00625 (< 7 course_period__ARRAY__13))))
(assert (and (=> (< 7 course_period__ARRAY__14) BOOL____00627) (=> BOOL____00627 (< 7 course_period__ARRAY__14))))
(assert (and (=> (< 7 course_period__ARRAY__15) BOOL____00629) (=> BOOL____00629 (< 7 course_period__ARRAY__15))))
(assert (and (=> (< 7 course_period__ARRAY__16) BOOL____00631) (=> BOOL____00631 (< 7 course_period__ARRAY__16))))
(assert (and (=> (< 7 course_period__ARRAY__17) BOOL____00633) (=> BOOL____00633 (< 7 course_period__ARRAY__17))))
(assert (and (=> (< 7 course_period__ARRAY__18) BOOL____00635) (=> BOOL____00635 (< 7 course_period__ARRAY__18))))
(assert (and (=> (< 7 course_period__ARRAY__19) BOOL____00637) (=> BOOL____00637 (< 7 course_period__ARRAY__19))))
(assert (and (=> (< 7 course_period__ARRAY__20) BOOL____00639) (=> BOOL____00639 (< 7 course_period__ARRAY__20))))
(assert (and (=> (< 7 course_period__ARRAY__21) BOOL____00641) (=> BOOL____00641 (< 7 course_period__ARRAY__21))))
(assert (and (=> (< 7 course_period__ARRAY__22) BOOL____00643) (=> BOOL____00643 (< 7 course_period__ARRAY__22))))
(assert (and (=> (< 7 course_period__ARRAY__23) BOOL____00645) (=> BOOL____00645 (< 7 course_period__ARRAY__23))))
(assert (and (=> (< 7 course_period__ARRAY__24) BOOL____00647) (=> BOOL____00647 (< 7 course_period__ARRAY__24))))
(assert (and (=> (< 7 course_period__ARRAY__25) BOOL____00649) (=> BOOL____00649 (< 7 course_period__ARRAY__25))))
(assert (and (=> (< 7 course_period__ARRAY__26) BOOL____00651) (=> BOOL____00651 (< 7 course_period__ARRAY__26))))
(assert (and (=> (< 7 course_period__ARRAY__27) BOOL____00653) (=> BOOL____00653 (< 7 course_period__ARRAY__27))))
(assert (and (=> (< 7 course_period__ARRAY__28) BOOL____00655) (=> BOOL____00655 (< 7 course_period__ARRAY__28))))
(assert (and (=> (< 7 course_period__ARRAY__29) BOOL____00657) (=> BOOL____00657 (< 7 course_period__ARRAY__29))))
(assert (and (=> (< 7 course_period__ARRAY__30) BOOL____00659) (=> BOOL____00659 (< 7 course_period__ARRAY__30))))
(assert (and (=> (< 7 course_period__ARRAY__31) BOOL____00661) (=> BOOL____00661 (< 7 course_period__ARRAY__31))))
(assert (and (=> (< 7 course_period__ARRAY__32) BOOL____00663) (=> BOOL____00663 (< 7 course_period__ARRAY__32))))
(assert (and (=> (< 7 course_period__ARRAY__33) BOOL____00665) (=> BOOL____00665 (< 7 course_period__ARRAY__33))))
(assert (and (=> (< 7 course_period__ARRAY__34) BOOL____00667) (=> BOOL____00667 (< 7 course_period__ARRAY__34))))
(assert (and (=> (< 7 course_period__ARRAY__35) BOOL____00669) (=> BOOL____00669 (< 7 course_period__ARRAY__35))))
(assert (and (=> (< 7 course_period__ARRAY__36) BOOL____00671) (=> BOOL____00671 (< 7 course_period__ARRAY__36))))
(assert (and (=> (< 7 course_period__ARRAY__37) BOOL____00673) (=> BOOL____00673 (< 7 course_period__ARRAY__37))))
(assert (and (=> (< 7 course_period__ARRAY__38) BOOL____00675) (=> BOOL____00675 (< 7 course_period__ARRAY__38))))
(assert (and (=> (< 7 course_period__ARRAY__39) BOOL____00677) (=> BOOL____00677 (< 7 course_period__ARRAY__39))))
(assert (and (=> (< 7 course_period__ARRAY__40) BOOL____00679) (=> BOOL____00679 (< 7 course_period__ARRAY__40))))
(assert (and (=> (< 7 course_period__ARRAY__41) BOOL____00681) (=> BOOL____00681 (< 7 course_period__ARRAY__41))))
(assert (and (=> (< 7 course_period__ARRAY__42) BOOL____00683) (=> BOOL____00683 (< 7 course_period__ARRAY__42))))
(assert (and (=> (< 7 course_period__ARRAY__43) BOOL____00685) (=> BOOL____00685 (< 7 course_period__ARRAY__43))))
(assert (and (=> (< 7 course_period__ARRAY__44) BOOL____00687) (=> BOOL____00687 (< 7 course_period__ARRAY__44))))
(assert (and (=> (< 7 course_period__ARRAY__45) BOOL____00689) (=> BOOL____00689 (< 7 course_period__ARRAY__45))))
(assert (and (=> (< 7 course_period__ARRAY__46) BOOL____00691) (=> BOOL____00691 (< 7 course_period__ARRAY__46))))
(assert (and (=> (< 7 course_period__ARRAY__47) BOOL____00693) (=> BOOL____00693 (< 7 course_period__ARRAY__47))))
(assert (and (=> (< 7 course_period__ARRAY__48) BOOL____00695) (=> BOOL____00695 (< 7 course_period__ARRAY__48))))
(assert (and (=> (< 7 course_period__ARRAY__49) BOOL____00697) (=> BOOL____00697 (< 7 course_period__ARRAY__49))))
(assert (and (=> (< 7 course_period__ARRAY__50) BOOL____00699) (=> BOOL____00699 (< 7 course_period__ARRAY__50))))
(assert (and (=> (< 8 course_period__ARRAY__1) BOOL____00701) (=> BOOL____00701 (< 8 course_period__ARRAY__1))))
(assert (and (=> (< 8 course_period__ARRAY__2) BOOL____00703) (=> BOOL____00703 (< 8 course_period__ARRAY__2))))
(assert (and (=> (< 8 course_period__ARRAY__3) BOOL____00705) (=> BOOL____00705 (< 8 course_period__ARRAY__3))))
(assert (and (=> (< 8 course_period__ARRAY__4) BOOL____00707) (=> BOOL____00707 (< 8 course_period__ARRAY__4))))
(assert (and (=> (< 8 course_period__ARRAY__5) BOOL____00709) (=> BOOL____00709 (< 8 course_period__ARRAY__5))))
(assert (and (=> (< 8 course_period__ARRAY__6) BOOL____00711) (=> BOOL____00711 (< 8 course_period__ARRAY__6))))
(assert (and (=> (< 8 course_period__ARRAY__7) BOOL____00713) (=> BOOL____00713 (< 8 course_period__ARRAY__7))))
(assert (and (=> (< 8 course_period__ARRAY__8) BOOL____00715) (=> BOOL____00715 (< 8 course_period__ARRAY__8))))
(assert (and (=> (< 8 course_period__ARRAY__9) BOOL____00717) (=> BOOL____00717 (< 8 course_period__ARRAY__9))))
(assert (and (=> (< 8 course_period__ARRAY__10) BOOL____00719) (=> BOOL____00719 (< 8 course_period__ARRAY__10))))
(assert (and (=> (< 8 course_period__ARRAY__11) BOOL____00721) (=> BOOL____00721 (< 8 course_period__ARRAY__11))))
(assert (and (=> (< 8 course_period__ARRAY__12) BOOL____00723) (=> BOOL____00723 (< 8 course_period__ARRAY__12))))
(assert (and (=> (< 8 course_period__ARRAY__13) BOOL____00725) (=> BOOL____00725 (< 8 course_period__ARRAY__13))))
(assert (and (=> (< 8 course_period__ARRAY__14) BOOL____00727) (=> BOOL____00727 (< 8 course_period__ARRAY__14))))
(assert (and (=> (< 8 course_period__ARRAY__15) BOOL____00729) (=> BOOL____00729 (< 8 course_period__ARRAY__15))))
(assert (and (=> (< 8 course_period__ARRAY__16) BOOL____00731) (=> BOOL____00731 (< 8 course_period__ARRAY__16))))
(assert (and (=> (< 8 course_period__ARRAY__17) BOOL____00733) (=> BOOL____00733 (< 8 course_period__ARRAY__17))))
(assert (and (=> (< 8 course_period__ARRAY__18) BOOL____00735) (=> BOOL____00735 (< 8 course_period__ARRAY__18))))
(assert (and (=> (< 8 course_period__ARRAY__19) BOOL____00737) (=> BOOL____00737 (< 8 course_period__ARRAY__19))))
(assert (and (=> (< 8 course_period__ARRAY__20) BOOL____00739) (=> BOOL____00739 (< 8 course_period__ARRAY__20))))
(assert (and (=> (< 8 course_period__ARRAY__21) BOOL____00741) (=> BOOL____00741 (< 8 course_period__ARRAY__21))))
(assert (and (=> (< 8 course_period__ARRAY__22) BOOL____00743) (=> BOOL____00743 (< 8 course_period__ARRAY__22))))
(assert (and (=> (< 8 course_period__ARRAY__23) BOOL____00745) (=> BOOL____00745 (< 8 course_period__ARRAY__23))))
(assert (and (=> (< 8 course_period__ARRAY__24) BOOL____00747) (=> BOOL____00747 (< 8 course_period__ARRAY__24))))
(assert (and (=> (< 8 course_period__ARRAY__25) BOOL____00749) (=> BOOL____00749 (< 8 course_period__ARRAY__25))))
(assert (and (=> (< 8 course_period__ARRAY__26) BOOL____00751) (=> BOOL____00751 (< 8 course_period__ARRAY__26))))
(assert (and (=> (< 8 course_period__ARRAY__27) BOOL____00753) (=> BOOL____00753 (< 8 course_period__ARRAY__27))))
(assert (and (=> (< 8 course_period__ARRAY__28) BOOL____00755) (=> BOOL____00755 (< 8 course_period__ARRAY__28))))
(assert (and (=> (< 8 course_period__ARRAY__29) BOOL____00757) (=> BOOL____00757 (< 8 course_period__ARRAY__29))))
(assert (and (=> (< 8 course_period__ARRAY__30) BOOL____00759) (=> BOOL____00759 (< 8 course_period__ARRAY__30))))
(assert (and (=> (< 8 course_period__ARRAY__31) BOOL____00761) (=> BOOL____00761 (< 8 course_period__ARRAY__31))))
(assert (and (=> (< 8 course_period__ARRAY__32) BOOL____00763) (=> BOOL____00763 (< 8 course_period__ARRAY__32))))
(assert (and (=> (< 8 course_period__ARRAY__33) BOOL____00765) (=> BOOL____00765 (< 8 course_period__ARRAY__33))))
(assert (and (=> (< 8 course_period__ARRAY__34) BOOL____00767) (=> BOOL____00767 (< 8 course_period__ARRAY__34))))
(assert (and (=> (< 8 course_period__ARRAY__35) BOOL____00769) (=> BOOL____00769 (< 8 course_period__ARRAY__35))))
(assert (and (=> (< 8 course_period__ARRAY__36) BOOL____00771) (=> BOOL____00771 (< 8 course_period__ARRAY__36))))
(assert (and (=> (< 8 course_period__ARRAY__37) BOOL____00773) (=> BOOL____00773 (< 8 course_period__ARRAY__37))))
(assert (and (=> (< 8 course_period__ARRAY__38) BOOL____00775) (=> BOOL____00775 (< 8 course_period__ARRAY__38))))
(assert (and (=> (< 8 course_period__ARRAY__39) BOOL____00777) (=> BOOL____00777 (< 8 course_period__ARRAY__39))))
(assert (and (=> (< 8 course_period__ARRAY__40) BOOL____00779) (=> BOOL____00779 (< 8 course_period__ARRAY__40))))
(assert (and (=> (< 8 course_period__ARRAY__41) BOOL____00781) (=> BOOL____00781 (< 8 course_period__ARRAY__41))))
(assert (and (=> (< 8 course_period__ARRAY__42) BOOL____00783) (=> BOOL____00783 (< 8 course_period__ARRAY__42))))
(assert (and (=> (< 8 course_period__ARRAY__43) BOOL____00785) (=> BOOL____00785 (< 8 course_period__ARRAY__43))))
(assert (and (=> (< 8 course_period__ARRAY__44) BOOL____00787) (=> BOOL____00787 (< 8 course_period__ARRAY__44))))
(assert (and (=> (< 8 course_period__ARRAY__45) BOOL____00789) (=> BOOL____00789 (< 8 course_period__ARRAY__45))))
(assert (and (=> (< 8 course_period__ARRAY__46) BOOL____00791) (=> BOOL____00791 (< 8 course_period__ARRAY__46))))
(assert (and (=> (< 8 course_period__ARRAY__47) BOOL____00793) (=> BOOL____00793 (< 8 course_period__ARRAY__47))))
(assert (and (=> (< 8 course_period__ARRAY__48) BOOL____00795) (=> BOOL____00795 (< 8 course_period__ARRAY__48))))
(assert (and (=> (< 8 course_period__ARRAY__49) BOOL____00797) (=> BOOL____00797 (< 8 course_period__ARRAY__49))))
(assert (and (=> (< 8 course_period__ARRAY__50) BOOL____00799) (=> BOOL____00799 (< 8 course_period__ARRAY__50))))
(assert (and (=> (< 9 course_period__ARRAY__1) BOOL____00801) (=> BOOL____00801 (< 9 course_period__ARRAY__1))))
(assert (and (=> (< 9 course_period__ARRAY__2) BOOL____00803) (=> BOOL____00803 (< 9 course_period__ARRAY__2))))
(assert (and (=> (< 9 course_period__ARRAY__3) BOOL____00805) (=> BOOL____00805 (< 9 course_period__ARRAY__3))))
(assert (and (=> (< 9 course_period__ARRAY__4) BOOL____00807) (=> BOOL____00807 (< 9 course_period__ARRAY__4))))
(assert (and (=> (< 9 course_period__ARRAY__5) BOOL____00809) (=> BOOL____00809 (< 9 course_period__ARRAY__5))))
(assert (and (=> (< 9 course_period__ARRAY__6) BOOL____00811) (=> BOOL____00811 (< 9 course_period__ARRAY__6))))
(assert (and (=> (< 9 course_period__ARRAY__7) BOOL____00813) (=> BOOL____00813 (< 9 course_period__ARRAY__7))))
(assert (and (=> (< 9 course_period__ARRAY__8) BOOL____00815) (=> BOOL____00815 (< 9 course_period__ARRAY__8))))
(assert (and (=> (< 9 course_period__ARRAY__9) BOOL____00817) (=> BOOL____00817 (< 9 course_period__ARRAY__9))))
(assert (and (=> (< 9 course_period__ARRAY__10) BOOL____00819) (=> BOOL____00819 (< 9 course_period__ARRAY__10))))
(assert (and (=> (< 9 course_period__ARRAY__11) BOOL____00821) (=> BOOL____00821 (< 9 course_period__ARRAY__11))))
(assert (and (=> (< 9 course_period__ARRAY__12) BOOL____00823) (=> BOOL____00823 (< 9 course_period__ARRAY__12))))
(assert (and (=> (< 9 course_period__ARRAY__13) BOOL____00825) (=> BOOL____00825 (< 9 course_period__ARRAY__13))))
(assert (and (=> (< 9 course_period__ARRAY__14) BOOL____00827) (=> BOOL____00827 (< 9 course_period__ARRAY__14))))
(assert (and (=> (< 9 course_period__ARRAY__15) BOOL____00829) (=> BOOL____00829 (< 9 course_period__ARRAY__15))))
(assert (and (=> (< 9 course_period__ARRAY__16) BOOL____00831) (=> BOOL____00831 (< 9 course_period__ARRAY__16))))
(assert (and (=> (< 9 course_period__ARRAY__17) BOOL____00833) (=> BOOL____00833 (< 9 course_period__ARRAY__17))))
(assert (and (=> (< 9 course_period__ARRAY__18) BOOL____00835) (=> BOOL____00835 (< 9 course_period__ARRAY__18))))
(assert (and (=> (< 9 course_period__ARRAY__19) BOOL____00837) (=> BOOL____00837 (< 9 course_period__ARRAY__19))))
(assert (and (=> (< 9 course_period__ARRAY__20) BOOL____00839) (=> BOOL____00839 (< 9 course_period__ARRAY__20))))
(assert (and (=> (< 9 course_period__ARRAY__21) BOOL____00841) (=> BOOL____00841 (< 9 course_period__ARRAY__21))))
(assert (and (=> (< 9 course_period__ARRAY__22) BOOL____00843) (=> BOOL____00843 (< 9 course_period__ARRAY__22))))
(assert (and (=> (< 9 course_period__ARRAY__23) BOOL____00845) (=> BOOL____00845 (< 9 course_period__ARRAY__23))))
(assert (and (=> (< 9 course_period__ARRAY__24) BOOL____00847) (=> BOOL____00847 (< 9 course_period__ARRAY__24))))
(assert (and (=> (< 9 course_period__ARRAY__25) BOOL____00849) (=> BOOL____00849 (< 9 course_period__ARRAY__25))))
(assert (and (=> (< 9 course_period__ARRAY__26) BOOL____00851) (=> BOOL____00851 (< 9 course_period__ARRAY__26))))
(assert (and (=> (< 9 course_period__ARRAY__27) BOOL____00853) (=> BOOL____00853 (< 9 course_period__ARRAY__27))))
(assert (and (=> (< 9 course_period__ARRAY__28) BOOL____00855) (=> BOOL____00855 (< 9 course_period__ARRAY__28))))
(assert (and (=> (< 9 course_period__ARRAY__29) BOOL____00857) (=> BOOL____00857 (< 9 course_period__ARRAY__29))))
(assert (and (=> (< 9 course_period__ARRAY__30) BOOL____00859) (=> BOOL____00859 (< 9 course_period__ARRAY__30))))
(assert (and (=> (< 9 course_period__ARRAY__31) BOOL____00861) (=> BOOL____00861 (< 9 course_period__ARRAY__31))))
(assert (and (=> (< 9 course_period__ARRAY__32) BOOL____00863) (=> BOOL____00863 (< 9 course_period__ARRAY__32))))
(assert (and (=> (< 9 course_period__ARRAY__33) BOOL____00865) (=> BOOL____00865 (< 9 course_period__ARRAY__33))))
(assert (and (=> (< 9 course_period__ARRAY__34) BOOL____00867) (=> BOOL____00867 (< 9 course_period__ARRAY__34))))
(assert (and (=> (< 9 course_period__ARRAY__35) BOOL____00869) (=> BOOL____00869 (< 9 course_period__ARRAY__35))))
(assert (and (=> (< 9 course_period__ARRAY__36) BOOL____00871) (=> BOOL____00871 (< 9 course_period__ARRAY__36))))
(assert (and (=> (< 9 course_period__ARRAY__37) BOOL____00873) (=> BOOL____00873 (< 9 course_period__ARRAY__37))))
(assert (and (=> (< 9 course_period__ARRAY__38) BOOL____00875) (=> BOOL____00875 (< 9 course_period__ARRAY__38))))
(assert (and (=> (< 9 course_period__ARRAY__39) BOOL____00877) (=> BOOL____00877 (< 9 course_period__ARRAY__39))))
(assert (and (=> (< 9 course_period__ARRAY__40) BOOL____00879) (=> BOOL____00879 (< 9 course_period__ARRAY__40))))
(assert (and (=> (< 9 course_period__ARRAY__41) BOOL____00881) (=> BOOL____00881 (< 9 course_period__ARRAY__41))))
(assert (and (=> (< 9 course_period__ARRAY__42) BOOL____00883) (=> BOOL____00883 (< 9 course_period__ARRAY__42))))
(assert (and (=> (< 9 course_period__ARRAY__43) BOOL____00885) (=> BOOL____00885 (< 9 course_period__ARRAY__43))))
(assert (and (=> (< 9 course_period__ARRAY__44) BOOL____00887) (=> BOOL____00887 (< 9 course_period__ARRAY__44))))
(assert (and (=> (< 9 course_period__ARRAY__45) BOOL____00889) (=> BOOL____00889 (< 9 course_period__ARRAY__45))))
(assert (and (=> (< 9 course_period__ARRAY__46) BOOL____00891) (=> BOOL____00891 (< 9 course_period__ARRAY__46))))
(assert (and (=> (< 9 course_period__ARRAY__47) BOOL____00893) (=> BOOL____00893 (< 9 course_period__ARRAY__47))))
(assert (and (=> (< 9 course_period__ARRAY__48) BOOL____00895) (=> BOOL____00895 (< 9 course_period__ARRAY__48))))
(assert (and (=> (< 9 course_period__ARRAY__49) BOOL____00897) (=> BOOL____00897 (< 9 course_period__ARRAY__49))))
(assert (and (=> (< 9 course_period__ARRAY__50) BOOL____00899) (=> BOOL____00899 (< 9 course_period__ARRAY__50))))

;
; GOAL
;
(minimize objective :lower 23 :upper 100)

;
; OPTIMIZATION 
;
(check-sat)
(get-objectives)

(exit)
